/*
 * Created on Jul 1, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package vp.graph;

import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

/**
 * @author Matt Holford
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public abstract class AbstractGraph implements Graph
{
	List listeners;
	List vertexList;
	List edgeList;
	int id;
	Object obj;
	/* (non-Javadoc)
	 * @see vp.graph.Graph#addEdge(vp.graph.Edge)
	 */
	public void addEdge(Edge edge)
	{
		boolean okToAdd = true;

		if (containsEdge(edge))
		{
			okToAdd = false;
		}

		if (okToAdd)
		{
			Vertex vert1 = edge.getVertexA();
			Vertex vert2 = edge.getVertexB();
			if (!containsVertex(vert1) || !containsVertex(vert2))
			{
				okToAdd = false;
			}
		}

		if (okToAdd)
		{
			GraphEvent event =
				new GraphEvent(this, GraphEvent.PRE_ADD_EDGE, edge);
			invokeGraphListeners(event);

			edgeList.add(edge);

			event = new GraphEvent(this, GraphEvent.AFT_ADD_EDGE, edge);
			invokeGraphListeners(event);
		}
	}

	/**
	    * Adds the specified GraphListener to the list of listeners.
	    */
	public void addGraphListener(GraphListener listener)
	{
		listeners.add(listener);
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#addVertex(vp.graph.Vertex)
	 */
	public void addVertex(Vertex vertex)
	{
		boolean okToAdd = true;

		if (containsVertex(vertex))
		{
			okToAdd = false;
		}
		if (okToAdd)
		{
			GraphEvent event =
				new GraphEvent(this, GraphEvent.PRE_ADD_VERT, vertex);
			invokeGraphListeners(event);

			vertexList.add(vertex);

			event = new GraphEvent(this, GraphEvent.AFT_ADD_VERT, vertex);
			invokeGraphListeners(event);
		}
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#containsEdge(vp.graph.Edge)
	 */
	public boolean containsEdge(Edge edge)
	{
		return edgeList.contains(edge);
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#containsVertex(vp.graph.Vertex)
	 */
	public boolean containsVertex(Vertex vertex)
	{
		return vertexList.contains(vertex);
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#getEdgeCount()
	 */
	public int getEdgeCount()
	{
		return edgeList.size();
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#getEdgeIterator()
	 */
	public ListIterator getEdgeIterator()
	{
		return edgeList.listIterator();
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#getEdgeList()
	 */
	public List getEdgeList()
	{
		return edgeList;
	}

	public List getEdges(Vertex vertex)
	{
		List edges = new ArrayList();
		ListIterator iter = edgeList.listIterator();
		while (iter.hasNext())
		{
			Edge current = (Edge)iter.next();
			if ((current.getVertexA() == vertex
				|| current.getVertexB() == vertex)
				&& !edges.contains(current))
			{
				edges.add(current);
			}

		}
		return edges;
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#getVertexCount()
	 */
	public int getVertexCount()
	{
		return vertexList.size();
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#getVertexIterator()
	 */
	public ListIterator getVertexIterator()
	{
		return vertexList.listIterator();
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#getVertexList()
	 */
	public List getVertexList()
	{
		return vertexList;
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#removeAllEdges()
	 */
	public void removeAllEdges()
	{
		ListIterator iter = getEdgeIterator();
		while (iter.hasNext())
		{
			Edge edge = (Edge)iter.next();
			removeEdge(edge);
		}
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#removeAllVertices()
	 */
	public void removeAllVertices()
	{
		ListIterator iter = getVertexIterator();
		while (iter.hasNext())
		{
			Vertex vertex = (Vertex)iter.next();
			removeVertex(vertex);
		}
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#removeEdge(vp.graph.Edge)
	 */
	public void removeEdge(Edge edge)
	{
		GraphEvent event = new GraphEvent(this, GraphEvent.PRE_DEL_EDGE, edge);
		invokeGraphListeners(event);

		edgeList.remove(edge);

		event = new GraphEvent(this, GraphEvent.AFT_DEL_EDGE, edge);
		invokeGraphListeners(event);
	}

	/**
	    * Removes the specified graph listener from the list of listeners.
	    */
	public void removeGraphListener(GraphListener listener)
	{
		listeners.remove(listener);
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#removeVertex(vp.graph.Vertex)
	 */
	public void removeVertex(Vertex vertex)
	{
		//First remove all attached edges.
		ListIterator iter = getEdges(vertex).listIterator();
		while (iter.hasNext())
		{
			removeEdge((Edge)iter.next());
		}

		//Now remove the vertex
		GraphEvent event =
			new GraphEvent(this, GraphEvent.PRE_DEL_VERT, vertex);
		invokeGraphListeners(event);

		vertexList.remove(vertex);

		event = new GraphEvent(this, GraphEvent.AFT_DEL_VERT, vertex);
		invokeGraphListeners(event);
	}

	/**
	     * @return
	     */
	public List getListeners()
	{
		return listeners;
	}

	/**
	     * @param list
	     */
	public void setListeners(ArrayList list)
	{
		listeners = list;
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#getObject()
	 */
	public Object getObject()
	{
		return obj;
	}

	/* (non-Javadoc)
	 * @see vp.graph.Graph#setObject(java.lang.Object)
	 */
	public void setObject(Object obj)
	{
		this.obj = obj;
	}

	/**
	 * @param event
	 */
	private void invokeGraphListeners(GraphEvent event)
	{
		int command = event.getCommand();
		ListIterator iter = listeners.listIterator();
		while (iter.hasNext())
		{
			GraphListener listener = (GraphListener)iter.next();
			switch (command)
			{
				case GraphEvent.PRE_ADD_VERT :
					listener.beforeVertexAdded(event);
					break;
				case GraphEvent.AFT_ADD_VERT :
					listener.afterVertexAdded(event);
					break;
				case GraphEvent.PRE_DEL_VERT :
					listener.beforeVertexRemoved(event);
					break;
				case GraphEvent.AFT_DEL_VERT :
					listener.afterVertexRemoved(event);
					break;
				case GraphEvent.PRE_ADD_EDGE :
					listener.beforeEdgeAdded(event);
					break;
				case GraphEvent.AFT_ADD_EDGE :
					listener.afterEdgeAdded(event);
					break;
				case GraphEvent.PRE_DEL_EDGE :
					listener.beforeEdgeRemoved(event);
					break;
				case GraphEvent.AFT_DEL_EDGE :
					listener.afterEdgeRemoved(event);
					break;
				default :
					break;
			}
		}

	}
}
