package vp.graph;

import java.util.ArrayList;
import java.util.ListIterator;

/**
* The interface used for maintaining decorations associated with a structural element.
* Currently, the edge is the only graphical element that uses this interface, but, theoretically, other elements may
* wish to employ it in the future.   It maintains an <code>ArrayList</code> containing all associated decorations
* as well as a list of listeners for additions to and removals from this list.
* It is implemented through the <code>DecorableElementImp</code> class.
*
* @see edu.yale.graph.DecorableElementImp
*/
public interface DecorableElement extends GraphElement
{
	/**
	* Add the specified decoration to the list of decorations.
	*/
	void addDecoration(Decoration dec);

	/**
	* Add the specified decoration listener to the list of decoration listeners.
	*/
	void addDecListener(DecorationListener listener);

	/**
	* Return true if the list of decorations contains the specified decoration, false otherwise.
	*/
	boolean containsDecoration(Decoration dec);
    
    Decoration getDecoration(int index);

	/**
	* Return the number of decorations in the list of decorations.
	*/
	int getDecorationCount();

	/**
	* Return an iteration of the list of decorations.
	*/
	ListIterator getDecorationIterator();

	/**
	* Return the full list of decorations.
	*/
	ArrayList getDecList();

	/**
	* Remove all decorations from the list of decorations.
	*/
	void removeAllDecorations();

	/**
	* Remove the specified decoration from the list of decorations.
	*/
	void removeDecoration(Decoration dec);

	/**
	* Remove the specified decoration listener from the list of decoration listeners.
	*/
	void removeDecListener(DecorationListener listener);
}
