package vp.graph;

/** 
* The common interface for all types of vertices.
* The decoration is one of the basic building blocks this program uses to conceptualize pathway graphs.
* Typically, a decoration represents an entity that modifies or is a part of an edge, e.g. an enzyme or
* expression value associated with a link. 
* Its movement and location are dependent on the element with which it is assocated.   Currently, only edges
* may have decorations, although it possible that in the future other elements will wish to have them.
* <code>Decoration</code> can be implemented generally with <code>DecorationImp</code> or more specifically using 
* <code>PWEnzyme</code>.   Its visual cognate is <code>VisualDecoration</code>.
*
* @see edu.yale.graph.DecorationImp
* @see edu.yale.graph.PWEnzyme
* @see edu.yale.graph.Enzyme
* @see edu.yale.graph.visual.VisualDecoration
*/
public interface Decoration extends GraphElement
{
	/**
	* Returns the element to which the decoration belongs.
	* Currently, only edges may have decorations, although other elements may wish them in the future.
	*
	* @see #setParent(DecorableElement e)
	*/
	DecorableElement getParent();

	/**
	* Sets the element to which the decoration belongs to the specified value.
	* Currently, only edges may have decorations, although other elements may wish them in the future.
	*
	* @see #getParent()
	*/
	void setParent(DecorableElement element);

	Graph getGraph();
    
    void setGraph(Graph graph);
    
    int getID();
    
    void setID(int id);
    
    Object getObject();
    
    void setObject(Object obj);
}
