package vp.graph;

import java.util.EventListener;

/**
* An event listener that detects when decorations are to be added or removed from
* their parent element.  Currently, <code>VisualEdge</code> is the only class to implement this interface.
*
* @see edu.yale.graph.visual.VisualEdge
*/
public interface DecorationListener extends EventListener
{
	/**
	* Delegates events to occur after a decoration has been added.
	*/
	void afterDecorationAdded(GraphEvent event);

	/**
	* Delegates events to occur after a decoration has been removed.
	*/
	void afterDecorationRemoved(GraphEvent event);

	/**
	* Delegates events to occur before a decoration is added.
	*/
	void beforeDecorationAdded(GraphEvent event);

	/**
	* Delegates events to occur before a decoration is removed.
	*/
	void beforeDecorationRemoved(GraphEvent event);
}
