package vp.graph;

import vp.util.GraphUtilities;
import vp.visual.type.InvisibleVertex;

/**
* A specific type of <code>PWVertex</code> that is attached to a link.
* It is completely invisible and its location is dependent on the link to which it is attached.
* Despite this, it is still technically a vertex because it serves as an endpoint to a link.
* Dummy vertices are used in the pathway graph to represent when a link connects to another link rather than a
* specific compound or pathway, e.g. to represent by-products of a reaction.  They employ a specific predefined
* visual type, <code>InvisibleVertex</code>, and can be identified in the database or XML documents
* by their default compound name, "dummy".
*
* @see edu.yale.graph.PWVertex
* @see edu.yale.graph.type.InvisibleVertex
*/
public class DummyVertex extends PathwayVertex
{
	public DummyVertex(Graph graph, int xPosition, int yPosition)
	{
		this(graph, xPosition, yPosition, -1, -1);
	}

	/**
	 * @param graph
	 * @param id
	 * @param dcl
	 * @param xPosition
	 * @param yPosition
	 * @param onLink
	 * @param onLinkPosition
	 */
	public DummyVertex(
		Graph graph,
		int xPosition,
		int yPosition,
		int onLink,
		double onLinkPosition)
	{
		super(
			graph,
			-3,
			GraphUtilities.HEADLESS_VERTEX,
			new InvisibleVertex(),
			xPosition,
			yPosition,
			onLink,
			onLinkPosition);
	}

}
