package vp.graph;

/**
* The common interface for all types of edges.
* The edge is one of the basic building blocks this program uses to conceptualize pathway graphs.
* Typically, an edge represents a connection between two vertices, which are usually compounds or other pathways.
* An edge's location is dependent on the positions of its two endpoints, although a <code>DummyVertex</code>
* may be located relative to the edge itself.  An edge can also be decorated, typically with graphical entities
* representing enzymes and expression values.   These entities move in conjunction with the edge and their location
* is based on the location of their parent edge.
* <code>Edge</code> can be implemented generally with <code>EdgeImp</code> or more specifically using <code>
* Link</code>.   Its visual cognate is <code>VisualEdge</code>.
*
* @see edu.yale.graph.EdgeImp
* @see edu.yale.graph.Link
* @see edu.yale.graph.visual.VisualEdge
*/
public interface Edge extends GraphElement
{
	/**
	* Returns the direction the edge is pointing.
	*
	* @see #setDirection(String direction)
	*/
	String getDirection();

	/**
	* Returns vertex A of the edge.
	*
	* @see #setVertexA(Vertex v)
	*/
	Vertex getVertexA();

	/**
	* Returns vertex B of the edge.
	*
	* @see #setVertexB(Vertex v)
	*/
	Vertex getVertexB();

	int getID();
    
    void setID(int id);
    
    Object getObject();
    
    void setObject(Object obj);
    
    Graph getGraph();

	/**
	* Sets the direction the edge is pointing to the specified direction.
	*
	* @see #getDirection()
	*/
	void setDirection(String direction);

	/**
	* Sets vertex A of the edge to the vertex specified.
	*
	* @see #getVertexA()
	*/
	void setVertexA(Vertex vertex);

	/**
	* Sets vertex B of the edge to the vertex specified.
	*
	* @see #getVertexB()
	*/
	void setVertexB(Vertex vertex);
    
    Vertex getOppositeVertex(Vertex vertex);
}
