package vp.graph;

import java.util.EventListener;

/**
* An event listener that detects when vertices and edges are to be added or removed from
* the pathway graph.  Currently, <code>VisualGraph</code> is the only class to implement this interface.
*
* @see edu.yale.graph.visual.VisualGraph
*/
public interface GraphListener extends EventListener
{
	/**
	* Delegates events to occur before an edge is added.
	*/
	void beforeEdgeAdded(GraphEvent event);

	/**
	* Delegates events to occur before an edge is removed.
	*/
	void beforeEdgeRemoved(GraphEvent event);

	/**
	* Delegates events to occur before a vertex is added.
	*/
	void beforeVertexAdded(GraphEvent event);

	/**
	* Delegates events to occur before a vertex is removed.
	*/
	void beforeVertexRemoved(GraphEvent event);

	/**
	* Delegates events to occur after an edge is added.
	*
	* @throws java.lang.Exception	The listener will not allow the addition and has blocked it.
	*/
	void afterEdgeAdded(GraphEvent event);

	/**
	* Delegates events to occur after an edge is removed.
	*
	* @throws java.lang.Exception	The listener will not allow the addition and has blocked it.
	*/
	void afterEdgeRemoved(GraphEvent event);

	/**
	* Delegates events to occur after a vertex is added.
	*
	* @throws java.lang.Exception	The listener will not allow the addition and has blocked it.
	*/
	void afterVertexAdded(GraphEvent event);

	/**
	* Delegates events to occur after a vertex is removed.
	*
	* @throws java.lang.Exception	The listener will not allow the addition and has blocked it.
	*/
	void afterVertexRemoved(GraphEvent event);
}
