package vp.graph;

/** A basic implementation of <code>LabeledElement</code>.
*
* @see edu.yale.graph.LabeledElement
*/
public class LabeledElementImp implements LabeledElement
{
	private String label;
    public static final String DEF_LABEL = "<Unlabeled>";
	/**
	* Creates a new labeled element with the default label.
	*/
	public LabeledElementImp()
	{
		this(DEF_LABEL);
	}

	/**
	* Creates a new labeled element with the specified label.
	*/
	public LabeledElementImp(String label)
	{
		this.label = label;
	}

	/**
	* Returns the element's label.
	*
	* @see #setLabel(String label)
	*/
	public String getLabel()
	{
		return label;
	}

	/**
	* Returns true if a label has been specified.
	*/
	public boolean hasLabel()
	{
		return (label != null && label != DEF_LABEL);
	}

	/**
	* Sets the element's label to the specified string.
	*
	* @see #getLabel()
	*/
	public void setLabel(String label)
	{
		try
		{
			this.label = label;
		}
		catch (NullPointerException e)
		{
			this.label = DEF_LABEL;
		}
	}

	/**
	* Returns the specified label or the default label if none has been specified.
	*/
	public String toString()
	{
		return label;
	}
}
