package vp.graph;

/** 
* The interface for structural elements that link to other elements or 
* outside information.  Theoretically, this can be vertices, edges and decorations, although it has not yet been
* implemented for any specific element.  It is implemented with <code>LinkableElementImp</code>.
*
* @see edu.yale.graph.LinkableElementImp
*/
public interface LinkableElement extends GraphElement
{
	/** 
	* A string describing the default link if none is specified.  Currently this is the empty string.
	*/
	String DEFAULT_LINK = "";

	/**
	* Returns a string representing the element's link.
	*
	* @see #setLink(String link)
	*/
	String getLink();

	/**
	* Returns true if a link has been specified.
	*/
	boolean hasLink();

	/**
	* Returns true if linking has been enabled.
	*
	* @see #setEnabled(boolean enabled)
	*/
	boolean getLinkEnabled();

	/**
	* Enables or disables linking for the element.
	*
	* @see #isEnabled()
	*/
	void setLinkEnabled(boolean enabled);

	/**
	* Sets the string representing the element's link to the specified string.
	*
	* @see #getLink()
	*/
	void setLink(String link);
}
