package vp.graph;

/** 
* A basic implementation of <code>LinkableElement</code>.
*
* @see edu.yale.graph.LinkableElement
*/
public class LinkableElementImp implements LinkableElement
{
	private boolean linkEnabled;
	private String link;

	/**
	* Creates a new linkable element with the specified string representing the link.
	* By default, linkability is enabled.
	*/
	public LinkableElementImp(String link)
	{
		this.link = link;
		linkEnabled = true;
	}

	/**
	* Returns a string representing the element's link.
	*
	* @see #setLink(String link)
	*/
	public String getLink()
	{
		return link;
	}

	/**
	* Returns true if a link has been specified.
	*/
	public boolean hasLink()
	{
		return (
			link != null
				&& link != LinkableElement.DEFAULT_LINK
				&& linkEnabled);
	}

	/**
	* Returns true if linking has been enabled.
	*
	* @see #setEnabled(boolean enabled)
	*/
	public boolean getLinkEnabled()
	{
		return linkEnabled;
	}

	/**
	* Enables or disables linking for the element.
	*
	* @see #isEnabled()
	*/
	public void setLinkEnabled(boolean flag)
	{
		linkEnabled = flag;
	}

	/**
	* Sets the string representing the element's link to the specified string.
	*
	* @see #getLink()
	*/
	public void setLink(String link)
	{
		this.link = link;
	}
}
