package vp.graph;

/** 
* The interface for structural elements that have tooltips.  
* Theoretically, this can be vertices, edges and decorations, although it has not yet been
* implemented for any specific element.  It is implemented with <code>ToolTippedElementImp</code>.
*
* @see edu.yale.graph.ToolTippedElementImp
*/
public interface TooltipElement extends GraphElement
{
	/** 
	* The default tooltip if none is specified.  Currently this is "<No tooltip assigned>".
	*/
	String DEF_TIP = "<No tooltip assigned>";

	/**
	* Returns the element's tooltip.
	*
	* @see #setToolTip(String tooltip)
	*/
	String getToolTip();

	/**
	* Returns true if a tooltip has been specified.
	*/
	boolean hasToolTip();

	/**
	* Returns true if tooltipping has been enabled.
	*
	* @see #setEnabled(boolean enabled)
	*/
	boolean isTipEnabled();

	/**
	* Enables or disables tooltipping for the element.
	*
	* @see #isEnabled()
	*/
	void setTipEnabled(boolean enabled);

	/**
	* Sets the element's tooltip to the specified value.
	*
	* @see #getToolTip()
	*/
	void setToolTip(String tooltip);
}
