package vp.graph;

/** 
* A basic implementation of <code>ToolTippedElement</code>.
*
* @see edu.yale.graph.ToolTippedElement
*/
public class TooltipElementImp implements TooltipElement
{
	private boolean tipEnabled;
	private String toolTip;

	/**
	* Creates a new element with the specified tooltip.
	* By default, tooltips are enabled.
	*/
	public TooltipElementImp(String toolTip)
	{
		tipEnabled = false;
		this.toolTip = toolTip;
	}

	/**
	* Returns the element's tooltip.
	*
	* @see #setToolTip(String tooltip)
	*/
	public String getToolTip()
	{
		return toolTip;
	}

	/**
	* Returns true if a tooltip has been specified.
	*/
	public boolean hasToolTip()
	{
		return (
			(toolTip != null)
				&& (toolTip != TooltipElement.DEF_TIP)
				&& (tipEnabled));
	}

	/**
	* Returns true if tooltipping has been enabled.
	*
	* @see #setEnabled(boolean enabled)
	*/
	public boolean isTipEnabled()
	{
		return tipEnabled;
	}

	/**
	* Enables or disables tooltipping for the element.
	*
	* @see #isEnabled()
	*/
	public void setTipEnabled(boolean flag)
	{
		tipEnabled = flag;
	}

	/**
	* Sets the element's tooltip to the specified value.
	*
	* @see #getToolTip()
	*/
	public void setToolTip(String toolTip)
	{
		this.toolTip = toolTip;
	}
}
