/*
 * Created on May 5, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package vp.graph;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import vp.gui.VPSettings;
import vp.visual.type.Type;

/**
 * @author Matt Holford
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class VPGradient
{
	private static BufferedImage bImage;
	private Color maxColor;
	private Color minColor;
	private Color midColor;
	private Color nullColor;
	private double maxValue;
	private double minValue;
	private static final Dimension DEF_DIM = new Dimension(5, 150);

	public VPGradient(
		Color minColor,
		Color maxColor,
		Color nullColor,
		double maxValue,
		double minValue)
	{
		this(minColor, maxColor, null, nullColor, maxValue, minValue);
	}

	public VPGradient(
		Color minColor,
		Color maxColor,
		Color midColor,
		Color nullColor,
		double maxValue,
		double minValue)
	{
		this.maxColor = maxColor;
		this.minColor = minColor;
		this.midColor = midColor;
		this.nullColor = nullColor;
		this.maxValue = maxValue;
		this.minValue = minValue;
		bImage =
			getGradientImage(minColor, maxColor, midColor, DEF_DIM);
	}

	public static BufferedImage getGradientImage(
		Color minColor,
		Color maxColor,
		Color midColor,
		Dimension size)
	{
		BufferedImage rImage =
			new BufferedImage(
				size.width,
				size.height,
				BufferedImage.TYPE_INT_ARGB);
		Graphics2D rig = rImage.createGraphics();
		if (midColor == null)
		{
			rig.setPaint(
				new GradientPaint(0, 0, maxColor, 0, size.height, minColor));
			rig.fillRect(0, 0, size.width, size.height);
		}
		else
		{
			rig.setPaint(
				new GradientPaint(
					0,
					0,
					maxColor,
					0,
					size.height / 2,
					midColor));
			rig.fillRect(0, 0, size.width, size.height / 2);
			rig.setPaint(
				new GradientPaint(
					0,
					size.height / 2,
					midColor,
					0,
					size.height,
					minColor));
			rig.fillRect(0, size.height / 2, size.width, size.height / 2);
		}
		return rImage;
	}

	public Type getType(String value)
	{
		double expValue;
		Color color;
		try
		{
			expValue = Double.parseDouble(value);
		}
		catch (NumberFormatException e)
		{
			expValue = -99999d;
		}
		if (expValue > -99999)
		{
			double percent = 1 - (expValue - minValue) / (maxValue - minValue);
			color = getColorAtValue(percent);
		}
		else
        {
            color = nullColor;
        }
		Type newType =
			new Type(
				Color.BLACK,
				color,
				Color.BLACK,
				VPSettings.SR_FONT,
				new GeneralPath(new Rectangle2D.Double(5, 5, 10, 10)),
				"Rectangle",
				"temp",
				"temp");
		return newType;
	}
	/**
	 * @param percent
	 * @return
	 */
	public Color getColorAtValue(double percent)
	{
		int vHeight = (int) ((DEF_DIM.height - 1) * percent);
		return new Color(bImage.getRGB(0, vHeight));
	}

	/**
	 * @return
	 */
	public Color getMaxColor()
	{
		return maxColor;
	}

	/**
	 * @return
	 */
	public double getMaxValue()
	{
		return maxValue;
	}

	/**
	 * @return
	 */
	public Color getMidColor()
	{
		return midColor;
	}

	/**
	 * @return
	 */
	public Color getMinColor()
	{
		return minColor;
	}

	/**
	 * @return
	 */
	public double getMinValue()
	{
		return minValue;
	}

	/**
	 * @return
	 */
	public Color getNullColor()
	{
		return nullColor;
	}

	/**
	 * @param color
	 */
	public void setMaxColor(Color color)
	{
		maxColor = color;
	}

	/**
	 * @param d
	 */
	public void setMaxValue(double dbl)
	{
		maxValue = dbl;
	}

	/**
	 * @param color
	 */
	public void setMidColor(Color color)
	{
		midColor = color;
	}

	/**
	 * @param color
	 */
	public void setMinColor(Color color)
	{
		minColor = color;
	}

	/**
	 * @param d
	 */
	public void setMinValue(double dbl)
	{
		minValue = dbl;
	}

	/**
	 * @param color
	 */
	public void setNullColor(Color color)
	{
		nullColor = color;
	}

}
