package vp.gui;

import vp.gui.components.VPColorChooser;
import vp.gui.components.VPFileChooser;
import vp.main.MainFrame;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Point;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.UIManager;
import javax.swing.plaf.ColorUIResource;

public class VPSettings
{
	/**
	 * @author Matt Holford
	 *
	 * To change the template for this generated type comment go to
	 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
	 */
	private static final String FONT_FAM = "Palatino Linotype";
	public static final String NEW_LINE = "\n";
	public static final Font B_FONT = new Font(FONT_FAM, Font.BOLD, 13);
	public static final Font SB_FONT = new Font(FONT_FAM, Font.BOLD, 12);
	public static final Font R_FONT = new Font(FONT_FAM, Font.PLAIN, 12);
	public static final Font SR_FONT = new Font(FONT_FAM, Font.PLAIN, 10);
	public static final Map ORGS_MAP;
	static {
		ORGS_MAP = new HashMap();
		ORGS_MAP.put("Homo sapiens", "Hs");
		ORGS_MAP.put("Rattus norvegicus", "Rn");
		ORGS_MAP.put("Mus musculus", "Mm");
	}
	public static final List ORGANISMS =
		Collections.unmodifiableList(new ArrayList(ORGS_MAP.keySet()));

	private VPSettings()
	{}

	public static Point centerChildPoint(Component parent, Component child)
	{
		int xPos =
			parent.getBounds().x
				+ (parent.getBounds().width / 2)
				- (child.getBounds().width / 2);
		int yPos =
			parent.getBounds().y
				+ (parent.getBounds().height / 2)
				- (child.getBounds().height / 2);
		return new Point(xPos, yPos);
	}

	public static JFileChooser getVPFileChooser(
		File defaultPath,
		MainFrame parent)
	{
		final Hashtable hash = new Hashtable();
		tempChangeUI(
			"ToolBarUI",
			"javax.swing.plaf.metal.MetalToolBarUI",
			hash);
		tempChangeUI(
			"ToolBar.background",
			new ColorUIResource(VPSettings.Colors.SIDE_PANEL),
			hash);
		tempChangeUI(
			"Panel.background",
			new ColorUIResource(VPSettings.Colors.SIDE_PANEL),
			hash);
		tempChangeUI("ToolBar.font", VPSettings.SB_FONT, hash);
		tempChangeUI("ComboBox.font", VPSettings.SB_FONT, hash);
		tempChangeUI("Button.font", VPSettings.SB_FONT, hash);
		tempChangeUI("List.font", VPSettings.SB_FONT, hash);
		tempChangeUI("Panel.font", VPSettings.SB_FONT, hash);
		tempChangeUI("TextField.font", VPSettings.R_FONT, hash);
		tempChangeUI("TextArea.font", VPSettings.R_FONT, hash);
		tempChangeUI("Label.font", VPSettings.SB_FONT, hash);
		tempChangeUI("ToggleButton.font", VPSettings.SB_FONT, hash);

		VPFileChooser chooser = new VPFileChooser(parent);
		chooser.setCurrentDirectory(defaultPath);
		Runnable reset = new Runnable()
		{
			public void run()
			{
				try
				{
					Thread.sleep(200);
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
				Iterator iter = hash.keySet().iterator();
				while (iter.hasNext())
				{
					Object next = iter.next();
					UIManager.put(next, hash.get(next));
				}
			}
		};
		Thread resetter = new Thread(reset);
		resetter.start();
		return chooser;
	}

	public static Color getVPColorChooser(
		JComponent component,
		String title,
		Color color)
	{
		final Hashtable hash = new Hashtable();
		tempChangeUI(
			"ToolBarUI",
			"javax.swing.plaf.metal.MetalToolBarUI",
			hash);
		tempChangeUI(
			"ToolBar.background",
			new ColorUIResource(VPSettings.Colors.SIDE_PANEL),
			hash);
		tempChangeUI(
			"Panel.background",
			new ColorUIResource(VPSettings.Colors.SIDE_PANEL),
			hash);
		tempChangeUI("ToolBar.font", VPSettings.SB_FONT, hash);
		tempChangeUI("ComboBox.font", VPSettings.SB_FONT, hash);
		tempChangeUI("Button.font", VPSettings.SB_FONT, hash);
		tempChangeUI("List.font", VPSettings.SB_FONT, hash);
		tempChangeUI("Panel.font", VPSettings.SB_FONT, hash);
		tempChangeUI("TextField.font", VPSettings.R_FONT, hash);
		tempChangeUI("TextArea.font", VPSettings.R_FONT, hash);
		tempChangeUI("Label.font", VPSettings.SB_FONT, hash);
		tempChangeUI("ToggleButton.font", VPSettings.SB_FONT, hash);
		tempChangeUI("Slider.background", VPSettings.Colors.SIDE_PANEL, hash);
		tempChangeUI("Spinner.font", VPSettings.R_FONT, hash);
		tempChangeUI(
			"RadioButtonUI",
			"javax.swing.plaf.metal.MetalRadioButtonUI",
			hash);
		tempChangeUI(
			"RadioButton.background",
			VPSettings.Colors.SIDE_PANEL,
			hash);

		Runnable reset = new Runnable()
		{
			public void run()
			{
				try
				{
					Thread.sleep(200);
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
				Iterator iter = hash.keySet().iterator();
				while (iter.hasNext())
				{
					Object next = iter.next();
					UIManager.put(next, hash.get(next));
				}
			}
		};
		Thread resetter = new Thread(reset);
		resetter.start();
		Color outColor = VPColorChooser.showDialog(component, title, color);
		return outColor;
	}
	/**
	 * @param string
	 * @param string2
	 * @param hs
	 */
	private static void tempChangeUI(Object key, Object value, Hashtable hash)
	{
		hash.put(key, UIManager.get(key));
		UIManager.put(key, value);
	}

	public static class Colors
	{
		private Colors()
		{}
		public static final Color D_RED = new Color(128, 0, 0);
		public static final Color M_RED = new Color(192, 0, 0);
		public static final Color M_GREEN = new Color(0, 192, 0);
		public static final Color AQUA = new Color(220, 220, 255);
		public static final Color L_BLUE = new Color(240, 240, 245);
		public static final Color LM_BLUE = new Color(220, 220, 240);
		public static final Color SKY_BLUE = new Color(210, 235, 255);
		public static final Color M_BLUE = new Color(190, 190, 240);
		public static final Color MD_BLUE = new Color(130, 130, 200);
		public static final Color D_BLUE = new Color(32, 32, 100);
		public static final Color CLEAR = new Color(255, 255, 255, 0);
		public static final Color C_GRAY = new Color(224, 224, 224, 84);
		public static final Color C_SKY_BLUE = new Color(200, 255, 255, 112);
		public static final Color BEIGE = new Color(245, 245, 220);
		public static final Color M_YELLOW = new Color(218, 165, 32);

		public static final Color SIDE_PANEL = BEIGE;
	}
}
