package vp.gui.components;

import vp.gui.*;
import vp.main.MainFrame;
import vp.util.ActionCenter;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
import java.util.StringTokenizer;
import javax.swing.JDialog;
import javax.swing.JSeparator;

public class DialogFactory extends JDialog
{
	private ActionListener altListener;
	private static Listener listener;
	private String okCommand;
	private String canCommand;

	public static final int OK_ONLY = 1000;
	public static final int CAN_ONLY = 1001;
	public static final int OK_CANCEL = 1002;

	private DialogFactory(
		MainFrame parent,
		BufferedImage icon,
		String title,
		String header,
		String body,
		int buttons,
		String okCommand,
		String canCommand,
		ActionListener altListener)
	{
		super(parent, title, true);
		this.altListener = altListener;
		this.okCommand = okCommand;
		this.canCommand = canCommand;
		listener = new Listener();

		getContentPane().setLayout(new GridBagLayout());
		getContentPane().setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints superc = new GridBagConstraints();
		superc.anchor = GridBagConstraints.WEST;
		superc.insets = new Insets(5, 5, 5, 5);
		superc.gridwidth = 2;

		getContentPane().add(new DialogHeader(icon, header), superc);

		superc.gridy = 1;
		superc.fill = GridBagConstraints.BOTH;
		getContentPane().add(new JSeparator(JSeparator.HORIZONTAL), superc);

		superc.gridy = 2;
		superc.anchor = GridBagConstraints.CENTER;
		superc.insets = new Insets(0, 5, 5, 5);
		VPRLabel bodyLabel = new VPRLabel();
		String bodyText = "<html>";
		StringTokenizer toke = new StringTokenizer(body, "\n");
		while (toke.hasMoreTokens())
		{
			bodyText += toke.nextToken() + "<br>";
		}
		bodyText = bodyText.substring(0, bodyText.lastIndexOf("<br>"));
		bodyLabel.setText(bodyText);
		getContentPane().add(bodyLabel, superc);

		superc.gridy = 3;
		superc.insets = new Insets(5, 5, 5, 5);
		String command = "";
		switch (buttons)
		{
			case OK_ONLY :
				superc.fill = GridBagConstraints.NONE;
				command =
					(okCommand == null)
						? ActionCenter.DISPOSE
						: ActionCenter.OK;
				getContentPane().add(
					new VPButton("OK", command, listener),
					superc);
				break;
			case CAN_ONLY :
				superc.fill = GridBagConstraints.NONE;
				command =
					(canCommand == null)
						? ActionCenter.DISPOSE
						: ActionCenter.CANCEL;
				getContentPane().add(
					new VPButton("Cancel", command, listener),
					superc);
				break;
			case OK_CANCEL :
				superc.gridwidth = 1;
				command =
					(okCommand == null)
						? ActionCenter.DISPOSE
						: ActionCenter.OK;
				getContentPane().add(
					new VPButton("OK", command, listener),
					superc);
				superc.gridx = 1;
				command =
					(canCommand == null)
						? ActionCenter.DISPOSE
						: ActionCenter.CANCEL;
				getContentPane().add(
					new VPButton("Cancel", command, listener),
					superc);
				break;
			default :
				break;
		}

		pack();
		setResizable(false);
		setLocation(
			(parent.getBounds().x + (parent.getBounds().width / 2))
				- (getBounds().width / 2),
			(parent.getBounds().y + (parent.getBounds().height / 2))
				- (getBounds().height / 2));
		addWindowListener(listener);
	}

	private class Listener implements ActionListener, WindowListener
	{
		Listener()
		{}
		public void actionPerformed(ActionEvent event)
		{
			String command = event.getActionCommand();
			if (command.equals(ActionCenter.DISPOSE))
			{
				dispose();
			}
			else if (command.equals(ActionCenter.OK))
			{
				ActionEvent newEvent =
					new ActionEvent(
						event.getSource(),
						event.getID(),
						okCommand,
						event.getWhen(),
						event.getModifiers());
				altListener.actionPerformed(newEvent);
				dispose();
			}
			else if (command.equals(ActionCenter.CANCEL))
			{
				ActionEvent newEvent =
					new ActionEvent(
						event.getSource(),
						event.getID(),
						canCommand,
						event.getWhen(),
						event.getModifiers());
				altListener.actionPerformed(newEvent);
				dispose();
			}
		}

		public void windowOpened(WindowEvent event)
		{}
		public void windowClosing(WindowEvent event)
		{
			dispose();
		}
		public void windowClosed(WindowEvent event)
		{}
		public void windowIconified(WindowEvent event)
		{}
		public void windowDeiconified(WindowEvent event)
		{}
		public void windowActivated(WindowEvent event)
		{}
		public void windowDeactivated(WindowEvent event)
		{}
	}

	public static void createDialog(
		MainFrame parent,
		BufferedImage icon,
		String title,
		String header,
		String body,
		int buttons)
	{
		createDialog(
			parent,
			icon,
			title,
			header,
			body,
			buttons,
			null,
			null,
			null);
	}

	public static void createDialog(
		MainFrame parent,
		BufferedImage icon,
		String title,
		String header,
		String body,
		int buttons,
		String okCommand,
		String canCommand,
		ActionListener altListener)
	{
		DialogFactory newDialog =
			new DialogFactory(
				parent,
				icon,
				title,
				header,
				body,
				buttons,
				okCommand,
				canCommand,
				altListener);
		newDialog.show();
	}
	/**
	 * @return
	 */
	public ActionListener getAltListener()
	{
		return altListener;
	}

	/**
	 * @return
	 */
	public String getCanCommand()
	{
		return canCommand;
	}

	/**
	 * @return
	 */
	public String getOkCommand()
	{
		return okCommand;
	}

	/**
	 * @param listener
	 */
	public void setAltListener(ActionListener listener)
	{
		altListener = listener;
	}

	/**
	 * @param string
	 */
	public void setCanCommand(String string)
	{
		canCommand = string;
	}

	/**
	 * @param string
	 */
	public void setOkCommand(String string)
	{
		okCommand = string;
	}

}
