package vp.gui.components;

import vp.graph.DataClass;
import vp.graph.PathwayDecoration;
import vp.graph.PathwayElement;
import vp.gui.VPSettings;
import vp.util.DecorationChangeEvent;
import vp.util.DecorationChangeListener;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.EventListener;
import java.util.List;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JViewport;
import javax.swing.event.ListSelectionListener;

public class EnzymeViewer
	extends JTabbedPane
	implements DecorationChangeListener
{
	private JList ecNumberList;
	private JList enzNameList;
	private static JScrollPane ecScroller;
	private static JScrollPane nameScroller;
	private List enzymes;
	private EventListener listener;
	private DefaultListModel ecModel, nameModel;

	public static final int ID_VIEW = 0;
	public static final int NAME_VIEW = 1;

	public EnzymeViewer(List enzymes, int rows, ListSelectionListener listener)
	{
		this.enzymes = enzymes;
		List ecNumbers = new ArrayList();
		List enzymeNames = new ArrayList();
		this.listener = listener;

		for (int i = 0; i < enzymes.size(); i++)
		{
			DataClass current = ((PathwayElement)enzymes.get(i)).getDataClass();
			ecNumbers.add(current.get("ECNumber"));
			enzymeNames.add(current.get("Name"));
		}

		ecModel = new DefaultListModel();
		for (int i = 0; i < ecNumbers.size(); i++)
			ecModel.add(ecModel.size(), ecNumbers.get(i));
		ecNumberList = new JList(ecModel);
		ecNumberList.setFont(VPSettings.R_FONT);

		ecNumberList.setVisibleRowCount(rows);
		ecNumberList.addListSelectionListener(listener);
		int listWidth =
			ecNumberList.getFontMetrics(VPSettings.R_FONT).stringWidth(
				"<No compound selected>");
		int listHeight =
			ecNumberList.getFontMetrics(VPSettings.R_FONT).getHeight();

		ecScroller =
			new JScrollPane(
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		JViewport view = new JViewport();
		view.setPreferredSize(new Dimension(listWidth, listHeight * 3));
		view.setView(ecNumberList);
		ecScroller.setViewport(view);

		nameModel = new DefaultListModel();
		for (int i = 0; i < enzymeNames.size(); i++)
			nameModel.add(nameModel.size(), enzymeNames.get(i));
		enzNameList = new JList(nameModel);
		enzNameList.setFont(VPSettings.R_FONT);
		enzNameList.setVisibleRowCount(rows);
		enzNameList.addListSelectionListener(listener);

		nameScroller =
			new JScrollPane(
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		JViewport view2 = new JViewport();
		view2.setPreferredSize(view.getPreferredSize());
		view2.setView(enzNameList);
		nameScroller.setViewport(view2);

		setFont(VPSettings.B_FONT);
		addTab("E.C.", ecScroller);
		addTab("Name", nameScroller);
		setPreferredSize(new Dimension(163, (listHeight * rows) + 48));
		setMinimumSize(new Dimension(163, (listHeight * rows) + 48));
		setMaximumSize(new Dimension(163, (listHeight * rows) + 48));

		if (enzymes.size() > 0)
			ecNumberList.setSelectedIndex(0);

	}

	public PathwayDecoration getSelEnzyme()
	{
		int s = ecNumberList.getSelectedIndex();
		return (s >= 0) ? (PathwayDecoration)enzymes.get(s) : null;
	}

	public JList getEcNumberList()
	{
		return ecNumberList;
	}

	public JList getEnzNameList()
	{
		return enzNameList;
	}

	public void changeType(int display)
	{
		if (display == ID_VIEW)
		{
			int selection = ecNumberList.getSelectedIndex();
			enzNameList.setSelectedIndex(selection);
		}
		else if (display == NAME_VIEW)
		{
			int selection = enzNameList.getSelectedIndex();
			ecNumberList.setSelectedIndex(selection);
		}
	}
	/**
	 * @param list
	 */
	public void setEnzNameList(JList list)
	{
		enzNameList = list;
	}

	/**
	 * @param list
	 */
	public void setEcNumberList(JList list)
	{
		ecNumberList = list;
	}

	/**
	 * @return
	 */
	public List getEnzymes()
	{
		return enzymes;
	}

	/**
	 * @return
	 */
	public EventListener getListener()
	{
		return listener;
	}

	/**
	 * @param list
	 */
	public void setEnzymes(ArrayList list)
	{
		enzymes = list;
	}

	/**
	 * @param listener
	 */
	public void setListener(EventListener listener)
	{
		this.listener = listener;
	}

	/* (non-Javadoc)
	 * @see vp.util.DecorationChangeListener#decorationsChanged(vp.util.DecorationChangeEvent)
	 */
	public void decorationsChanged(DecorationChangeEvent de)
	{}

	/* (non-Javadoc)
	 * @see vp.util.DecorationChangeListener#addDecoration(vp.util.DecorationChangeEvent)
	 */
	public void addDecoration(DecorationChangeEvent de)
	{
		PathwayDecoration dec = (PathwayDecoration)de.getDecoration();
		enzymes.add(dec);
		ecModel.add(ecModel.size(), dec.getDataClass().get("ECNumber"));
		nameModel.add(nameModel.size(), dec.getDataClass().get("Name"));
		ecNumberList.setSelectedIndex(ecModel.size() - 1);

		//enzNameList.setSelectedIndex(nameModel.size());
	}

	/* (non-Javadoc)
	 * @see vp.util.DecorationChangeListener#removeDecoration(vp.util.DecorationChangeEvent)
	 */
	public void removeDecoration(DecorationChangeEvent de)
	{
		PathwayDecoration dec = (PathwayDecoration)de.getDecoration();
		enzymes.remove(dec);
		int removeIndex = ecModel.indexOf(dec.getDataClass().get("ECNumber"));
		ecModel.remove(removeIndex);
		nameModel.remove(removeIndex);
		if (ecModel.size() > removeIndex)
			ecNumberList.setSelectedIndex(removeIndex);
		else
			ecNumberList.setSelectedIndex(removeIndex -1);
	}

}
