package vp.gui.components;

import vp.graph.DummyVertex;
import vp.graph.PathwayDecoration;
import vp.graph.PathwayEdge;
import vp.graph.PathwayElement;
import vp.graph.PathwayVertex;
import vp.gui.VPSettings;
import vp.main.MainFrame;
import vp.visual.type.Type;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButton;
import javax.swing.plaf.metal.MetalRadioButtonUI;

public class TypeButton extends JRadioButton
{
	private PathwayElement element;
	private MainFrame parent;
	private JPopupMenu pop;
	private JLabel popImage;
	private Ellipse2D.Double zoomCircle;
	private TypeButton thisButton;
	private Listener listener;

	public static final int VERTEX = 0;
	public static final int EDGE = 1;
	public static final int DECORATION = 2;

	public TypeButton(MainFrame parent, PathwayElement element)
	{
		this.parent = parent;
		this.element = element;
		thisButton = this;
		listener = new Listener();
		setBackground(VPSettings.Colors.SIDE_PANEL);
		setUI(new MetalRadioButtonUI());

		//setPreferredSize(new Dimension(48, 48));
		BufferedImage bi =
			new BufferedImage(50, 50, BufferedImage.TYPE_INT_RGB);

		if (element instanceof PathwayEdge)
		{
			initEdge(element, bi);
		}

		else if (element instanceof PathwayVertex)
		{
			initVert(element, bi);
		}

		else if (element instanceof PathwayDecoration)
		{
			initDec(element, bi);
		}

		BufferedImage returnImage =
			new BufferedImage(54, 54, BufferedImage.TYPE_INT_RGB);
		Graphics2D rg2 = returnImage.createGraphics();
		rg2.drawImage(bi, 2, 2, null);
		rg2.setColor(Color.white);
		rg2.drawRect(0, 0, 53, 53);
		rg2.drawRect(1, 1, 51, 51);
		ImageIcon icon = new ImageIcon(returnImage);
		setIcon(icon);

		BufferedImage selectedImage =
			new BufferedImage(54, 54, BufferedImage.TYPE_INT_RGB);
		Graphics2D sg2 = selectedImage.createGraphics();
		sg2.setColor(VPSettings.Colors.MD_BLUE);
		sg2.drawImage(bi, 2, 2, null);
		sg2.drawRect(0, 0, 53, 53);
		sg2.drawRect(1, 1, 51, 51);

		ImageIcon selectedIcon = new ImageIcon(selectedImage);
		setSelectedIcon(selectedIcon);
	}
	/**
	 * @param element
	 * @param bi
	 */
	private void initDec(PathwayElement element, BufferedImage bi)
	{
		PathwayDecoration dec = (PathwayDecoration)element;
		boolean fullX, fullY;

		Rectangle bounds = dec.getBounds();
		double xOffset = 0 - bounds.getX();
		double yOffset = 0 - bounds.getY();
		AffineTransform at = new AffineTransform();
		at.translate(xOffset, yOffset);
		dec.getGeneralPath().transform(at);

		BufferedImage vImage =
			new BufferedImage(
				bounds.width + 1,
				bounds.height + 1,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D vg2 = vImage.createGraphics();
		vg2.setColor(Color.white);
		vg2.fillRect(0, 0, vImage.getWidth(), vImage.getHeight());
		dec.paint(vg2);

		double scaleX, scaleY;

		if (vImage.getWidth() > 40)
		{
			scaleX = 1 / (vImage.getWidth() / 40d);
			fullX = true;
		}
		else
		{
			scaleX = 1;
			fullX = false;
		}
		if (vImage.getHeight() > 30)
		{
			scaleY = 1 / (vImage.getHeight() / 30d);
			fullY = true;
		}
		else
		{
			scaleY = 1;
			fullY = false;
		}

		AffineTransform tx = new AffineTransform();
		tx.scale(scaleX, scaleY);
		AffineTransformOp op =
			new AffineTransformOp(tx, AffineTransformOp.TYPE_BILINEAR);
		BufferedImage newVImage = op.filter(vImage, null);

		int imageX = fullX ? 5 : (50 - bounds.width) / 2;
		int imageY = fullY ? 5 : (50 - bounds.height) / 2;

		Graphics2D g2 = bi.createGraphics();
		g2.setColor(Color.white);
		g2.fillRect(0, 0, bi.getWidth(), bi.getHeight());
		g2.drawImage(newVImage, imageX, imageY, null);

		if (fullX || fullY)
		{
			g2.setRenderingHint(
				RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
			g2.setColor(Color.black);
			g2.drawLine(25, 38, 25, 46);
			g2.drawLine(21, 42, 29, 42);

			zoomCircle = new Ellipse2D.Double(20, 37, 10, 10);
			g2.draw(zoomCircle);
			VPPanel p = new VPPanel();
			popImage = new JLabel(new ImageIcon(vImage));
			p.add(popImage);
			pop = new JPopupMenu();
			pop.add(p);

			addMouseMotionListener(listener);
		}
	}
	private void initVert(PathwayElement element, BufferedImage bi)
	{
		PathwayVertex vert = (PathwayVertex)element;
		boolean fullX, fullY;

		Rectangle bounds = vert.getBounds();
		double xOffset = 0 - bounds.getX();
		double yOffset = 0 - bounds.getY();
		AffineTransform at = new AffineTransform();
		at.translate(xOffset, yOffset);
		vert.getGeneralPath().transform(at);

		BufferedImage vImage =
			new BufferedImage(
				bounds.width + 1,
				bounds.height + 1,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D vg2 = vImage.createGraphics();
		vg2.setColor(Color.white);
		vg2.fillRect(0, 0, vImage.getWidth(), vImage.getHeight());
		vert.paint(vg2);

		double scaleX, scaleY;

		if (vImage.getWidth() > 40)
		{
			scaleX = 1 / (vImage.getWidth() / 40d);
			fullX = true;
		}
		else
		{
			scaleX = 1;
			fullX = false;
		}
		if (vImage.getHeight() > 30)
		{
			scaleY = 1 / (vImage.getHeight() / 30d);
			fullY = true;
		}
		else
		{
			scaleY = 1;
			fullY = false;
		}

		AffineTransform tx = new AffineTransform();
		tx.scale(scaleX, scaleY);
		AffineTransformOp op =
			new AffineTransformOp(tx, AffineTransformOp.TYPE_BILINEAR);
		BufferedImage newVImage = op.filter(vImage, null);

		int imageX = fullX ? 5 : (50 - bounds.width) / 2;
		int imageY = fullY ? 5 : (50 - bounds.height) / 2;

		Graphics2D g2 = bi.createGraphics();
		g2.setColor(Color.white);
		g2.fillRect(0, 0, bi.getWidth(), bi.getHeight());
		g2.drawImage(newVImage, imageX, imageY, null);

		if (fullX || fullY)
		{
			g2.setRenderingHint(
				RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
			g2.setColor(Color.black);
			g2.drawLine(25, 38, 25, 46);
			g2.drawLine(21, 42, 29, 42);

			zoomCircle = new Ellipse2D.Double(20, 37, 10, 10);
			g2.draw(zoomCircle);
			VPPanel p = new VPPanel();
			popImage = new JLabel(new ImageIcon(vImage));
			p.add(popImage);
			pop = new JPopupMenu();
			pop.add(p);

			addMouseMotionListener(listener);
		}
	}
	private void initEdge(PathwayElement element, BufferedImage bi)
	{
		PathwayEdge edge = (PathwayEdge)element;

		//Create invisible vertices at the edge of the button
		PathwayVertex edgeVertex1 = new DummyVertex(null, 4, 4);
		PathwayVertex edgeVertex2 = new DummyVertex(null, 46, 46);
		edge.setVertexA(edgeVertex1);
		edge.setVertexB(edgeVertex2);
		Graphics2D g2 = bi.createGraphics();
		g2.setColor(Color.white);
		g2.fillRect(0, 0, bi.getWidth(), bi.getHeight());
		edgeVertex1.paint(g2);
		edgeVertex2.paint(g2);

		edge.paint(g2);
	}

	private class Listener implements MouseListener, MouseMotionListener
	{
		public void mouseMoved(MouseEvent evt)
		{
			Point point = new Point(evt.getX() - 5, evt.getY() - 5);
			if (zoomCircle.getBounds().contains(point))
			{
				Point screenLoc =
					((Component)evt.getSource()).getLocationOnScreen();
				pop.setLocation(screenLoc.x, screenLoc.y);
				pop.setVisible(true);
				pop.addMouseListener(this);
			}
			else
				pop.setVisible(false);

		}
		public void mouseDragged(MouseEvent evt)
		{}
		public void mouseClicked(MouseEvent evt)
		{}
		public void mouseEntered(MouseEvent evt)
		{}
		public void mouseExited(MouseEvent evt)
		{
			pop.setVisible(false);
		}
		public void mousePressed(MouseEvent evt)
		{}
		public void mouseReleased(MouseEvent evt)
		{}
	}
	/**
	 * @return
	 */
	public Type getType()
	{
		return element.getRenderer().getType();
	}
	/**
	 * @return
	 */
	public PathwayElement getElement()
	{
		return element;
	}

}
