package vp.gui.components;

import vp.graph.PathwayElement;
import vp.gui.VPSettings;
import vp.main.MainFrame;
import vp.util.ActionCenter;
import vp.util.GraphUtilities;
import vp.visual.type.Type;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.EventListener;
import java.util.List;
import java.util.Vector;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JViewport;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionListener;

public class TypeViewer extends JTabbedPane
{
	private JViewport typeImageView;
	public static final int IMAGE = 0;
	public static final int NAME = 1;

	private ButtonGroup typeGroup;
	private VPPanel typeImagePanel;
	private JList typeNameList;
	private JScrollPane typeImagePanelScroller;
	private JScrollPane typeNameListScroller;
	private int selectedNumber;

	public TypeViewer(
		MainFrame parent,
		List elements,
		boolean edit,
		PathwayElement current,
		EventListener listener)
	{

		typeImagePanel = new VPPanel();
		typeGroup = new ButtonGroup();
		TypeButton[] typeButtons = new TypeButton[elements.size()];
		TypeButton newButton = null;

		for (int i = 0; i < typeButtons.length; i++)
		{
			PathwayElement temp = (PathwayElement)elements.get(i);
            typeButtons[i] = new TypeButton(parent, temp);
			typeButtons[i].setActionCommand(ActionCenter.CHANGE_TYPE);
			typeButtons[i].addActionListener((ActionListener)listener);
			typeGroup.add(typeButtons[i]);
        }

		boolean typeSelected = false;

		if (edit)
		{
			int selectIndex = 0;
			if (current != null)
			{

				for (int i = 0; i < typeButtons.length; i++)
				{

					if (current
						.getRenderer()
						.getType()
						.equals(typeButtons[i].getType()))
					{
						selectIndex = i;
						typeSelected = true;
					}
				}
			}

			else
				typeSelected = true;
			typeButtons[selectIndex].setSelected(true);
			selectedNumber = selectIndex;
			if (!typeSelected)
			{
				newButton =
					new TypeButton(
						parent,
						GraphUtilities.getHeadlessElement(
							current.getRenderer().getType(),
							current.getElementType()));
				newButton.setActionCommand(ActionCenter.CHANGE_TYPE);
				newButton.addActionListener((ActionListener)listener);
				selectedNumber = typeButtons.length + 1;
				typeGroup.add(newButton);
				selectIndex = selectedNumber;
				newButton.setSelected(true);
			}

		}

		else
			typeButtons[0].setSelected(true);
		int rows;
		int size = 0;
		for (Enumeration e = typeGroup.getElements();
			e.hasMoreElements();
			e.nextElement())
		{
			size++;
		}

		if (size > 4)
		{
			rows = size / 2;
			rows = (size % 2 == 0) ? rows : (rows + 1);
		}
		else
			rows = 2;
		typeImagePanel.setBorder(new EmptyBorder(3, 4, 5, 0));
		typeImagePanel.setLayout(new GridLayout(rows, 2, 0, 2));
		typeImagePanel.setBackground(VPSettings.Colors.SIDE_PANEL);
		for (int i = 0; i < typeButtons.length; i++)
		{
			typeImagePanel.add(typeButtons[i]);
		}

		if (newButton != null)
			typeImagePanel.add(newButton);
		for (int i = 0; i < (rows * 2) - size; i++)
		{
			typeImagePanel.add(new JLabel());
		} //typeImagePanel.setPreferredSize(new Dimension(122, 122));
		typeImagePanelScroller =
			new JScrollPane(
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		typeImageView = new JViewport();
		typeImageView.setPreferredSize(new Dimension(128, 130));
		typeImageView.setView(typeImagePanel);
		typeImageView.setViewPosition(
			new Point(
				0,
				(typeImagePanel.getPreferredSize().height / rows)
					* (selectedNumber / 2)));
		typeImagePanelScroller.setViewport(typeImageView);
		Vector typeNames = new Vector();
		for (Enumeration e = typeGroup.getElements(); e.hasMoreElements();)
		{
			String name = ((TypeButton)e.nextElement()).getType().getName();
			typeNames.add(name);
		}

		typeNameList = new JList(typeNames);
		typeNameList.setFont(VPSettings.R_FONT);
		typeNameList.setSelectedValue(getSelectedType().getName(), true);
		typeNameList.addListSelectionListener((ListSelectionListener)listener);
		//typeNameList.setPreferredSize(new Dimension(122, 122));
		typeNameListScroller =
			new JScrollPane(
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		JViewport view2 = new JViewport();
		view2.setPreferredSize(typeImageView.getPreferredSize());
		view2.setView(typeNameList);
		typeNameListScroller.setViewport(view2);
		setFont(VPSettings.B_FONT);
		addTab("Image", typeImagePanelScroller);
		addTab("Name", typeNameListScroller);
	}

	public TypeButton getSelectedTypeButton()
	{
		TypeButton tb = null;
		for (Enumeration e = typeGroup.getElements(); e.hasMoreElements();)
		{
			TypeButton current = (TypeButton)e.nextElement();
			if (current.isSelected())
			{
				tb = current;
				break;
			}
		}
		return tb;
	}

	public Type getSelectedType()
	{
		return getSelectedTypeButton().getType();
	}

	public PathwayElement getSelectedElement()
	{
		return getSelectedTypeButton().getElement();
	}

	public JList getTypeNameList()
	{
		return typeNameList;
	}

	public void changeType(int display)
	{
		if (display == IMAGE)
		{
			typeNameList.setSelectedValue(getSelectedType().getName(), true);
		}
		else if (display == NAME)
		{
			String name = typeNameList.getSelectedValue().toString();
			for (Enumeration e = typeGroup.getElements(); e.hasMoreElements();)
			{
				TypeButton current = (TypeButton)e.nextElement();
				if (current.getType().getName().equals(name))
				{
					current.setSelected(true);
					if (!typeImageView
						.getViewRect()
						.contains(current.getLocation()))
						typeImageView.setViewPosition(
							new Point(0, current.getLocation().y));
					break;
				}
			}
		}
	}

}
