package vp.gui.dialogs;

import vp.graph.DataClass;
import vp.graph.Decoration;
import vp.graph.DummyVertex;
import vp.graph.PathwayEdge;
import vp.graph.PathwayElement;
import vp.graph.PathwayGraph;
import vp.graph.PathwayVertex;
import vp.gui.VPSettings;
import vp.gui.components.DialogHeader;
import vp.gui.components.ExpandableVPRLabel;
import vp.gui.components.PreviewPanel;
import vp.gui.components.TypeViewer;
import vp.gui.components.VPBLabel;
import vp.gui.components.VPButton;
import vp.gui.components.VPPanel;
import vp.main.MainFrame;
import vp.util.ActionCenter;
import vp.util.DecorationChangeEvent;
import vp.util.DecorationChangeListener;
import vp.util.EditEdgeList;
import vp.util.GraphUtilities;
import vp.util.IconFactory;
import vp.util.StringBreaker;
import vp.util.TypeChangeEvent;
import vp.util.TypeChangeListener;
import vp.util.database.DBConstants;
import vp.util.database.DBQuery;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.geom.Point2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.EventListener;
import java.util.List;
import java.util.ListIterator;

import javax.swing.JComboBox;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class AddEdgeDialog extends VPPanel
{
	private MainFrame parent;
	private PathwayElement vgeA;
	private PathwayElement vgeB;
	private Listener listener;
	private JComboBox directionBox;
	private AddEdgeDialog thisDialog;
	private PreviewPanel pl;
	private JSpinner linkPos1;
	private JSpinner linkPos2;
	private boolean edit, directionChanged;
	private PathwayEdge oldLink;
	private TypeChangeListener typeChangeListener;
	private DecorationChangeListener decorationChangeListener;
	private VPPanel typeImagePanel;
	private TypeViewer tview;
	private PathwayGraph pGraph;
	private boolean decChanged;
	private DataClass dataClass;
	private PathwayEdge currentElement;
	private List typeChangeListeners;
	private PathwayVertex vertexA, vertexB;

	public AddEdgeDialog(
		MainFrame parent,
		PathwayElement vgeA,
		PathwayElement vgeB)
	{
		this(parent, vgeA, vgeB, false, null, null, null);
	}

	public AddEdgeDialog(
		MainFrame parent,
		PathwayElement vgeA,
		PathwayElement vgeB,
		PathwayEdge oldLink,
		EventListener vpListener)
	{
		this(parent, vgeA, vgeB, true, oldLink, vpListener, vpListener);
	}

	public AddEdgeDialog(
		MainFrame parent,
		PathwayElement vgeA,
		PathwayElement vgeB,
		boolean edit,
		PathwayEdge oldLink,
		EventListener typeListener,
		EventListener decListener)
	{
		this.parent = parent;
		this.vgeA = vgeA;
		this.vgeB = vgeB;
		this.edit = edit;
		this.oldLink = oldLink;
		typeChangeListener = (TypeChangeListener)typeListener;
		decorationChangeListener = (DecorationChangeListener)decListener;
		thisDialog = this;
		listener = new Listener();
		directionChanged = false;
		pGraph = parent.getPathwayGraph();
		typeChangeListeners = new ArrayList();

		if (edit)
			dataClass = oldLink.getDataClass();
		else
			dataClass = GraphUtilities.HEADLESS_EDGE;

		pGraph.setLocked(true);
		setLayout(new GridBagLayout());
		setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints superc = new GridBagConstraints();
		superc.anchor = GridBagConstraints.WEST;
		superc.insets = new Insets(5, 5, 5, 6);

		VPPanel p =
			new DialogHeader(
				IconFactory.getEdgeIcon(),
				"Describe the link using the menus below");
		superc.gridwidth = 3;
		add(p, superc);

		superc.gridy = 1;
		superc.fill = GridBagConstraints.BOTH;
		JSeparator jsep = new JSeparator(JSeparator.HORIZONTAL);
		add(jsep, superc);

		superc.gridy = 2;
		superc.gridwidth = 1;
		superc.fill = GridBagConstraints.NONE;
		superc.anchor = GridBagConstraints.CENTER;

		VPPanel leftPanel = new VPPanel();
		leftPanel.setLayout(new GridBagLayout());
		leftPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints leftc = new GridBagConstraints();
		leftc.anchor = GridBagConstraints.CENTER;

		leftPanel.add(new VPBLabel("Visual Types"), leftc);

		leftc.gridy = 1;
		typeImagePanel = new VPPanel();
		typeImagePanel.setBackground(VPSettings.Colors.SIDE_PANEL);
		tview =
			new TypeViewer(
				parent,
				GraphUtilities.getHeadlessElements(
					pGraph.getAvailableEdgeTypes(),
					DBConstants.EDGE),
				edit,
				oldLink,
				listener);
		typeImagePanel.add(tview);
		leftPanel.add(typeImagePanel, leftc);

		leftc.gridy = 2;
		leftc.insets = new Insets(4, 5, 5, 4);
		String[] directions =
			{
				PathwayEdge.FORWARD,
				PathwayEdge.BACKWARD,
				PathwayEdge.BOTH,
				PathwayEdge.NEITHER };
		directionBox = new JComboBox(directions);
		directionBox.setFont(VPSettings.R_FONT);

		if (edit)
		{
			directionBox.setSelectedItem(oldLink.getDirection());
		}
		else
			directionBox.setSelectedItem(PathwayEdge.FORWARD);
		directionBox.addItemListener(listener);
		leftPanel.add(directionBox, leftc);

		add(leftPanel, superc);

		superc.gridx = 1;
		superc.fill = GridBagConstraints.BOTH;
		superc.insets = new Insets(5, 6, 5, 6);
		jsep = new JSeparator(JSeparator.VERTICAL);
		add(jsep, superc);

		superc.gridx = 2;
		superc.insets = new Insets(5, 5, 5, 5);
		VPPanel rightPanel = new VPPanel();
		rightPanel.setLayout(new GridBagLayout());
		rightPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints rightc = new GridBagConstraints();
		rightc.anchor = GridBagConstraints.CENTER;
		rightc.gridwidth = 2;
		rightc.insets = new Insets(0, 0, 0, 0);
		rightPanel.add(new VPBLabel("Endpoints"), rightc);

		rightc.gridy = 1;
		rightc.anchor = GridBagConstraints.WEST;
		p = new VPPanel();
		p.setBackground(VPSettings.Colors.SIDE_PANEL);
		p.setLayout(new GridBagLayout());

		if (vgeA instanceof PathwayVertex)
		{
			String displayString =
				StringBreaker.breakStringForLabel(
					(String) ((PathwayVertex)vgeA).getDataClass().get("Name"));
			p.add(new VPBLabel("1.: "));
			p.add(new ExpandableVPRLabel(parent, displayString, 18));
			vertexA = (PathwayVertex)vgeA;
		}

		else if (vgeA instanceof PathwayEdge)
		{
			p.add(new VPBLabel("1.: Link Pos.: "));
			linkPos1 = new JSpinner(new SpinnerNumberModel(0.5, 0.0, 1.0, 0.1));
			linkPos1.setPreferredSize(
				new Dimension(
					linkPos1.getPreferredSize().width * 2,
					linkPos1.getPreferredSize().height));
			p.add(linkPos1);

			double linkp = ((Number)linkPos1.getValue()).doubleValue();
			Point2D.Double dummyPos =
				((PathwayEdge)vgeA).getLinkPosition(linkp);

			vertexA =
				new DummyVertex(
					pGraph,
					(int)dummyPos.x,
					(int)dummyPos.y,
					((PathwayEdge)vgeA).getID(),
					linkp);

		}

		rightPanel.add(p, rightc);

		rightc.gridy = 2;
		rightc.insets = new Insets(0, 0, 0, 0);
		p = new VPPanel();
		p.setBackground(VPSettings.Colors.SIDE_PANEL);
		p.setLayout(new GridBagLayout());

		if (vgeB instanceof PathwayVertex)
		{
			String displayString =
				StringBreaker.breakStringForLabel(
					(String) ((PathwayVertex)vgeB).getDataClass().get("Name"));
			p.add(new VPBLabel("2.: "));
			p.add(new ExpandableVPRLabel(parent, displayString, 18));
			vertexB = (PathwayVertex)vgeB;
		}

		else if (vgeB instanceof PathwayEdge)
		{
			p.add(new VPBLabel("2.: Link Pos.: "));
			linkPos2 = new JSpinner(new SpinnerNumberModel(0.5, 0.0, 1.0, 0.1));
			linkPos2.setPreferredSize(
				new Dimension(
					linkPos2.getPreferredSize().width * 2,
					linkPos2.getPreferredSize().height));
			p.add(linkPos2);

			double linkp = ((Number)linkPos2.getValue()).doubleValue();
			Point2D.Double dummyPos =
				((PathwayEdge)vgeA).getLinkPosition(linkp);

			vertexB =
				new DummyVertex(
					pGraph,
					(int)dummyPos.x,
					(int)dummyPos.y,
					((PathwayEdge)vgeB).getID(),
					linkp);
		}

		rightPanel.add(p, rightc);

		rightc.gridy = 3;
		rightc.fill = GridBagConstraints.BOTH;
		rightc.insets = new Insets(5, 0, 5, 0);
		rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

		rightc.gridy = 4;
		rightc.insets = new Insets(5, 5, 5, 5);
		rightPanel.add(
			new VPButton(
				"Edit Visual Properties",
				ActionCenter.EDIT_EDGE_PROPERTIES,
				listener),
			rightc);

		rightc.gridy = 5;
		rightPanel.add(
			new VPButton("Add Enzyme(s)", ActionCenter.DECORATE_EDGE, listener),
			rightc);

		rightc.gridy = 6;
		rightc.insets = new Insets(5, 0, 5, 0);
		rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

		rightc.gridy = 7;
		rightc.gridwidth = 1;
		rightc.insets = new Insets(5, 5, 5, 5);
		rightPanel.add(new VPButton("OK", ActionCenter.OK, listener), rightc);
		rightc.gridx = 1;
		rightPanel.add(
			new VPButton("Cancel", ActionCenter.CANCEL, listener),
			rightc);

		add(rightPanel, superc);

		superc.gridy = 3;
		superc.gridx = 0;
		superc.gridwidth = 3;
		superc.fill = GridBagConstraints.BOTH;
		superc.insets = new Insets(4, 6, 5, 5);
		add(new JSeparator(JSeparator.HORIZONTAL), superc);

		superc.gridy = 4;
		superc.anchor = GridBagConstraints.CENTER;
		superc.fill = GridBagConstraints.NONE;
		superc.insets = new Insets(5, 6, 5, 5);

		currentElement =
			(oldLink == null)
				? new PathwayEdge(
					pGraph,
					-2,
					dataClass,
					tview.getSelectedType(),
					vertexA,
					vertexB,
					directionBox.getSelectedItem().toString())
				: oldLink;

		pl = new PreviewPanel(parent, currentElement, new Dimension(200, 20));
		add(pl, superc);

		tview.changeType(TypeViewer.IMAGE);
		addTypeChangeListener(pl);
	}

	public void addTypeChangeListener(TypeChangeListener tcl)
	{
		typeChangeListeners.add(tcl);
	}

	public void fireTypeChange(TypeChangeEvent event)
	{
		for (int i = 0; i < typeChangeListeners.size(); i++)
		{
			((TypeChangeListener)typeChangeListeners.get(i)).typeChanged(event);
		}
	}

	private void update()
	{
		currentElement.setType(tview.getSelectedType());
		currentElement.setDataClass(dataClass);
		currentElement.setDirection(directionBox.getSelectedItem().toString());
		fireTypeChange(new TypeChangeEvent(thisDialog, pGraph, currentElement));
	}

	private class Listener
		implements ActionListener, ItemListener, ListSelectionListener, TypeChangeListener
	{
		public void actionPerformed(ActionEvent event)
		{
			String command = event.getActionCommand();
			if (command.equals(ActionCenter.OK))
			{
				if (edit)
				{
					if (directionChanged)
						((EditEdgeList)typeChangeListener).changeDirection(
							directionBox.getSelectedItem().toString());
					if (decChanged)
					{
                        decorationChangeListener.decorationsChanged(
							new DecorationChangeEvent(
								this,
								pGraph,
								oldLink.getDecList()));
					}
					typeChangeListener.typeChanged(
						new TypeChangeEvent(
							this,
							pGraph,
							tview.getSelectedElement()));

				}
				else
				{

					dataClass.setStoredType(currentElement.getType());
					parent.getStack().store(pGraph);

					if (!pGraph.containsVertex(vertexA))
					{
						vertexA.setID(DBQuery.getUniqueID());
						try
						{
							pGraph.addVertex(vertexA);
						}
						catch (Exception e)
						{
							e.printStackTrace();
						}
					}

					if (!pGraph.containsVertex(vertexB))
					{
						vertexB.setID(DBQuery.getUniqueID());
						try
						{
							pGraph.addVertex(vertexB);
						}
						catch (Exception e)
						{
							e.printStackTrace();
						}
					}
					currentElement.setVertexA(vertexA);
					currentElement.setVertexB(vertexB);
					currentElement.setID(DBQuery.getUniqueID());

					try
					{
						pGraph.addEdge(currentElement);
					}
					catch (Exception e)
					{
						e.printStackTrace();
					}
				}
				pGraph.setLocked(false);
				parent.clearEditPanel();
				pGraph.clearHighlightedElements();
				pGraph.removeAllAnimatedElements();
			}

			else if (command.equals(ActionCenter.CANCEL))
			{
				pGraph.setLocked(false);
				parent.clearEditPanel();
				pGraph.clearHighlightedElements();
				pGraph.removeAllAnimatedElements();
			}

			else if (command.equals(ActionCenter.CHANGE_TYPE))
			{
				tview.changeType(TypeViewer.IMAGE);
				update();
			}

			else if (command.equals(ActionCenter.EDIT_EDGE_PROPERTIES))
				parent.addToEditPanel(
					new EditEdgeDialog(
						parent,
						directionBox.getSelectedItem().toString(),
						new PathwayEdge(currentElement),
						this));
			else if (command.equals(ActionCenter.DECORATE_EDGE))
			{
				PathwayVertex vertexA = null;
				PathwayVertex vertexB = null;
				PathwayEdge pe = null;

				if (vgeA instanceof PathwayVertex)
				{
					vertexA = (PathwayVertex)vgeA;
				}

				if (vgeB instanceof PathwayVertex)
				{
					vertexB = (PathwayVertex)vgeB;
				}
				String dir = directionBox.getSelectedItem().toString();

				if (edit)
				{
					//To reflect visual type change for existing edge.
					//before it gets passed to the decoration editor.
					pe =
						new PathwayEdge(
							pGraph,
							DBQuery.getUniqueID(),
							(DataClass)oldLink.getObject(),
							tview.getSelectedType(),
							(PathwayVertex)oldLink.getVertexA(),
							(PathwayVertex)oldLink.getVertexB(),
							dir);
					ListIterator decIter = oldLink.getDecorationIterator();
					while (decIter.hasNext())
					{
						try
						{
							pe.addDecoration((Decoration)decIter.next());
						}
						catch (Exception e)
						{
							e.printStackTrace();
						}
					}

				}

				else
				{
					//To define the new edge so far
					//before it gets passed to the Decoration editor.
					pe =
						new PathwayEdge(
							pGraph,
							DBQuery.getUniqueID(),
							GraphUtilities.HEADLESS_EDGE,
							tview.getSelectedType(),
							vertexA,
							vertexB,
							dir);

					if (decorationChangeListener == null)
					{
						decorationChangeListener = new EditEdgeList(pGraph);
						((EditEdgeList)decorationChangeListener).add(pe);
						typeChangeListener =
							(TypeChangeListener)decorationChangeListener;
					}
				}

                parent.addToEditPanel(
					new DecorateEdgeDialog(parent, thisDialog, pe));
			}
		}

		public void itemStateChanged(ItemEvent event)
		{
			if (event.getSource() == directionBox)
			{
				if (event.getStateChange() == ItemEvent.SELECTED)
				{
					update();
					directionChanged = true;
				}
			}
		}

		public void valueChanged(ListSelectionEvent event)
		{
			tview.changeType(TypeViewer.NAME);
			update();
		}

		public void typeChanged(TypeChangeEvent event)
		{
			if (event != null)
			{
				typeImagePanel.remove(tview);
				PathwayElement newRenderer = event.getType();

				tview =
					new TypeViewer(
						parent,
						GraphUtilities.getHeadlessElements(
							pGraph.getAvailableEdgeTypes(),
							DBConstants.EDGE),
						true,
						newRenderer,
						listener);
				typeImagePanel.add(tview);
				update();
			}
			parent.addToEditPanel(thisDialog);
		}

	}

	/**
	 * @return
	 */
	public TypeChangeListener getTypeChangeListener()
	{
		return typeChangeListener;
	}

	/**
	 * @return
	 */
	public PathwayElement getVgeA()
	{
		return vgeA;
	}

	/**
	 * @return
	 */
	public PathwayElement getVgeB()
	{
		return vgeB;
	}

	/**
	 * @param listener
	 */
	public void setTypeChangeListener(TypeChangeListener listener)
	{
		typeChangeListener = listener;
	}

	/**
	 * @param element
	 */
	public void setVgeA(PathwayElement element)
	{
		vgeA = element;
	}

	/**
	 * @param element
	 */
	public void setVgeB(PathwayElement element)
	{
		vgeB = element;
	}

	/**
	 * @return
	 */
	public boolean isDecChanged()
	{
		return decChanged;
	}

	/**
	 * @param b
	 */
	public void setDecChanged(boolean b)
	{
		decChanged = b;
	}

	/**
	 * @return
	 */
	public DecorationChangeListener getDecorationChangeListener()
	{
		return decorationChangeListener;
	}

	/**
	 * @param listener
	 */
	public void setDecorationChangeListener(DecorationChangeListener listener)
	{
		decorationChangeListener = listener;
	}

}
