package vp.gui.dialogs;

import vp.graph.DataClass;
import vp.graph.PathwayElement;
import vp.graph.PathwayGraph;
import vp.graph.PathwayVertex;
import vp.gui.VPSettings;
import vp.gui.components.DialogFactory;
import vp.gui.components.DialogHeader;
import vp.gui.components.ExpandableVPRLabel;
import vp.gui.components.PreviewPanel;
import vp.gui.components.TypeViewer;
import vp.gui.components.VPBLabel;
import vp.gui.components.VPButton;
import vp.gui.components.VPPanel;
import vp.main.MainFrame;
import vp.util.ActionCenter;
import vp.util.EditVertexList;
import vp.util.GraphUtilities;
import vp.util.IconFactory;
import vp.util.StringBreaker;
import vp.util.TypeChangeEvent;
import vp.util.TypeChangeListener;
import vp.util.database.DBConstants;
import vp.util.database.DBQuery;

import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JSeparator;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class AddVertexDialog extends VPPanel
{
	private PathwayGraph pGraph;
	private MainFrame parent;
	private int xPosition, yPosition;
	private Listener listener;
	private ExpandableVPRLabel vertexText;
	private PreviewPanel pl;
	private AddVertexDialog thisDialog;
	private DataClass dataClass;
	private String label;
	private boolean edit, dataClassChanged;
	private PathwayVertex oldVertex;
	private TypeChangeListener typeChangeListener;
	private TypeViewer tview;
	private VPPanel leftPanel;
	private boolean done;
	private List typeChangeListeners;
	private PathwayVertex currentElement;

	public AddVertexDialog(MainFrame parent, int xPosition, int yPosition)
	{
		this(parent, xPosition, yPosition, false, null, null);
	}

	public AddVertexDialog(
		MainFrame parent,
		PathwayVertex oldVertex,
		TypeChangeListener typeChangeListener)
	{
		this(parent, 0, 0, true, oldVertex, typeChangeListener);
	}

	public AddVertexDialog(
		MainFrame parent,
		int xPosition,
		int yPosition,
		boolean edit,
		PathwayVertex oldVertex,
		TypeChangeListener typeChangeListener)
	{
		this.parent = parent;
		this.xPosition = xPosition;
		this.yPosition = yPosition;
		this.edit = edit;
		this.oldVertex = oldVertex;
		this.typeChangeListener = typeChangeListener;
		typeChangeListeners = new ArrayList();
		thisDialog = this;
		pGraph = parent.getPathwayGraph();

		if (edit)
		{
			dataClass = (DataClass)oldVertex.getDataClass();
			oldVertex.setTexturePaint(null);
		}
		else
		{
			dataClass = GraphUtilities.UNSELECTED_VERTEX;
		}

		dataClassChanged = false;
		listener = new Listener();
		label = (String)dataClass.get("Name");

		pGraph.setLocked(true);
		setLayout(new GridBagLayout());
		setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints superc = new GridBagConstraints();
		superc.anchor = GridBagConstraints.WEST;
		superc.insets = new Insets(5, 5, 5, 5);

		VPPanel p =
			new DialogHeader(
				IconFactory.getPencilIcon(),
				"Describe the vertex using the menus below");
		superc.gridwidth = 3;
		add(p, superc);

		superc.gridy = 1;
		superc.fill = GridBagConstraints.BOTH;
		JSeparator jsep = new JSeparator(JSeparator.HORIZONTAL);
		add(jsep, superc);

		superc.gridy = 2;
		superc.gridwidth = 1;
		superc.anchor = GridBagConstraints.CENTER;

		leftPanel = new VPPanel();
		leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.Y_AXIS));
		leftPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
		//leftPanel.setBackground(new Color(240, 240, 245));

		leftPanel.add(new VPBLabel("Visual Types"));
		leftPanel.add(Box.createVerticalStrut(5));

		tview =
			new TypeViewer(
				parent,
				GraphUtilities.getHeadlessElements(
					pGraph.getAvailableVertexTypes(),
					DBConstants.VERTEX),
				edit,
				oldVertex,
				listener);
		leftPanel.add(tview);

		add(leftPanel, superc);

		superc.gridx = 1;
		jsep = new JSeparator(JSeparator.VERTICAL);
		add(jsep, superc);

		superc.gridx = 2;

		VPPanel rightPanel = new VPPanel();
		rightPanel.setLayout(new GridBagLayout());
		rightPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
		//rightPanel.setBackground(new Color(240, 240, 245));
		GridBagConstraints rightc = new GridBagConstraints();
		rightc.anchor = GridBagConstraints.CENTER;

		rightc.gridwidth = 2;
		rightPanel.add(new VPBLabel("Compound"), rightc);

		rightc.gridy = 1;
		rightc.insets = new Insets(2, 5, 1, 5);
		vertexText =
			new ExpandableVPRLabel(
				parent,
				StringBreaker.breakStringForLabel(label),
				22);

		rightPanel.add(vertexText, rightc);

		rightc.gridy = 2;
		rightc.fill = GridBagConstraints.BOTH;
		rightc.insets = new Insets(5, 5, 5, 5);

		rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

		rightc.gridy = 3;
		rightPanel.add(
			new VPButton(
				"Select Compound",
				ActionCenter.SELECT_COMPOUND,
				listener),
			rightc);

		rightc.gridy = 4;
		rightPanel.add(
			new VPButton(
				"Edit Visual Properties",
				ActionCenter.EDIT_VERTEX_PROPERTIES,
				listener),
			rightc);

		rightc.gridy = 5;
		rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

		rightc.gridy = 6;
		rightc.gridwidth = 1;
		rightPanel.add(new VPButton("OK", ActionCenter.OK, listener), rightc);
		rightc.gridx = 1;
		rightPanel.add(
			new VPButton("Cancel", ActionCenter.CANCEL, listener),
			rightc);

		add(rightPanel, superc);

		superc.gridy = 3;
		superc.gridx = 0;
		superc.gridwidth = 3;
		superc.fill = GridBagConstraints.BOTH;
		superc.insets = new Insets(10, 5, 5, 5);
		jsep = new JSeparator(JSeparator.HORIZONTAL);
		add(jsep, superc);

		superc.gridy = 4;
		superc.anchor = GridBagConstraints.CENTER;
		superc.fill = GridBagConstraints.NONE;
		superc.insets = new Insets(9, 5, 5, 5);

		currentElement =
			(oldVertex == null)
				? new PathwayVertex(
					pGraph,
					-2,
					dataClass,
					tview.getSelectedType(),
					xPosition,
					yPosition,
					-1,
					-1)
				: oldVertex;

		pl = new PreviewPanel(parent, currentElement);
		add(pl, superc);

		tview.changeType(TypeViewer.IMAGE);

		addTypeChangeListener(pl);
		//System.out.println(getPreferredSize());
	}

	/**
	 * @param pl
	 */
	public void addTypeChangeListener(TypeChangeListener pl)
	{
		typeChangeListeners.add(pl);
	}

	public void fireTypeChange(TypeChangeEvent event)
	{
		for (int i = 0; i < typeChangeListeners.size(); i++)
		{
			((TypeChangeListener)typeChangeListeners.get(i)).typeChanged(event);
		}
	}

	public void setDataClass(DataClass dataClass)
	{
		this.dataClass = dataClass;
		label = (String)dataClass.get("Name");
		vertexText.resetText(StringBreaker.breakStringForLabel(label));
		update();
	}

	private void update()
	{
		currentElement.setType(tview.getSelectedType());
		currentElement.setDataClass(dataClass);
		fireTypeChange(new TypeChangeEvent(thisDialog, pGraph, currentElement));
	}

	private class Listener
		implements ActionListener, ListSelectionListener, TypeChangeListener
	{
		public void actionPerformed(ActionEvent event)
		{
			String command = event.getActionCommand();
			if (command.equals(ActionCenter.OK))
			{
				if ((dataClass.get("Name").equals("Not selected"))
					|| (((String)dataClass.get("Name")).length() == 0))
				{
					String title = "Error";
					String header = "No entity selected!";
					String body =
						"Please select an entity from \n"
							+ "the database or create a new one.";
					DialogFactory.createDialog(
						parent,
						IconFactory.getErrorIcon(),
						title,
						header,
						body,
						DialogFactory.OK_ONLY);
				}
				else
				{
					pGraph.setLocked(false);

					dataClass.setStoredType(
						currentElement.getRenderer().getType());
					parent.getStack().store(pGraph);

					if (edit)
					{
						if (dataClassChanged)
							((EditVertexList)typeChangeListener)
								.changeDataClass(dataClass);
						typeChangeListener.typeChanged(
							new TypeChangeEvent(this, pGraph, currentElement));

					}
					else
					{

						currentElement.setID(DBQuery.getUniqueID());
						try
						{
							pGraph.addVertex(currentElement);

						}
						catch (Exception e)
						{
							e.printStackTrace();
						}
					}
					parent.clearEditPanel();
					pGraph.removeAllAnimatedElements();
				}

			}

			else if (command.equals(ActionCenter.CANCEL))
			{
				pGraph.setLocked(false);

				parent.clearEditPanel();
				pGraph.removeAllAnimatedElements();
				pGraph.clearHighlightedElements();

			}

			else if (command.equals(ActionCenter.SELECT_COMPOUND))
			{
				parent.addToEditPanel(
					new SelectCompoundDialog(parent, thisDialog));
				if (edit)
					dataClassChanged = true;
			}
			else if (command.equals(ActionCenter.CHANGE_TYPE))
			{
				tview.changeType(TypeViewer.IMAGE);

				update();
			}
			else if (command.equals(ActionCenter.EDIT_VERTEX_PROPERTIES))
			{
				//try to pass clone
				parent.addToEditPanel(
					new EditVertexDialog(
						parent,
						label,
						new PathwayVertex(currentElement),
						this));
			}

		}

		public void valueChanged(ListSelectionEvent event)
		{
			tview.changeType(TypeViewer.NAME);

			update();
		}

		public void typeChanged(TypeChangeEvent event)
		{
			if (event != null)
			{
				leftPanel.remove(tview);
				PathwayElement newElement = event.getType();

				tview =
					new TypeViewer(
						parent,
						GraphUtilities.getHeadlessElements(
							pGraph.getAvailableVertexTypes(),
							DBConstants.VERTEX),
						true,
						newElement,
						listener);
				leftPanel.add(tview);

				update();
			}
			parent.addToEditPanel(thisDialog);
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	protected void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		tview.changeType(TypeViewer.IMAGE);
		if (!done)
		{
			repaint();
			done = true;
		}

	}
}
