/*
 * Created on Mar 19, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package vp.gui.dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import javax.imageio.ImageIO;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JViewport;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import vp.graph.DataClass;
import vp.graph.PathwayGraph;
import vp.gui.VPSettings;
import vp.gui.components.AnnotationArea;
import vp.gui.components.VPBLabel;
import vp.gui.components.VPButton;
import vp.gui.components.VPPanel;
import vp.gui.components.VPRLabel;
import vp.main.MainFrame;
import vp.util.ActionCenter;
import vp.util.BrowserLauncher;
import vp.util.StringBreaker;

/**
 * @author Matt
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class CompoundDisplayPanel extends VPPanel
{
	/**
	 * @author Matt
	 *
	 * To change the template for this generated type comment go to
	 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
	 */

	private VPBLabel compoundLabel;
	private VPButton annotateButton;
	private JViewport jv;
	private AnnotationArea annotationArea;
	private PathwayGraph pGraph;
	private MainFrame parent;
	private DataClass compound;
	private CompoundDisplayPanel thisPanel;

	public CompoundDisplayPanel(MainFrame parent, DataClass compound)
	{
		this.parent = parent;
		this.compound = compound;
		thisPanel = this;
		pGraph = parent.getPathwayGraph();
		Listener listener = new Listener();
		setLayout(new GridBagLayout());
		setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints superc = new GridBagConstraints();
		superc.anchor = GridBagConstraints.WEST;
        superc.fill = GridBagConstraints.BOTH;
		superc.insets = new Insets(5, 5, 5, 5);
        superc.weighty = 2.0;

		VPPanel topPanel = new VPPanel();
		topPanel.setLayout(new GridBagLayout());
		topPanel.setBackground(VPSettings.Colors.SIDE_PANEL);

		GridBagConstraints topc = new GridBagConstraints();
		Insets regular = new Insets(0, 10, 0, 5);
		Insets indent = new Insets(0, 50, 5, 5);
		topc.anchor = GridBagConstraints.WEST;
		topc.fill = GridBagConstraints.BOTH;
		topc.weightx = 2;
		topc.insets = new Insets(10, 10, 0, 5);
		compoundLabel = new VPBLabel("Compound:");
		topPanel.add(compoundLabel, topc);
		topc.gridy = 1;
		topc.insets = indent;
		topc.weighty = 0;
		String pathwayName =
			(compound != null)
				? StringBreaker.breakStringForLabel((String)compound.get("Name"), 50)
				: "N/A";
		topPanel.add(new VPRLabel(pathwayName), topc);
		topc.gridy = 2;
		topc.insets = indent;
		BufferedImage bi = getCompoundImage(compound);
		if (bi != null)
			topPanel.add(new ChemicalPanel(bi), topc);
		else
			topPanel.add(new VPRLabel("Image not found"), topc);

		topc.gridy = 3;
		topc.insets = regular;
		topPanel.add(new VPBLabel("Annotations:"), topc);
		topc.gridy = 4;
		topc.insets = new Insets(5, 50, 10, 5);
		topc.fill = GridBagConstraints.NONE;
		topc.weighty = 2;
        
        String annotation = (String) compound.get("annotation");
        annotation = (annotation == null) ? "" : annotation;
		annotationArea = new AnnotationArea(parent, annotation);
		annotationArea.getDocument().addDocumentListener(listener);
		topPanel.add(annotationArea, topc);
		jv = new JViewport();
		jv.setView(topPanel);
		JScrollPane jsp =
			new JScrollPane(
				jv,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		jsp.setViewport(jv);
        jsp.setPreferredSize(new Dimension(371, jsp.getPreferredSize().height));
        jsp.setMinimumSize(new Dimension(371, jsp.getPreferredSize().height));
        add(jsp, superc);

		superc.gridy = 1;
        superc.weighty = 0.1;
		superc.fill = GridBagConstraints.BOTH;
		add(new JSeparator(JSeparator.HORIZONTAL), superc);

		VPPanel bottomPanel = new VPPanel();
		bottomPanel.setLayout(new GridBagLayout());
		bottomPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints bottomc = new GridBagConstraints();
		bottomc.anchor = GridBagConstraints.CENTER;
		//bottomc.fill = GridBagConstraints.BOTH;
		bottomc.insets = new Insets(0, 10, 5, 10);
		annotateButton =
			new VPButton("Annotate", ActionCenter.ANNOTATE, listener);
		bottomPanel.add(annotateButton, bottomc);
		bottomc.gridx = 1;
		bottomPanel.add(
			new VPButton("Go To KEGG", ActionCenter.GO_TO_KEGG, listener),
			bottomc);

		superc.gridy = 2;
        superc.weighty = 0;
		superc.insets = new Insets(5, 5, 0, 5);
		add(bottomPanel, superc);
	}

	private BufferedImage getCompoundImage(DataClass compound)
	{
		URL url = null;
		BufferedImage bi = null;
		try
		{
			url =
				new URL(
					"http://www.genome.ad.jp/Fig/compound/"
						+ compound.get("id")
						+ ".gif");
		}
		catch (MalformedURLException e)
		{
			e.printStackTrace();
		}
		try
		{
			bi = ImageIO.read(url);
		}
		catch (IOException e1)
		{
			e1.printStackTrace();
		}
		return bi;
	}

	public class Listener implements ActionListener, DocumentListener
	{

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e)
		{
			String command = e.getActionCommand();
			if (command.equals(ActionCenter.GO_TO_KEGG))
			{
				try
				{
					BrowserLauncher.openURL(
						"http://www.genome.ad.jp/dbget-bin/www_bget?compound+"
							+ compound.get("id"));
				}
				catch (IOException e1)
				{
					e1.printStackTrace();
				}
			}
			else if (command.equals(ActionCenter.ANNOTATE))
			{
				annotationArea.setAnnotate();
				jv.setViewPosition(
					new Point(
						annotationArea.getBounds().x
							+ annotationArea.getBounds().width,
						annotationArea.getBounds().y
							+ annotationArea.getBounds().height));

				annotationArea.requestFocus();
				annotateButton.setText("Save Note");
				annotateButton.setActionCommand(ActionCenter.SAVE_NOTE);
			}
			else if (command.equals(ActionCenter.SAVE_NOTE))
			{
				annotationArea.setSave();
                //compound.setAnnotation(annotationArea.getText());
				jv.setViewPosition(new Point(0, 0));
				annotateButton.setText("Annotate");
				annotateButton.setActionCommand(ActionCenter.ANNOTATE);
                compound.put("annotation", annotationArea.getText());
			}
		}

        
		/* (non-Javadoc)
		 * @see javax.swing.event.DocumentListener#changedUpdate(javax.swing.event.DocumentEvent)
		 */
		public void changedUpdate(DocumentEvent e)
		{
			annotationArea.noteChange();
		}

		/* (non-Javadoc)
		 * @see javax.swing.event.DocumentListener#insertUpdate(javax.swing.event.DocumentEvent)
		 */
		public void insertUpdate(DocumentEvent e)
		{
			annotationArea.noteChange();
		}

		/* (non-Javadoc)
		 * @see javax.swing.event.DocumentListener#removeUpdate(javax.swing.event.DocumentEvent)
		 */
		public void removeUpdate(DocumentEvent e)
		{
			annotationArea.noteChange();
		}

	}
}
