package vp.gui.dialogs;

import vp.graph.DataClass;
import vp.graph.Decoration;
import vp.graph.PathwayDecoration;
import vp.graph.PathwayEdge;
import vp.graph.PathwayGraph;
import vp.gui.VPSettings;
import vp.gui.components.DialogHeader;
import vp.gui.components.EnzymeViewer;
import vp.gui.components.PreviewPanel;
import vp.gui.components.TypeViewer;
import vp.gui.components.VPBLabel;
import vp.gui.components.VPButton;
import vp.gui.components.VPPanel;
import vp.main.MainFrame;
import vp.util.ActionCenter;
import vp.util.DecorationChangeEvent;
import vp.util.DecorationChangeListener;
import vp.util.GraphUtilities;
import vp.util.IconFactory;
import vp.util.TypeChangeEvent;
import vp.util.TypeChangeListener;
import vp.util.database.DBConstants;
import vp.util.database.DBQuery;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JSeparator;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class DecorateEdgeDialog extends VPPanel
{
	private List typeChangeListeners, decorationChangeListeners;
	private MainFrame parent;
	private PathwayEdge link;
	private Listener listener;
	private PreviewPanel pl, decpl;
	private JComponent referrer;
	private DecorateEdgeDialog thisDialog;
	private VPPanel leftPanel;
	private VPPanel typeViewerPanel;
	private VPPanel enzymeViewerPanel;
	private TypeViewer tview;
	private PathwayGraph pGraph;
	private List decorations;
	private EnzymeViewer eview;
	private PathwayDecoration currentElement;
	private DataClass dataClass;

	public DecorateEdgeDialog(
		MainFrame parent,
		JComponent referrer,
		PathwayEdge link)
	{
		this.parent = parent;
		this.referrer = referrer;
		this.link = link;
		listener = new Listener();
		thisDialog = this;
		pGraph = parent.getPathwayGraph();
		typeChangeListeners = new ArrayList();
		decorationChangeListeners = new ArrayList();

		decorations = link.getDecList();

		eview = new EnzymeViewer(decorations, 3, listener);
		addDecorationChangeListener(eview);

		setLayout(new GridBagLayout());
		setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints superc = new GridBagConstraints();
		superc.anchor = GridBagConstraints.WEST;
		superc.insets = new Insets(5, 5, 3, 3);

		VPPanel p =
			new DialogHeader(
				IconFactory.getEdgeIcon(),
				"Add enzymes using the menus below");
		superc.gridwidth = 3;
		add(p, superc);

		superc.gridy = 1;
		superc.insets = new Insets(5, 5, 5, 3);
		superc.fill = GridBagConstraints.BOTH;
		add(new JSeparator(JSeparator.HORIZONTAL), superc);

		superc.gridy = 2;
		superc.gridwidth = 1;
		superc.anchor = GridBagConstraints.CENTER;
		superc.fill = GridBagConstraints.NONE;

		leftPanel = new VPPanel();
		leftPanel.setLayout(new GridBagLayout());
		leftPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints leftc = new GridBagConstraints();
		leftc.anchor = GridBagConstraints.CENTER;

		leftPanel.add(new VPBLabel("Visual Types"));

		leftc.gridy = 1;
		typeViewerPanel = new VPPanel();
		typeViewerPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
		tview =
			new TypeViewer(
				parent,
				GraphUtilities.getHeadlessElements(
					pGraph.getAvailableDecorationTypes(),
					DBConstants.DECORATION),
				true,
				currentElement,
				listener);
		typeViewerPanel.add(tview);
		leftPanel.add(typeViewerPanel, leftc);
		leftc.insets = new Insets(5, 0, 5, 0);
		leftc.gridy = 2;
		leftc.fill = GridBagConstraints.BOTH;
		leftPanel.add(new JSeparator(JSeparator.HORIZONTAL), leftc);

		leftc.gridy = 3;
		leftc.fill = GridBagConstraints.NONE;

		currentElement =
			(eview.getSelEnzyme() != null) ? eview.getSelEnzyme() : null;

		Dimension decplDim = new Dimension(100, 41);
		decpl = new PreviewPanel(parent, currentElement, decplDim);
		leftPanel.add(decpl, leftc);

		tview.changeType(TypeViewer.IMAGE);
		addTypeChangeListener(decpl);

		add(leftPanel, superc);

		superc.gridx = 1;
		superc.fill = GridBagConstraints.BOTH;
		superc.insets = new Insets(5, 5, 5, 5);
		add(new JSeparator(JSeparator.VERTICAL), superc);

		superc.gridx = 2;
		superc.insets = new Insets(0, 3, 5, 4);
		superc.fill = GridBagConstraints.NONE;

		VPPanel rightPanel = new VPPanel();
		rightPanel.setLayout(new GridBagLayout());
		rightPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints rightc = new GridBagConstraints();
		rightc.anchor = GridBagConstraints.CENTER;
		rightc.gridwidth = 4;
		rightc.insets = new Insets(0, 5, 0, 5);

		rightPanel.add(new VPBLabel("Enzymes"), rightc);

		rightc.gridy = 1;
		enzymeViewerPanel = new VPPanel();
		enzymeViewerPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
		enzymeViewerPanel.add(eview);
		rightc.insets = new Insets(0, 5, 0, 5);
		rightPanel.add(enzymeViewerPanel, rightc);

		rightc.gridy = 2;
		rightc.fill = GridBagConstraints.BOTH;
		rightc.insets = new Insets(5, 5, 6, 5);
		rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

		rightc.gridy = 3;
		rightPanel.add(
			new VPButton("Select Enzyme", ActionCenter.SELECT_ENZYME, listener),
			rightc);

		rightc.gridy = 4;
		rightc.gridwidth = 2;
		rightPanel.add(
			new VPButton(
				"Edit",
				ActionCenter.EDIT_DECORATION_PROPERTIES,
				listener),
			rightc);
		rightc.gridx = 2;
		rightPanel.add(
			new VPButton("Delete", ActionCenter.DELETE_ENZYME, listener),
			rightc);

		rightc.gridy = 5;
		rightc.gridx = 0;
		rightc.gridwidth = 4;
		rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

		rightc.gridy = 6;
		rightc.gridwidth = 1;
		rightPanel.add(new VPButton("OK", ActionCenter.OK, listener), rightc);
		rightc.gridwidth = 3;
		rightc.gridx = 1;
		rightPanel.add(
			new VPButton("Cancel", ActionCenter.CANCEL, listener),
			rightc);

		add(rightPanel, superc);

	}

	/**
	 * @param decpl
	 */
	private void addTypeChangeListener(TypeChangeListener decpl)
	{
		typeChangeListeners.add(decpl);
	}

	public void fireTypeChange(TypeChangeEvent event)
	{
		for (int i = 0; i < typeChangeListeners.size(); i++)
		{
			((TypeChangeListener)typeChangeListeners.get(i)).typeChanged(event);
		}
	}

	private void addDecorationChangeListener(DecorationChangeListener dcl)
	{
		decorationChangeListeners.add(dcl);
	}

	public void fireAddDecoration(DecorationChangeEvent event)
	{
		for (int i = 0; i < decorationChangeListeners.size(); i++)
		{
			(
				(DecorationChangeListener)decorationChangeListeners.get(
					i)).addDecoration(
				event);
		}
	}

	public void fireRemoveDecoration(DecorationChangeEvent event)
	{
		for (int i = 0; i < decorationChangeListeners.size(); i++)
		{
			(
				(DecorationChangeListener)decorationChangeListeners.get(
					i)).removeDecoration(
				event);
		}
	}

	private void update()
	{
		update(false);
	}

	private void update(boolean storedType)
	{
		currentElement = eview.getSelEnzyme();

		if (currentElement != null)
		{
			if (!storedType)
				currentElement.setType(tview.getSelectedType());
			else
			{
				typeViewerPanel.remove(tview);

				tview =
					new TypeViewer(
						parent,
						GraphUtilities.getHeadlessElements(
							pGraph.getAvailableDecorationTypes(),
							DBConstants.DECORATION),
						true,
						currentElement,
						listener);
				typeViewerPanel.add(tview);
				thisDialog.validate();
			}

		}
		fireTypeChange(new TypeChangeEvent(thisDialog, pGraph, currentElement));
	}

	public void addDecoration(PathwayDecoration dec)
	{
		dec.setID(DBQuery.getUniqueID());
		dec.setType(tview.getSelectedType());
		dec.setParent(link);
		dec.setLabel((String)dec.getDataClass().get("ECNumber"));
		//decorations.add(dec);
		fireAddDecoration(new DecorationChangeEvent(thisDialog, pGraph, dec));
		update();
	}

	private void removeDecoration(PathwayDecoration dec)
	{
		decorations.remove(dec);
		fireRemoveDecoration(
			new DecorationChangeEvent(thisDialog, pGraph, dec));
		update();
	}

	private class Listener
		implements ActionListener, ListSelectionListener, TypeChangeListener
	{
		public void actionPerformed(ActionEvent event)
		{
			String command = event.getActionCommand();

			if (command.equals(ActionCenter.OK))
			{
				//vGraph.removeEdge(link);
				//vGraph.repaint();
				PathwayEdge returnLink =
					new PathwayEdge(
						pGraph,
						DBQuery.getUniqueID(),
						(DataClass)link.getObject(),
						link.getType(),
						link.getVertexA(),
						link.getVertexB(),
						link.getDirection());

				for (int i = 0; i < decorations.size(); i++)
				{
					try
					{
						returnLink.addDecoration(
							(Decoration)decorations.get(i));
					}
					catch (Exception xe)
					{
						xe.printStackTrace();
					}
				}
				if (referrer instanceof AddEdgeDialog)
				{
					AddEdgeDialog aed = (AddEdgeDialog)referrer;
					AddEdgeDialog newAed =
						new AddEdgeDialog(
							parent,
							aed.getVgeA(),
							aed.getVgeB(),
							true,
							returnLink,
							aed.getTypeChangeListener(),
							aed.getDecorationChangeListener());
					newAed.setDecChanged(true);
					parent.addToEditPanel(newAed);

				}
			}

			else if (command.equals(ActionCenter.CANCEL))
				parent.addToEditPanel(referrer);

			else if (command.equals(ActionCenter.CHANGE_TYPE))
			{
				tview.changeType(TypeViewer.IMAGE);

				update();

			}

			else if (command.equals(ActionCenter.SELECT_ENZYME))
				parent.addToEditPanel(
					new SelectEnzymeDialog(parent, thisDialog));

			else if (command.equals(ActionCenter.DELETE_ENZYME))
			{
				removeDecoration(eview.getSelEnzyme());
				update();
			}

			else if (command.equals(ActionCenter.EDIT_DECORATION_PROPERTIES))
			{
				String label =
					(eview.getSelEnzyme() != null)
						? (String)eview.getSelEnzyme().getDataClass().get(
							"ECNumber")
						: "Dec:";
				PathwayDecoration decToEdit =
					(currentElement == null)
						? (PathwayDecoration)tview.getSelectedElement()
						: new PathwayDecoration(currentElement);
				parent.addToEditPanel(
					new EditDecorationDialog(parent, label, decToEdit, this));
			}
		}

		public void valueChanged(ListSelectionEvent event)
		{
			if ((tview != null)
				&& (event.getSource() == tview.getTypeNameList()))
			{
				tview.changeType(TypeViewer.NAME);
				update();
			}
			else if (
				(eview != null)
					&& (event.getSource() == eview.getEcNumberList()))
			{
				eview.changeType(EnzymeViewer.ID_VIEW);
				update(true);
			}
			else if (
				(eview != null)
					&& (event.getSource() == eview.getEnzNameList()))
			{
				eview.changeType(EnzymeViewer.NAME_VIEW);
				update(true);
			}
		}

		public void typeChanged(TypeChangeEvent event)
		{
			if (event != null)
			{
				typeViewerPanel.remove(tview);

				tview =
					new TypeViewer(
						parent,
						GraphUtilities.getHeadlessElements(
							pGraph.getAvailableDecorationTypes(),
							DBConstants.DECORATION),
						true,
						event.getType(),
						listener);
				typeViewerPanel.add(tview);
				update();

			}

			parent.addToEditPanel(thisDialog);
		}

	}
}
