package vp.gui.dialogs;

import vp.graph.PathwayDecoration;
import vp.graph.PathwayElement;
import vp.graph.PathwayGraph;
import vp.gui.VPSettings;
import vp.gui.components.ColorButton;
import vp.gui.components.DialogHeader;
import vp.gui.components.PreviewPanel;
import vp.gui.components.VPBLabel;
import vp.gui.components.VPButton;
import vp.gui.components.VPPanel;
import vp.gui.components.VPRLabel;
import vp.main.MainFrame;
import vp.util.ActionCenter;
import vp.util.FileExtensionFilter;
import vp.util.IconFactory;
import vp.util.SaveType;
import vp.util.TypeChangeEvent;
import vp.util.TypeChangeListener;
import vp.visual.DecorationRenderer;
import vp.visual.type.Type;
import java.awt.Color;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JViewport;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class EditDecorationDialog extends VPPanel
{
    private PathwayGraph pGraph;

    private List typeChangeListeners;

    private MainFrame parent;

    private PathwayDecoration decoration;

    private String text;

    private TypeChangeListener typeChangeListener;

    private EditDecorationDialog thisDialog;

    private Listener listener;

    private ColorButton outlineColor;

    private ColorButton fillColor;

    private ColorButton fontColor;

    private PreviewPanel pl;

    private JList fontFamilies;

    private JComboBox fontStyle;

    private JComboBox fontPoint;

    private JComboBox vertexShape;

    private PathwayElement currentElement;

    public EditDecorationDialog(MainFrame parent, String text,
        PathwayDecoration decoration,
        TypeChangeListener typeChangeListener)
    {
        this.parent = parent;
        this.text = text;
        this.decoration = decoration;
        this.typeChangeListener = typeChangeListener;
        thisDialog = this;
        pGraph = parent.getPathwayGraph();
        listener = new Listener();
        typeChangeListeners = new ArrayList();

        setLayout(new GridBagLayout());
        setBackground(VPSettings.Colors.SIDE_PANEL);
        GridBagConstraints superc = new GridBagConstraints();
        superc.anchor = GridBagConstraints.WEST;
        superc.insets = new Insets(5, 5, 5, 5);

        superc.gridwidth = 3;
        VPPanel p = new DialogHeader(IconFactory.getPencilIcon(),
            "Edit appearance using the menus below");
        add(p, superc);

        superc.gridy = 1;
        superc.fill = GridBagConstraints.BOTH;
        add(new JSeparator(JSeparator.HORIZONTAL), superc);

        superc.gridy = 2;
        superc.gridwidth = 1;
        // superc.fill = GridBagConstraints.NONE;
        superc.anchor = GridBagConstraints.CENTER;
        superc.insets = new Insets(0, 5, 5, 5);

        VPPanel leftPanel = new VPPanel();
        leftPanel.setLayout(new GridBagLayout());
        leftPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
        GridBagConstraints leftc = new GridBagConstraints();

        leftc.anchor = GridBagConstraints.WEST;
        leftc.gridwidth = 2;

        leftPanel.add(new VPBLabel("Font"), leftc);

        leftc.gridy = 1;
        leftc.insets = new Insets(5, 5, 5, 5);
        GraphicsEnvironment local = GraphicsEnvironment
            .getLocalGraphicsEnvironment();
        String[] aff = local.getAvailableFontFamilyNames();
        fontFamilies = new JList(aff);
        fontFamilies.setFont(VPSettings.R_FONT);
        fontFamilies.setVisibleRowCount(3);
        int fontWidth = fontFamilies
            .getFontMetrics(VPSettings.R_FONT).stringWidth(
                "Franklin Gothic Medium Con");
        fontFamilies.setFixedCellWidth(fontWidth + 2);
        fontFamilies.setSelectedValue(decoration.getFont()
            .getFamily(), true);
        fontFamilies.ensureIndexIsVisible(fontFamilies
            .getSelectedIndex());

        fontFamilies.addListSelectionListener(listener);
        JScrollPane jsp = new JScrollPane(
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
            JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        JViewport fontFamilyView = new JViewport();
        fontFamilyView.setView(fontFamilies);
        fontFamilyView.setViewPosition(new Point(0, fontFamilies
            .getCellBounds(fontFamilies.getSelectedIndex(),
                fontFamilies.getSelectedIndex()).y));

        jsp.setViewport(fontFamilyView);
        leftPanel.add(jsp, leftc);

        leftc.gridy = 2;
        leftc.gridwidth = 1;
        leftc.fill = GridBagConstraints.BOTH;
        String[] styles = { "Plain", "Bold", "Italic" };
        fontStyle = new JComboBox(styles);
        fontStyle.setFont(VPSettings.R_FONT);
        // fontStyle.setBackground(new ColorUIResource(224, 224,
        // 224));
        int currentFontStyle = decoration.getFont().getStyle();
        switch (currentFontStyle)
        {
            case Font.PLAIN:
                fontStyle.setSelectedItem("Plain");
                break;
            case Font.BOLD:
                fontStyle.setSelectedItem("Bold");
                break;
            case Font.ITALIC:
                fontStyle.setSelectedItem("Italic");
                break;
        }
        fontStyle.addItemListener(listener);
        leftPanel.add(fontStyle, leftc);

        leftc.gridx = 1;
        String[] points = { "6 Point", "8 Point", "9 Point",
                "10 Point", "11 Point", "12 Point", "14 Point",
                "16 Point", "18 Point", "24 Point" };
        fontPoint = new JComboBox(points);
        fontPoint.setFont(VPSettings.R_FONT);
        // fontPoint.setBackground(new ColorUIResource(224, 224,
        // 224));
        fontPoint.setSelectedItem(String.valueOf(decoration.getFont()
            .getSize())
            + " Point");
        fontPoint.addItemListener(listener);
        leftPanel.add(fontPoint, leftc);

        leftc.gridy = 4;
        leftc.gridx = 0;
        leftc.gridwidth = 2;
        leftc.anchor = GridBagConstraints.WEST;
        leftc.fill = GridBagConstraints.BOTH;
        leftPanel.add(new JSeparator(JSeparator.HORIZONTAL), leftc);

        leftc.gridy = 5;
        leftc.insets = new Insets(0, 5, 5, 5);
        leftPanel.add(new VPBLabel("Shape"), leftc);

        leftc.gridy = 6;
        leftc.insets = new Insets(0, 5, 5, 5);
        // leftc.fill = GridBagConstraints.CENTER;
        String[] shapes = { "Rectangle", "Ellipse",
                "Rounded Rectangle" };
        vertexShape = new JComboBox(shapes);
        vertexShape.setFont(VPSettings.R_FONT);
        // vertexShape.setBackground(new ColorUIResource(224, 224,
        // 224));
        vertexShape.setSelectedItem(decoration.getShape());
        vertexShape.addItemListener(listener);
        leftPanel.add(vertexShape, leftc);

        add(leftPanel, superc);

        superc.gridx = 1;
        superc.insets = new Insets(5, 5, 5, 5);
        add(new JSeparator(JSeparator.VERTICAL), superc);

        superc.gridx = 2;
        superc.insets = new Insets(0, 5, 5, 5);

        VPPanel rightPanel = new VPPanel();
        rightPanel.setLayout(new GridBagLayout());
        rightPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
        GridBagConstraints rightc = new GridBagConstraints();
        Insets reg = new Insets(0, 0, 0, 0);
        Insets insert = new Insets(0, 20, 0, 5);
        rightc.insets = reg;
        rightc.anchor = GridBagConstraints.WEST;

        rightc.gridwidth = 4;
        rightPanel.add(new VPBLabel("Colors"), rightc);

        rightc.gridy = 1;
        rightc.gridwidth = 2;
        rightc.anchor = GridBagConstraints.WEST;
        rightc.gridx = 0;

        rightc.insets = insert;
        rightPanel.add(new VPRLabel("Outline:"), rightc);
        rightc.gridx = 2;
        rightc.insets = reg;
        rightc.anchor = GridBagConstraints.CENTER;
        outlineColor = new ColorButton(decoration.getOutlineColor(),
            ActionCenter.EDIT_VERTEX_OUTLINE_COLOR, listener);
        rightPanel.add(outlineColor, rightc);

        rightc.gridy = 2;
        rightc.gridx = 0;
        rightc.anchor = GridBagConstraints.WEST;
        rightc.insets = insert;
        rightPanel.add(new VPRLabel("Fill:"), rightc);
        rightc.gridx = 2;
        rightc.anchor = GridBagConstraints.CENTER;
        rightc.insets = reg;
        fillColor = new ColorButton(decoration.getFillColor(),
            ActionCenter.EDIT_VERTEX_FILL_COLOR, listener);
        rightPanel.add(fillColor, rightc);

        rightc.gridy = 3;
        rightc.gridx = 0;
        rightc.anchor = GridBagConstraints.WEST;
        rightc.insets = insert;
        rightPanel.add(new VPRLabel("Text:"), rightc);
        rightc.gridx = 2;
        rightc.anchor = GridBagConstraints.CENTER;
        rightc.insets = reg;
        fontColor = new ColorButton(decoration.getFontColor(),
            ActionCenter.EDIT_VERTEX_FONT_COLOR, listener);
        rightPanel.add(fontColor, rightc);

        rightc.gridy = 4;
        rightc.gridx = 0;
        rightc.gridwidth = 4;
        rightc.insets = new Insets(5, 5, 5, 5);
        rightc.fill = GridBagConstraints.BOTH;
        rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

        rightc.gridy = 5;
        rightPanel.add(new VPButton("Save Type",
            ActionCenter.SAVE_TYPE, listener), rightc);

        rightc.gridy = 6;
        rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

        rightc.gridy = 7;
        rightc.gridwidth = 1;
        rightPanel.add(new VPButton("OK", ActionCenter.OK, listener),
            rightc);
        rightc.gridx = 1;
        rightc.gridwidth = 3;
        rightPanel.add(new VPButton("Cancel", ActionCenter.CANCEL,
            listener), rightc);

        add(rightPanel, superc);

        superc.gridy = 3;
        superc.gridwidth = 3;
        superc.gridx = 0;
        superc.insets = new Insets(5, 5, 5, 5);
        add(new JSeparator(JSeparator.HORIZONTAL), superc);

        superc.gridy = 4;
        superc.fill = GridBagConstraints.NONE;

        currentElement = decoration;
        // currentElement.getDataClass().put("Name", "Vert");
        // ((PathwayVertex)currentElement).setLabel("Vert");
        currentElement.getRenderer().getType()
            .setName("Unsaved type");
        pl = new PreviewPanel(parent, currentElement);
        add(pl, superc);

        addTypeChangeListener(pl);
    }

    public void addTypeChangeListener(TypeChangeListener pl)
    {
        typeChangeListeners.add(pl);
    }

    public void fireTypeChange(TypeChangeEvent event)
    {
        for (int i = 0; i < typeChangeListeners.size(); i++)
        {
            ((TypeChangeListener) typeChangeListeners.get(i))
                .typeChanged(event);
        }
    }

    private void update()
    {
        fireTypeChange(new TypeChangeEvent(thisDialog, pGraph,
            currentElement));
    }

    public int getFontStyle(String s)
    {
        int style = 0;
        if (s.equals("Plain"))
        {
            style = Font.PLAIN;
        }
        else if (s.equals("Bold"))
        {
            style = Font.BOLD;
        }
        else if (s.equals("Italic"))
        {
            style = Font.ITALIC;
        }
        return style;
    }

    public int getFontPoint(String s)
    {
        return Integer.parseInt(s.substring(0, s.indexOf(" ")));
    }

    private class Listener implements ActionListener, ItemListener,
        ListSelectionListener
    {
        public void actionPerformed(ActionEvent event)
        {
            String command = event.getActionCommand();
            if (command
                .equals(ActionCenter.EDIT_VERTEX_OUTLINE_COLOR))
            {
                Color color = VPSettings.getVPColorChooser(
                    thisDialog, "Select Outline Color", decoration
                        .getOutlineColor());
                outlineColor.changeColor(color);
                ((PathwayDecoration) currentElement)
                    .setOutlineColor(color);
                update();
            }
            else if (command
                .equals(ActionCenter.EDIT_VERTEX_FILL_COLOR))
            {
                Color color = VPSettings.getVPColorChooser(
                    thisDialog, "Select Fill Color", decoration
                        .getFillColor());
                fillColor.changeColor(color);
                ((PathwayDecoration) currentElement)
                    .setFillColor(color);
                update();
            }
            else if (command
                .equals(ActionCenter.EDIT_VERTEX_FONT_COLOR))
            {
                Color color = VPSettings.getVPColorChooser(
                    thisDialog, "Select Font Color", decoration
                        .getFontColor());
                fontColor.changeColor(color);
                ((PathwayDecoration) currentElement)
                    .setFontColor(color);
                update();
            }
            else if (command.equals(ActionCenter.OK))
            {
                typeChangeListener.typeChanged(new TypeChangeEvent(
                    this, parent.getPathwayGraph(), currentElement));

            }
            else if (command.equals(ActionCenter.SAVE_TYPE))
            {
                File defaultPath = new File("etc" + File.separator
                    + "types" + File.separator + "decoration");

                JFileChooser chooser = VPSettings.getVPFileChooser(
                    defaultPath, parent);

                chooser
                    .addChoosableFileFilter(new FileExtensionFilter(
                        "type"));
                int result = chooser.showSaveDialog(parent);
                File selectedFile = chooser.getSelectedFile();
                if (result == JFileChooser.APPROVE_OPTION)
                    SaveType.saveType(parent, Type.DECORATION,
                        currentElement.getRenderer().getType(),
                        selectedFile);
            }
            else if (command.equals(ActionCenter.CANCEL))
            {
                typeChangeListener.typeChanged(null);
            }
        }

        public void itemStateChanged(ItemEvent event)
        {
            if (event.getSource() == fontStyle)
            {
                Font font = new Font(fontFamilies.getSelectedValue()
                    .toString(), getFontStyle(fontStyle
                    .getSelectedItem().toString()),
                    getFontPoint(fontPoint.getSelectedItem()
                        .toString()));
                ((PathwayDecoration) currentElement).setFont(font);
                update();
            }
            if (event.getSource() == fontPoint)
            {
                Font font = new Font(fontFamilies.getSelectedValue()
                    .toString(), getFontStyle(fontStyle
                    .getSelectedItem().toString()),
                    getFontPoint(fontPoint.getSelectedItem()
                        .toString()));
                ((PathwayDecoration) currentElement).setFont(font);
                update();

            }
            if (event.getSource() == vertexShape)
            {
                String shape = vertexShape.getSelectedItem()
                    .toString();
                if (shape.equals("Ellipse"))
                {
                    ((DecorationRenderer) currentElement
                        .getRenderer())
                        .setGeneralPath(new GeneralPath(
                            new Ellipse2D.Double(5, 10, 5, 10)));
                    ((PathwayDecoration) currentElement)
                        .setShape(shape);
                }
                else if (shape.equals("Rounded Rectangle"))
                {
                    ((DecorationRenderer) currentElement
                        .getRenderer())
                        .setGeneralPath(new GeneralPath(
                            new RoundRectangle2D.Double(5, 10, 5, 10,
                                5, 5)));
                    ((PathwayDecoration) currentElement)
                        .setShape(shape);
                }
                else if (vertexShape.getSelectedItem().toString()
                    .equals("Rectangle"))
                {
                    ((DecorationRenderer) currentElement
                        .getRenderer())
                        .setGeneralPath(new GeneralPath(
                            new Rectangle2D.Double(5, 10, 5, 10)));
                    ((PathwayDecoration) currentElement)
                        .setShape(shape);
                }
                update();
            }
        }

        public void valueChanged(ListSelectionEvent event)
        {
            if (event.getSource() == fontFamilies)
            {
                Font font = new Font(fontFamilies.getSelectedValue()
                    .toString(), getFontStyle(fontStyle
                    .getSelectedItem().toString()),
                    getFontPoint(fontPoint.getSelectedItem()
                        .toString()));

                ((PathwayDecoration) currentElement).setFont(font);
                update();
            }
        }

    }
}
