package vp.gui.dialogs;

import vp.graph.PathwayEdge;
import vp.graph.PathwayElement;
import vp.graph.PathwayGraph;
import vp.gui.VPSettings;
import vp.gui.components.ColorButton;
import vp.gui.components.DialogHeader;
import vp.gui.components.PreviewPanel;
import vp.gui.components.VPBLabel;
import vp.gui.components.VPButton;
import vp.gui.components.VPPanel;
import vp.main.MainFrame;
import vp.util.ActionCenter;
import vp.util.FileExtensionFilter;
import vp.util.IconFactory;
import vp.util.SaveType;
import vp.util.TypeChangeEvent;
import vp.util.TypeChangeListener;
import vp.visual.type.Type;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class EditEdgeDialog extends VPPanel
{
    private PathwayGraph pGraph;

    private List typeChangeListeners;

    private MainFrame parent;

    private String direction;

    private PathwayEdge edge;

    private TypeChangeListener typeChangeListener;

    private EditEdgeDialog thisDialog;

    private Listener listener;

    private ColorButton outlineColor;

    private PreviewPanel pl;

    private JComboBox edgeStyle;

    private JSpinner curveRate;

    private float curvature;

    private PathwayElement currentElement;

    public EditEdgeDialog(MainFrame parent, String direction,
        PathwayEdge edge, TypeChangeListener myListener)
    {
        this.parent = parent;
        this.direction = direction;
        this.edge = edge;
        this.typeChangeListener = myListener;

        pGraph = parent.getPathwayGraph();
        thisDialog = this;
        listener = new Listener();
        curvature = edge.getCurvature();
        typeChangeListeners = new ArrayList();

        setLayout(new GridBagLayout());
        setBackground(VPSettings.Colors.SIDE_PANEL);
        GridBagConstraints superc = new GridBagConstraints();
        superc.anchor = GridBagConstraints.WEST;
        superc.insets = new Insets(8, 5, 5, 5);

        VPPanel p = new DialogHeader(IconFactory.getEdgeIcon(),
            "Edit appearance using the menus below");
        superc.gridwidth = 3;
        add(p, superc);

        superc.gridy = 1;
        superc.fill = GridBagConstraints.BOTH;
        add(new JSeparator(JSeparator.HORIZONTAL), superc);

        superc.gridy = 2;
        superc.gridwidth = 1;
        VPPanel leftPanel = new VPPanel();
        leftPanel.setLayout(new GridBagLayout());
        leftPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
        GridBagConstraints leftc = new GridBagConstraints();
        leftc.insets = new Insets(5, 0, 5, 0);
        leftc.anchor = GridBagConstraints.WEST;

        leftPanel.add(new VPBLabel("Arrow Type:"), leftc);
        leftc.gridy = 1;
        leftc.insets = new Insets(0, 35, 0, 5);
        leftPanel.add(new VPBLabel("Body:"), leftc);
        leftc.gridx = 1;
        leftc.insets = new Insets(5, 5, 5, 5);
        leftc.anchor = GridBagConstraints.CENTER;
        String[] styles = { Type.SOLID_LINE, Type.DOTTED_LINE };
        edgeStyle = new JComboBox(styles);
        edgeStyle.setFont(VPSettings.R_FONT);
        // edgeStyle.setBackground(new ColorUIResource(224, 224,
        // 224));
        edgeStyle.setSelectedItem(edge.getShape());
        edgeStyle.addItemListener(listener);
        leftPanel.add(edgeStyle, leftc);

        leftc.gridy = 2;
        leftc.gridx = 0;
        leftc.insets = new Insets(10, 5, 10, 5);
        leftc.fill = GridBagConstraints.BOTH;
        leftc.gridwidth = 2;
        leftPanel.add(new JSeparator(JSeparator.HORIZONTAL), leftc);

        leftc.gridy = 3;
        leftc.insets = new Insets(5, 5, 5, 5);
        leftc.anchor = GridBagConstraints.WEST;
        leftc.gridwidth = 1;
        leftPanel.add(new VPBLabel("Curvature:"), leftc);

        leftc.gridy = 4;
        leftc.insets = new Insets(0, 35, 0, 5);
        leftPanel.add(new VPBLabel("Rate:"), leftc);

        leftc.gridx = 1;
        leftc.anchor = GridBagConstraints.CENTER;
        leftc.insets = new Insets(5, 5, 5, 5);
        curveRate = new JSpinner(new SpinnerNumberModel(curvature,
            0.0, 1.0, 0.1));
        curveRate.setPreferredSize(new Dimension(curveRate
            .getPreferredSize().width * 2, curveRate
            .getPreferredSize().height));
        curveRate.addChangeListener(listener);
        leftPanel.add(curveRate, leftc);

        add(leftPanel, superc);

        superc.gridx = 1;
        superc.insets = new Insets(8, 18, 5, 18);
        add(new JSeparator(JSeparator.VERTICAL), superc);

        superc.gridx = 2;
        superc.insets = new Insets(8, 5, 5, 5);
        VPPanel rightPanel = new VPPanel();
        rightPanel.setLayout(new GridBagLayout());
        rightPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
        GridBagConstraints rightc = new GridBagConstraints();
        rightc.insets = new Insets(5, 5, 5, 5);
        rightc.anchor = GridBagConstraints.WEST;
        rightc.fill = GridBagConstraints.BOTH;
        rightc.gridwidth = 4;
        rightPanel.add(new VPBLabel("Colors:"), rightc);

        rightc.gridy = 1;
        rightc.gridwidth = 2;
        rightc.insets = new Insets(0, 35, 0, 5);
        rightc.anchor = GridBagConstraints.WEST;
        rightc.fill = GridBagConstraints.NONE;
        rightPanel.add(new VPBLabel("Arrow:"), rightc);
        rightc.gridx = 2;
        rightc.anchor = GridBagConstraints.CENTER;
        rightc.insets = new Insets(5, 5, 5, 5);
        outlineColor = new ColorButton(edge.getOutlineColor(),
            ActionCenter.EDIT_EDGE_OUTLINE_COLOR, listener);
        rightPanel.add(outlineColor, rightc);

        rightc.gridy = 2;
        rightc.gridx = 0;
        rightc.gridwidth = 4;
        rightc.insets = new Insets(10, 5, 10, 5);
        rightc.fill = GridBagConstraints.BOTH;
        rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

        rightc.gridy = 3;
        rightc.insets = new Insets(5, 5, 5, 5);
        rightPanel.add(new VPButton("Save Type",
            ActionCenter.SAVE_TYPE, listener), rightc);

        rightc.gridy = 4;
        rightc.fill = GridBagConstraints.BOTH;
        rightc.insets = new Insets(10, 5, 10, 5);
        rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

        rightc.gridy = 5;
        rightc.gridwidth = 1;
        rightc.insets = new Insets(5, 5, 5, 5);
        rightPanel.add(new VPButton("OK", ActionCenter.OK, listener),
            rightc);
        rightc.gridx = 1;
        rightc.gridwidth = 3;
        rightPanel.add(new VPButton("Cancel", ActionCenter.CANCEL,
            listener), rightc);

        add(rightPanel, superc);

        superc.gridy = 3;
        superc.gridwidth = 3;
        superc.gridx = 0;
        add(new JSeparator(JSeparator.HORIZONTAL), superc);

        superc.gridy = 4;
        superc.insets = new Insets(10, 5, 5, 5);
        superc.fill = GridBagConstraints.NONE;
        superc.anchor = GridBagConstraints.CENTER;

        currentElement = edge;
        currentElement.getRenderer().getType()
            .setName("Unsaved Type");
        pl = new PreviewPanel(parent, edge);
        add(pl, superc);
        addTypeChangeListener(pl);
    }

    public void addTypeChangeListener(TypeChangeListener tcl)
    {
        typeChangeListeners.add(tcl);
    }

    public void fireTypeChange(TypeChangeEvent event)
    {
        for (int i = 0; i < typeChangeListeners.size(); i++)
        {
            ((TypeChangeListener) typeChangeListeners.get(i))
                .typeChanged(event);
        }
    }

    private void update()
    {
        fireTypeChange(new TypeChangeEvent(thisDialog, pGraph,
            currentElement));
    }

    private class Listener implements ActionListener, ChangeListener,
        ItemListener
    {
        public void actionPerformed(ActionEvent event)
        {
            String command = event.getActionCommand();
            if (command.equals(ActionCenter.EDIT_EDGE_OUTLINE_COLOR))
            {
                Color color = VPSettings.getVPColorChooser(
                    thisDialog, "Select Outline Color", edge
                        .getOutlineColor());
                ((PathwayEdge) currentElement).setOutlineColor(color);
                outlineColor.changeColor(color);
                update();
            }
            else if (command.equals(ActionCenter.OK))
            {
                typeChangeListener.typeChanged(new TypeChangeEvent(
                    this, parent.getPathwayGraph(), edge));
            }
            else if (command.equals(ActionCenter.SAVE_TYPE))
            {
                File defaultPath = new File("etc" + File.separator
                    + "types" + File.separator + "edge");

                JFileChooser chooser = VPSettings.getVPFileChooser(
                    defaultPath, parent);

                chooser
                    .addChoosableFileFilter(new FileExtensionFilter(
                        "type"));
                int result = chooser.showSaveDialog(parent);
                File selectedFile = chooser.getSelectedFile();
                if (result == JFileChooser.APPROVE_OPTION)
                    SaveType.saveType(parent, Type.EDGE, edge
                        .getType(), selectedFile);
            }
            else if (command.equals(ActionCenter.CANCEL))
                typeChangeListener.typeChanged(null);
        }

        public void stateChanged(ChangeEvent event)
        {
            float newCurve = ((Number) curveRate.getValue())
                .floatValue();
            ((PathwayEdge) currentElement).setCurvature(newCurve);
            update();
        }

        public void itemStateChanged(ItemEvent event)
        {
            if (event.getSource() == edgeStyle)
            {
                ((PathwayEdge) currentElement).setShape(edgeStyle
                    .getSelectedItem().toString());
                pl.repaint();
            }
        }
    }
}
