package vp.gui.dialogs;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.JFileChooser;
import javax.swing.JSeparator;

import vp.graph.PathwayGraph;
import vp.graph.VPGradient;
import vp.gui.VPSettings;
import vp.gui.components.DialogHeader;
import vp.gui.components.VPBLabel;
import vp.gui.components.VPButton;
import vp.gui.components.VPPanel;
import vp.gui.components.VPRLabel;
import vp.main.MainFrame;
import vp.util.ActionCenter;
import vp.util.FileExtensionFilter;
import vp.util.IconFactory;
import vp.util.xml.XMLToGraph;

public class LoadFlatFileDialog extends VPPanel
{
    private int currentIndex = -1;

    private int setSize = 0;

    private class ExperimentSet
    {
        List experiments;

        ExperimentSet()
        {
            experiments = new ArrayList();
        }

        HashMap getExperiment(int index)
        {
            return (HashMap) experiments.get(index);
        }

        void addExperiment(HashMap experiment)
        {
            experiments.add(experiment);
        }

        public int getSize()
        {
            return experiments.size();
        }

    }

    public class Listener implements ActionListener, KeyListener
    {

        public void actionPerformed(ActionEvent e)
        {
            String command = e.getActionCommand();

            if (command.equals(ActionCenter.LOAD_FILE))
            {
                File defaultPath = new File("etc" + File.separator
                    + "files" + File.separator);
                JFileChooser chooser = VPSettings.getVPFileChooser(
                    defaultPath, parent);
                chooser
                    .addChoosableFileFilter(new FileExtensionFilter(
                        "txt"));
                int result = chooser
                    .showOpenDialog(LoadFlatFileDialog.this);
                experimentFile = chooser.getSelectedFile();

                if (result == JFileChooser.APPROVE_OPTION)
                {
                    experimentSet = parseExperimentFile(experimentFile);

                    setSize = experimentSet.getSize();

                    pGraph.swapEnzymesForGenes(false);
                    showFirstExperiment();
                }
            }

            if (command.equals(ActionCenter.FIRST_EXPERIMENT))
                showFirstExperiment();
            else if (command.equals(ActionCenter.NEXT_EXPERIMENT))
                showNextExperiment();
            else if (command.equals(ActionCenter.PREV_EXPERIMENT))
                showPrevExperiment();
            else if (command.equals(ActionCenter.LAST_EXPERIMENT))
                showLastExperiment();
        }

        public void keyTyped(KeyEvent e)
        {
            // TODO Auto-generated method stub

        }

        public void keyPressed(KeyEvent e)
        {
            if (e.isConsumed())
                return;

            if (e.getKeyCode() == KeyEvent.VK_PAGE_DOWN)
            {
                showNextExperiment();
                e.consume();
            }
            else if (e.getKeyCode() == KeyEvent.VK_PAGE_UP)
            {
                showPrevExperiment();
                e.consume();
            }
            else if (e.getKeyCode() == KeyEvent.VK_HOME)
            {
                showFirstExperiment();
                e.consume();
            }
            else if (e.getKeyCode() == KeyEvent.VK_END)
            {
                showLastExperiment();
                e.consume();
            }
        }

        public void keyReleased(KeyEvent e)
        {
            // TODO Auto-generated method stub

        }

    }

    private MainFrame parent;

    private PathwayGraph pGraph;

    private Listener listener;

    private File experimentFile = null;

    private ExperimentSet experimentSet = null;

    private HashMap currentExp = null;

    private VPGradient gradient = null;

    private Color nullColor = Color.GRAY;

    private Color minColor = Color.YELLOW;

    private Color maxColor = Color.RED;

    private Color midColor = null;

    private VPRLabel expLabel;

    public LoadFlatFileDialog(MainFrame parent)
    {
        this.parent = parent;
        pGraph = parent.getPathwayGraph();

        listener = new Listener();
        setFocusable(true);
        requestFocusInWindow();
        addKeyListener(listener);

        setLayout(new GridBagLayout());
        setBackground(VPSettings.Colors.SIDE_PANEL);
        GridBagConstraints superc = new GridBagConstraints();
        superc.anchor = GridBagConstraints.WEST;
        superc.insets = new Insets(8, 5, 8, 5);

        VPPanel p = new DialogHeader(IconFactory.getPencilIcon(),
            "Select a file and choose experiment number");
        superc.gridwidth = 4;
        add(p, superc);

        superc.gridy = 1;
        superc.fill = GridBagConstraints.BOTH;
        JSeparator jsep = new JSeparator(JSeparator.HORIZONTAL);
        add(jsep, superc);

        superc.gridy = 2;
        superc.anchor = GridBagConstraints.CENTER;
        add(new VPButton("Load File", ActionCenter.LOAD_FILE,
            listener), superc);

        superc.gridy = 3;
        jsep = new JSeparator(JSeparator.HORIZONTAL);
        add(jsep, superc);

        superc.gridy = 4;
        add(new VPButton("EditGradient", ActionCenter.EDIT_GRADIENT,
            listener), superc);

        superc.gridy = 5;
        jsep = new JSeparator(JSeparator.HORIZONTAL);
        add(jsep, superc);

        superc.gridy = 6;
        superc.gridwidth = 1;
        add(new VPBLabel("Pathway:"), superc);

        superc.gridx = 1;
        superc.gridwidth = 3;
        add(new VPRLabel((String) pGraph.getGraphDataClass().get(
            "Name")), superc);

        superc.gridy = 7;
        superc.gridx = 0;
        superc.gridwidth = 1;
        add(new VPBLabel("Experiment:"), superc);

        superc.gridx = 1;
        superc.gridwidth = 3;
        expLabel = new VPRLabel((currentIndex + 1) + " of " + setSize);
        add(expLabel, superc);

        superc.gridy = 8;
        superc.gridx = 0;
        superc.gridwidth = 1;
        add(new VPButton("<<", ActionCenter.FIRST_EXPERIMENT,
            listener), superc);

        superc.gridx = 1;
        add(
            new VPButton("<", ActionCenter.PREV_EXPERIMENT, listener),
            superc);

        superc.gridx = 2;
        add(
            new VPButton(">", ActionCenter.NEXT_EXPERIMENT, listener),
            superc);

        superc.gridx = 3;
        add(
            new VPButton(">>", ActionCenter.LAST_EXPERIMENT, listener),
            superc);

    }

    public void showLastExperiment()
    {
        currentIndex = setSize - 1;
        currentExp = experimentSet.getExperiment(currentIndex);
        pGraph.setExpressionDataByGene(currentExp, gradient, null);
        resetLabel();
    }

    public void showPrevExperiment()
    {
        currentIndex = (currentIndex > 0) ? currentIndex - 1 : 0;
        currentExp = experimentSet.getExperiment(currentIndex);
        pGraph.setExpressionDataByGene(currentExp, gradient, null);
        resetLabel();
    }

    public void showNextExperiment()
    {
        currentIndex = (currentIndex < setSize) ? currentIndex + 1
            : experimentSet.getSize();
        currentExp = experimentSet.getExperiment(currentIndex);
        pGraph.setExpressionDataByGene(currentExp, gradient, null);
        resetLabel();
    }

    public void showFirstExperiment()
    {
        currentIndex = 0;
        currentExp = experimentSet.getExperiment(currentIndex);
        pGraph.setExpressionDataByGene(currentExp, gradient, null);
        resetLabel();
    }

    /**
     * 
     */
    private void resetLabel()
    {
        expLabel.setText((currentIndex + 1) + " of " + setSize);
    }

    public ExperimentSet parseExperimentFile(File file)
    {
        ExperimentSet eSet = new ExperimentSet();

        double min = Double.MAX_VALUE;
        double max = Double.MIN_VALUE;

        BufferedReader sb = null;
        String line = null;
        StringTokenizer stkr = null;

        try
        {
            sb = new BufferedReader(new FileReader(file));

            // skip the first line
            sb.readLine();

            // first line of data
            line = sb.readLine();
            stkr = new StringTokenizer(line);
            String key = stkr.nextToken();
            while (stkr.hasMoreTokens())
            {
                String value = stkr.nextToken();
                double v = Double.parseDouble(value);
                max = (v > max) ? v : max;
                min = (v < min) ? v : min;
                HashMap map = new HashMap();
                map.put(key, value);
                eSet.addExperiment(map);
            }

            while ((line = sb.readLine()) != null)
            {
                stkr = new StringTokenizer(line);
                int i = 0;
                key = stkr.nextToken();
                while (stkr.hasMoreTokens())
                {
                    String value = stkr.nextToken();
                    double v = Double.parseDouble(value);
                    max = (v > max) ? v : max;
                    min = (v < min) ? v : min;
                    ((HashMap) eSet.experiments.get(i++)).put(key,
                        value);
                }
            }

            sb.close();
        }
        catch (FileNotFoundException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        gradient = new VPGradient(minColor, maxColor, midColor,
            nullColor, max, min);
        return eSet;

    }
}
