package vp.gui.dialogs;

import vp.graph.GraphDataClass;
import vp.gui.VPSettings;
import vp.gui.components.DialogFactory;
import vp.gui.components.DialogHeader;
import vp.gui.components.VPBLabel;
import vp.gui.components.VPButton;
import vp.gui.components.VPTextField;
import vp.main.MainFrame;
import vp.util.ActionCenter;
import vp.util.IconFactory;
import vp.util.database.DBQuery;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JSeparator;

public class NewPathwayDialog extends JDialog
{
	private MainFrame parent;
	private Listener listener;
	private VPTextField idField, nameField;
	private String pathID, pathName;
	private NewPathwayDialog thisDialog;

	public NewPathwayDialog(MainFrame parent)
	{
		super(parent, "New Pathway", false);
		this.parent = parent;
		listener = new Listener();
		int id = DBQuery.getUniqueID();
		String pname = "Test" + id;
		thisDialog = this;

		getContentPane().setLayout(new GridBagLayout());
        getContentPane().setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints superc = new GridBagConstraints();
		superc.anchor = GridBagConstraints.WEST;
		superc.insets = new Insets(5, 10, 5, 10);
		superc.gridwidth = 2;

		JPanel p = new DialogHeader(IconFactory.getVPIcon(), "Name pathway");
		getContentPane().add(p, superc);

		superc.gridy = 1;
		superc.fill = GridBagConstraints.BOTH;
		getContentPane().add(new JSeparator(JSeparator.HORIZONTAL), superc);

		superc.gridy = 2;
		getContentPane().add(new VPBLabel("Number"), superc);

		superc.gridy = 3;
		superc.gridwidth = 1;
		superc.weightx = 1;
		idField = new VPTextField(String.valueOf(id));
		getContentPane().add(idField, superc);

		superc.gridy = 4;
		superc.gridwidth = 2;
		getContentPane().add(new VPBLabel("Name"), superc);

		superc.gridy = 5;
		nameField = new VPTextField(pname);
		getContentPane().add(nameField, superc);

		superc.gridy = 6;
        superc.insets = new Insets(10, 10, 5, 10);
        getContentPane().add(new JSeparator(JSeparator.HORIZONTAL), superc);

		superc.gridy = 7;
		superc.gridwidth = 1;
        getContentPane().add(
			new VPButton("OK", ActionCenter.OK, listener),
			superc);
		superc.gridx = 1;
		getContentPane().add(
			new VPButton("Cancel", ActionCenter.CANCEL, listener),
			superc);

		pack();
		setResizable(false);
		setLocation(
			(parent.getBounds().x + (parent.getBounds().width / 2))
				- (getBounds().width / 2),
			(parent.getBounds().y + (parent.getBounds().height / 2))
				- (getBounds().height / 2));
		addWindowListener(listener);
		show();
	}

	public void setPathway(GraphDataClass pathway)
	{
		idField.setText(String.valueOf(pathway.get("id")));
		nameField.setText((String)pathway.get("Name"));
	}

	private class Listener implements ActionListener, WindowListener
	{
		public void actionPerformed(ActionEvent event)
		{
			String command = event.getActionCommand();
			if (command.equals(ActionCenter.OK))
			{
				pathID = idField.getText();
				pathName = nameField.getText();

				if (pathID.length() <= 0 || pathName.length() <= 0)
				{
					String title = "Error";
					String header = "No pathway specified!";
					String body =
						"Please specify both a name and \nID number for this pathway.";
					DialogFactory.createDialog(
						parent,
						IconFactory.getErrorIcon(),
						title,
						header,
						body,
						DialogFactory.OK_ONLY);
				}

				else
				{
					try
					{
						GraphDataClass pw = new GraphDataClass(pathID);
						parent.setGraphDataClass(pw);
						parent.showNewGraph();
						dispose();
					}
					catch (NumberFormatException nfe)
					{
						String title = "Error";
						String header = "ID format error!";
						String body = "Pathway ID number must /nbe an integer.";
						DialogFactory.createDialog(
							parent,
							IconFactory.getErrorIcon(),
							title,
							header,
							body,
							DialogFactory.OK_ONLY);
					}
				}
			}
			else if (command.equals(ActionCenter.CANCEL))
				dispose();
			
		}

		public void windowOpened(WindowEvent event)
		{}
		public void windowClosing(WindowEvent event)
		{
			dispose();
		}
		public void windowClosed(WindowEvent event)
		{}
		public void windowIconified(WindowEvent event)
		{}
		public void windowDeiconified(WindowEvent event)
		{}
		public void windowActivated(WindowEvent event)
		{}
		public void windowDeactivated(WindowEvent event)
		{}
	}
}
