package vp.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.color.ColorSpace;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
import java.awt.image.ColorConvertOp;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;

import vp.main.MainFrame;
import vp.util.ActionCenter;

public class PrintPreviewDialog extends JDialog
{
	private MainFrame parent;
	private Listener listener;
	private BufferedImage image;

	public PrintPreviewDialog(MainFrame parent, BufferedImage image)
	{
		super(parent, "Print Preview", true);
		this.parent = parent;
		this.image = image;
		listener = new Listener();
		Font boldFont = new Font("Palatino Linotype", Font.BOLD, 14);

		JPanel p = new JPanel();
		PreviewImagePanel pip = new PreviewImagePanel();
		JScrollPane jsp =
			new JScrollPane(
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		JViewport jv = new JViewport();
		jv.setView(pip);
		jsp.setViewport(jv);
		Border b1 = new EtchedBorder();
		Border b2 = new EmptyBorder(10, 10, 10, 10);
		jsp.setBorder(new CompoundBorder(b2, b1));
		jsp.setPreferredSize(new Dimension(700, 500));
		p.add(jsp, BorderLayout.CENTER);
		getContentPane().add(p, BorderLayout.CENTER);

		p = new JPanel();
		p.setLayout(new FlowLayout());
		JButton print = new JButton("Print");
		print.setActionCommand(ActionCenter.PRINT);
		print.addActionListener(listener);
		print.setFont(boldFont);
		p.add(print);
		JButton cancel = new JButton("Cancel");
		cancel.setActionCommand(ActionCenter.CANCEL);
		cancel.addActionListener(listener);
		cancel.setFont(boldFont);
		p.add(cancel);
		getContentPane().add(p, BorderLayout.SOUTH);

		pack();
		setResizable(false);
		setLocation(
			(parent.getBounds().x + (parent.getBounds().width / 2))
				- (getBounds().width / 2),
			Math.max(
				0,
				(parent.getBounds().y + (parent.getBounds().height / 2))
					- (getBounds().height / 2)));
		addWindowListener(listener);
		show();
	}

	private class Listener implements ActionListener, WindowListener
	{
		public void actionPerformed(ActionEvent event)
		{
			String command = event.getActionCommand();

			if (command.equals(ActionCenter.PRINT))
			{
				parent.onPrint();
				dispose();
			}
			else if (command.equals(ActionCenter.CANCEL))
				dispose();
		}
		public void windowOpened(WindowEvent event)
		{}
		public void windowClosing(WindowEvent event)
		{
			dispose();
		}
		public void windowClosed(WindowEvent event)
		{}
		public void windowIconified(WindowEvent event)
		{}
		public void windowDeiconified(WindowEvent event)
		{}
		public void windowActivated(WindowEvent event)
		{}
		public void windowDeactivated(WindowEvent event)
		{}
	}

	private class PreviewImagePanel extends JPanel
	{
		PreviewImagePanel()
		{
			setPreferredSize(
				new Dimension(image.getWidth(), image.getHeight()));
		}

		public void paintComponent(Graphics g)
		{
			super.paintComponent(g);
			Graphics2D g2 = (Graphics2D)g;
			ColorSpace cs = ColorSpace.getInstance(ColorSpace.CS_GRAY);
			ColorConvertOp op = new ColorConvertOp(cs, null);
			image = op.filter(image, null);
			g2.setColor(Color.red);
			g2.fillRect(0, 0, image.getWidth(), image.getHeight());
			g2.drawImage(image, 0, 0, null);
		}
	}
}
