package vp.gui.dialogs;

import vp.graph.DataClass;
import vp.graph.PathwayGraph;
import vp.gui.VPSettings;
import vp.gui.components.DialogFactory;
import vp.gui.components.DialogHeader;
import vp.gui.components.VPBLabel;
import vp.gui.components.VPButton;
import vp.gui.components.VPPanel;
import vp.gui.components.VPTextField;
import vp.main.MainFrame;
import vp.util.ActionCenter;
import vp.util.IconFactory;
import vp.util.VPSwingWorker;
import vp.util.database.DBAccess;
import vp.util.database.DBQuery;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class SelectCompoundDialog extends VPPanel
{
	private PathwayGraph pGraph;
	private ArrayList matches;
	private String searchString;
	private VPTextField add;
	private MainFrame parent;
	private AddVertexDialog referrer;
	private SelectCompoundDialog thisDialog;
	private Listener listener;
	private JList compoundList;
	private DefaultListModel compounds;
	private VPTextField search;
	private DataClass selectedCompound;

	public SelectCompoundDialog(MainFrame parent, AddVertexDialog referrer)
	{
		this.parent = parent;
		this.referrer = referrer;
		pGraph = parent.getPathwayGraph();
		thisDialog = this;
		listener = new Listener();
		selectedCompound = null;

		setLayout(new GridBagLayout());
		setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints superc = new GridBagConstraints();
		superc.anchor = GridBagConstraints.WEST;
		superc.insets = new Insets(5, 5, 5, 5);

		VPPanel p =
			new DialogHeader(
				IconFactory.getPencilIcon(),
				"Search for a compound or add a new one");
		superc.gridwidth = 3;
		add(p, superc);

		superc.gridy = 1;
		superc.fill = GridBagConstraints.BOTH;
		JSeparator jsep = new JSeparator(JSeparator.HORIZONTAL);
		add(jsep, superc);

		superc.gridy = 2;
		superc.gridwidth = 1;
		superc.anchor = GridBagConstraints.WEST;

		VPPanel leftPanel = new VPPanel();
		leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.Y_AXIS));
		leftPanel.setBackground(VPSettings.Colors.SIDE_PANEL);

		leftPanel.add(new VPBLabel("Compounds"));

		compounds = new DefaultListModel();
		compoundList = new JList(compounds);
		compoundList.addListSelectionListener(listener);
		compoundList.setFont(VPSettings.R_FONT);
		int listWidth =
			compoundList.getFontMetrics(VPSettings.R_FONT).stringWidth(
				"<No compound selected>" + 20);
		//compoundList.setPrototypeCellValue("<No compound selected>");
		compoundList.setVisibleRowCount(10);
		JScrollPane compoundScroller =
			new JScrollPane(
				compoundList,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		compoundScroller.setPreferredSize(
			new Dimension(
				listWidth + 8,
				compoundList.getPreferredSize().height));
		leftPanel.add(compoundScroller);

		add(leftPanel, superc);

		superc.gridx = 1;
		superc.insets = new Insets(5, 10, 5, 6);
		jsep = new JSeparator(JSeparator.VERTICAL);
		add(jsep, superc);

		superc.gridx = 2;
		superc.insets = new Insets(5, 5, 5, 5);

		VPPanel rightPanel = new VPPanel();
		rightPanel.setLayout(new GridBagLayout());
		rightPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints rightc = new GridBagConstraints();
		rightc.anchor = GridBagConstraints.WEST;
		rightc.fill = GridBagConstraints.BOTH;
		rightc.gridwidth = 4;
		rightc.insets = new Insets(0, 5, 5, 5);

		rightPanel.add(new VPBLabel("Search"), rightc);

		rightc.gridy = 1;

		search = new VPTextField(ActionCenter.FIND_COMPOUND, listener);
		rightPanel.add(search, rightc);

		rightc.gridy = 2;
		rightc.insets = new Insets(5, 5, 5, 5);
		rightc.gridwidth = 2;
		rightc.weightx = 1;
		rightPanel.add(
			new VPButton("Find", ActionCenter.FIND_COMPOUND, listener),
			rightc);
		rightc.gridx = 2;
		rightPanel.add(
			new VPButton("Clear", ActionCenter.CLEAR, listener),
			rightc);

		rightc.gridy = 3;
		rightc.gridx = 0;
		rightc.gridwidth = 4;
		rightc.insets = new Insets(10, 5, 5, 5);
		rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

		rightc.gridy = 4;
		rightc.insets = new Insets(5, 5, 5, 5);
		rightPanel.add(new VPBLabel("Compound Name"), rightc);

		rightc.gridy = 5;
		rightc.insets = new Insets(0, 5, 5, 5);
		add = new VPTextField(ActionCenter.ADD_COMPOUND, listener);
		rightPanel.add(add, rightc);

		rightc.gridy = 6;
		rightc.insets = new Insets(6, 5, 5, 5);
		rightPanel.add(
			new VPButton(
				"Add new compound",
				ActionCenter.ADD_COMPOUND,
				listener),
			rightc);

		rightc.gridy = 7;
		rightc.insets = new Insets(10, 5, 5, 5);
		rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

		rightc.gridy = 8;
		rightc.gridx = 0;
		rightc.gridwidth = 1;
		rightc.weightx = 0;
		rightPanel.add(new VPButton("OK", ActionCenter.OK, listener), rightc);
		rightc.gridx = 1;
		rightc.gridwidth = 3;
		rightPanel.add(
			new VPButton("Cancel", ActionCenter.CANCEL, listener),
			rightc);

		add(rightPanel, superc);

	}

	public AddVertexDialog getReferrer()
	{
		return referrer;
	}

	private void findCompound(final String searchString)
	{
		this.searchString = searchString;
		matches = new ArrayList();
		compounds.clear();
		int start = 0;
		//Search locally first
		List localClasses = parent.getPathwayGraph().getNewDataClasses();
		Iterator iter = localClasses.iterator();
		while (iter.hasNext())
		{
			DataClass current = (DataClass)iter.next();
			if (current.getClassName().equals("Compound"))
			{
				String compoundName = (String)current.get("Name");
				if (compoundName.indexOf(searchString) != -1)
					matches.add(current);
			}

		}
		if (!DBAccess.isInitialized())
		{
			start = 51;
			VPSwingWorker access =
				new VPSwingWorker(
					parent,
					parent.getGraphPanel(),
					"InitializeDatabase",
					0,
					50)
			{
				protected void doNonUI() throws RuntimeException
				{
					DBAccess.initialize();
				}

				protected void doUI() throws RuntimeException
				{}
			};
			access.start();
		}

		VPSwingWorker match =
			new VPSwingWorker(
				parent,
				parent.getGraphPanel(),
				"MatchCompounds",
				start,
				95)
		{

			protected void doNonUI() throws RuntimeException
			{
				matches.addAll(DBQuery.findCompound(searchString, pGraph));

			}

			protected void doUI() throws RuntimeException
			{
				for (int i = 0; i < matches.size(); i++)
					compounds.addElement((DataClass)matches.get(i));
				if (compounds.isEmpty())
				{
					String title = "Error";
					String header = "Compound not found!";
					String body =
						"The compound you are searching for \n"
							+ "is not in our database.\n"
							+ "Please enter a different search \n"
							+ "or add a new compound.";
					DialogFactory.createDialog(
						parent,
						IconFactory.getErrorIcon(),
						title,
						header,
						body,
						DialogFactory.OK_ONLY);
				}
			}
		};
		match.start();
	}

	private class Listener implements ActionListener, ListSelectionListener
	{
		public void actionPerformed(ActionEvent event)
		{
			String command = event.getActionCommand();

			if (command.equals(ActionCenter.FIND_COMPOUND))
			{
				try
				{
					findCompound(search.getText());
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}

			}
			else if (command.equals(ActionCenter.CLEAR))
			{
				compounds.clear();
				search.setText("");
			}

			else if (command.equals(ActionCenter.ADD_COMPOUND))
			{
				String title = "Not enabled";
				String header = "Feature not yet enabled!";
				String body =
					"This feature has not yet been "
						+ "\nenabled.   Please check future"
						+ "\nversions for availability";
				DialogFactory.createDialog(
					parent,
					IconFactory.getVPIcon(),
					title,
					header,
					body,
					DialogFactory.OK_ONLY);
				/*
				if (add.getText().length() <= 0)
				{
					String title = "Error";
					String header = "No compound specified!";
					String body =
						"No compound is specified.  \nPlease provide a name \nfor the new compound";
					DialogFactory.createDialog(
						parent,
						IconFactory.getErrorIcon(),
						title,
						header,
						body,
						DialogFactory.OK_ONLY);
				}
				else
				{
					DataClass newCompound =
						new DataClass("C" + DBQuery.getUniqueID(), pGraph);
					newCompound.put("Name", add.getText());
					referrer.setDataClass(newCompound);
					compounds.addElement(newCompound);
					selectedCompound = newCompound;
					parent.getPathwayGraph().addNewDataClass(newCompound);
				}
				*/
			}
			//new AddCompoundDialog(parent, thisDialog);

			else if (command.equals(ActionCenter.OK))
			{
				if (selectedCompound == null)
				{
					String title = "Error";
					String header = "No compound selected!";
					String body =
						"No compound has been selected.  \n"
							+ "Please select a compound from \n"
							+ "the database or create a new one.";
					DialogFactory.createDialog(
						parent,
						IconFactory.getErrorIcon(),
						title,
						header,
						body,
						DialogFactory.OK_ONLY);
				}
				else
				{
					referrer.setDataClass(selectedCompound);
					parent.addToEditPanel(referrer);
				}
			}
			else if (command.equals(ActionCenter.CANCEL))
			{
				parent.addToEditPanel(referrer);
			}
		}

		public void valueChanged(ListSelectionEvent event)
		{
			if (event.getSource() == compoundList)
			{
				selectedCompound = (DataClass)compoundList.getSelectedValue();
			}
		}

	}
}
