package vp.gui.dialogs;

import vp.graph.DataClass;
import vp.graph.PathwayDecoration;
import vp.graph.PathwayGraph;
import vp.gui.VPSettings;
import vp.gui.components.DialogFactory;
import vp.gui.components.DialogHeader;
import vp.gui.components.EnzymeViewer;
import vp.gui.components.VPBLabel;
import vp.gui.components.VPButton;
import vp.gui.components.VPPanel;
import vp.gui.components.VPTextField;
import vp.main.MainFrame;
import vp.util.ActionCenter;
import vp.util.IconFactory;
import vp.util.VPSwingWorker;
import vp.util.database.DBAccess;
import vp.util.database.DBQuery;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.swing.JSeparator;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class SelectEnzymeDialog extends VPPanel
{
	private PathwayGraph pGraph;
	private VPPanel typeImagePanel;
	private ArrayList matches;
	private String searchString;
	private VPTextField id;
	private VPTextField name;
	private MainFrame parent;
	private DecorateEdgeDialog referrer;
	private SelectEnzymeDialog thisDialog;
	private Listener listener;
	private PathwayDecoration selectedEnzyme;
	private ArrayList enzymes;
	private EnzymeViewer eview;
	private VPTextField search;
	private VPPanel leftPanel;

	public SelectEnzymeDialog(MainFrame parent, DecorateEdgeDialog referrer)
	{
		this.parent = parent;
		this.referrer = referrer;
		pGraph = parent.getPathwayGraph();
		listener = new Listener();
		selectedEnzyme = null;
		thisDialog = this;
		enzymes = new ArrayList();

		setLayout(new GridBagLayout());
		setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints superc = new GridBagConstraints();
		superc.anchor = GridBagConstraints.WEST;
		superc.insets = new Insets(5, 6, 5, 10);
		superc.gridwidth = 3;

		VPPanel p =
			new DialogHeader(
				IconFactory.getEdgeIcon(),
				"Select an enzyme or add a new one");
		add(p, superc);

		superc.gridy = 1;
		superc.fill = GridBagConstraints.BOTH;
		add(new JSeparator(JSeparator.HORIZONTAL), superc);

		superc.gridy = 2;
		superc.gridwidth = 1;
		superc.insets = new Insets(0, 6, 5, 10);
		leftPanel = new VPPanel();
		leftPanel.setLayout(new GridBagLayout());
		GridBagConstraints leftc = new GridBagConstraints();
		leftc.anchor = GridBagConstraints.CENTER;
		leftPanel.setBackground(VPSettings.Colors.SIDE_PANEL);

		leftPanel.add(new VPBLabel("Enzymes"), leftc);

		leftc.gridy = 1;
		typeImagePanel = new VPPanel();
		typeImagePanel.setLayout(new GridLayout(1, 1));
		typeImagePanel.setBackground(VPSettings.Colors.SIDE_PANEL);
		eview = new EnzymeViewer(enzymes, 12, listener);
		typeImagePanel.add(eview);
		leftPanel.add(typeImagePanel, leftc);

		add(leftPanel, superc);

		superc.gridx = 1;
		superc.insets = new Insets(0, 5, 5, 9);
		add(new JSeparator(JSeparator.VERTICAL), superc);

		superc.gridx = 2;
		superc.insets = new Insets(0, 6, 5, 6);
		VPPanel rightPanel = new VPPanel();
		rightPanel.setLayout(new GridBagLayout());
		rightPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
		GridBagConstraints rightc = new GridBagConstraints();
		rightc.anchor = GridBagConstraints.WEST;
		rightc.fill = GridBagConstraints.BOTH;
		rightc.gridwidth = 4;
		rightc.insets = new Insets(0, 5, 5, 5);

		rightPanel.add(new VPBLabel("Search"), rightc);

		rightc.gridy = 1;
		search = new VPTextField(ActionCenter.FIND_ENZYME, listener);
		rightPanel.add(search, rightc);

		rightc.gridy = 2;
		rightc.insets = new Insets(5, 5, 5, 5);
		rightc.gridwidth = 2;
		rightc.weightx = 1;
		rightPanel.add(
			new VPButton("Find", ActionCenter.FIND_ENZYME, listener),
			rightc);
		rightc.gridx = 2;
		rightPanel.add(
			new VPButton("Clear", ActionCenter.CLEAR, listener),
			rightc);

		rightc.gridy = 3;
		rightc.gridx = 0;
		rightc.gridwidth = 4;
		rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

		rightc.gridy = 4;
		rightc.gridwidth = 2;
		rightc.insets = new Insets(0, 5, 0, 5);
		rightPanel.add(new VPBLabel("Enzyme ID"), rightc);
		rightc.gridx = 2;
		rightc.fill = GridBagConstraints.HORIZONTAL;
		id = new VPTextField();
		rightPanel.add(id, rightc);

		rightc.gridy = 5;
		rightc.gridwidth = 4;
		rightc.gridx = 0;
		rightc.insets = new Insets(0, 5, 2, 5);
		rightc.fill = GridBagConstraints.BOTH;
		rightPanel.add(new VPBLabel("Enzyme Name"), rightc);

		rightc.gridy = 6;
		rightc.insets = new Insets(3, 5, 5, 5);
		name = new VPTextField();
		rightPanel.add(name, rightc);

		rightc.gridy = 7;
		rightPanel.add(
			new VPButton("Add new enzyme", ActionCenter.ADD_ENZYME, listener),
			rightc);

		rightc.gridy = 8;
		rightc.insets = new Insets(5, 5, 5, 5);
		rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

		rightc.gridy = 9;
		rightc.gridx = 0;
		rightc.gridwidth = 1;
		rightc.weightx = 0;
		rightPanel.add(new VPButton("OK", ActionCenter.OK, listener), rightc);
		rightc.gridx = 1;
		rightc.gridwidth = 3;
		rightPanel.add(
			new VPButton("Cancel", ActionCenter.CANCEL, listener),
			rightc);

		add(rightPanel, superc);
	}

	public DecorateEdgeDialog getReferrer()
	{
		return referrer;
	}

	private void findEnzyme(final String searchString)
	{
		this.searchString = searchString;
		matches = new ArrayList();
		enzymes.clear();
		int start = 0;
		//Search locally first
		List localClasses = parent.getPathwayGraph().getNewDataClasses();
		Iterator iter = localClasses.iterator();
		while (iter.hasNext())
		{
			DataClass current = (DataClass)iter.next();
			if (current.getClassName().equals("Enzyme"))
			{
				String enzymeName = (String)current.get("Name");
				if (enzymeName.indexOf(searchString) != -1)
					matches.add(
						new PathwayDecoration(pGraph, -3, current, null, 1));
				else if (
					((String)current.get("ECNumber")).indexOf(searchString)
						!= -1)
					matches.add(
						new PathwayDecoration(pGraph, -3, current, null, 1));
			}
		}
		if (!DBAccess.isInitialized())
		{
			start = 51;
			VPSwingWorker access =
				new VPSwingWorker(
					parent,
					parent.getGraphPanel(),
					"InitializeDatabase",
					0,
					50)
			{
				protected void doNonUI() throws RuntimeException
				{
					DBAccess.initialize();
				}

				protected void doUI() throws RuntimeException
				{}
			};
			access.start();
		}

		VPSwingWorker match =
			new VPSwingWorker(
				parent,
				parent.getGraphPanel(),
				"MatchEnzymes",
				start,
				95)
		{

			protected void doNonUI() throws RuntimeException
			{
				matches.addAll(DBQuery.findEnzyme(searchString, pGraph));

			}

			protected void doUI() throws RuntimeException
			{
				for (int i = 0; i < matches.size(); i++)
					enzymes.add((PathwayDecoration)matches.get(i));
				if (enzymes.isEmpty())
				{
					String title = "Error";
					String header = "Enzyme not found!";
					String body =
						"The enzyme you are searching for "
							+ "\nis not in our database."
							+ "\nPlease enter a different search \nor add a new enzyme.";
					DialogFactory.createDialog(
						parent,
						IconFactory.getErrorIcon(),
						title,
						header,
						body,
						DialogFactory.OK_ONLY);
				}
				else
				{
					typeImagePanel.remove(eview);
					eview = new EnzymeViewer(enzymes, 12, listener);
					typeImagePanel.add(eview);
				}
			}
		};
		match.start();

	}

	private class Listener implements ActionListener, ListSelectionListener
	{
		public void actionPerformed(ActionEvent event)
		{
			String command = event.getActionCommand();

			if (command.equals(ActionCenter.FIND_ENZYME))
			{
				try
				{
					findEnzyme(search.getText());
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}

			}
			else if (command.equals(ActionCenter.CLEAR))
			{
				enzymes.clear();
				search.setText("");
			}
			else if (command.equals(ActionCenter.ADD_ENZYME))
			{
				String title = "Not enabled";
				String header = "Feature not yet enabled!";
				String body =
					"This feature has not yet been "
						+ "\nenabled.   Please check future"
						+ "\nversions for availability";
				DialogFactory.createDialog(
					parent,
					IconFactory.getVPIcon(),
					title,
					header,
					body,
					DialogFactory.OK_ONLY);
				/*
				if ((id.getText().length() <= 0)
					|| (name.getText().length() <= 0))
				{
					String title = "Error";
					String header = "No enzyme selected!";
					String body =
						"No enzyme has been selected.  "
							+ "\nPlease select an enzyme from "
							+ "\nthe database or create a new one.";
					DialogFactory.createDialog(
						parent,
						IconFactory.getErrorIcon(),
						title,
						header,
						body,
						DialogFactory.OK_ONLY);
				}
				else
				{
					enzymes.clear();
				
					DataClass newEnzyme =
						new DataClass("EN" + DBQuery.getUniqueID(), pGraph);
					newEnzyme.put("ECNumber", id.getText());
					newEnzyme.put("Name", name.getText());
					enzymes.add(newEnzyme);
					selectedEnzyme =
						new PathwayDecoration(pGraph, -3, newEnzyme, null, 1);
					parent.getPathwayGraph().addNewDataClass(newEnzyme);
					search.setText("");
					id.setText("");
					name.setText("");
					typeImagePanel.remove(eview);
					eview = new EnzymeViewer(enzymes, 12, listener);
					typeImagePanel.add(eview);
				
					thisDialog.validate();
				}
				*/
			}
			else if (command.equals(ActionCenter.OK))
			{
				selectedEnzyme = eview.getSelEnzyme();
				if (selectedEnzyme == null)
				{
					String title = "Error";
					String header = "No enzyme selected!";
					String body =
						"No enzyme has been selected.  "
							+ "\nPlease select an enzyme from "
							+ "\nthe database or create a new one.";
					DialogFactory.createDialog(
						parent,
						IconFactory.getErrorIcon(),
						title,
						header,
						body,
						DialogFactory.OK_ONLY);
				}
				else
				{
					referrer.addDecoration(selectedEnzyme);
					parent.addToEditPanel(referrer);
				}
			}
			else if (command.equals(ActionCenter.CANCEL))
			{
				parent.addToEditPanel(referrer);
			}
		}

		public void valueChanged(ListSelectionEvent event)
		{
			if (event.getSource() == eview.getEcNumberList())
				eview.changeType(EnzymeViewer.ID_VIEW);
			else if (event.getSource() == eview.getEnzNameList())
				eview.changeType(EnzymeViewer.NAME_VIEW);
			selectedEnzyme = eview.getSelEnzyme();
		}

	}
}
