package vp.gui.dialogs;

import vp.graph.DataClass;
import vp.graph.Experiment;
import vp.graph.PathwayDecoration;
import vp.graph.PathwayEdge;
import vp.graph.PathwayGraph;
import vp.graph.VPGradient;
import vp.gui.VPSettings;
import vp.gui.components.DialogFactory;
import vp.gui.components.DialogHeader;
import vp.gui.components.ExpandableVPRLabel;
import vp.gui.components.ExperimentDetailPanel;
import vp.gui.components.VPBLabel;
import vp.gui.components.VPButton;
import vp.gui.components.VPPanel;
import vp.gui.components.VPTextField;
import vp.main.MainFrame;
import vp.util.ActionCenter;
import vp.util.IconFactory;
import vp.util.VPSwingWorker;
import vp.util.database.DBQuery;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Map;
import java.util.Set;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.JViewport;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class SelectExperimentDialog extends VPPanel
{
    private SelectExperimentDialog thisDialog;

    private VPButton detailButton;

    private VPPanel displaySearchPanel;

    private VPPanel searchPanel;

    private ExpandableVPRLabel expLabel2;

    private ExpandableVPRLabel expLabel1;

    private JTabbedPane experimentPane;

    private MainFrame parent;

    private PathwayGraph pGraph;

    private Listener listener;

    private JList experimentList1, experimentList2;

    private DefaultListModel experiments1, experiments2;

    private VPTextField search;

    private Experiment selectedExperiment1, selectedExperiment2;

    private VPGradient gradient;

    public SelectExperimentDialog(MainFrame parent)
    {
        this.parent = parent;
        thisDialog = this;
        pGraph = parent.getPathwayGraph();
        listener = new Listener();
        selectedExperiment1 = null;
        selectedExperiment2 = null;
        gradient = null;

        parent.getToolBar().getExperimentButton().setSelected(true);
        setLayout(new GridBagLayout());
        setBackground(VPSettings.Colors.SIDE_PANEL);
        GridBagConstraints superc = new GridBagConstraints();
        superc.anchor = GridBagConstraints.WEST;
        superc.insets = new Insets(6, 5, 5, 8);

        VPPanel p = new DialogHeader(IconFactory.getVPIcon(),
            "Select an experiment from the menu");
        superc.gridwidth = 3;
        add(p, superc);

        superc.gridy = 1;
        superc.fill = GridBagConstraints.BOTH;
        superc.insets = new Insets(5, 5, 5, 5);
        add(new JSeparator(JSeparator.HORIZONTAL), superc);

        superc.gridy = 2;
        superc.gridwidth = 1;
        superc.anchor = GridBagConstraints.WEST;

        VPPanel leftPanel = new VPPanel();
        leftPanel.setLayout(new GridBagLayout());
        GridBagConstraints leftc = new GridBagConstraints();
        leftc.anchor = GridBagConstraints.WEST;
        leftc.insets = new Insets(2, 0, 7, 0);
        leftc.gridwidth = 2;
        leftPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
        leftPanel.add(new VPBLabel("Experiments"), leftc);

        leftc.gridy = 1;
        leftc.fill = GridBagConstraints.NONE;

        experiments1 = new DefaultListModel();
        experimentList1 = new JList(experiments1);
        experimentList1.addListSelectionListener(listener);
        experimentList1.setFont(VPSettings.R_FONT);
        experimentList1.setVisibleRowCount(8);
        int listWidth1 = experimentList1.getFontMetrics(
            VPSettings.R_FONT)
            .stringWidth("<No experiment selected>") + 21;
        int listHeight1 = (experimentList1.getFontMetrics(
            VPSettings.R_FONT).getHeight() * experimentList1
            .getVisibleRowCount()) + 4;

        JScrollPane experimentScroller1 = new JScrollPane(
            experimentList1,
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        experimentScroller1.setPreferredSize(new Dimension(
            listWidth1, listHeight1));

        experiments2 = new DefaultListModel();
        experimentList2 = new JList(experiments2);
        experimentList2.addListSelectionListener(listener);
        experimentList2.setFont(VPSettings.R_FONT);
        experimentList2.setVisibleRowCount(8);

        int listWidth2 = experimentList2.getFontMetrics(
            VPSettings.R_FONT)
            .stringWidth("<No experiment selected>") + 21;
        int listHeight2 = (experimentList2.getFontMetrics(
            VPSettings.R_FONT).getHeight() * experimentList2
            .getVisibleRowCount()) + 4;
        JScrollPane experimentScroller2 = new JScrollPane(
            experimentList2,
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        experimentScroller2.setPreferredSize(new Dimension(
            listWidth2, listHeight2));

        experimentPane = new JTabbedPane();
        experimentPane.setFont(VPSettings.B_FONT);
        experimentPane.addTab("1", experimentScroller1);
        experimentPane.addTab("2", experimentScroller2);
        // experimentPane.setPreferredSize(experimentScroller1.getPreferredSize());

        leftPanel.add(experimentPane, leftc);

        leftc.gridy = 2;
        leftc.gridwidth = 1;
        leftc.insets = new Insets(2, 0, 0, 0);
        leftc.fill = GridBagConstraints.NONE;
        leftPanel.add(new VPBLabel("1:"), leftc);
        leftc.gridx = 1;
        leftc.weightx = 2;
        String expString1 = (selectedExperiment1 == null) ? "None"
            : selectedExperiment1.getName();
        expLabel1 = new ExpandableVPRLabel(parent, expString1, 17);
        leftPanel.add(expLabel1, leftc);

        leftc.gridy = 3;
        leftc.gridx = 0;
        leftc.weightx = 0;
        leftPanel.add(new VPBLabel("2:"), leftc);
        leftc.gridx = 1;
        leftc.weightx = 2;
        String expString2 = (selectedExperiment2 == null) ? "None"
            : selectedExperiment2.getName();
        expLabel2 = new ExpandableVPRLabel(parent, expString2, 17);
        leftPanel.add(expLabel2, leftc);

        add(leftPanel, superc);

        superc.gridx = 1;
        superc.insets = new Insets(5, 10, 5, 8);
        add(new JSeparator(JSeparator.VERTICAL), superc);

        superc.gridx = 2;
        superc.insets = new Insets(5, 10, 5, 5);

        VPPanel rightPanel = new VPPanel();
        rightPanel.setLayout(new GridBagLayout());
        rightPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
        GridBagConstraints rightc = new GridBagConstraints();
        rightc.anchor = GridBagConstraints.WEST;
        rightc.fill = GridBagConstraints.BOTH;
        rightc.gridwidth = 4;
        // rightc.insets = new Insets(2, 5, 7, 5);

        searchPanel = new VPPanel();
        searchPanel.setLayout(new GridBagLayout());
        searchPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
        GridBagConstraints searchc = new GridBagConstraints();
        searchc.anchor = GridBagConstraints.WEST;
        searchc.fill = GridBagConstraints.BOTH;
        searchc.gridwidth = 2;
        searchPanel.add(new VPBLabel("Search Database"), searchc);

        searchc.gridy = 1;
        searchc.insets = new Insets(2, 5, 7, 5);
        search = new VPTextField(ActionCenter.FIND_EXPERIMENT,
            listener);
        searchPanel.add(search, searchc);

        searchc.gridy = 2;
        searchc.insets = new Insets(6, 5, 7, 5);
        searchc.gridwidth = 1;
        searchc.weightx = 1;
        searchPanel.add(new VPButton("Find",
            ActionCenter.FIND_EXPERIMENT, listener), searchc);
        searchc.gridx = 1;
        searchPanel.add(new VPButton("Clear", ActionCenter.CLEAR,
            listener), searchc);

        searchc.gridy = 3;
        searchc.gridx = 0;
        searchc.gridwidth = 2;
        searchPanel.add(new JSeparator(JSeparator.HORIZONTAL),
            searchc);

        searchc.gridy = 4;
        searchPanel.add(new VPButton("Load Data File",
            ActionCenter.LOAD_EXPERIMENT_FILE, listener), searchc);

        displaySearchPanel = new VPPanel();
        displaySearchPanel.setLayout(new GridLayout());
        displaySearchPanel.add(searchPanel);
        rightPanel.add(displaySearchPanel, rightc);
        rightc.insets = new Insets(6, 5, 7, 5);
        rightc.gridy = 1;
        rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

        rightc.gridy = 2;
        rightc.gridwidth = 2;
        detailButton = new VPButton("Details",
            ActionCenter.EXPERIMENT_DETAILS, listener);
        rightPanel.add(detailButton, rightc);
        rightc.gridx = 2;
        rightPanel.add(new VPButton("Edit",
            ActionCenter.EDIT_GRADIENT, listener), rightc);

        rightc.gridy = 3;
        rightc.gridx = 0;
        rightc.gridwidth = 4;
        rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

        rightc.gridy = 4;
        rightc.gridwidth = 1;
        rightc.weightx = 0;
        rightPanel.add(new VPButton("OK", ActionCenter.OK, listener),
            rightc);
        rightc.gridx = 1;
        rightc.gridwidth = 3;
        rightPanel.add(new VPButton("Cancel", ActionCenter.CANCEL,
            listener), rightc);

        add(rightPanel, superc);
    }

    private class DetailScroller extends JScrollPane
    {
        public DetailScroller()
        {
            super(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
            Experiment exp = (experimentPane.getSelectedIndex() == 0) ? selectedExperiment1
                : selectedExperiment2;

            JViewport jv = new JViewport();
            ExperimentDetailPanel expPanel = new ExperimentDetailPanel(
                exp);
            jv.setView(expPanel);
            setPreferredSize(new Dimension(106, 160));
            setViewport(jv);

        }
    }

    private void findExperiment(String searchString)
    {
        DefaultListModel currentModel = (experimentPane
            .getSelectedIndex() == 0) ? experiments1 : experiments2;
        currentModel.clear();

        ArrayList matches = DBQuery.findExperiment(searchString);
        for (int i = 0; i < matches.size(); i++)
            currentModel.addElement((Experiment) matches.get(i));
    }

    private class Listener implements ActionListener,
        ListSelectionListener
    {
        private Map exp;

        public void actionPerformed(ActionEvent event)
        {
            String command = event.getActionCommand();

            if (command.equals(ActionCenter.FIND_EXPERIMENT))
            {
                DefaultListModel currentModel = (experimentPane
                    .getSelectedIndex() == 0) ? experiments1
                    : experiments2;
                findExperiment(search.getText());

                if (currentModel.isEmpty())
                {
                    String title = "Error";
                    String header = "Experiment not found!";
                    String body = "The experiment you are searching for \n"
                        + "is not in out database. \n"
                        + "Please enter a different search.";
                    DialogFactory.createDialog(parent, IconFactory
                        .getErrorIcon(), title, header, body,
                        DialogFactory.OK_ONLY);
                }
            }
            else if (command.equals(ActionCenter.CLEAR))
            {
                DefaultListModel currentModel = (experimentPane
                    .getSelectedIndex() == 0) ? experiments1
                    : experiments2;
                currentModel.clear();
                search.setText("");
            }

            else if (command.equals(ActionCenter.OK))
            {
                if (selectedExperiment1 == null)
                {
                    String title = "Error";
                    String header = "No experiment selected!";
                    String body = "No experiment has been selected. \n"
                        + "Please select an experiment \n"
                        + "from the database.";
                    DialogFactory.createDialog(parent, IconFactory
                        .getErrorIcon(), title, header, body,
                        DialogFactory.OK_ONLY);
                }
                else if (gradient == null)
                {
                    String title = "Error";
                    String header = "No gradient selected!";
                    String body = "No gradient has been selected.  \n"
                        + "Please select a gradient \n"
                        + "by clicking the edit button.";
                    DialogFactory.createDialog(parent, IconFactory
                        .getErrorIcon(), title, header, body,
                        DialogFactory.OK_ONLY);
                }
                else
                {
                    // if (exp == null)
                    // exp =
                    // DBQuery.getExpressions(
                    // selectedExperiment1,
                    // vGraph.getPathway());
                    // vGraph.setExpressionData(exp, gradient);

                    if (exp == null)
                    {
                        exp = new HashMap();
                        ListIterator allEdges = pGraph
                            .getEdgeIterator();
                        while (allEdges.hasNext())
                        {
                            ListIterator allDecs = ((PathwayEdge) allEdges
                                .next()).getDecorationIterator();
                            while (allDecs.hasNext())
                            {
                                PathwayDecoration currDec = (PathwayDecoration) allDecs
                                    .next();
                                String oldEC = (String) currDec
                                    .getDataClass().get("ECNumber");
                                HashMap valueMap = DBQuery
                                    .getExpByGene(
                                        selectedExperiment1, pGraph
                                            .getGraphDataClass(),
                                        oldEC, pGraph);
                                ((HashMap) exp).put(oldEC, valueMap);
                            }
                        }
                    }
                    pGraph.setExpressionDataByGene(exp, gradient,
                        selectedExperiment1);
                    parent.clearEditPanel();
                    parent.getToolBar().getExperimentButton()
                        .setSelected(false);
                }
            }

            else if (command.equals(ActionCenter.CANCEL))
            {
                parent.clearEditPanel();
                parent.getToolBar().getExperimentButton()
                    .setSelected(false);
            }

            else if (command.equals(ActionCenter.EXPERIMENT_DETAILS))
            {
                detailButton.setText("Search");
                detailButton
                    .setActionCommand(ActionCenter.EXPERIMENT_SEARCH);
                displaySearchPanel.removeAll();
                DetailScroller ds = new DetailScroller();

                displaySearchPanel.add(ds);

            }

            else if (command.equals(ActionCenter.EXPERIMENT_SEARCH))
            {
                detailButton.setText("Details");
                detailButton
                    .setActionCommand(ActionCenter.EXPERIMENT_DETAILS);
                displaySearchPanel.removeAll();

                displaySearchPanel.add(searchPanel);
            }

            else if (command.equals(ActionCenter.EDIT_GRADIENT))
            {
                if (selectedExperiment1 == null)
                {
                    String title = "Error";
                    String header = "No experiment selected!";
                    String body = "No experiment has been selected.  \n"
                        + "Please select one or more \n"
                        + "experiments before editing \n"
                        + "the gradient.";
                    DialogFactory.createDialog(parent, IconFactory
                        .getErrorIcon(), title, header, body,
                        DialogFactory.OK_ONLY);
                }
                else
                {
                    // exp =
                    // DBQuery.getExpressions(
                    // selectedExperiment1,
                    // vGraph.getPathway());
                    final long startTime = System.currentTimeMillis();

                    VPSwingWorker values = new VPSwingWorker(parent,
                        parent.getGraphPanel(),
                        "loadExperimentValues", 0, 95, pGraph
                            .getEdgeCount())
                    {
                        protected void doNonUI()
                            throws RuntimeException
                        {
                            boolean replace = false;
                            if (pGraph.isExperimentData())
                                replace = true;

                            exp = getExpValues(selectedExperiment2 != null);

                        }

                        protected void doUI() throws RuntimeException
                        {
                            long stopTime = System
                                .currentTimeMillis();
                            parent
                                .addToEditPanel(new EditGradientDialog(
                                    parent, exp, thisDialog));
                        }
                    };
                    values.start();

                }
            }
        }

        private Map getExpValues(boolean foldChange)
        {
            exp = new HashMap();
            int numDecs = 0;
            boolean isGeneData = pGraph.isGeneData();
            ListIterator allEdges = pGraph.getEdgeIterator();
            while (allEdges.hasNext())
            {
                PathwayEdge pe = (PathwayEdge) allEdges.next();

                ListIterator allDecIter = pe.getDecorationIterator();
                while (allDecIter.hasNext())
                {
                    PathwayDecoration currDec = (PathwayDecoration) allDecIter
                        .next();

                    if (!isGeneData)
                    {
                        String oldEC = (String) currDec
                            .getDataClass().get("ECNumber");

                        
                        HashMap valueMap = DBQuery.getExpByGene(
                            selectedExperiment1, pGraph
                                .getGraphDataClass(), oldEC, pGraph);
                        if (foldChange)
                        {
                            HashMap valueMap2 = DBQuery.getExpByGene(
                                selectedExperiment2, pGraph
                                    .getGraphDataClass(), oldEC,
                                pGraph);
                            // See if there's values for each gene
                            Set vmKeys = valueMap.keySet();
                            Iterator vmKeyIter = vmKeys.iterator();
                            while (vmKeyIter.hasNext())
                            {
                                String fold = null;
                                DataClass next = (DataClass) vmKeyIter
                                    .next();
                                if (valueMap2.get(next) != null)
                                {
                                    try
                                    {
                                        double value1 = Double
                                            .parseDouble(((DataClass) valueMap
                                                .get(next)).get(
                                                "Value").toString());
                                        double value2 = Double
                                            .parseDouble(((DataClass) valueMap2
                                                .get(next)).get(
                                                "Value").toString());
                                        double prefold = value1
                                            / value2;
                                        fold = String
                                            .valueOf((prefold) > 1 ? prefold
                                                : -(1 / prefold));
                                    }
                                    catch (NumberFormatException nfe)
                                    {
                                        fold = "N/A";
                                    }
                                }
                                valueMap.put(next, fold);

                            }
                        }
                        exp.put(oldEC, valueMap);
                    }

                    else
                    {
                        /*
                         * If its not an ecNumber or gene then clear
                         * it because we want to override.
                         */
                        if (!((String) currDec.getDataClass()
                            .getObjectID()).startsWith("GE"))
                        {
                            //pe.removeDecoration(currDec);

                        }

                        else
                        {
                            String oldSymbol = (String) currDec
                                .getDataClass().get("Name");
                            

                            String geneValue = DBQuery.getGeneValue(
                                selectedExperiment1, pGraph
                                    .getGraphDataClass(), oldSymbol,
                                pGraph);

                            if (foldChange)
                            {
                                String fold = null;

                                String geneValue2 = DBQuery
                                    .getGeneValue(
                                        selectedExperiment2, pGraph
                                            .getGraphDataClass(),
                                        oldSymbol, pGraph);
                                // See if there's values for each gene
                                try
                                {
                                    double value1 = Double
                                        .parseDouble(geneValue);
                                    double value2 = Double
                                        .parseDouble(geneValue2);
                                    double prefold = value1 / value2;
                                    fold = String
                                        .valueOf((prefold) > 1 ? prefold
                                            : -(1 / prefold));
                                }
                                catch (NumberFormatException nfe)
                                {
                                    fold = "N/A";
                                }

                                geneValue = fold;

                            }

                            exp.put(oldSymbol, geneValue);
                        }
                    }
                }
            }
            return exp;
        }

        public void valueChanged(ListSelectionEvent event)
        {
            if (event.getSource() == experimentList1)
            {
                selectedExperiment1 = (Experiment) experimentList1
                    .getSelectedValue();
                expLabel1.resetText((experimentList1
                    .getSelectedIndex() != -1) ? selectedExperiment1
                    .getName() : "None");
                if (detailButton.getText().equals("Search"))
                {
                    displaySearchPanel.removeAll();
                    DetailScroller ds = new DetailScroller();
                    displaySearchPanel.add(ds);
                }
            }
            else if (event.getSource() == experimentList2)
            {
                selectedExperiment2 = (Experiment) experimentList2
                    .getSelectedValue();
                expLabel2.resetText((experimentList2
                    .getSelectedIndex() != -1) ? selectedExperiment2
                    .getName() : "None");
                if (detailButton.getText().equals("Search"))
                {
                    displaySearchPanel.removeAll();
                    DetailScroller ds = new DetailScroller();
                    displaySearchPanel.add(ds);
                }
            }
        }

    }

    /**
     * @return
     */
    public VPGradient getGradient()
    {
        return gradient;
    }

    /**
     * @param gradient
     */
    public void setGradient(VPGradient gradient)
    {
        this.gradient = gradient;
    }

}
