package vp.gui.dialogs;

import vp.graph.GraphDataClass;
import vp.gui.VPSettings;
import vp.gui.components.DialogFactory;
import vp.gui.components.DialogHeader;
import vp.gui.components.VPBLabel;
import vp.gui.components.VPButton;
import vp.gui.components.VPPanel;
import vp.gui.components.VPTextField;
import vp.main.MainFrame;
import vp.util.ActionCenter;
import vp.util.IconFactory;
import vp.util.database.DBQuery;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;

public class SelectPathwayDialog extends VPPanel
{
	private MainFrame parent;
	private NewPathwayDialog referrer;
	private Listener listener;
	private JList pathwayList;
	private DefaultListModel pathways;
	private VPTextField search;
	private GraphDataClass selectedPathway;

	public SelectPathwayDialog(MainFrame parent, NewPathwayDialog referrer)
	{
		this.parent = parent;
		this.referrer = referrer;
		listener = new Listener();

		setLayout(new GridBagLayout());
		GridBagConstraints superc = new GridBagConstraints();
		superc.anchor = GridBagConstraints.WEST;
		superc.insets = new Insets(5, 5, 5, 5);
		superc.gridwidth = 3;

		VPPanel p =
			new DialogHeader(
				IconFactory.getVPIcon(),
				"Search for a defined pathway");
		add(p, superc);

		superc.gridy = 1;
		superc.fill = GridBagConstraints.BOTH;
		add(new JSeparator(JSeparator.HORIZONTAL), superc);

		superc.gridy = 2;
		superc.gridwidth = 1;
		superc.anchor = GridBagConstraints.WEST;

		VPPanel leftPanel = new VPPanel();
		leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.Y_AXIS));

		leftPanel.add(new VPBLabel("Pathways"));

		pathways = new DefaultListModel();
		pathwayList = new JList(pathways);
		pathwayList.setFont(VPSettings.R_FONT);
		int pathWidth =
			pathwayList.getFontMetrics(VPSettings.R_FONT).stringWidth(
				"<No pathway selected>")
				+ 20;
		pathwayList.setVisibleRowCount(6);
		JScrollPane pathScroller =
			new JScrollPane(
				pathwayList,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		pathScroller.setPreferredSize(
			new Dimension(pathWidth, pathwayList.getPreferredSize().height));
		leftPanel.add(pathScroller);

		add(leftPanel, superc);

		superc.gridx = 1;
		add(new JSeparator(JSeparator.VERTICAL), superc);

		superc.gridx = 2;
		VPPanel rightPanel = new VPPanel();
		rightPanel.setLayout(new GridBagLayout());
		GridBagConstraints rightc = new GridBagConstraints();
		rightc.anchor = GridBagConstraints.WEST;
		rightc.fill = GridBagConstraints.BOTH;
		rightc.gridwidth = 4;
		rightc.insets = new Insets(0, 5, 5, 5);

		rightPanel.add(new VPBLabel("Search"), rightc);

		rightc.gridy = 1;
		search = new VPTextField(ActionCenter.FIND_PATHWAY, listener);
		rightPanel.add(search, rightc);

		rightc.gridy = 2;
		rightc.insets = new Insets(5, 5, 5, 5);
		rightc.gridwidth = 2;
		rightc.weightx = 1;
		rightPanel.add(
			new VPButton("Find", ActionCenter.FIND_PATHWAY, listener),
			rightc);
		rightc.gridx = 2;
		rightPanel.add(
			new VPButton("Clear", ActionCenter.CLEAR, listener),
			rightc);

		rightc.gridy = 3;
		rightc.gridx = 0;
		rightc.gridwidth = 4;
		rightPanel.add(new JSeparator(JSeparator.HORIZONTAL), rightc);

		rightc.gridy = 4;
		rightc.gridwidth = 1;
		rightc.weightx = 0;
		rightPanel.add(new VPButton("OK", ActionCenter.OK, listener), rightc);
		rightc.gridx = 1;
		rightc.gridwidth = 3;
		rightPanel.add(
			new VPButton("Cancel", ActionCenter.CANCEL, listener),
			rightc);

		add(rightPanel, superc);
	}

	private void findPathway(String searchString)
	{
		pathways.clear();
		ArrayList matches = DBQuery.findPathway(searchString);
		for (int i = 0; i < matches.size(); i++)
			pathways.addElement((String)matches.get(i));
	}

	private class Listener implements ActionListener
	{
		public void actionPerformed(ActionEvent event)
		{
			String command = event.getActionCommand();
			if (command.equals(ActionCenter.FIND_PATHWAY))
			{
				findPathway(search.getText());

				if (pathways.isEmpty())
				{
					String title = "Error";
					String header = "Pathway not found!";
					String body =
						"The pathway you are searching for \n"
							+ "is not in out database. \n"
							+ "Please enter a different search \n"
							+ "or add a new compound.";
					DialogFactory.createDialog(
						parent,
						IconFactory.getErrorIcon(),
						title,
						header,
						body,
						DialogFactory.OK_ONLY);
				}
			}
			else if (command.equals(ActionCenter.CLEAR))
			{
				pathways.clear();
				search.setText("");
			}
			else if (command.equals(ActionCenter.OK))
			{
				selectedPathway = (GraphDataClass)pathwayList.getSelectedValue();

				if (selectedPathway == null)
				{
					String title = "Error";
					String header = "No pathway selected!";
					String body =
						"No pathway has been selected. \nPlease select one from the database \nor cancel and create a new one.";
					DialogFactory.createDialog(
						parent,
						IconFactory.getErrorIcon(),
						title,
						header,
						body,
						DialogFactory.OK_ONLY);
				}
				else
				{
					referrer.setPathway(selectedPathway);
				}
			}
			else if (command.equals(ActionCenter.CANCEL))
				parent.clearEditPanel();
		}

	}
}
