package vp.gui.table;

import vp.gui.table.components.*;
import vp.visual.type.Type;

import java.util.ArrayList;

public class EdgeData implements VPData
{
	private Boolean show;
	private String vertexA;
	private Type type;
	private String vertexB;
	private ArrayList enzymes;
	private String direction;
	private int enzymeValue;

	public static final String FORWARD = "Forwards";
	public static final String BACKWARD = "Backwards";
	public static final String BOTH = "Both ways";
	public static final String NEITHER = "Neither way";

	public EdgeData(boolean show, String vertexA, Type type, String vertexB, ArrayList enzymes, String direction)
	{
		this.show = new Boolean(show);
		this.vertexA = vertexA;
		this.type = type;
		this.vertexB = vertexB;
		this.enzymes = enzymes;
		this.direction = direction;
		enzymeValue = 0;
	}

	public Object getData(int index)
	{
		switch (index)
		{
		case 0:
			return show;
		case 1:
			return vertexA;
		case 2:
			return type;
		case 3:
			return vertexB;
		case 4:
			return enzymes;
		case 5:
			return direction;
		default:
			return new Object();
		}
	}

	public int getEnzymeValue()
	{
		return enzymeValue;
	}

	public void setEnzymeValue(int enzymeValue)
	{
		this.enzymeValue = enzymeValue;
	}

	public Boolean getShow()
	{
		return show;
	}

	public void setShow(boolean show)
	{
		this.show = new Boolean(show);
	}

	public String getVertexA()
	{
		return vertexA;
	}

	public void setVertexA(String vertexA)
	{
		this.vertexA = vertexA;
	}

	public Type getType()
	{
		return type;
	}

	public void setType(Type type)
	{
		this.type = type;
	}

	public String getVertexB()
	{
		return vertexB;
	}

	public void setVertexB(String vertexB)
	{
		this.vertexB = vertexB;
	}

	public ArrayList getEnzymes()
	{
		return enzymes;
	}

	public void setEnzymes(ArrayList enzymes)
	{
		this.enzymes = enzymes;
	}

	public String getDirection()
	{
		return direction;
	}

	public void setDirection(String direction)
	{
		this.direction = direction;
	}

	/* (non-Javadoc)
	 * @see vp.src.edu.yale.vitapad.gui.table.VPData#get(javax.swing.JTable, int, int)
	 */
	public Object get(VPTable table, int row, int col)
	{
		VPCellComponent vpcc;
		VPTableButton vptb;
		VPTableComponent vptc;
		
		switch(col)
		{
		case 0:
			vpcc = new CheckCellComponent(show, row, col);
			vptb = new VPTableButton(VPTableButton.LOOKUP);
			vptc = new VPTableComponent(table, vpcc, vptb);
			return vptc;
		case 1:
			vpcc = new PlainCellComponent(vertexA, row, col);
			vptb = new VPTableButton(VPTableButton.GOTO);
			vptc = new VPTableComponent(table, vpcc, vptb);
			return vptc;
		case 2:
			vpcc = new TypeCellComponent(table, type, row, col);
			vptb = new VPTableButton(VPTableButton.EDIT);
			vptc = new VPTableComponent(table, vpcc, vptb);
			return vptc;
		case 3:
			vpcc = new PlainCellComponent(vertexB, row, col);
			vptb = new VPTableButton(VPTableButton.GOTO);
			vptc = new VPTableComponent(table, vpcc, vptb);
			return vptc;
		case 4:
			vpcc = new ArrayCellComponent(table, enzymes, row, col);
			vptb = new VPTableButton(VPTableButton.GOTO);
			vptc = new VPTableComponent(table, vpcc, vptb);
			return vptc;
		case 5:
			vpcc = new PlainCellComponent(direction, row, col);
			vptb = null;
			vptc = new VPTableComponent(table, vpcc, vptb);
			return vptc;
		default:
			return new Object();
		
		}
	}
}
