package vp.gui.table;

import vp.graph.PathwayDecoration;
import vp.graph.PathwayEdge;
import vp.graph.PathwayGraph;
import vp.graph.PathwayVertex;
import vp.visual.type.Type;

import java.util.ArrayList;
import java.util.ListIterator;
import java.util.Vector;
import javax.swing.JLabel;

public class EdgeTableData extends VPTableData
{

	private PathwayGraph pGraph;

	public EdgeTableData(PathwayGraph pGraph, VPTable vpTable)
	{
		this.pGraph = pGraph;
		this.vpTable = vpTable;
		columns = new ColumnData[6];
		columns[0] = new ColumnData("?", 45, JLabel.CENTER, true);
		columns[1] = new ColumnData("VertexA", 160, JLabel.LEFT, true);
		columns[2] = new ColumnData("Type", 120, JLabel.LEFT, false);
		columns[3] = new ColumnData("VertexB", 160, JLabel.LEFT, true);
		columns[4] = new ColumnData("Enzymes", 100, JLabel.LEFT, false);
		columns[5] = new ColumnData("Direction", 80, JLabel.CENTER, true);
		v = new Vector();
		setData(pGraph);
	}

	public void setData(PathwayGraph pGraph)
	{
		v.removeAllElements();
		ListIterator viter = pGraph.getEdgeIterator();
		while (viter.hasNext())
		{
			PathwayEdge current = (PathwayEdge)viter.next();
			String vertexA = ((PathwayVertex)current.getVertexA()).getLabel();
			String vertexB = ((PathwayVertex)current.getVertexB()).getLabel();
			Type type = current.getType();

			ListIterator deciter = current.getDecorationIterator();
			ArrayList enzymes = new ArrayList();
			while (deciter.hasNext())
			{
				PathwayDecoration currdec = (PathwayDecoration)deciter.next();
				if (currdec.getDataClass().getClassName().equals("Enzyme"))
					enzymes.add(currdec.getDataClass().get("ECNumber"));
			}
			String dir = current.getDirection();

			v.addElement(
				new EdgeData(true, vertexA, type, vertexB, enzymes, dir));
		}
	}

	public Object getValueAt(int nRow, int nCol)
	{
		EdgeData row = (EdgeData)v.elementAt(nRow);
		return row.get(vpTable, nRow, nCol);
	}

	public void setValueAt(Object value, int nRow, int nCol)
	{
		EdgeData row = (EdgeData)v.elementAt(nRow);
		switch (nCol)
		{
			case 0 :
				row.setShow(((Boolean)value).booleanValue());
		}
	}

	public Object getUnderlyingValueAt(int nRow, int nCol)
	{
		EdgeData row = (EdgeData)v.elementAt(nRow);
		return row.getData(nCol);
	}
}
