package vp.gui.table;

import vp.graph.PathwayGraph;
import vp.gui.VPSettings;
import vp.gui.table.components.*;
import vp.main.MainFrame;
import java.awt.Cursor;
import java.awt.Dimension;
import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.JTableHeader;

public abstract class VPTable extends JTable
{
	PathwayGraph pGraph;
	VPTableData tableData;
	TableViewer viewer;
	Cell selectedCell;
	Cell highlightedCell;

	public static final int ROW_HEIGHT = 28;
	public static final int TYPE_WIDTH = 90;

	public void init()
	{
		selectedCell = null;
		highlightedCell = null;
		setAutoCreateColumnsFromModel(false);
		setModel(tableData);
		setRowHeight(VPTable.ROW_HEIGHT);
		setRowMargin(0);
		getColumnModel().setColumnMargin(0);
		setFont(VPSettings.R_FONT);
		setShowGrid(false);

		for (int i = 0; i < tableData.getColumnCount(); i++)
		{
			TableCellEditor editor = null;
			TableCellRenderer renderer = new VPCellRenderer();
			TableCellRenderer headerRenderer = null;

			if (tableData.getColumn(i).isSortable())
				headerRenderer = new SortableHeaderCellRenderer(this);

			String colName = tableData.getColumn(i).getName();

			if (colName.equals("?"))
			{
                editor = new DefaultCellEditor(new JCheckBox());
			}

			TableColumn column =
				new TableColumn(
					i,
					tableData.getColumn(i).getWidth(),
					renderer,
					editor);
			column.setHeaderRenderer(headerRenderer);
			addColumn(column);
		}
		VPTableListener tableListener = new VPTableListener(this);
		addMouseListener(tableListener);
		addMouseMotionListener(tableListener);
		JTableHeader header = getTableHeader();
		header.setFont(VPSettings.B_FONT);
		header.setPreferredSize(
			new Dimension(header.getPreferredSize().width, VPTable.ROW_HEIGHT));
		VPColumnListener headerListener = new VPColumnListener(this);
		header.addMouseListener(headerListener);
		header.addMouseMotionListener(headerListener);
	}

	public void fixColumnWidth(int columnNo, int width)
	{
		TableColumn column = getColumnModel().getColumn(columnNo);
		column.setMaxWidth(width);
		column.setMinWidth(width);
		column.setPreferredWidth(width);
	}

	public MainFrame getMainFrame()
	{
		return pGraph.getParent();
	}

	public TableViewer getTableViewer()
	{
		return viewer;
	}
	/* (non-Javadoc)
	 * @see javax.swing.JTable#resizeAndRepaint()
	 */

	public void fix()
	{
		resizeAndRepaint();
	}
	/**
	 * @return
	 */
	public Cell getHighlightedCell()
	{
		return highlightedCell;
	}

	/**
	 * @return
	 */
	public Cell getSelectedCell()
	{
		return selectedCell;
	}

	/**
	 * @param cell
	 */
	public void setHighlightedCell(Cell cell)
	{
		if (selectedCell != null)
		{
			VPTableComponent sVptc =
				(VPTableComponent)getValueAt(selectedCell.row,
					selectedCell.column);
			sVptc.getIconButton().setSelected();
			(
				(VPCellRenderer)getCellRenderer(
					selectedCell.row,
					selectedCell.column)).render(
				sVptc);			
		}
		if (highlightedCell != null)
		{
			setCursor(Cursor.getDefaultCursor());
			int row = highlightedCell.row;
			int col = highlightedCell.column;
			VPTableComponent vptc = (VPTableComponent)getValueAt(row, col);
			vptc.getIconButton().setOff();
			((VPCellRenderer)getCellRenderer(row, col)).render(null);
			highlightedCell = null;
		}
		if (cell != null)
		{
			setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			highlightedCell = cell;
			VPTableComponent newVptc =
				(VPTableComponent)getValueAt(cell.row, cell.column);
			newVptc.getIconButton().setRollover();
			((VPCellRenderer)getCellRenderer(cell.row, cell.column)).render(
				newVptc);
		}
        fix();
	}

	/**
	 * @param cell
	 */
	public void setSelectedCell(Cell cell)
	{
		if (selectedCell != null)
		{
			setCursor(Cursor.getDefaultCursor());
			int row = selectedCell.row;
			int col = selectedCell.column;
			VPTableComponent vptc = (VPTableComponent)getValueAt(row, col);
			vptc.getIconButton().setOff();
			((VPCellRenderer)getCellRenderer(row, col)).render(null);
			selectedCell = null;
		}
		if (cell != null)
		{
			selectedCell = cell;
			VPTableComponent newVptc =
				(VPTableComponent)getValueAt(cell.row, cell.column);
			newVptc.getIconButton().setSelected();
			((VPCellRenderer)getCellRenderer(cell.row, cell.column)).render(
				newVptc);
			setCursor(Cursor.getDefaultCursor());
		}
        fix();
	}

	public boolean hasSelectedCell()
	{
		return selectedCell != null;
	}

	public boolean hasHighlightedCell()
	{
		return highlightedCell != null;
	}

}
