package vp.gui.table.components;

import vp.graph.DataClass;
import vp.graph.PathwayEdge;
import vp.graph.PathwayVertex;
import vp.gui.VPSettings;
import vp.gui.components.VPPanel;
import vp.main.MainFrame;
import vp.util.database.DBQuery;
import vp.visual.Renderer;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import javax.swing.border.EtchedBorder;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

public class PreviewLabel extends JLabel
{

	private MainFrame parent;
	private Renderer element;
	private int width, height;
	private static JPopupMenu pop;
	private static JLabel popImage;
	private static Ellipse2D.Double zoomCircle;
	private static Listener listener;
	private static PreviewLabel thisPanel;
	private static BufferedImage vImage;

	private static final int DEF_WIDTH = 200;
	private static final int DEF_HEIGHT = 50;

	/**
	     * @param frame
	     * @param renderer
	     * @param dimension
	     */
	public PreviewLabel(MainFrame parent, Renderer element, Dimension size)
	{
		this.parent = parent;
		this.element = element;

		listener = new Listener();
		thisPanel = this;
		width = (size != null) ? size.width : DEF_WIDTH;
		height = (size != null) ? size.height : DEF_HEIGHT;

		setPreferredSize(new Dimension(width, height));
		setBorder(new EtchedBorder(VPSettings.Colors.MD_BLUE, Color.lightGray));
	}

	public void paintComponent(Graphics graphics)
	{
		super.paintComponent(graphics);
		Graphics2D g2D = (Graphics2D)graphics;
		g2D.setColor(Color.white);
		g2D.fillRect(0, 0, width, height);
		g2D.setRenderingHint(
			RenderingHints.KEY_ANTIALIASING,
			RenderingHints.VALUE_ANTIALIAS_ON);

		if (element instanceof PathwayVertex)
		{
			PathwayVertex vert = (PathwayVertex)element;
			boolean fullX, fullY;

			Rectangle bounds = vert.getBounds();
			double xOffset = 0 - bounds.getX();
			double yOffset = 0 - bounds.getY();
			AffineTransform trans = new AffineTransform();
			trans.translate(xOffset, yOffset);
			vert.getGeneralPath().transform(trans);

			vImage =
				new BufferedImage(
					bounds.width + 1,
					bounds.height + 1,
					BufferedImage.TYPE_INT_RGB);
			Graphics2D vg2 = vImage.createGraphics();
			vg2.setColor(Color.white);
			vg2.fillRect(0, 0, vImage.getWidth(), vImage.getHeight());
			vert.paint(vg2);

			double scaleX, scaleY;
			double fullWidth = width * .75;
			double fullHeight = height * .6;

			if (vImage.getWidth() > fullWidth)
			{
				scaleX = 1 / (vImage.getWidth() / fullWidth);
				fullX = true;
			}
			else
			{
				scaleX = 1;
				fullX = false;
			}
			if (vImage.getHeight() > fullHeight)
			{
				scaleY = 1 / (vImage.getHeight() / fullHeight);
				fullY = true;
			}
			else
			{
				scaleY = 1;
				fullY = false;
			}

			AffineTransform trans2 = new AffineTransform();
			trans2.scale(scaleX, scaleY);
			AffineTransformOp transOp =
				new AffineTransformOp(trans2, AffineTransformOp.TYPE_BILINEAR);
			BufferedImage newVImage = transOp.filter(vImage, null);

			double startX = width / 8d;
			double startY = height / 10d;

			int imageX = fullX ? (int)startX : (width - bounds.width) / 2;
			int imageY = fullY ? (int)startY : (height - bounds.height) / 2;

			g2D.drawImage(newVImage, imageX, imageY, null);

			if (fullX || fullY)
			{
				g2D.setRenderingHint(
					RenderingHints.KEY_ANTIALIASING,
					RenderingHints.VALUE_ANTIALIAS_ON);
				g2D.setColor(Color.black);
				g2D.drawLine(width / 2, height - 12, width / 2, height - 4);
				g2D.drawLine(
					(width / 2) - 4,
					height - 8,
					(width / 2) + 4,
					height - 8);

				zoomCircle =
					new Ellipse2D.Double((width / 2) - 5, height - 13, 10, 10);
				g2D.draw(zoomCircle);
				VPPanel panel = new VPPanel();
				popImage = new JLabel(new ImageIcon(vImage));
				panel.add(popImage);
				panel.setBackground(Color.WHITE);
				pop = new JPopupMenu();
				pop.add(panel);

				addMouseMotionListener(listener);
			}
		}

		else if (element instanceof PathwayEdge)
		{
			PathwayEdge edge = (PathwayEdge)element;

			boolean moveUp = edge.getDecorationCount() > 0;
			double drawHeight = moveUp ? height / 4 : height / 2;

			PathwayVertex edgeVertex1 =
				new PathwayVertex(
					null,
					DBQuery.getUniqueID(),
					new DataClass("_BLANK", null),
					10,
					(int)drawHeight);
			PathwayVertex edgeVertex2 =
				new PathwayVertex(
					null,
					DBQuery.getUniqueID(),
					new DataClass("_BLANK", null),
					width - 10,
					(int)drawHeight);
			edge.setVertexA(edgeVertex1);
			edge.setVertexB(edgeVertex2);
			edgeVertex1.paint(g2D);
			edgeVertex2.paint(g2D);
			edge.paint(g2D);
		}
	}

	private class Listener implements MouseListener, MouseMotionListener
	{
		Listener()
		{}
		public void mouseMoved(MouseEvent evt)
		{
			Point2D.Float point = new Point2D.Float(evt.getX(), evt.getY());
			Point converted =
				SwingUtilities.convertPoint(
					thisPanel,
					(width - vImage.getWidth()) / 2,
					0,
					parent);
			if (zoomCircle.getBounds().contains(point))
			{
				pop.setLocation(converted.x, converted.y);
				pop.setVisible(true);
				pop.addMouseListener(this);
			}
			else
			{
				pop.setVisible(false);
			}

		}
		public void mouseDragged(MouseEvent evt)
		{}
		public void mouseClicked(MouseEvent evt)
		{}
		public void mouseEntered(MouseEvent evt)
		{}
		public void mouseExited(MouseEvent evt)
		{
			pop.setVisible(false);
		}
		public void mousePressed(MouseEvent evt)
		{}
		public void mouseReleased(MouseEvent evt)
		{}
	}
}
