package vp.gui.table.components;

import vp.gui.components.VPBLabel;
import vp.gui.components.VPPanel;
import vp.gui.table.VPTable;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import javax.swing.JTable;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;

public class SortableHeaderCellRenderer extends VPPanel implements TableCellRenderer
{
	private JTableHeader th;
	private Cursor defaultCursor;
	private VPTable vpTable;
    private TableViewer tv;
	private VPTableButton upButton, downButton;
	private String value;
	private boolean upSelected, downSelected, upRollover, downRollover;
	Cursor handCursor, prevCursor;
	
	public SortableHeaderCellRenderer(VPTable vpTable)
	{
		this.vpTable = vpTable;
        tv = vpTable.getTableViewer();
        th = vpTable.getTableHeader();
		upButton = new VPTableButton(VPTableButton.ASCENDING);
		downButton = new VPTableButton(VPTableButton.DESCENDING);
		handCursor = Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);
        defaultCursor = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);
	}

	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
	{
		removeAll();
		setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		if(((String)value).length() > 0)
		{
			gbc.weightx = 999.0;
			add(new VPBLabel((String)value), gbc);
		}
		gbc.gridx = 1;
		gbc.weightx = 1.0;
		gbc.insets = new Insets(0, 3, 0, 3);
		add(upButton, gbc);
		gbc.gridx = 2;
		gbc.insets = new Insets(0, 0, 0, 3);
		add(downButton, gbc);
		
		this.value = (String)value;
		return this;
	}

	public VPTableButton getUpButton()
	{
		return upButton;
	}

	public VPTableButton getDownButton()
	{
		return downButton;
	}

	public boolean isUpButton(int x, int y)
	{
		if(upButton == null)
			return false;
		
		if(upButton.getBounds().contains(x, y))
			return true;
	
		return false;
	}

	public boolean isDownButton(int x, int y)
	{
		if(downButton == null)
			return false;
		
		if(downButton.getBounds().contains(x, y))
			return true;
		
		return false;
	}

	public void reset()
	{
		if(upSelected || upRollover)
		{
			upButton.setOff();
			upSelected = false;
		}
		if(downSelected || downRollover)
		{
			downButton.setOff();
			downSelected = false;
		}
	}

	public boolean isUpSelected()
	{
		return upSelected;
	}

	public boolean isDownSelected()
	{
		return downSelected;
	}

	public boolean isUpRollover()
	{
		return upRollover;
	}

	public boolean isDownRollover()
	{
		return downRollover;
	}

	public void setUpSelected(boolean upSelected)
	{
		this.upSelected = upSelected;

		if(upSelected)
			upButton.setSelected();

		else
			upButton.setOff();
	}

	public void setDownSelected(boolean downSelected)
	{
		this.downSelected = downSelected;
		if(downSelected)
			downButton.setSelected();
		else
			downButton.setOff();
	}
    
	public void setUpRollover(boolean upRollover)
	{
		this.upRollover = upRollover;
		if(upRollover)
		{
			upButton.setRollover();
            if(prevCursor == null)
                prevCursor = th.getCursor();
            th.setCursor(handCursor);
		}
		else
		{
			upButton.setOff();
            th.setCursor(prevCursor);
            prevCursor = null;
		}
        vpTable.getTableHeader().resizeAndRepaint();
	}

	public void setDownRollover(boolean downRollover)
	{
		this.downRollover = downRollover;
		if(downRollover)
		{
			downButton.setRollover();
			if(prevCursor == null)
                prevCursor = th.getCursor();
            th.setCursor(handCursor);
		}
		else
		{
			downButton.setOff();
            th.setCursor(prevCursor);
            prevCursor = null;
		}
        vpTable.getTableHeader().resizeAndRepaint();
	}

	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);

		g.setColor(Color.white);
		g.drawLine(0, 0, getWidth() -1, 0);	//top
		g.drawLine(0, 0, 0, getHeight() - 1);	//left side
		g.setColor(Color.GRAY);
		g.drawLine(0, getHeight() -1, getWidth() - 1, getHeight() - 1);	//bottom
		g.drawLine(getWidth() - 1, 0, getWidth() - 1, getHeight() - 1);	//right side
	}

	
}

