package vp.gui.table.components;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;

import javax.swing.CellRendererPane;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.ListCellRenderer;
import javax.swing.border.EmptyBorder;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicComboBoxUI;
import javax.swing.plaf.metal.MetalComboBoxIcon;

import vp.gui.VPSettings;
/**
 * The KunststoffComboBoxUI is a little bit tricky, but it should work fine in
 * most cases. It currently draws only correctly if the renderer of the combo
 * box is an instance of <code>JComponent</code> and the background color is an
 * instance of <code>ColorUIResource</code>. In a default <code>JComboBox</code>
 * with a default renderer this should be the case.
 */
public class TableComboBoxUI extends BasicComboBoxUI
{

	public static ComponentUI createUI(JComponent c)
	{
		return new TableComboBoxUI();
	}

	/**
	* Installs MyMetalComboBoxButton
	*/
	protected JButton createArrowButton()
	{
		JButton button =
			new MyMetalComboBoxButton(
				comboBox,
				new MetalComboBoxIcon(),
				comboBox.isEditable() ? true : false,
				currentValuePane,
				listBox);
		button.setMargin(new Insets(0, 1, 1, 3));
		return button;
	}

	/*
	* This inner class finally fixed a nasty bug with the combo box. Thanks to
	* Matthew Philips for providing the bugfix.
	* Thanks to Ingo Kegel for fixing two compiling issues for jikes.
	*/
	private final class MyMetalComboBoxButton
		extends VPTableComboBoxButton
	{

		public MyMetalComboBoxButton(
			JComboBox cb,
			Icon i,
			boolean onlyIcon,
			CellRendererPane pane,
			JList list)
		{
			super(cb, i, onlyIcon, pane, list);
			setBorder(new EmptyBorder(5, 3, 5, 3));
		}

		public void paintComponent(Graphics g)
		{
			if (!iconOnly && MyMetalComboBoxButton.this.comboBox != null)
			{
                boolean isSetRendererOpaque = false;
				ListCellRenderer renderer =
					MyMetalComboBoxButton.this.comboBox.getRenderer();
				if (renderer instanceof JComponent)
				{
					JComponent jRenderer = (JComponent)renderer;
					if (jRenderer.isOpaque()
						&& jRenderer.getBackground() instanceof ColorUIResource)
					{
						isSetRendererOpaque = true;
						// remember to set the renderer opaque again
						jRenderer.setOpaque(false);
					}
				}
				super.paintComponent(g);
				if (isSetRendererOpaque)
				{
					((JComponent)renderer).setOpaque(true);
				}
			}
			else
			{
                super.paintComponent(g);
			}
		}
	}
	/* (non-Javadoc)
	 * @see javax.swing.plaf.basic.BasicComboBoxUI#createRenderer()
	 */
	protected ListCellRenderer createRenderer()
	{
		return new TableComboBoxRenderer.UIResource();
	}

	/* (non-Javadoc)
	 * @see javax.swing.plaf.basic.BasicComboBoxUI#paintCurrentValue(java.awt.Graphics, java.awt.Rectangle, boolean)
	 */


	/* (non-Javadoc)
	 * @see javax.swing.plaf.basic.BasicComboBoxUI#paintCurrentValueBackground(java.awt.Graphics, java.awt.Rectangle, boolean)
	 */
	public void paintCurrentValueBackground(
		Graphics g,
		Rectangle bounds,
		boolean hasFocus)
	{
        Color inv = VPSettings.Colors.CLEAR;
        Color t = g.getColor();
		if (comboBox.isEnabled())
			g.setColor(inv);
		else
			g.setColor(inv);
		g.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);
		g.setColor(t);
	}

}
