package vp.gui.table.components;

import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Collections;
import java.util.Vector;

import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;

import vp.gui.table.*;

public class VPColumnListener implements MouseListener, MouseMotionListener
{
	private VPTable vpTable;
    private JTableHeader th;
	public VPColumnListener(VPTable vpTable)
	{
		this.vpTable = vpTable;
        th = vpTable.getTableHeader();
	}

	public void mouseClicked(MouseEvent event)
	{
		TableColumnModel colModel = vpTable.getColumnModel();
		int columnModelIndex = colModel.getColumnIndexAtX(event.getX());
		int modelIndex = colModel.getColumn(columnModelIndex).getModelIndex();

		VPTableData model = (VPTableData)vpTable.getModel();
		
		TableCellRenderer tcr = colModel.getColumn(modelIndex).getHeaderRenderer();
		
		if (tcr instanceof SortableHeaderCellRenderer)
		{
			SortableHeaderCellRenderer shcr = (SortableHeaderCellRenderer)tcr;
			Point p = new Point(convertX(colModel, columnModelIndex, event.getX()), event.getY());
			
			if(shcr.isUpButton(p.x, p.y) && !shcr.isUpSelected())
			{
                shcr.setUpRollover(false);
				shcr.setUpSelected(true);
				shcr.setDownSelected(false);
				vpTable.getTableHeader().resizeAndRepaint();
				resetOthers(colModel, modelIndex);
				sortAllRowsBy(model, modelIndex, true);
				return;
			}

			if(shcr.isDownButton(p.x, p.y) && !shcr.isDownSelected())
			{
                shcr.setDownRollover(false);
				shcr.setDownSelected(true);
				shcr.setUpSelected(false);
				vpTable.getTableHeader().resizeAndRepaint();
				resetOthers(colModel, modelIndex);
				sortAllRowsBy(model, modelIndex, false);
				return;
			}
		}
	}
	
	public void mouseMoved(MouseEvent event)
	{
		TableColumnModel colModel = vpTable.getColumnModel();
		int columnModelIndex = colModel.getColumnIndexAtX(event.getX());
		int modelIndex = colModel.getColumn(columnModelIndex).getModelIndex();

		TableCellRenderer tcr = colModel.getColumn(modelIndex).getHeaderRenderer();
		
		if (tcr instanceof SortableHeaderCellRenderer)
		{
			SortableHeaderCellRenderer shcr = (SortableHeaderCellRenderer)tcr;
			Point p = new Point(convertX(colModel, columnModelIndex, event.getX()), event.getY());
			
			if(shcr.isUpButton(p.x, p.y))
			{
                if(shcr.isUpSelected())
                    return;
                else
                {
                    if(shcr.isDownRollover())
                        shcr.setDownRollover(false);
                      
                    shcr.setUpRollover(true);
                    th.resizeAndRepaint();
                }
            }

			else if(shcr.isDownButton(p.x, p.y))
			{
                if(shcr.isDownSelected())
                    return;
                else
                {
                    if(shcr.isUpRollover())
                       shcr.setUpRollover(false);
                    shcr.setDownRollover(true);
                    th.resizeAndRepaint();
                }
			}
            else
            {
                if(shcr.isUpRollover())
                {
                    shcr.setUpRollover(false);
                    th.resizeAndRepaint();
                }
                if(shcr.isDownRollover())
                {
                    shcr.setDownRollover(false);
                    th.resizeAndRepaint();
                }
            }
		}
	}

	public void mouseEntered(MouseEvent event)
	{
	}

	public void mouseExited(MouseEvent event)
	{
		TableColumnModel colModel = vpTable.getColumnModel();
		int k = -1;
		
		for (int i = 0;i < colModel.getColumnCount() ;i++ )
		{
			TableCellRenderer tcr = colModel.getColumn(i).getHeaderRenderer();
		
			if (tcr instanceof SortableHeaderCellRenderer)
			{
				SortableHeaderCellRenderer shcr = (SortableHeaderCellRenderer)tcr;

				if(shcr.isUpSelected())
				{
					if(shcr.isDownRollover())
						shcr.setDownRollover(false);
					k = i;
					break;
				}

				else if(shcr.isDownSelected())
				{
					if(shcr.isUpRollover())
						shcr.setUpRollover(false);
					k = i;
					break;
				}
			}
		}
		resetOthers(colModel, k);
	}

	public void mousePressed(MouseEvent event)
	{
	}

	public void mouseReleased(MouseEvent event)
	{
	}

	public void mouseDragged(MouseEvent event)
	{
	}

	private int convertX(TableColumnModel colModel, int index, int xPos)
	{
		int push = 0;
		for(int i = 0; i < index; i++)
		{
			push += colModel.getColumn(i).getWidth();
		}
		return xPos - push;
	}

	private void resetOthers(TableColumnModel colModel, int index)
	{
		for (int i = 0;i < colModel.getColumnCount();i++ )
		{
			if (i != index)
			{
				TableCellRenderer tcr = colModel.getColumn(i).getHeaderRenderer();
				if(tcr instanceof SortableHeaderCellRenderer)
					((SortableHeaderCellRenderer)tcr).reset();
			}
		}
	}

	private void sortAllRowsBy(VPTableData model, int colIndex, boolean ascending)
	{
		Vector data = model.getDataVector();
		Collections.sort(data, new ColumnSorter(colIndex, ascending));
		model.fireTableStructureChanged();
	}
}
