/*
 * Created on Mar 31, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package vp.gui.table.components;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.CellRendererPane;
import javax.swing.DefaultButtonModel;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.UIManager;
import javax.swing.plaf.metal.MetalLookAndFeel;

import vp.gui.VPSettings;

/**
 * @author Matt Holford
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class VPTableComboBoxButton extends JButton
{

	protected JComboBox comboBox;
	protected JList listBox;
	protected CellRendererPane rendererPane;
	protected Icon comboIcon;
	protected boolean iconOnly = false;

	public final JComboBox getComboBox()
	{
		return comboBox;
	}
	public final void setComboBox(JComboBox cb)
	{
		comboBox = cb;
	}

	public final Icon getComboIcon()
	{
		return comboIcon;
	}
	public final void setComboIcon(Icon i)
	{
		comboIcon = i;
	}

	public final boolean isIconOnly()
	{
		return iconOnly;
	}
	public final void setIconOnly(boolean isIconOnly)
	{
		iconOnly = isIconOnly;
	}

	VPTableComboBoxButton()
	{
		super("");
		DefaultButtonModel model = new DefaultButtonModel()
		{
			public void setArmed(boolean armed)
			{
				super.setArmed(isPressed() ? true : armed);
			}
		};
		setModel(model);
	}

	public VPTableComboBoxButton(
		JComboBox cb,
		Icon i,
		CellRendererPane pane,
		JList list)
	{
		this();
		comboBox = cb;
		comboIcon = i;
		rendererPane = pane;
		listBox = list;
		setEnabled(comboBox.isEnabled());
	}

	public VPTableComboBoxButton(
		JComboBox cb,
		Icon i,
		boolean onlyIcon,
		CellRendererPane pane,
		JList list)
	{
		this(cb, i, pane, list);
		iconOnly = onlyIcon;
	}

	
	public void setEnabled(boolean enabled)
	{
		super.setEnabled(enabled);

		// Set the background and foreground to the combobox colors.
		if (enabled)
		{
			setBackground(VPSettings.Colors.CLEAR);
			setForeground(comboBox.getForeground());
		}
		else
		{
			setBackground(VPSettings.Colors.CLEAR);
			setForeground(UIManager.getColor("ComboBox.disabledForeground"));
		}
	}

	public void paintComponent(Graphics g)
	{
        boolean leftToRight =
			comboBox.getComponentOrientation().isLeftToRight();

		// Paint the button as usual
		//super.paintComponent(g);

		Insets insets = getInsets();

		int width = getWidth() - (insets.left + insets.right);
		int height = getHeight() - (insets.top + insets.bottom);

		if (height <= 0 || width <= 0)
		{
			return;
		}

		int left = insets.left;
		int top = insets.top;
		int right = left + (width - 1);
		int bottom = top + (height - 1);

		int iconWidth = 0;
		int iconLeft = (leftToRight) ? right : left;

		// Paint the icon
		if (comboIcon != null)
		{
			iconWidth = comboIcon.getIconWidth();
			int iconHeight = comboIcon.getIconHeight();
			int iconTop = 0;

			if (iconOnly)
			{
				iconLeft = (getWidth() / 2) - (iconWidth / 2);
				iconTop = (getHeight() / 2) - (iconHeight / 2);
			}
			else
			{
				if (leftToRight)
				{
					iconLeft = (left + (width - 1)) - iconWidth;
				}
				else
				{
					iconLeft = left;
				}
				iconTop = (top + ((bottom - top) / 2)) - (iconHeight / 2);
			}

			comboIcon.paintIcon(this, g, iconLeft, iconTop);

			// Paint the focus
			if (comboBox.hasFocus())
			{
				g.setColor(MetalLookAndFeel.getFocusColor());
				g.drawRect(left - 1, top - 1, width + 3, height + 1);
			}
		}

		// Let the renderer paint
		if (!iconOnly && comboBox != null)
		{
			ListCellRenderer renderer = comboBox.getRenderer();
			Component c;
			boolean renderPressed = getModel().isPressed();
			c =
				renderer.getListCellRendererComponent(
					listBox,
					comboBox.getSelectedItem(),
					-1,
					renderPressed,
					false);
			c.setFont(rendererPane.getFont());

			if (model.isArmed() && model.isPressed())
			{
				if (isOpaque())
				{
					c.setBackground(UIManager.getColor("Button.select"));
				}
				c.setForeground(comboBox.getForeground());
			}
			else if (!comboBox.isEnabled())
			{
				if (isOpaque())
				{
					c.setBackground(VPSettings.Colors.CLEAR);
				}
				c.setForeground(
					UIManager.getColor("ComboBox.disabledForeground"));
			}
			else
			{
				c.setForeground(comboBox.getForeground());
				c.setBackground(VPSettings.Colors.CLEAR);
			}

			int cWidth = width - (insets.right + iconWidth);

			// Fix for 4238829: should lay out the JPanel.
			boolean shouldValidate = false;
			if (c instanceof JPanel)
			{
				shouldValidate = true;
			}

			if (leftToRight)
			{
				rendererPane.paintComponent(
					g,
					c,
					this,
					left,
					top,
					cWidth,
					height,
					shouldValidate);
			}
			else
			{
				rendererPane.paintComponent(
					g,
					c,
					this,
					left + iconWidth,
					top,
					cWidth,
					height,
					shouldValidate);
			}
		}
	}
}
