/*
 * Created on Mar 6, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package vp.gui.table.components;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;

import javax.swing.JComponent;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableColumnModel;

import vp.gui.VPSettings;
import vp.gui.components.VPPanel;
import vp.gui.table.*;

/**
 * @author Matt
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class VPTableComponent extends VPPanel
{

	private boolean paintLeft;
	private boolean paintRight;
	private boolean paintBottom;
	private boolean paintTop;
	private TableColumnModel colModel;
	private Cursor defaultCursor;
	private VPTable vpTable;
	private Cursor handCursor;
	private boolean rollover;
	private boolean selected;
	private VPTableButton iconButton;
	private VPCellComponent mainComponent;
	private final Color borderColor = VPSettings.Colors.D_BLUE;

	public VPTableComponent(
		VPTable vpTable,
		VPCellComponent mainComponent,
		VPTableButton iconButton)
	{
		this.vpTable = vpTable;
		this.mainComponent = mainComponent;
		this.iconButton = iconButton;
		colModel = vpTable.getColumnModel();

		handCursor = Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);
		defaultCursor = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);

		removeAll();
		//setUI(new TablePanelUI());
		//setBackground(new Color(255, 255, 255, 0));
		setBorder(new EmptyBorder(0, 0, 0, 0));
		setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.weightx = 999.0;
		if (!(mainComponent instanceof TypeCellComponent))
			gbc.insets = new Insets(5, 0, 0, 0);
		if (mainComponent instanceof TypeCellComponent
			|| mainComponent.getValue() instanceof Number)
			gbc.anchor = GridBagConstraints.CENTER;
		else
			gbc.anchor = GridBagConstraints.WEST;
		add((JComponent)mainComponent, gbc);
		if (iconButton != null)
		{
			gbc.gridx = 1;
			gbc.weightx = 1.0;
			gbc.insets = new Insets(0, 3, 0, 3);
			add(iconButton, gbc);
		}
	}

	public VPTableButton getIconButton()
	{

		return iconButton;
	}
	/*
	 * Workaround- Cant call getBounds() because component is not laid out on screen.
	 */
	public boolean isIconButton(int col, int x, int y)
	{
		if (iconButton == null)
			return false;

		int colWidth = colModel.getColumn(col).getWidth();
		int iconX = (colWidth - 5) - iconButton.getPreferredSize().width;
		int iconY =
			((getPreferredSize().height - iconButton.getPreferredSize().height)
				/ 2)
				+ 2;
		Rectangle r =
			new Rectangle(
				iconX,
				iconY,
				iconButton.getPreferredSize().width,
				iconButton.getPreferredSize().height);
		if (r.contains(x, y))
			return true;
		return false;
	}

	public void setIconSelected(int row, int col, boolean selected)
	{
		this.selected = selected;
		if (selected)
		{
			iconButton.setSelected();
			vpTable.setCursor(defaultCursor);
			((VPCellRenderer)vpTable.getCellRenderer(row, col)).render(this);
		}
		else
		{
			iconButton.setOff();
			vpTable.setCursor(defaultCursor);
			((VPCellRenderer)vpTable.getCellRenderer(row, col)).render(null);
		}
	}

	public boolean isIconSelected()
	{
		return selected;
	}

	public void reset()
	{
		if (selected || rollover)
		{
			iconButton.setOff();
			selected = false;
		}
	}

	public void setIconRollover(int row, int col, boolean rollover)
	{
		if (iconButton != null)
		{
			this.rollover = rollover;

			if (rollover)
			{
				iconButton.setRollover();
				vpTable.setCursor(handCursor);
				((VPCellRenderer)vpTable.getCellRenderer(row, col)).render(
					this);
			}
			else
			{
				iconButton.setOff();
				vpTable.setCursor(defaultCursor);
				((VPCellRenderer)vpTable.getCellRenderer(row, col)).render(
					null);
			}
		}
	}

	public boolean isIconRollover()
	{
		return rollover;
	}

	/**
	 * @return
	 */
	public VPCellComponent getMainComponent()
	{
		return mainComponent;
	}

	/**
	 * @param component
	 */
	public void setMainComponent(VPCellComponent component)
	{
		mainComponent = component;
	}

	public void setSelectedBorder(
		boolean top,
		boolean left,
		boolean bottom,
		boolean right)
	{
		paintTop = top;
		paintBottom = bottom;
		paintRight = right;
		paintLeft = left;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj)
	{
		if (!(obj instanceof VPTableComponent))
			return false;
		if (((VPTableComponent)obj).getMainComponent().getValue().hashCode()
			== mainComponent.getValue().hashCode())
		{
			return true;
		}

		return false;
	}

	/**
	 * @return
	 */
	public int getColumn()
	{
		return mainComponent.getColumn();
	}

	/**
	 * @return
	 */
	public int getRow()
	{
		return mainComponent.getRow();
	}

	/**
	 * @return
	 */
	public Object getValue()
	{
		return mainComponent.getValue();
	}

	/**
	 * @return
	 */
	public boolean hasIcon()
	{
		return iconButton != null;
	}

	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	protected void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		if (paintTop)
		{
			g.setColor(borderColor);
			g.drawLine(0, 0, getBounds().width, 0);
			g.drawLine(0, 1, getBounds().width, 1);
		}
		if (paintBottom)
		{
			g.setColor(borderColor);
			g.drawLine(
				0,
				getBounds().height - 1,
				getBounds().width,
				getBounds().height - 1);
			g.drawLine(
				0,
				getBounds().height - 2,
				getBounds().width,
				getBounds().height - 2);
		}
		if (paintRight)
		{
			g.setColor(borderColor);
			g.drawLine(
				getBounds().width - 2,
				0,
				getBounds().width - 2,
				getBounds().height);
			g.drawLine(
				getBounds().width - 1,
				0,
				getBounds().width - 1,
				getBounds().height);

		}
		if (paintLeft)
		{
			g.setColor(borderColor);
			g.drawLine(0, 0, 0, getBounds().height);
			g.drawLine(1, 0, 1, getBounds().height);
		}
	}

}
