package vp.main;

import vp.graph.PathwayGraph;
import vp.gui.components.VPPanel;
import java.awt.BorderLayout;

/**
* The highest level component that is placed into the application window.  It contains a scroll pane that holds the 
* graph panel and a UI toolbar.  It is then loaded by the application into the </code>MainFrame</code>.  
*
* @see edu.yale.vitapad.MainFrame
* @see edu.yale.graph.visual.GraphScrollPane
* @see edu.yale.graph.visual.GraphToolBar
*/
public class GraphEditor extends VPPanel
{
	private GraphScrollPane gsp;
	private GraphToolBar gtb;
	private MainFrame frame;

	/**
	* Constructs a new graph editor with the specified application window and a pre-defined scrollpane housing the 
	* graph panel.
	*/
	public GraphEditor(MainFrame frame, GraphScrollPane gsp)
	{
		this.gsp = gsp;
		this.frame = frame;
		gtb = new GraphToolBar(this);
		setLayout(new BorderLayout());
		//add(gtb, BorderLayout.NORTH);
		add(gsp, BorderLayout.CENTER);
	}

	/**
	* Returns the graph panel which holds the visual graph.
	*
	* @see #setGraphPanel(GraphPanel gp)
	*/
	public GraphPanel getGraphPanel()
	{
		return gsp.getGraphPanel();
	}

	/**
	* Returns the UI tool bar for the graph editor.
	*
	* @see #setGraphToolBar(GraphToolBar gtb)
	*/
	public GraphToolBar getGraphToolBar()
	{
		return gtb;
	}

	/**
	* Returns the application window that holds the graph editor.
	*/
	public MainFrame getMainFrame()
	{
		return frame;
	}

	/**
	* Returns the visual graph held by the graph panel.
	*
	* @see #setVisualGraph(VisualGraph vg)
	*/
	public PathwayGraph getPathwayGraph()
	{
		return gsp.getPathwayGraph();
	}

	/**
	* Sets the graph panel which holds the visual graph to the specified one.
	*
	* @see #getGraphPanel()
	*/
	public void setGraphPanel(GraphPanel gp)
	{
		gsp.setGraphPanel(gp);
	}

	/**
	* Sets the UI tool bar for the graph editor to the one specified.
	*
	* @see #getGraphToolBar()
	*/
	public void setGraphToolBar(GraphToolBar gtb)
	{
		remove(this.gtb);
		this.gtb = gtb;
		add(gtb, BorderLayout.NORTH);
	}

	/**
	* Sets the visual graph held by the graph panel to the one specified.
	*
	* @see #getVisualGraph()
	*/
	public void setPathwayGraph(PathwayGraph vg)
	{
		gsp.setPathwayGraph(vg);
	}
}
