package vp.main;

import vp.graph.PathwayEdge;
import vp.graph.PathwayElement;
import vp.graph.PathwayVertex;
import vp.gui.dialogs.AddEdgeDialog;
import vp.util.geom.Intersection;
import vp.visual.Renderer;
import vp.visual.graphics.AnimatedArrow;

import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.geom.Line2D;

/**
* The UI mode for adding edges to the visual graph.  In this mode, the user can add edges to connect any previously
* existing edge or vertex with another.
*/
public class GraphPanelEdgeMode extends GraphPanelMode
{
	private PathwayElement vgeA;
	private PathwayElement vgeB;
	private Cursor edgeCursor;
	private Cursor defaultCursor;
	private int clicked;

	/**
	* Constructs a UI mode object for the specified panel.
	*/
	public GraphPanelEdgeMode(GraphPanel gPanel)
	{
		super(gPanel);
		this.gPanel = gPanel;
		pGraph = gPanel.getPathwayGraph();
		mainFrame = pGraph.getParent();
		edgeCursor = new Cursor(Cursor.CROSSHAIR_CURSOR);
		defaultCursor = new Cursor(Cursor.DEFAULT_CURSOR);
		clicked = 0;
	}

	/**
	* Registers which two elements the edge is to connect.  After clicking the first element, if the user clicks a 
	* second the menu is brought up.  Otherwise, the count is reset and the user must click a first element again.
	*/
	public GraphPanelMode mousePressed(MouseEvent e)
	{
		switch (clicked)
		{
			case 0 :
				vgeA = pGraph.getSelectedVertex(e.getX(), e.getY());
				if (vgeA == null)
				{
					vgeA = pGraph.getSelectedEdge(e.getX(), e.getY());
				}

				if (vgeA != null)
					clicked++;
				break;
			case 1 :
				vgeB = pGraph.getSelectedVertex(e.getX(), e.getY());
				if (vgeB == null)
				{
					vgeB = pGraph.getSelectedEdge(e.getX(), e.getY());
				}
				if (vgeB != null)
				{
					mainFrame.addToEditPanel(
						new AddEdgeDialog(pGraph.getParent(), vgeA, vgeB));
					pGraph.removeAllAnimatedElements();
					Rectangle vABounds = ((Renderer)vgeA).getBounds();
					Rectangle vBBounds = ((Renderer)vgeB).getBounds();
					Line2D.Double hypLine =
						new Line2D.Double(
							vABounds.getCenterX(),
							vABounds.getCenterY(),
							vBBounds.getCenterX(),
							vBBounds.getCenterY());
					Point hi =
						Intersection.getIntersection(
							hypLine,
							((Renderer)vgeA).getGeneralPath());
					Point hi2 =
						Intersection.getIntersection(
							hypLine,
							((Renderer)vgeB).getGeneralPath());
					Point[] locations = { hi, hi2 };
					AnimatedArrow aa =
						AnimatedArrow.getInstance(pGraph, locations);
					pGraph.addAnimatedElement(aa);
					clicked = 0;
				}
				else
					clicked = 0;
				break;
		}

		return this;
	}

	/**
	* Changes the cursor depending upon whether the mouse is over an element which can serve as the end point of a link.
	*/
	public GraphPanelMode mouseMoved(MouseEvent e)
	{
		PathwayVertex pv = pGraph.getSelectedVertex(e.getX(), e.getY());
		PathwayEdge pe = pGraph.getSelectedEdge(e.getX(), e.getY());
		if (pv != null || pe != null)
			gPanel.setCursor(edgeCursor);
		else
			gPanel.setCursor(defaultCursor);

		return this;
	}

	/**
	* Calls the visual graph's paint command.
	*/
	public void paint(Graphics2D g2)
	{
		pGraph.paint(g2);
	}
}
