package vp.main;

import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

import vp.graph.PathwayVertex;

/**
* The UI mode for moving elements on the visual graph.   In this mode, the user can drag vertices to new locations
*/
public class GraphPanelMoveMode extends GraphPanelMode
{
	private PathwayVertex targetVertex;
	private Cursor prevCursor;
	private Cursor moveCursor;
	private int prevX;
	private int prevY;
	private boolean store = false;
	private Thread storageTest;

	/**
	* Constructs a UI mode object for the specified panel.
	*/
	public GraphPanelMoveMode(GraphPanel gPanel)
	{
		super(gPanel);
		moveCursor = new Cursor(Cursor.MOVE_CURSOR);
		mainFrame = gPanel.getMainFrame();
	}

	/**
	* Constructs a UI mode object with no specific graph panel.
	*/
	public GraphPanelMoveMode()
	{
		this(null);
	}

	/**
	* Resets the location of the dragged vertex.
	*/
	public GraphPanelMode mouseDragged(MouseEvent e)
	{
		store = false;
		if (storageTest != null)
			storageTest.interrupt();
		if (targetVertex != null)
			dragVertex(e.getX(), e.getY());

		return this;
	}

	public GraphPanelMode mousePressed(MouseEvent e)
	{
		if ((storageTest == null) || (!storageTest.isAlive()))
		{
			mainFrame.getStack().store(pGraph);
		}
		return this;
	}

	public GraphPanelMode mouseReleased(MouseEvent e)
	{
		store = true;
		Runnable testStore = new Runnable()
		{
			public void run()
			{
				try
				{
					Thread.sleep(500);
				}
				catch (InterruptedException e)
				{
					Thread.currentThread().interrupt();
				}

			}
		};
		storageTest = new Thread(testStore);
		storageTest.start();
		return this;
	}

	/**
	* Changes the graph panel's cursor depending on whether the mouse enters or leaves a vertex or edge.
	*/
	public GraphPanelMode mouseMoved(MouseEvent e)
	{
		pGraph = gPanel.getPathwayGraph();
		targetVertex = pGraph.getSelectedVertex(e.getX(), e.getY());

		if (targetVertex != null)
		{
			if (prevCursor == null)
				prevCursor = gPanel.getCursor();
			gPanel.setCursor(moveCursor);
		}
		else
		{
			gPanel.setCursor(prevCursor);
			prevCursor = null;
		}

		return this;
	}

	/**
	* Calls the visual graph's paint command.
	*/
	public void paint(Graphics2D g2)
	{
		gPanel.getPathwayGraph().paint(g2);
	}

	private void dragVertex(int x, int y)
	{
		Rectangle vertexRect = targetVertex.getBounds();

		targetVertex.setXPosition(x);
		targetVertex.setYPosition(y);

		int deltaX = vertexRect.x + x - prevX < 0 ? -vertexRect.x : x - prevX;
		int deltaY = vertexRect.y + y - prevY < 0 ? -vertexRect.y : y - prevY;

		targetVertex.setLocation(x, y);
		prevX = prevX + deltaX;
		prevY = prevY + deltaY;

		pGraph.repaint();
	}
}
