package vp.main;

import vp.gui.dialogs.AddVertexDialog;
import vp.visual.graphics.*;

import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;

/**
* The UI mode for adding vertices to the visual graph.  In this mode, the user can add a vertex at the point clicked on 
* the graph.
*/
public class GraphPanelVertexMode extends GraphPanelMode
{
	private Cursor crossCursor;

	/**
	* Constructs a UI mode object for the specified panel.
	*/
	public GraphPanelVertexMode(GraphPanel gPanel)
	{
		super(gPanel);
		this.gPanel = gPanel;
		pGraph = gPanel.getPathwayGraph();
		mainFrame = pGraph.getParent();
		crossCursor = new Cursor(Cursor.CROSSHAIR_CURSOR);
		gPanel.setCursor(crossCursor);
	}

	/**
	* Brings up the menu to select a vertex to add at the requested point.
	*/
	public GraphPanelMode mousePressed(MouseEvent e)
	{
		mainFrame.addToEditPanel(
			new AddVertexDialog(pGraph.getParent(), e.getX(), e.getY()));
		pGraph.removeAllAnimatedElements();
		Point[] locations = { new Point(e.getX(), e.getY())};
		AnimatedCrosshair ac = AnimatedCrosshair.getInstance(pGraph, locations);
		pGraph.addAnimatedElement(ac);
		return this;
	}

	/**
	* Calls the visual graph's paint command.
	*/
	public void paint(Graphics2D g2)
	{
		pGraph.paint(g2);
	}
}
