package vp.main;

import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import vp.graph.DataClass;
import vp.graph.PathwayDecoration;
import vp.graph.PathwayVertex;
import vp.gui.dialogs.CompoundDisplayPanel;
import vp.gui.dialogs.EnzymeDisplayPanel;
import vp.gui.dialogs.GeneDisplayPanel;
import vp.gui.dialogs.PathwayDisplayPanel;
import vp.util.VPSwingWorker;

/**
* Shows the visual graph in its final, uneditable mode.  In this mode, tooltips and links will be active.
*/
public class GraphPanelViewMode extends GraphPanelMode
{
	private Cursor defaultCursor;
	private PathwayDecoration active;
	private Cursor handCursor;
	private DataClass enzyme;
    private DataClass compound;
	/**
	* Constructs a UI mode object for the specified panel.
	*/
	public GraphPanelViewMode(GraphPanel gPanel)
	{
		super(gPanel);
		this.gPanel = gPanel;
		mainFrame = gPanel.getMainFrame();
		active = null;
		pGraph = gPanel.getPathwayGraph();
		handCursor = new Cursor(Cursor.HAND_CURSOR);
		defaultCursor = new Cursor(Cursor.DEFAULT_CURSOR);
	}

	/**
	* Constructs a UI mode object with no specific graph panel.
	*/
	public GraphPanelViewMode()
	{
		this(null);
	}

	/**
	* Notifies if the mouse is over an edge or vertex.
	*/
	public GraphPanelMode mouseMoved(MouseEvent e)
	{
		PathwayVertex pv = pGraph.getSelectedVertex(e.getX(), e.getY());
		PathwayDecoration pd = pGraph.getSelectedDecoration(e.getX(), e.getY());
		if (pv != null)
		{
			gPanel.setCursor(handCursor);
		}
		else if (
			(pd != null)
				&& (pd.getDataClass().getClassName().equals("Enzyme")))
		{
			gPanel.setCursor(handCursor);
			pd.setTipEnabled(true);
			active = pd;
			pGraph.repaint();
		}
		else
		{
			if (active != null)
			{
				active.setTipEnabled(false);
				pGraph.repaint();
				active = null;
			}
			gPanel.setCursor(defaultCursor);
		}
		return this;
	}

	/**
	* Calls the visual graph's paint command.
	*/
	public void paint(Graphics2D g2)
	{
		gPanel.getPathwayGraph().paint(g2);
	}
	/* (non-Javadoc)
	 * @see vp.src.edu.yale.graph.visual.GraphPanelMode#mouseClicked(java.awt.event.MouseEvent)
	 */
	public GraphPanelMode mouseClicked(MouseEvent e)
	{
		PathwayVertex pv = pGraph.getSelectedVertex(e.getX(), e.getY());
		PathwayDecoration pd = pGraph.getSelectedDecoration(e.getX(), e.getY());

		if (pv != null)
		{
            this.compound = pv.getDataClass();
            mainFrame.clearDisplayPanel();
            
			mainFrame.addToDisplayPanel(
				new CompoundDisplayPanel(mainFrame, pv.getDataClass()));
		}

		else if (pd != null)
		{
			if (pd.getDataClass().getClassName().equals("Enzyme"))
			{
				this.enzyme = pd.getDataClass();
				mainFrame.clearDisplayPanel();
                mainFrame.addToDisplayPanel(new EnzymeDisplayPanel(mainFrame, enzyme));
//				if ((enzyme.get("Substrates") != null)
//					&& enzyme.get("Products") != null)
//					mainFrame.addToDisplayPanel(
//						new EnzymeDisplayPanel(mainFrame, enzyme));
//				else
//				{
//					VPSwingWorker work =
//						new VPSwingWorker(
//							mainFrame,
//							gPanel,
//							"LoadEnzymeDisplayPanel")
//					{
//						public void doNonUI()
//						{
//							loadEnzymeData();
//						}
//
//						protected void doUI() throws RuntimeException
//						{
//							mainFrame.addToDisplayPanel(
//								new EnzymeDisplayPanel(mainFrame, enzyme));
//						}
//					};
//
//					work.start();
//				}
			}

			else if (pd.getDataClass().getClassName().equals("Gene"))
			{
				mainFrame.clearDisplayPanel();
				mainFrame.addToDisplayPanel(
					new GeneDisplayPanel(
						mainFrame,
						pd.getDataClass()));
			}
		}

		else
		{
			mainFrame.addToDisplayPanel(new PathwayDisplayPanel(mainFrame));
		}

		return this;
	}

	private void loadEnzymeData()
	{
		BufferedReader br = null;
		PrintWriter pw = null;
		//String request = "/dbget-bin/www_bget?enzyme+" + enzyme.getID();
		//URLConnection kc = KEGGConnection.getInstance(request);

		String s;
		int c;

		URL url = null;
		try
		{
			url =
				new URL(
					"http://www.genome.ad.jp/dbget-bin/www_bget?enzyme+"
						+ enzyme.get("ECNumber"));
		}
		catch (MalformedURLException e)
		{
			e.printStackTrace();
		}

		try
		{
			br = new BufferedReader(new InputStreamReader(url.openStream()));
		}
		catch (IOException e1)
		{
			e1.printStackTrace();
		}
		try
		{
			while ((s = br.readLine()) != null)
			{
				if (s.indexOf("REACTION</A>") != -1)
				{
					String reaction =
						s.substring(s.indexOf("REACTION</A>") + 12);
					reaction = reaction.trim();
					StringTokenizer stkr = new StringTokenizer(reaction, "=");
					String allSubstrates = stkr.nextToken();
					String allProducts = stkr.nextToken();

					allSubstrates = allSubstrates.replace('+', '&');
					allSubstrates = allSubstrates.replaceAll(" & ", "@");
					allSubstrates = allSubstrates.replace('&', '+');
					StringTokenizer subtkr =
						new StringTokenizer(allSubstrates, "@");
					List substrates = new ArrayList();
					int i = 0;
					while (subtkr.hasMoreTokens())
					{
						substrates.add(subtkr.nextToken());
						i++;
					}
					enzyme.put("Substrates", substrates);

					allProducts = allProducts.replace('+', '&');
					allProducts = allProducts.replaceAll(" & ", "@");
					allProducts = allProducts.replace('&', '+');
					StringTokenizer protkr =
						new StringTokenizer(allProducts, "@");
					List products = new ArrayList();
					i = 0;
					while (protkr.hasMoreTokens())
					{
						products.add(protkr.nextToken());
						i++;
					}
					enzyme.put("Products", products);
					break;
				}
			}
		}

		catch (IOException e2)
		{
			e2.printStackTrace();
		}
	}
}
