package vp.main;

import javax.swing.JScrollPane;
import javax.swing.JViewport;

import vp.graph.PathwayGraph;

/**
* A <code>JScrollPane</code> to hold the graph panel.
*/
public class GraphScrollPane extends JScrollPane
{
	private GraphPanel gPanel;
	
	/**
	* Creates a new scroll pane with the specified values.  By default, the pane is set to always show horizontal and
	* vertical scroll bars.
	*/
	public GraphScrollPane(GraphPanel gPanel)
	{
		super(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		this.gPanel = gPanel;
		gPanel.setGPContainer(this);
		JViewport viewport;
		viewport = new JViewport();
		viewport.setView(gPanel);
		setViewport(viewport);
	}
	
	/**
	* Returns the graph panel held within the scroll pane.
	*
	* @see #setGraphPanel(GraphPanel gPanel)
	*/
	public GraphPanel getGraphPanel()
	{
		return gPanel;
	}

	/**
	* Returns the visual graph held within the graph panel which is in turn held within the scroll pane.
	*
	* @see #setVisualGraph(VisualGraph vg)
	*/
	public PathwayGraph getPathwayGraph()
	{
		return gPanel.getPathwayGraph();
	}
	
	/**
	* Sets the graph panel held within the scroll pane to the one specified.
	*
	* @see #getGraphPanel()
	*/
	public void setGraphPanel(GraphPanel gPanel)
	{
		this.gPanel = gPanel;
	}

	/**
	* Sets the visual graph held within the graph panel which is in turn held within the scroll pane to the one 
	* specified.
	*
	* @see #getVisualGraph()
	*/
	public void setPathwayGraph(PathwayGraph pg)
	{
		gPanel.setPathwayGraph(pg);
	}
}
