package vp.main;

import vp.gui.VPSettings;
import vp.gui.components.VPPanel;
import vp.util.ActionCenter;
import vp.util.IconFactory;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JProgressBar;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.JToolBar;

public class GraphToolBar extends JToolBar
{
	private JRadioButton experimentButton;
	private JRadioButton enzymeButton;
	private JRadioButton geneButton;
	private JProgressBar progBar;
	private VPPanel alertPanel;
	private JTextField alertField;
	/**
	 * @author Matt Holford
	 *
	 * To change the template for this generated type comment go to
	 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
	 */
	private GraphEditor gEdit;
	private ButtonGroup modeGroup;
	private ButtonGroup userGroup;
	private ButtonGroup levelGroup;
	private JRadioButton viewButton;
	private JRadioButton selectButton;
	private JRadioButton vertexButton;
	private JRadioButton edgeButton;
	private JRadioButton moveButton;
	private JRadioButton editButton;
	private JRadioButton deleteButton;
	private JRadioButton undoButton;
	private JRadioButton redoButton;
	private JRadioButton zoomInButton;
	private JRadioButton zoomOutButton;
	private ActionListener listener;

	private final Insets regular = new Insets(0, 0, 0, 0);
	private final Insets indented = new Insets(0, 5, 0, 5);

	public GraphToolBar(GraphEditor gEdit)
	{
		super();
		setRollover(true);
		this.gEdit = gEdit;
		modeGroup = new ButtonGroup();
		userGroup = new ButtonGroup();
		levelGroup = new ButtonGroup();
		listener = gEdit.getMainFrame().getListener();
		setFloatable(false);
		setLayout(new GridBagLayout());

		GridBagConstraints toolc = new GridBagConstraints();
		toolc.insets = regular;

		viewButton =
			getButton(
				IconFactory.getLockIcon(),
				IconFactory.getSelectedLockIcon(),
				IconFactory.getRolloverLockIcon(),
				"Lock Graph",
				ActionCenter.VIEW_MODE);
		viewButton.setSelected(true);
		modeGroup.add(viewButton);
		add(viewButton, toolc);

		toolc.gridx = 1;
		selectButton =
			getButton(
				IconFactory.getArrowIcon(),
				IconFactory.getSelectedArrowIcon(),
				IconFactory.getRolloverArrowIcon(),
				"Select Graph Elements",
				ActionCenter.EDIT_MODE);
		modeGroup.add(selectButton);
		add(selectButton, toolc);

		toolc.gridx = 2;
		vertexButton =
			getButton(
				IconFactory.getPencilIcon(),
				IconFactory.getSelectedPencilIcon(),
				IconFactory.getRolloverPencilIcon(),
				"Add Vertex",
				ActionCenter.VERTEX_MODE);
		modeGroup.add(vertexButton);
		add(vertexButton, toolc);

		toolc.gridx = 3;
		edgeButton =
			getButton(
				IconFactory.getEdgeIcon(),
				IconFactory.getSelectedEdgeIcon(),
				IconFactory.getRolloverEdgeIcon(),
				"Add Edge",
				ActionCenter.EDGE_MODE);
		modeGroup.add(edgeButton);
		add(edgeButton, toolc);

		toolc.gridx = 4;
		moveButton =
			getButton(
				IconFactory.getMoveIcon(),
				IconFactory.getSelectedMoveIcon(),
				IconFactory.getRolloverMoveIcon(),
				"Move Elements",
				ActionCenter.MOVE_MODE);
		modeGroup.add(moveButton);
		add(moveButton, toolc);

		toolc.gridx = 5;
		toolc.fill = GridBagConstraints.BOTH;
		toolc.insets = indented;
		add(new JSeparator(JSeparator.VERTICAL), toolc);
		//addSeparator(new Dimension(10, getPreferredSize().height));

		toolc.gridx = 6;
		toolc.insets = regular;
		editButton =
			getButton(
				IconFactory.getEditIcon(),
				IconFactory.getSelectedEditIcon(),
				IconFactory.getRolloverEditIcon(),
				"Edit Selected Elements",
				ActionCenter.EDIT_SELECTED);
		editButton.setSelected(false);
		add(editButton, toolc);

		toolc.gridx = 7;
		deleteButton =
			getButton(
				IconFactory.getDeleteIcon(),
				IconFactory.getSelectedDeleteIcon(),
				IconFactory.getRolloverDeleteIcon(),
				"Delete Selected Elements",
				ActionCenter.DELETE_SELECTED);
		add(deleteButton, toolc);

		toolc.gridx = 8;
		toolc.insets = indented;
		add(new JSeparator(JSeparator.VERTICAL), toolc);
		//addSeparator(new Dimension(10, getPreferredSize().height));

		toolc.gridx = 9;
		toolc.insets = regular;
		undoButton =
			getButton(
				IconFactory.getUndoIcon(),
				IconFactory.getSelectedUndoIcon(),
				IconFactory.getRolloverUndoIcon(),
				"Undo",
				ActionCenter.UNDO);
		undoButton.setDisabledIcon(
			new ImageIcon(IconFactory.getDisabledUndoIcon()));
		undoButton.setEnabled(false);
		//userGroup.add(undoButton);
		add(undoButton, toolc);

		toolc.gridx = 10;
		redoButton =
			getButton(
				IconFactory.getRedoIcon(),
				IconFactory.getSelectedRedoIcon(),
				IconFactory.getRolloverRedoIcon(),
				"Redo",
				ActionCenter.REDO);
		redoButton.setDisabledIcon(
			new ImageIcon(IconFactory.getDisabledRedoIcon()));
		redoButton.setEnabled(false);
		//userGroup.add(redoButton);
		add(redoButton, toolc);

		toolc.gridx = 11;
		zoomInButton =
			getButton(
				IconFactory.getZoomInIcon(),
				IconFactory.getSelectedZoomInIcon(),
				IconFactory.getRolloverZoomInIcon(),
				"Zoom In",
				ActionCenter.ZOOM_IN);

		add(zoomInButton, toolc);

		toolc.gridx = 12;
		zoomOutButton =
			getButton(
				IconFactory.getZoomOutIcon(),
				IconFactory.getSelectedZoomOutIcon(),
				IconFactory.getRolloverZoomOutIcon(),
				"Zoom Out",
				ActionCenter.ZOOM_OUT);

		add(zoomOutButton, toolc);

		toolc.gridx = 13;
		toolc.insets = indented;
		add(new JSeparator(JSeparator.VERTICAL), toolc);

		toolc.gridx = 14;
		toolc.insets = regular;
		experimentButton =
			getButton(
				IconFactory.getExperimentIcon(),
				IconFactory.getSelectedExperimentIcon(),
				IconFactory.getRolloverExperimentIcon(),
				"Show Genes",
				ActionCenter.LOAD_EXPERIMENT);
		add(experimentButton, toolc);

		toolc.gridx = 15;
		toolc.insets = regular;
		geneButton =
			getButton(
				IconFactory.getGeneIcon(),
				IconFactory.getSelectedGeneIcon(),
				IconFactory.getRolloverGeneIcon(),
				"Show Genes",
				ActionCenter.SHOW_GENES);
		geneButton.setEnabled(false);
		levelGroup.add(geneButton);
		add(geneButton, toolc);

		toolc.gridx = 16;
		toolc.fill = GridBagConstraints.NONE;
		toolc.anchor = GridBagConstraints.WEST;
		enzymeButton =
			getButton(
				IconFactory.getEnzymeIcon(),
				IconFactory.getSelectedEnzymeIcon(),
				IconFactory.getRolloverEnzymeIcon(),
				"Show Genes",
				ActionCenter.SHOW_ENZYMES);
		enzymeButton.setEnabled(false);
		levelGroup.add(enzymeButton);
		add(enzymeButton, toolc);

		toolc.gridx = 17;
		toolc.anchor = GridBagConstraints.EAST;
		toolc.insets = new Insets(0, 3, 0, 15);
		toolc.weightx = 2;

		alertPanel = new VPPanel();
		alertPanel.setLayout(new GridBagLayout());
		alertPanel.setOpaque(false);
		GridBagConstraints alertc = new GridBagConstraints();
		alertc.insets = new Insets(0, 0, 0, 10);

		alertField = new JTextField();
		alertField.setFont(VPSettings.SR_FONT);
		alertField.setBackground(VPSettings.Colors.BEIGE);
		alertField.setPreferredSize(
			new Dimension(200, alertField.getPreferredSize().height));
		alertField.setText("Accessing database.");
		alertPanel.add(alertField, alertc);

		alertc.gridx = 1;
		alertc.insets = new Insets(0, 10, 0, 0);
		progBar = new JProgressBar(0, 100);
		progBar.setBackground(VPSettings.Colors.SKY_BLUE);
		progBar.setForeground(VPSettings.Colors.D_BLUE);
		progBar.setFont(VPSettings.R_FONT);
		progBar.setValue(57);
		//progBar.setStringPainted(true);
		progBar.setPreferredSize(
			new Dimension(150, alertField.getPreferredSize().height - 3));
		alertPanel.add(progBar, alertc);

		add(alertPanel, toolc);

	}

	public JRadioButton getViewButton()
	{
		return viewButton;
	}

	public JRadioButton getSelectButton()
	{
		return selectButton;
	}

	public JRadioButton getVertexButton()
	{
		return vertexButton;
	}

	public JRadioButton getEdgeButton()
	{
		return edgeButton;
	}

	public JRadioButton getMoveButton()
	{
		return moveButton;
	}

	public JRadioButton getEditButton()
	{
		return editButton;
	}

	public JRadioButton getDeleteButton()
	{
		return deleteButton;
	}

	public JRadioButton getUndoButton()
	{
		return undoButton;
	}

	public JRadioButton getRedoButton()
	{
		return redoButton;
	}

	public JRadioButton getZoomInButton()
	{
		return zoomInButton;
	}

	public JRadioButton getZoomOutButton()
	{
		return zoomOutButton;
	}

	private JRadioButton getButton(
		BufferedImage off,
		BufferedImage on,
		BufferedImage roll,
		String toolTip,
		String actionCommand)
	{
		ImageIcon onIcon = new ImageIcon(on);
		ImageIcon offIcon = new ImageIcon(off);
		ImageIcon rollIcon = new ImageIcon(roll);
		JRadioButton newButton = new JRadioButton(offIcon);

		newButton.setRolloverEnabled(true);
		newButton.setToolTipText(toolTip);
		newButton.setSelectedIcon(onIcon);
		newButton.setRolloverIcon(rollIcon);
		newButton.setVerticalTextPosition(AbstractButton.BOTTOM);
		newButton.setHorizontalTextPosition(AbstractButton.CENTER);
		newButton.setActionCommand(actionCommand);
		newButton.addActionListener(listener);
		return newButton;
	}
	/**
	 * @return
	 */
	public JTextField getAlertField()
	{
		return alertField;
	}

	/**
	 * @return
	 */
	public VPPanel getAlertPanel()
	{
		return alertPanel;
	}

	/**
	 * @return
	 */
	public JProgressBar getProgBar()
	{
		return progBar;
	}

	/**
	 * @param field
	 */
	public void setAlertField(JTextField field)
	{
		alertField = field;
	}

	/**
	 * @param panel
	 */
	public void setAlertPanel(VPPanel panel)
	{
		alertPanel = panel;
	}

	/**
	 * @param bar
	 */
	public void setProgBar(JProgressBar bar)
	{
		progBar = bar;
	}

	/**
	 * @return
	 */
	public JRadioButton getExperimentButton()
	{
		return experimentButton;
	}

	/**
	 * @param button
	 */
	public void setZoomOutButton(JRadioButton button)
	{
		zoomOutButton = button;
	}

	/**
	 * @return
	 */
	public JRadioButton getEnzymeButton()
	{
		return enzymeButton;
	}

	/**
	 * @return
	 */
	public JRadioButton getGeneButton()
	{
		return geneButton;
	}

}
