package vp.main;

import vp.graph.GraphDataClass;
import vp.graph.PathwayEdge;
import vp.graph.PathwayGraph;
import vp.graph.PathwayVertex;
import vp.gui.VPSettings;
import vp.gui.components.DialogFactory;
import vp.gui.components.VPPanel;
import vp.gui.dialogs.AddEdgeDialog;
import vp.gui.dialogs.AddVertexDialog;
import vp.gui.dialogs.LoadFlatFileDialog;
import vp.gui.dialogs.NewPathwayDialog2;
import vp.gui.dialogs.PathwayDetailsDialog;
import vp.gui.dialogs.PathwayDisplayPanel;
import vp.gui.dialogs.SelectDBPathwayDialog;
import vp.gui.dialogs.SelectExperimentDialog;
import vp.gui.kunststoff.KunststoffLookAndFeel;
import vp.gui.table.components.TableViewer;
import vp.util.ActionCenter;
import vp.util.EditEdgeList;
import vp.util.EditVertexList;
import vp.util.FileExtensionFilter;
import vp.util.GraphUtilities;
import vp.util.GraphicsFactory;
import vp.util.IconFactory;
import vp.util.PrintFactory;
import vp.util.UndoRedoStack;
import vp.util.VPProperties;
import vp.util.VPSwingWorker;
import vp.util.database.DBAccess;
import vp.util.database.DBConstants;
import vp.util.database.DBQuery;
import vp.util.graphviz.GraphvizToGraph;
import vp.util.xml.GraphToXML;
import vp.util.xml.XMLToGraph;
import vp.visual.type.Type;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JProgressBar;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

public class MainFrame extends JFrame
{
    private GraphDataClass currentGraphDataClass;

    private JSplitPane split;

    private MainFrame thisFrame;

    private static MainFrame currentFrame;

    private VPPanel displayPanel;

    private VPPanel editPanel;

    private PathwayGraph pGraph;

    private GraphEditor gEdit;

    private GraphPanel gPanel;

    private Listener listener;

    private MainMenu menu;

    private GraphToolBar toolBar;

    private UndoRedoStack stack;

    private List vertexClasses, edgeClasses, decorationClasses;

    private Map tableNames, defaultTypes, classNames;

    private JComponent currEditComponent;

    public MainFrame()
    {
        super("VitaPad");

        try
        {
            UIManager.setLookAndFeel(new KunststoffLookAndFeel());
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        thisFrame = this;
        listener = new Listener();
        GraphPanel initPanel = new GraphPanel(this,
            new GraphPanelEditMode());
        GraphScrollPane gsp = new GraphScrollPane(initPanel);
        gEdit = new GraphEditor(this, gsp);
        gPanel = gEdit.getGraphPanel();
        toolBar = gEdit.getGraphToolBar();

        stack = new UndoRedoStack(this);

        classNames = new HashMap();
        tableNames = new HashMap();
        defaultTypes = new HashMap();

        menu = new MainMenu(this);
        setJMenuBar(menu);
        ImageIcon icon = new ImageIcon(IconFactory.getVPIcon());
        setIconImage(icon.getImage());

        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(listener);

        Dimension smallSize = new Dimension(1000, 700);
        setSize(smallSize);
        setLocation(200, 200);

        // connect to database
        Runnable dbconnect = new Runnable()
        {

            public void run()
            {
                DBAccess.initialize();
                vertexClasses = DBQuery.getClasses("_DEFPATH",
                    DBConstants.VERTEX);
                edgeClasses = DBQuery.getClasses("_DEFPATH",
                    DBConstants.EDGE);
                decorationClasses = DBQuery.getClasses("_DEFPATH",
                    DBConstants.DECORATION);

                for (int i = 0; i < vertexClasses.size(); i++)
                {
                    Integer classID = (Integer) vertexClasses.get(i);

                    classNames.put(classID, DBQuery.getClassName(
                        "_DEFPATH", classID.intValue()));
                    tableNames.put(classID, DBQuery.getTableName(
                        "_DEFPATH", classID.intValue()));
                    defaultTypes.put(classID, DBQuery.getDefaultType(
                        "_DEFPATH", classID.intValue()));
                }

                for (int i = 0; i < edgeClasses.size(); i++)
                {
                    Integer classID = (Integer) edgeClasses.get(i);

                    classNames.put(classID, DBQuery.getClassName(
                        "_DEFPATH", classID.intValue()));
                    tableNames.put(classID, DBQuery.getTableName(
                        "_DEFPATH", classID.intValue()));
                    defaultTypes.put(classID, DBQuery.getDefaultType(
                        "_DEFPATH", classID.intValue()));
                }

                for (int i = 0; i < decorationClasses.size(); i++)
                {
                    Integer classID = (Integer) decorationClasses
                        .get(i);

                    classNames.put(classID, DBQuery.getClassName(
                        "_DEFPATH", classID.intValue()));
                    tableNames.put(classID, DBQuery.getTableName(
                        "_DEFPATH", classID.intValue()));
                    defaultTypes.put(classID, DBQuery.getDefaultType(
                        "_DEFPATH", classID.intValue()));
                }

            }
        };
        Thread db = new Thread(dbconnect);
        db.start();
        db.setPriority(Thread.MIN_PRIORITY);

        new OpeningScreen(this);
        // onNew();
    }

    public void showGraph(PathwayGraph pGraph)
    {
        this.pGraph = pGraph;
        getContentPane().removeAll();
        repaint();
        pGraph.setExperimentData(false);

        gEdit.setPathwayGraph(pGraph);

        getContentPane().add(toolBar, BorderLayout.NORTH);
        VPPanel sidebox = new VPPanel();

        sidebox.setLayout(new GridBagLayout());

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.WEST;

        editPanel = new VPPanel();
        editPanel.setPreferredSize(new Dimension(390, 370));
        editPanel.setMaximumSize(new Dimension(390, 370));
        editPanel.setMinimumSize(new Dimension(390, 370));
        editPanel.setBorder(new TitledBorder(new EtchedBorder(
            VPSettings.Colors.M_BLUE, Color.gray), "Edit Panel",
            TitledBorder.CENTER, TitledBorder.DEFAULT_POSITION,
            new Font("Palatino Linotype", Font.BOLD, 14),
            VPSettings.Colors.D_BLUE));
        editPanel.setBackground(VPSettings.Colors.SIDE_PANEL);

        sidebox.add(editPanel, gbc);

        gbc.weighty = 2.0;
        gbc.gridy = 1;
        gbc.fill = GridBagConstraints.BOTH;
        displayPanel = new VPPanel();
        displayPanel.setPreferredSize(new Dimension(390, 228));
        displayPanel.setLayout(new BorderLayout());
        displayPanel.setBorder(new TitledBorder(new EtchedBorder(
            VPSettings.Colors.M_BLUE, Color.gray), "Display Panel",
            TitledBorder.CENTER, TitledBorder.DEFAULT_POSITION,
            new Font("Palatino Linotype", Font.BOLD, 14),
            VPSettings.Colors.D_BLUE));
        displayPanel.setBackground(VPSettings.Colors.SIDE_PANEL);
        displayPanel.add(new PathwayDisplayPanel(this));

        sidebox.add(displayPanel, gbc);

        split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, gEdit,
            sidebox);
        split.setOneTouchExpandable(true);
        split.setContinuousLayout(true);
        split.setResizeWeight(1.0);
        getContentPane().add(split, BorderLayout.CENTER);

        getAlertField().setVisible(false);
        getProgBar().setVisible(false);
        toolBar.getViewButton().setSelected(true);
        validate();
    }

    public Listener getListener()
    {
        return listener;
    }

    public GraphEditor getGraphEditor()
    {
        return gEdit;
    }

    public GraphPanel getGraphPanel()
    {
        return gPanel;
    }

    public PathwayGraph getPathwayGraph()
    {
        return pGraph;
    }

    public GraphDataClass getGraphDataClass()
    {
        return currentGraphDataClass;
    }

    public void setGraphDataClass(GraphDataClass currentGraphDataClass)
    {
        this.currentGraphDataClass = currentGraphDataClass;
    }

    public void addToEditPanel(JComponent jc)
    {
        editPanel.removeAll();
        editPanel.add(jc);
        editPanel.repaint();
        validate();
        currEditComponent = jc;
    }

    public void addToDisplayPanel(JComponent jc)
    {
        displayPanel.removeAll();
        displayPanel.add(jc);
        displayPanel.repaint();
        validate();
    }

    public void clearEditPanel()
    {
        editPanel.removeAll();
        editPanel.repaint();
        validate();
        currEditComponent = null;
    }

    public void clearDisplayPanel()
    {
        displayPanel.removeAll();
        displayPanel.repaint();
        validate();
    }

    protected void onNew()
    {
        new NewPathwayDialog2(this);
    }

    public void showNewGraph()
    {
        PathwayGraph pg;
        pg = (currentGraphDataClass != null) ? new PathwayGraph(this,
            DBQuery.getUniqueID(), currentGraphDataClass)
            : new PathwayGraph(this, DBQuery.getUniqueID(),
                GraphUtilities.TEMP_GRAPH_CLASS);

        try
        {
            showGraph(pg);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    private void onLoadFromDB()
    {
        if (getPathwayGraph() != null)
            getPathwayGraph().removeAllAnimatedElements();
        addToEditPanel(new SelectDBPathwayDialog(this));
    }

    protected void onLoadFile()
    {
        if (getPathwayGraph() != null)
            getPathwayGraph().removeAllAnimatedElements();
        File defaultPath = new File("etc" + File.separator + "files"
            + File.separator);
        JFileChooser chooser = VPSettings.getVPFileChooser(
            defaultPath, this);
        chooser
            .addChoosableFileFilter(new FileExtensionFilter("xml"));
        int result = chooser.showOpenDialog(this);
        File selectedFile = chooser.getSelectedFile();

        if (result == JFileChooser.APPROVE_OPTION)
        {
            // onNew();
            showNewGraph();
            PathwayGraph xmltg = XMLToGraph.loadXMLFile(pGraph,
                selectedFile);

            try
            {
                showGraph(xmltg);
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
    }

    public void loadDBGraph(GraphDataClass graphData)
    {
        showNewGraph();

        final GraphvizToGraph gvtg = new GraphvizToGraph(pGraph,
            graphData);

        VPSwingWorker query = new VPSwingWorker(this, gPanel,
            "queryDatabase")
        {
            protected void doNonUI() throws RuntimeException
            {
                gvtg.query();
            }

            protected void doUI() throws RuntimeException
            {
                gvtg.generateGraph();
                try
                {
                    gvtg.runAlgorithm();
                }
                catch (FileNotFoundException e)
                {
                    e.printStackTrace();
                }
                showGraph(gvtg.getGraph());
            }
        };
        query.start();

    }

    private void onLoadExperiment()
    {
        getPathwayGraph().removeAllAnimatedElements();
        addToEditPanel(new SelectExperimentDialog(this));
    }

    public void onSaveFile()
    {
        File defaultPath = new File("etc" + File.separator + "files"
            + File.separator);
        JFileChooser chooser = new JFileChooser(defaultPath);
        chooser
            .addChoosableFileFilter(new FileExtensionFilter("xml"));
        int result = chooser.showSaveDialog(this);
        File selectedFile = chooser.getSelectedFile();

        if (result == JFileChooser.APPROVE_OPTION)
        {
            GraphToXML.saveXMLFile(pGraph, selectedFile);
        }
    }

    public void onSaveImage()
    {
        File defaultPath = new File("etc" + File.separator + "images"
            + File.separator);
        JFileChooser chooser = new JFileChooser(defaultPath);
        chooser
            .addChoosableFileFilter(new FileExtensionFilter("png"));
        int result = chooser.showSaveDialog(this);

        File selectedFile = chooser.getSelectedFile();
        if (result == JFileChooser.APPROVE_OPTION)
        {
            BufferedImage bi = new BufferedImage(gPanel.getWidth(),
                gPanel.getHeight(), BufferedImage.TYPE_INT_RGB);
            Graphics2D big = bi.createGraphics();
            big.setColor(Color.white);
            big.fillRect(0, 0, gPanel.getWidth(), gPanel.getHeight());
            pGraph.paint(big);
            try
            {
                ImageIO.write(bi, "png", selectedFile);
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
    }

    public void onSaveFileIntoDB()
    {

        String title = "Not enabled";
        String header = "Feature not yet enabled!";
        String body = "This feature has not yet been "
            + "\nenabled.   Please check future"
            + "\nversions for availability";
        DialogFactory.createDialog(this, IconFactory.getVPIcon(),
            title, header, body, DialogFactory.OK_ONLY);

        // DBQuery.saveGraphToDB(pGraph);
    }

    public void onPrint()
    {
        BufferedImage bi = new BufferedImage(gEdit.getWidth(), gEdit
            .getHeight(), BufferedImage.TYPE_INT_RGB);
        Graphics2D big = bi.createGraphics();
        big.setColor(Color.white);
        big.fillRect(0, 0, gEdit.getWidth(), gEdit.getHeight());
        pGraph.paint(big);
        PrintFactory.printImage(bi);
    }

    private void onPrintPreview()
    {
        // BufferedImage bi =
        // new BufferedImage(
        // gEdit.getWidth(),
        // gEdit.getHeight(),
        // BufferedImage.TYPE_INT_RGB);
        // Graphics2D big = bi.createGraphics();
        // big.setColor(Color.white);
        // big.fillRect(0, 0, gEdit.getWidth(), gEdit.getHeight());
        // vGraph.paint(big);
        // BufferedImage previewImage =
        // PrintFactory.getPreviewImage(bi);
        // new PrintPreviewDialog(this, previewImage);

        String title = "Not enabled";
        String header = "Feature not yet enabled!";
        String body = "This feature has not yet been "
            + "\nenabled.   Please check future"
            + "\nversions for availability";
        DialogFactory.createDialog(this, IconFactory.getVPIcon(),
            title, header, body, DialogFactory.OK_ONLY);
    }

    private void onExit()
    {
        String title = "Exit";
        String header = "Exit VitaPad";
        String body = "Do you wish to exit?";
        DialogFactory.createDialog(this, IconFactory.getVPIcon(),
            title, header, body, DialogFactory.OK_CANCEL,
            ActionCenter.SYSTEM_EXIT, null, listener);
    }

    private void onSelectAllCompounds()
    {
        EditVertexList highlightedVertices = pGraph
            .getHighlightedVertices();
        highlightedVertices.clear();
        highlightedVertices.addAll(pGraph.getVertexList());
        for (int i = 0; i < highlightedVertices.size(); i++)
        {
            ((PathwayVertex) highlightedVertices.get(i))
                .setTexturePaint(GraphicsFactory.getTexturePaint());
        }
        pGraph.repaint();
    }

    private void onSelectAllLinks()
    {
        EditEdgeList highlightedEdges = pGraph.getHighlightedEdges();
        highlightedEdges.clear();
        highlightedEdges.addAll(pGraph.getEdgeList());
        for (int i = 0; i < highlightedEdges.size(); i++)
        {
            ((PathwayEdge) highlightedEdges.get(i))
                .setTexturePaint(GraphicsFactory.getTexturePaint());
        }
        pGraph.repaint();
    }

    private void onTableView()
    {
        split.remove(gEdit);
        TableViewer tv = new TableViewer(this, pGraph);
        split.setLeftComponent(tv);
        validate();
    }

    private void onGraphView()
    {
        try
        {
            showGraph(pGraph);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    private void onViewMode()
    {
        if (!(gPanel.getCurrentMode() instanceof GraphPanelViewMode))
        {
            clearEditPanel();
            getPathwayGraph().removeAllAnimatedElements();
            getPathwayGraph().clearHighlightedElements();
            toolBar.getViewButton().setSelected(true);
            menu.getViewModeButton().setSelected(true);
            gPanel.setViewMode();
        }
    }

    private void onEditMode()
    {
        if (!(gPanel.getCurrentMode() instanceof GraphPanelEditMode))
        {
            clearEditPanel();
            getPathwayGraph().removeAllAnimatedElements();
            toolBar.getSelectButton().setSelected(true);
            menu.getEditModeButton().setSelected(true);
            gPanel.setEditMode();
        }
    }

    private void onVertexMode()
    {
        if (!(gPanel.getCurrentMode() instanceof GraphPanelVertexMode))
        {
            clearEditPanel();
            getPathwayGraph().removeAllAnimatedElements();
            getPathwayGraph().clearHighlightedElements();
            toolBar.getVertexButton().setSelected(true);
            menu.getVertexModeButton().setSelected(true);
            gPanel.setVertexMode();
        }
    }

    private void onEdgeMode()
    {
        if (!(gPanel.getCurrentMode() instanceof GraphPanelEdgeMode))
        {
            clearEditPanel();
            getPathwayGraph().removeAllAnimatedElements();
            getPathwayGraph().clearHighlightedElements();
            toolBar.getEdgeButton().setSelected(true);
            menu.getEdgeModeButton().setSelected(true);
            gPanel.setEdgeMode();
        }
    }

    private void onMoveMode()
    {
        if (!(gPanel.getCurrentMode() instanceof GraphPanelMoveMode))
        {
            if (!(currEditComponent instanceof LoadFlatFileDialog))
            {
                 clearEditPanel();
            }
            getPathwayGraph().removeAllAnimatedElements();
            getPathwayGraph().clearHighlightedElements();
            toolBar.getMoveButton().setSelected(true);
            menu.getMoveModeButton().setSelected(true);
            gPanel.setMoveMode();
        }
    }

    private void onEditSelected()
    {
        EditVertexList highlightedVertices = pGraph
            .getHighlightedVertices();
        EditEdgeList highlightedEdges = pGraph.getHighlightedEdges();

        if (highlightedVertices.size() > 0)
            addToEditPanel(new AddVertexDialog(this, 0, 0, true,
                new PathwayVertex(highlightedVertices.getVertex(0)),
                highlightedVertices));

        if (highlightedEdges.size() > 0)
            addToEditPanel(new AddEdgeDialog(this,
                (PathwayVertex) highlightedEdges.getEdge(0)
                    .getVertexA(), (PathwayVertex) highlightedEdges
                    .getEdge(0).getVertexB(), true, new PathwayEdge(
                    highlightedEdges.getEdge(0)), highlightedEdges,
                highlightedEdges));

        toolBar.getEditButton().setSelected(false);
    }

    private void onDeleteSelected()
    {
        stack.store(pGraph);
        EditVertexList highlightedVertices = pGraph
            .getHighlightedVertices();
        EditEdgeList highlightedEdges = pGraph.getHighlightedEdges();

        if (highlightedVertices.size() > 0)
        {
            for (int i = 0; i < highlightedVertices.size(); i++)
            {
                PathwayVertex current = highlightedVertices
                    .getVertex(i);
                try
                {
                    pGraph.removeVertex(current);
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
        }

        if (highlightedEdges.size() > 0)
        {
            for (int i = 0; i < highlightedEdges.size(); i++)
            {
                PathwayEdge current = highlightedEdges.getEdge(i);
                try
                {
                    pGraph.removeEdge(current);
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
        }

        toolBar.getDeleteButton().setSelected(false);
    }

    private void onPathwayDetails()
    {
        getPathwayGraph().removeAllAnimatedElements();
        addToEditPanel(new PathwayDetailsDialog(this));
    }

    private void onLoadFlatFile()
    {
        if (getPathwayGraph() != null)
            getPathwayGraph().removeAllAnimatedElements();
        addToEditPanel(new LoadFlatFileDialog(this));
    }

    public void undo()
    {
        PathwayGraph pg = stack.undo(pGraph);
        showNewGraph();
        showGraph(pg);
        toolBar.getUndoButton().setSelected(false);
    }

    public void redo()
    {
        PathwayGraph pg = stack.redo(pGraph);
        showNewGraph();
        showGraph(pg);
        toolBar.getRedoButton().setSelected(false);
    }

    private class Listener implements WindowListener, ActionListener
    {
        public void actionPerformed(ActionEvent event)
        {
            String command = event.getActionCommand();

            if (command.equals(ActionCenter.NEW))
                onNew();
            else if (command.equals(ActionCenter.LOAD_FILE))
                onLoadFile();
            else if (command.equals(ActionCenter.LOAD_FROM_DB))
                onLoadFromDB();
            else if (command.equals(ActionCenter.LOAD_FLAT_FILE))
                onLoadFlatFile();
            else if (command.equals(ActionCenter.LOAD_EXPERIMENT))
                onLoadExperiment();
            else if (command.equals(ActionCenter.SAVE_FILE))
                onSaveFile();
            else if (command.equals(ActionCenter.SAVE_IMAGE))
                onSaveImage();
            else if (command.equals(ActionCenter.SAVE_FILE_INTO_DB))
                onSaveFileIntoDB();
            else if (command.equals(ActionCenter.PRINT))
                onPrint();
            else if (command.equals(ActionCenter.PRINT_PREVIEW))
                onPrintPreview();
            else if (command.equals(ActionCenter.EXIT))
                onExit();
            else if (command
                .equals(ActionCenter.SELECT_ALL_COMPOUNDS))
                onSelectAllCompounds();
            else if (command.equals(ActionCenter.SELECT_ALL_LINKS))
                onSelectAllLinks();
            else if (command.equals(ActionCenter.TABLE_VIEW))
                onTableView();
            else if (command.equals(ActionCenter.GRAPH_VIEW))
                onGraphView();
            else if (command.equals(ActionCenter.VIEW_MODE))
                onViewMode();
            else if (command.equals(ActionCenter.EDIT_MODE))
                onEditMode();
            else if (command.equals(ActionCenter.VERTEX_MODE))
                onVertexMode();
            else if (command.equals(ActionCenter.EDGE_MODE))
                onEdgeMode();
            else if (command.equals(ActionCenter.MOVE_MODE))
                onMoveMode();
            else if (command.equals(ActionCenter.SYSTEM_EXIT))
            {
                VPProperties.writeProperties();
                System.exit(0);
            }
            else if (command.equals(ActionCenter.EDIT_SELECTED))
                onEditSelected();
            else if (command.equals(ActionCenter.DELETE_SELECTED))
                onDeleteSelected();
            else if (command.equals(ActionCenter.UNDO))
                undo();
            else if (command.equals(ActionCenter.REDO))
                redo();
            else if (command.equals(ActionCenter.PATHWAY_DETAILS))
                onPathwayDetails();
            else if (command.equals(ActionCenter.ZOOM_IN))
            {

                String title = "Not enabled";
                String header = "Feature not yet enabled!";
                String body = "This feature has not yet been "
                    + "\nenabled.   Please check future"
                    + "\nversions for availability";
                DialogFactory.createDialog(thisFrame, IconFactory
                    .getVPIcon(), title, header, body,
                    DialogFactory.OK_ONLY);
                toolBar.getZoomInButton().setSelected(false);
            }

            else if (command.equals(ActionCenter.ZOOM_OUT))
            {
                String title = "Not enabled";
                String header = "Feature not yet enabled!";
                String body = "This feature has not yet been "
                    + "\nenabled.   Please check future"
                    + "\nversions for availability";
                DialogFactory.createDialog(thisFrame, IconFactory
                    .getVPIcon(), title, header, body,
                    DialogFactory.OK_ONLY);
                toolBar.getZoomOutButton().setSelected(false);
            }
        }

        /**
         * 
         */

        public void windowOpened(WindowEvent event)
        {
        }

        public void windowClosing(WindowEvent event)
        {
            onExit();
        }

        public void windowClosed(WindowEvent event)
        {
        }

        public void windowIconified(WindowEvent event)
        {
        }

        public void windowDeiconified(WindowEvent event)
        {
        }

        public void windowActivated(WindowEvent event)
        {
        }

        public void windowDeactivated(WindowEvent event)
        {
        }
    }

    /**
     * @return
     */
    public VPPanel getDisplayPanel()
    {
        return displayPanel;
    }

    /**
     * @return
     */
    public VPPanel getEditPanel()
    {
        return editPanel;
    }

    /**
     * @param panel
     */
    public void setDisplayPanel(VPPanel panel)
    {
        displayPanel = panel;
    }

    /**
     * @param panel
     */
    public void setEditPanel(VPPanel panel)
    {
        editPanel = panel;
    }

    /**
     * @param clipSize
     * @param clipFile
     */
    public void captureScreenClip(Rectangle clipSize, File clipFile)
    {
        BufferedImage screenBi = new BufferedImage(gEdit.getWidth(),
            gEdit.getHeight(), BufferedImage.TYPE_INT_RGB);
        Graphics2D screenBig = screenBi.createGraphics();
        screenBig.setColor(Color.white);
        screenBig.fillRect(0, 0, gEdit.getWidth(), gEdit.getHeight());
        pGraph.paint(screenBig);

        BufferedImage bi = new BufferedImage(clipSize.width,
            clipSize.height, BufferedImage.TYPE_INT_RGB);
        Graphics2D big = bi.createGraphics();
        big.setColor(Color.white);
        big.fillRect(0, 0, bi.getWidth(), bi.getHeight());
        big.drawImage(screenBi, 0, 0, clipSize.width,
            clipSize.height, clipSize.x, clipSize.y, clipSize.x
                + clipSize.width, clipSize.y + clipSize.height, this);

        try
        {
            ImageIO.write(bi, "png", clipFile);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * @return
     */
    public JTextField getAlertField()
    {
        return toolBar.getAlertField();
    }

    /**
     * @return
     */
    public VPPanel getAlertPanel()
    {
        return toolBar.getAlertPanel();
    }

    /**
     * @return
     */
    public JProgressBar getProgBar()
    {
        return toolBar.getProgBar();
    }

    /**
     * @return
     */
    public UndoRedoStack getStack()
    {
        return stack;
    }

    /**
     * @return
     */
    public GraphToolBar getToolBar()
    {
        return toolBar;
    }

    /**
     * @param bar
     */
    public void setToolBar(GraphToolBar bar)
    {
        toolBar = bar;
    }

    /**
     * @param classID
     */
    public Type getDefaultType(int classID)
    {
        return (Type) defaultTypes.get(new Integer(classID));
    }

    public void setDefaultType(int classID, Type type)
    {
        defaultTypes.put(new Integer(classID), type);
    }

    public String getTableName(int classID)
    {
        return (String) tableNames.get(new Integer(classID));
    }

    public String getClassName(int classID)
    {
        return (String) classNames.get(new Integer(classID));
    }

    public List getClasses(int elementType)
    {
        switch (elementType)
        {
            case DBConstants.VERTEX:
                return vertexClasses;
            case DBConstants.EDGE:
                return edgeClasses;
            case DBConstants.DECORATION:
                return decorationClasses;
            default:
                return null;
        }
    }

    /**
     * @return Returns the currEditComponent.
     */
    public JComponent getCurrEditComponent()
    {
        return currEditComponent;
    }
    

}
