package vp.main;

import java.awt.Font;
import java.awt.event.ActionListener;
import javax.swing.ButtonGroup;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;

import vp.util.ActionCenter;

public class MainMenu extends JMenuBar
{
    private MainFrame frame;

    private ActionListener listener;

    private JRadioButtonMenuItem viewModeButton;

    private JRadioButtonMenuItem editModeButton;

    private JRadioButtonMenuItem vertexModeButton;

    private JRadioButtonMenuItem edgeModeButton;

    private JRadioButtonMenuItem moveModeButton;

    private Font boldFont;

    private ButtonGroup modeGroup;

    public MainMenu(MainFrame frame)
    {
        this.frame = frame;
        listener = frame.getListener();
        boldFont = new Font("Palatino Linotype", Font.BOLD, 13);

        JMenu fileMenu = new JMenu("File");
        fileMenu.setFont(boldFont);
        fileMenu
                .add(createMenuItem("New", ActionCenter.NEW, listener));
        fileMenu.addSeparator();
        fileMenu.add(createMenuItem("Load File",
                ActionCenter.LOAD_FILE, listener));
        fileMenu.add(createMenuItem("Load From DB",
                ActionCenter.LOAD_FROM_DB, listener));
        fileMenu.add(createMenuItem("Load Experiment",
                ActionCenter.LOAD_EXPERIMENT, listener));
        fileMenu.add(createMenuItem("Load From Flat File",
                ActionCenter.LOAD_FLAT_FILE, listener));
        fileMenu.addSeparator();
        fileMenu.add(createMenuItem("Save File",
                ActionCenter.SAVE_FILE, listener));
        fileMenu.add(createMenuItem("Save Image",
                ActionCenter.SAVE_IMAGE, listener));
        fileMenu.add(createMenuItem("Save To Database",
                ActionCenter.SAVE_FILE_INTO_DB, listener));
        fileMenu.addSeparator();
        fileMenu.add(createMenuItem("Print", ActionCenter.PRINT,
                listener));
        fileMenu.add(createMenuItem("Print Preview",
                ActionCenter.PRINT_PREVIEW, listener));
        fileMenu.addSeparator();
        fileMenu.add(createMenuItem("Exit", ActionCenter.EXIT,
                listener));
        add(fileMenu);

        JMenu editMenu = new JMenu("Edit");
        editMenu.setFont(boldFont);
        JMenu selectAllMenu = new JMenu("Select All");
        selectAllMenu.setFont(boldFont);
        // selectAllMenu.setFont(boldFont);
        selectAllMenu.add(createMenuItem("Compounds",
                ActionCenter.SELECT_ALL_COMPOUNDS, listener));
        selectAllMenu.add(createMenuItem("Links",
                ActionCenter.SELECT_ALL_LINKS, listener));
        editMenu.add(selectAllMenu);
        editMenu.addSeparator();
        editMenu.add(createMenuItem("Table View",
                ActionCenter.TABLE_VIEW, listener));
        editMenu.add(createMenuItem("Graph View",
                ActionCenter.GRAPH_VIEW, listener));
        editMenu.addSeparator();
        editMenu.add(createMenuItem("Pathway Details",
                ActionCenter.PATHWAY_DETAILS, listener));
        add(editMenu);

        JMenu modeMenu = new JMenu("Mode");
        modeMenu.setFont(boldFont);
        // modeMenu.setFont(bigBoldFont);
        modeGroup = new ButtonGroup();
        viewModeButton = createRadioButtonMenuItem("View Mode",
                ActionCenter.VIEW_MODE, listener, modeGroup, true);
        modeMenu.add(viewModeButton);
        editModeButton = createRadioButtonMenuItem("Edit Mode",
                ActionCenter.EDIT_MODE, listener, modeGroup, false);
        modeMenu.add(editModeButton);
        vertexModeButton = createRadioButtonMenuItem("Vertex Mode",
                ActionCenter.VERTEX_MODE, listener, modeGroup, false);
        modeMenu.add(vertexModeButton);
        edgeModeButton = createRadioButtonMenuItem("Edge Mode",
                ActionCenter.EDGE_MODE, listener, modeGroup, false);
        modeMenu.add(edgeModeButton);
        moveModeButton = createRadioButtonMenuItem("Move Mode",
                ActionCenter.MOVE_MODE, listener, modeGroup, false);
        modeMenu.add(moveModeButton);

        add(modeMenu);
    }

    public JRadioButtonMenuItem getViewModeButton()
    {
        return viewModeButton;
    }

    public JRadioButtonMenuItem getEditModeButton()
    {
        return editModeButton;
    }

    public JRadioButtonMenuItem getVertexModeButton()
    {
        return vertexModeButton;
    }

    public JRadioButtonMenuItem getEdgeModeButton()
    {
        return edgeModeButton;
    }

    public JRadioButtonMenuItem getMoveModeButton()
    {
        return moveModeButton;
    }

    private JMenuItem createMenuItem(String name, String command,
            ActionListener listener)
    {
        JMenuItem item = new JMenuItem(name);
        item.setFont(boldFont);
        item.setActionCommand(command);
        item.addActionListener(listener);
        return item;
    }

    private JRadioButtonMenuItem createRadioButtonMenuItem(
            String name, String command, ActionListener listener,
            ButtonGroup group, boolean selected)
    {
        JRadioButtonMenuItem item = new JRadioButtonMenuItem(name,
                selected);
        item.setFont(boldFont);
        group.add(item);
        item.setActionCommand(command);
        item.addActionListener(listener);
        return item;
    }
}
