package vp.util;

import vp.graph.DataClass;
import vp.graph.Decoration;
import vp.graph.PathwayEdge;
import vp.graph.PathwayGraph;
import vp.util.database.DBQuery;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

public class EditEdgeList
	implements List, TypeChangeListener, DecorationChangeListener
{
	private DataClass newDataClass;
	private String newDirection = null;
	private PathwayGraph pGraph;
	private final List list = new ArrayList();

	public EditEdgeList(PathwayGraph pGraph)
	{
		this.pGraph = pGraph;
	}

	public PathwayEdge getEdge(int index)
	{
		return (PathwayEdge)get(index);
	}

	public void changeDirection(String newDirection)
	{
		this.newDirection = newDirection;
	}

	public void changeDataClass(DataClass newDataClass)
	{
		this.newDataClass = newDataClass;
	}

	public void typeChanged(TypeChangeEvent event)
	{
		PathwayEdge newElement = (PathwayEdge)event.getType();
		this.pGraph = event.getPathwayGraph();

		for (int i = 0; i < size(); i++)
		{
			PathwayEdge old = (PathwayEdge)get(i);
			PathwayEdge l;
			String direction =
				(newDirection == null) ? old.getDirection() : newDirection;
			DataClass dataClass =
				(newDataClass == null) ? old.getDataClass() : newDataClass;

			l =
				new PathwayEdge(
					pGraph,
					DBQuery.getUniqueID(),
					dataClass,
					newElement.getType(),
					old.getVertexA(),
					old.getVertexB(),
					direction);

			Iterator decIter = old.getDecorationIterator();
			while (decIter.hasNext())
			{
				try
				{
					l.addDecoration((Decoration)decIter.next());
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}
			}
			try
			{
				pGraph.removeEdge(old);
				pGraph.addEdge(l);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
	}

	/* (non-Javadoc)
	 * @see vp.src.edu.yale.vitapad.util.DecorationChangeListener#decorationsChanged(vp.src.edu.yale.vitapad.util.DecorationChangeEvent)
	 */
	public void decorationsChanged(DecorationChangeEvent de)
	{
		List newDecs = de.getDecorations();
		PathwayGraph vGraph = de.getVGraph();
		for (int i = 0; i < size(); i++)
		{
			PathwayEdge l = (PathwayEdge)get(i);
			l.removeAllDecorations();
		}
		for (int i = 0; i < size(); i++)
		{
			PathwayEdge l = (PathwayEdge)get(i);
			for (int j = 0; j < newDecs.size(); j++)
			{
				l.addDecoration((Decoration)newDecs.get(j));
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.util.List#size()
	 */
	public int size()
	{
		return list.size();
	}

	/* (non-Javadoc)
	 * @see java.util.List#clear()
	 */
	public void clear()
	{
		for (int i = 0; i < size(); i++)
		{
			PathwayEdge pe = getEdge(i);
			pe.setTexturePaint(null);
		}
		list.clear();
	}

	/* (non-Javadoc)
	 * @see java.util.List#isEmpty()
	 */
	public boolean isEmpty()
	{
		return list.isEmpty();
	}

	/* (non-Javadoc)
	 * @see java.util.List#toArray()
	 */
	public Object[] toArray()
	{
		return list.toArray();
	}

	/* (non-Javadoc)
	 * @see java.util.List#get(int)
	 */
	public Object get(int index)
	{
		return list.get(index);
	}

	/* (non-Javadoc)
	 * @see java.util.List#remove(int)
	 */
	public Object remove(int index)
	{
		return list.remove(index);
	}

	/* (non-Javadoc)
	 * @see java.util.List#add(int, java.lang.Object)
	 */
	public void add(int index, Object element) throws ClassCastException
	{
		if (element instanceof PathwayEdge)
			list.add(index, element);
		else
			throw new ClassCastException("Element to be added must be a PathwayVertex");
	}

	/* (non-Javadoc)
	 * @see java.util.List#indexOf(java.lang.Object)
	 */
	public int indexOf(Object o)
	{
		return list.indexOf(o);
	}

	/* (non-Javadoc)
	 * @see java.util.List#lastIndexOf(java.lang.Object)
	 */
	public int lastIndexOf(Object o)
	{
		return list.lastIndexOf(o);
	}

	/* (non-Javadoc)
	 * @see java.util.List#add(java.lang.Object)
	 */
	public boolean add(Object o) throws ClassCastException
	{
		if (o instanceof PathwayEdge)
			return list.add(o);
		else
			throw new ClassCastException("Element to be added must be a PathwayEdge");
	}

	/* (non-Javadoc)
	 * @see java.util.List#contains(java.lang.Object)
	 */
	public boolean contains(Object o)
	{
		return list.contains(o);
	}

	/* (non-Javadoc)
	 * @see java.util.List#remove(java.lang.Object)
	 */
	public boolean remove(Object o)
	{
		return list.remove(o);
	}

	/* (non-Javadoc)
	 * @see java.util.List#addAll(int, java.util.Collection)
	 */
	public boolean addAll(int index, Collection c) throws ClassCastException
	{
		Iterator it = c.iterator();
		while (it.hasNext())
		{
			if (!(it.next() instanceof PathwayEdge))
				throw new ClassCastException("All elements to be added must be PathwayEdges");
		}
		return list.addAll(index, c);
	}

	/* (non-Javadoc)
	 * @see java.util.List#addAll(java.util.Collection)
	 */
	public boolean addAll(Collection c) throws ClassCastException
	{
		Iterator it = c.iterator();
		while (it.hasNext())
		{
			if (!(it.next() instanceof PathwayEdge))
				throw new ClassCastException("All elements to be added must be PathwayEdges");
		}
		return list.addAll(c);
	}

	/* (non-Javadoc)
	 * @see java.util.List#containsAll(java.util.Collection)
	 */
	public boolean containsAll(Collection c)
	{
		return list.containsAll(c);
	}

	/* (non-Javadoc)
	 * @see java.util.List#removeAll(java.util.Collection)
	 */
	public boolean removeAll(Collection c)
	{
		return list.removeAll(c);
	}

	/* (non-Javadoc)
	 * @see java.util.List#retainAll(java.util.Collection)
	 */
	public boolean retainAll(Collection c)
	{
		return list.retainAll(c);
	}

	/* (non-Javadoc)
	 * @see java.util.List#iterator()
	 */
	public Iterator iterator()
	{
		return list.iterator();
	}

	/* (non-Javadoc)
	 * @see java.util.List#subList(int, int)
	 */
	public List subList(int fromIndex, int toIndex)
	{
		return list.subList(fromIndex, toIndex);
	}

	/* (non-Javadoc)
	 * @see java.util.List#listIterator()
	 */
	public ListIterator listIterator()
	{
		return list.listIterator();
	}

	/* (non-Javadoc)
	 * @see java.util.List#listIterator(int)
	 */
	public ListIterator listIterator(int index)
	{
		return list.listIterator(index);
	}

	/* (non-Javadoc)
	 * @see java.util.List#set(int, java.lang.Object)
	 */
	public Object set(int index, Object element) throws ClassCastException
	{
		if (element instanceof PathwayEdge)
			return list.set(index, element);
		else
			throw new ClassCastException("Element to be added must be a PathwayEdge");
	}

	/* (non-Javadoc)
	 * @see java.util.List#toArray(java.lang.Object[])
	 */
	public Object[] toArray(Object[] a)
	{
		return list.toArray(a);
	}

	/* (non-Javadoc)
	 * @see vp.util.DecorationChangeListener#addDecoration(vp.util.DecorationChangeEvent)
	 */
	public void addDecoration(DecorationChangeEvent de)
	{
		
	}

	/* (non-Javadoc)
	 * @see vp.util.DecorationChangeListener#removeDecoration(vp.util.DecorationChangeEvent)
	 */
	public void removeDecoration(DecorationChangeEvent de)
	{
		
	}

}
