package vp.util;

import vp.graph.DataClass;
import vp.graph.Decoration;
import vp.graph.Edge;
import vp.graph.PathwayEdge;
import vp.graph.PathwayGraph;
import vp.graph.PathwayVertex;
import vp.graph.Vertex;
import vp.util.database.DBQuery;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Set;

public class EditVertexList implements TypeChangeListener, List
{
	private DataClass newDataClass;
	private PathwayGraph pGraph;
	private final List list = new ArrayList();

	public EditVertexList(PathwayGraph pGraph)
	{
		this.pGraph = pGraph;
	}

	public PathwayVertex getVertex(int index)
	{
		return (PathwayVertex)get(index);
	}

	public void changeDataClass(DataClass newDataClass)
	{
		this.newDataClass = newDataClass;
	}

	public void typeChanged(TypeChangeEvent event)
	{
		PathwayVertex newElement = (PathwayVertex)event.getType();
		this.pGraph = event.getPathwayGraph();
		Map verts = new HashMap();
		Map newEdges = new HashMap();

		for (int i = 0; i < size(); i++)
		{
			DataClass dataClass;
			PathwayVertex old = getVertex(i);
			if (newDataClass == null)
				dataClass = old.getDataClass();
			else
				dataClass = newDataClass;

			PathwayVertex v =
				new PathwayVertex(
					pGraph,
					DBQuery.getUniqueID(),
					dataClass,
					newElement.getType(),
					old.getXPosition(),
					old.getYPosition(),
					old.getOnLink(),
					old.getOnLinkPosition());
			verts.put(old, v);
		}

		newDataClass = null;

		Iterator it = pGraph.getEdgeIterator();
		while (it.hasNext())
		{
			PathwayEdge current = (PathwayEdge)it.next();
			if (verts.containsKey(current.getVertexA())
				|| verts.containsKey(current.getVertexB()))
			{
				PathwayEdge newEdge =
					new PathwayEdge(
						pGraph,
						DBQuery.getUniqueID(),
						current.getDataClass(),
						current.getType(),
						current.getVertexA(),
						current.getVertexB(),
						current.getDirection());

				if (verts.containsKey(current.getVertexA()))
					newEdge.setVertexA(
						(PathwayVertex)verts.get(current.getVertexA()));

				if (verts.containsKey(current.getVertexB()))
					newEdge.setVertexB(
						(PathwayVertex)verts.get(current.getVertexB()));

				Iterator decIter = current.getDecorationIterator();
				while (decIter.hasNext())
				{
					try
					{
						newEdge.addDecoration((Decoration)decIter.next());
					}
					catch (Exception e)
					{
						e.printStackTrace();
					}
				}
				newEdges.put(current, newEdge);
			}
		}

		Set set = verts.entrySet();
		Iterator iter = set.iterator();

		while (iter.hasNext())
		{
			Map.Entry entry = (Map.Entry)iter.next();
			try
			{
				pGraph.removeVertex((Vertex)entry.getKey());
				pGraph.addVertex((Vertex)entry.getValue());
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}

        set = newEdges.entrySet();
        iter = set.iterator();

        while (iter.hasNext())
        {
            Map.Entry entry = (Map.Entry)iter.next();
            try
            {
                pGraph.removeEdge((Edge)entry.getKey());
                pGraph.addEdge((Edge)entry.getValue());
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
	}

	/* (non-Javadoc)
	 * @see java.util.List#size()
	 */
	public int size()
	{
		return list.size();
	}

	/* (non-Javadoc)
	 * @see java.util.List#clear()
	 */
	public void clear()
	{
		for (int i = 0; i < size(); i++)
		{
			PathwayVertex pv = getVertex(i);
			pv.setTexturePaint(null);
		}
		list.clear();

	}

	/* (non-Javadoc)
	 * @see java.util.List#isEmpty()
	 */
	public boolean isEmpty()
	{
		return list.isEmpty();
	}

	/* (non-Javadoc)
	 * @see java.util.List#toArray()
	 */
	public Object[] toArray()
	{
		return list.toArray();
	}

	/* (non-Javadoc)
	 * @see java.util.List#get(int)
	 */
	public Object get(int index)
	{
		return list.get(index);
	}

	/* (non-Javadoc)
	 * @see java.util.List#remove(int)
	 */
	public Object remove(int index)
	{
		return list.remove(index);
	}

	/* (non-Javadoc)
	 * @see java.util.List#add(int, java.lang.Object)
	 */
	public void add(int index, Object element) throws ClassCastException
	{
		if (element instanceof PathwayVertex)
			list.add(index, element);
		else
			throw new ClassCastException("Element to be added must be a PathwayVertex");
	}

	/* (non-Javadoc)
	 * @see java.util.List#indexOf(java.lang.Object)
	 */
	public int indexOf(Object o)
	{
		return list.indexOf(o);
	}

	/* (non-Javadoc)
	 * @see java.util.List#lastIndexOf(java.lang.Object)
	 */
	public int lastIndexOf(Object o)
	{
		return list.lastIndexOf(o);
	}

	/* (non-Javadoc)
	 * @see java.util.List#add(java.lang.Object)
	 */
	public boolean add(Object o) throws ClassCastException
	{
		if (o instanceof PathwayVertex)
			return list.add(o);
		else
			throw new ClassCastException("Element to be added must be a PathwayVertex");
	}

	/* (non-Javadoc)
	 * @see java.util.List#contains(java.lang.Object)
	 */
	public boolean contains(Object o)
	{
		return list.contains(o);
	}

	/* (non-Javadoc)
	 * @see java.util.List#remove(java.lang.Object)
	 */
	public boolean remove(Object o)
	{
		return list.remove(o);
	}

	/* (non-Javadoc)
	 * @see java.util.List#addAll(int, java.util.Collection)
	 */
	public boolean addAll(int index, Collection c) throws ClassCastException
	{
		Iterator it = c.iterator();
		while (it.hasNext())
		{
			if (!(it.next() instanceof PathwayVertex))
				throw new ClassCastException("All elements to be added must be PathwayVertices");
		}
		return list.addAll(index, c);
	}

	/* (non-Javadoc)
	 * @see java.util.List#addAll(java.util.Collection)
	 */
	public boolean addAll(Collection c)
	{
		Iterator it = c.iterator();
		while (it.hasNext())
		{
			if (!(it.next() instanceof PathwayVertex))
				throw new ClassCastException("All elements to be added must be PathwayVertices");
		}
		return list.addAll(c);
	}

	/* (non-Javadoc)
	 * @see java.util.List#containsAll(java.util.Collection)
	 */
	public boolean containsAll(Collection c)
	{
		return list.containsAll(c);
	}

	/* (non-Javadoc)
	 * @see java.util.List#removeAll(java.util.Collection)
	 */
	public boolean removeAll(Collection c)
	{
		return list.removeAll(c);
	}

	/* (non-Javadoc)
	 * @see java.util.List#retainAll(java.util.Collection)
	 */
	public boolean retainAll(Collection c)
	{
		return list.retainAll(c);
	}

	/* (non-Javadoc)
	 * @see java.util.List#iterator()
	 */
	public Iterator iterator()
	{
		return list.iterator();
	}

	/* (non-Javadoc)
	 * @see java.util.List#subList(int, int)
	 */
	public List subList(int fromIndex, int toIndex)
	{
		return list.subList(fromIndex, toIndex);
	}

	/* (non-Javadoc)
	 * @see java.util.List#listIterator()
	 */
	public ListIterator listIterator()
	{
		return list.listIterator();
	}

	/* (non-Javadoc)
	 * @see java.util.List#listIterator(int)
	 */
	public ListIterator listIterator(int index)
	{
		return list.listIterator(index);
	}

	/* (non-Javadoc)
	 * @see java.util.List#set(int, java.lang.Object)
	 */
	public Object set(int index, Object element) throws ClassCastException
	{
		if (element instanceof PathwayVertex)
			return list.set(index, element);
		else
			throw new ClassCastException("Element to be added must be a PathwayVertex");
	}

	/* (non-Javadoc)
	 * @see java.util.List#toArray(java.lang.Object[])
	 */
	public Object[] toArray(Object[] a)
	{
		return list.toArray(a);
	}

}
