/*
 * Created on Jul 12, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package vp.util;

import java.awt.geom.GeneralPath;
import java.awt.geom.PathIterator;
import java.util.ArrayList;
import java.util.List;

import vp.graph.DataClass;
import vp.graph.GraphDataClass;
import vp.graph.PathwayDecoration;
import vp.graph.PathwayEdge;
import vp.graph.PathwayElement;
import vp.graph.PathwayVertex;
import vp.util.database.DBConstants;
import vp.visual.type.Type;

/**
 * @author Matt Holford
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class GraphUtilities
{
	public static final DataClass HEADLESS_EDGE = getHeadlessEdgeClass();
	private static DataClass unselectedVertexClass;
	private static DataClass headlessVertexClass;
	private static DataClass headlessEdgeClass;
	private static DataClass headlessDecorationClass;
	private static GraphDataClass tempGraphDataClass;
	public static final DataClass UNSELECTED_VERTEX =
		getUnselectedVertexClass();
	public static final DataClass HEADLESS_VERTEX = getHeadlessVertexClass();
	public static final DataClass HEADLESS_DECORATION =
		getHeadlessDecorationClass();
	public static final GraphDataClass TEMP_GRAPH_CLASS =
		getTempGraphDataClass();

	public static List getHeadlessElements(List types, int elementType)
	{
		List out = new ArrayList();
		for (int i = 0; i < types.size(); i++)
		{

			Type currentType = (Type)types.get(i);
			if (currentType != null)
			{
				PathwayElement pe =
					getHeadlessElement(currentType, elementType);
				boolean hasMatch = false;
				for (int j = 0; j < out.size(); j++)
				{
					Type jtype =
						((PathwayElement)out.get(j)).getRenderer().getType();
					if (jtype.equals(pe.getRenderer().getType()))
						hasMatch = true;

				}
				if (!hasMatch)
					out.add(pe);
			}
		}
		return out;
	}

	private static GraphDataClass getTempGraphDataClass()
	{
		if (tempGraphDataClass == null)
			initTempGraphDataClass();
		return tempGraphDataClass;
	}

	/**
	 * 
	 */
	private static void initTempGraphDataClass()
	{
		tempGraphDataClass = new GraphDataClass("X40001");
		tempGraphDataClass.put("id", "X40001");
		tempGraphDataClass.put("Name", "Temp");
	}

	/**
	 * @return
	 */
	private static DataClass getHeadlessVertexClass()
	{
		if (headlessVertexClass == null)
			initHeadlessVertexClass();
		return headlessVertexClass;
	}

	private static DataClass getHeadlessEdgeClass()
	{
		if (headlessEdgeClass == null)
			initHeadlessEdgeClass();
		return headlessEdgeClass;
	}

	private static DataClass getHeadlessDecorationClass()
	{
		if (headlessDecorationClass == null)
			initHeadlessDecorationClass();
		return headlessDecorationClass;
	}

	/**
	 * 
	 */
	private static void initHeadlessEdgeClass()
	{
		headlessEdgeClass = new DataClass("X20001", null);
		headlessEdgeClass.put("id", "X20001");
		headlessEdgeClass.put("Name", "Edge");
	}

	/**
	 * @return
	 */
	private static DataClass getUnselectedVertexClass()
	{
		if (unselectedVertexClass == null)
			initUnselectedVertexClass();
		return unselectedVertexClass;
	}

	/**
	 * 
	 */
	private static void initUnselectedVertexClass()
	{
		unselectedVertexClass = new DataClass("X10001", null);
		unselectedVertexClass.put("id", "X10001");
		unselectedVertexClass.put("Name", "Not selected");
	}

	private static void initHeadlessDecorationClass()
	{
		headlessDecorationClass = new DataClass("X30001", null);
		headlessDecorationClass.put("id", "X30001");
		headlessDecorationClass.put("Name", "Dec");
	}

	/**
	 * @param currentType
	 * @param elementType
	 * @return
	 */
	public static PathwayElement getHeadlessElement(
		Type currentType,
		int elementType)
	{
		PathwayElement pe;
		DataClass dataClass = new DataClass("_BLANK", null);
		dataClass.put("id", "_BLANK");
		switch (elementType)
		{
			case DBConstants.VERTEX :
				pe =
					new PathwayVertex(
						null,
						-1,
						HEADLESS_VERTEX,
						currentType,
						0,
						0);
				((PathwayVertex)pe).rescale();
				break;

			case DBConstants.EDGE :
				pe =
					new PathwayEdge(
						null,
						-1,
						HEADLESS_EDGE,
						currentType,
						null,
						null,
						PathwayEdge.BOTH);
				break;

			case DBConstants.DECORATION :
				pe =
					new PathwayDecoration(
						null,
						-1,
						HEADLESS_DECORATION,
						currentType,
						null,
						0);
				((PathwayDecoration)pe).rescale();
				break;

			default :
				pe = null;
				break;
		}
		//r.setType(currentType);

		return pe;
	}

	/**
	 * 
	 */
	private static void initHeadlessVertexClass()
	{
		headlessVertexClass = new DataClass("X10002", null);
		headlessVertexClass.put("id", "X10002");
		headlessVertexClass.put("Name", "Vert");
	}

	public static void describePath(GeneralPath gp)
	{
		PathIterator pi = gp.getPathIterator(null);

		while (pi.isDone() == false)
		{
			describeCurrentSegment(pi);
			pi.next();
		}
	}

	private static void describeCurrentSegment(PathIterator pi)
	{
		double[] coordinates = new double[6];
		int type = pi.currentSegment(coordinates);

		switch (type)
		{
			case PathIterator.SEG_MOVETO :
				System.out.println(
					"move to " + coordinates[0] + ", " + coordinates[1]);
				break;
			case PathIterator.SEG_LINETO :
				System.out.println(
					"line to " + coordinates[0] + ", " + coordinates[1]);
				break;
			case PathIterator.SEG_QUADTO :
				System.out.println(
					"quadratic to "
						+ coordinates[0]
						+ ", "
						+ coordinates[1]
						+ ", "
						+ coordinates[2]
						+ ", "
						+ coordinates[3]);
				break;
			case PathIterator.SEG_CUBICTO :
				System.out.println(
					"cubic to "
						+ coordinates[0]
						+ ", "
						+ coordinates[1]
						+ ", "
						+ coordinates[2]
						+ ", "
						+ coordinates[3]
						+ ", "
						+ coordinates[4]
						+ ", "
						+ coordinates[5]);
				break;
			case PathIterator.SEG_CLOSE :
				System.out.println("close");
				break;
			default :
				break;
		}
	}
}
