package vp.util;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.TexturePaint;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.DataBuffer;
import java.awt.image.IndexColorModel;

import vp.gui.VPSettings.Colors;

public class GraphicsFactory
{
	private static BufferedImage texture;
	private static TexturePaint texturePaint;
	private static float rgbh, rgbs, rgbv;

	private GraphicsFactory()
	{}

	private static BufferedImage getTexture()
	{
		texture = new BufferedImage(4, 4, BufferedImage.TYPE_INT_RGB);
		Graphics g = texture.getGraphics();
		g.setColor(Color.lightGray);
		g.fillRect(0, 0, 2, 2);
		g.fillRect(2, 2, 2, 2);
		g.setColor(Color.white);
		g.fillRect(0, 2, 2, 2);
		g.fillRect(2, 0, 2, 2);
		return texture;
	}

	public static TexturePaint getTexturePaint()
	{
		texturePaint =
			new TexturePaint(getTexture(), new Rectangle(0, 0, 4, 4));
		return texturePaint;
	}

	public static BasicStroke getDashed()
	{
		float[] dash = { 5.0f };
		return new BasicStroke(
			1.0f,
			BasicStroke.CAP_BUTT,
			BasicStroke.JOIN_MITER,
			5.0f,
			dash,
			0.0f);
	}

	public static BufferedImage antiAliasImage(
		BufferedImage image,
		float sensitivity,
		int brightness)
	{
		BufferedImage outImage =
			new BufferedImage(
				image.getWidth(),
				image.getHeight(),
				BufferedImage.TYPE_INT_ARGB);
		Graphics2D oig = outImage.createGraphics();
		oig.drawImage(image, 0, 0, null);
		int px[] = new int[9];
		int rav[] = new int[9];
		int gav[] = new int[9];
		int bav[] = new int[9];

		sensitivity = Math.abs(sensitivity);
		sensitivity = (float) ((sensitivity > .15) ? sensitivity : .15);
		sensitivity = (float) ((sensitivity < .5) ? sensitivity : .5);

		brightness = Math.abs(brightness);
		brightness = (brightness > 15) ? brightness : 15;
		brightness = (brightness < 75) ? brightness : 75;

		for (int x = 0; x < image.getWidth(); x++)
		{
			for (int y = 0; y < image.getHeight(); y++)
			{
				int a, b;
				int rax = 0;
				int gax = 0;
				int bax = 0;
				a = ((y == 0) || (y > (outImage.getHeight() - 2))) ? 0 : 1;
				b = ((x == 0) || (x > (outImage.getWidth() - 2))) ? 0 : 1;

				px[0] = outImage.getRGB(x, y - a);
				px[1] = outImage.getRGB(x, y + a);
				px[2] = outImage.getRGB(x + b, y);
				px[3] = outImage.getRGB(x - b, y);
				px[4] = outImage.getRGB(x, y);
				px[5] = outImage.getRGB(x + b, y - a);
				px[6] = outImage.getRGB(x - b, y + a);
				px[7] = outImage.getRGB(x + b, y - a);
				px[8] = outImage.getRGB(x - b, y + a);

				rgb_hsv(px[4]);
				float z4 = rgbv;
				float h4 = rgbh - 180;

				rgb_hsv(px[1]);
				float z1 = rgbv;
				float h1 = rgbh - 180;

				rgb_hsv(px[2]);
				float z2 = rgbv;
				float h2 = rgbh - 180;

				if ((px[4] != px[1]
					&& (Math.abs(z4 - z1) > sensitivity)
					|| Math.abs(h4 - h1) > brightness)
					|| (px[4] != px[2]
						&& (Math.abs(z4 - z2) > sensitivity
							|| Math.abs(h4 - h2) > brightness)))
				{
					for (int i = 0; i <= 8; i++)
					{
						rav[i] = (px[i] >> 16) & 0xff;
						gav[i] = (px[i] >> 8) & 0xff;
						bav[i] = px[i] & 0xff;
					}

					for (int i = 0; i <= 8; i++)
					{
						rax = rax + rav[i];
						gax = gax + gav[i];
						bax = bax + bav[i];
					}

					rax = rax / 9;
					gax = gax / 9;
					bax = bax / 9;

					int rgb = ((rax << 16) + (gax << 8) + bax);
					rgb = - (rgb);
					outImage.setRGB(x, y, rgb);
					
				}
			}
		}

		return outImage;

	}

	/**
	 * @param i
	 */
	private static void rgb_hsv(int rgb)
	{
		float r = (float) ((rgb >> 16 & 0xff) / 255.0);
		float g = (float) ((rgb >> 8 & 0xff) / 255.0);
		float b = (float) ((rgb & 0xff) / 255.0);

		float minVal = (r < g) ? r : g;
		minVal = (b < minVal) ? b : minVal;

		float maxVal = (r > g) ? r : g;
		maxVal = (b > maxVal) ? b : maxVal;

		float diff = maxVal - minVal;

		rgbv = maxVal;

		if (maxVal == 0)
		{
			rgbv = 0;
			rgbh = -1;
		}
		else
		{
			rgbs = diff / maxVal;

			if (r == maxVal)
				rgbh = (g - b) / diff;
			else if (g == maxVal)
				rgbh = 2 + (b - r) / diff;

			else
				rgbh = 4 + (r - g) / diff;

			rgbh = rgbh * 60;
			if (rgbh < 0)
				rgbh = rgbh + 360;
		}
	}

	public static BufferedImage colorConvert(
		BufferedImage image,
		int oldColorLow,
		int oldColorHigh,
		int newColor)
	{
		int pixelCount = 0;
		int color = Colors.SIDE_PANEL.getRGB();
		BufferedImage outImage =
			new BufferedImage(
				image.getWidth(),
				image.getHeight(),
				BufferedImage.TYPE_INT_RGB);
		for (int x = 0; x < image.getWidth(); x++)
		{
			for (int y = 0; y < image.getHeight(); y++)
			{
				int pixel = image.getRGB(x, y);
				if ((pixel <= oldColorHigh) && (pixel >= oldColorLow))
				{
					outImage.setRGB(x, y, newColor);
					pixelCount++;
				}
				else
					outImage.setRGB(x, y, pixel);
			}
		}
		return outImage;

	}

	public static BufferedImage makeTransparent(BufferedImage image)
	{
		ColorModel cm = image.getColorModel();
		if (cm instanceof IndexColorModel)
		{
			IndexColorModel icm = (IndexColorModel)cm;
			int[] cmap = new int[icm.getMapSize()];
			icm.getRGBs(cmap);
			int white = 0xFFFFFFFF;
			int i;
			for (i = 0; i < cmap.length; ++i)
				if (cmap[i] == white)
					break;
			if (i != cmap.length)
			{
				cmap[i] = 0;
				//transparent                
				IndexColorModel newicm =
					new IndexColorModel(
						8,
						cmap.length,
						cmap,
						0,
						true,
						Transparency.BITMASK,
						DataBuffer.TYPE_BYTE);
				image =
					new BufferedImage(newicm, image.getRaster(), false, null);
			}

		}
		return image;

	}
}
