package vp.util;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterJob;

public class PrintFactory implements Printable
{
	private BufferedImage printImage;

	public static void printImage(BufferedImage printImage)
	{
		new PrintFactory(printImage).print();
	}

	public static BufferedImage getPreviewImage(BufferedImage previewImage)
	{
		return new PrintFactory(previewImage).getPrintImage();
	}

	PrintFactory(BufferedImage printImage)
	{
		this.printImage = printImage;
	}

	public void print()
	{
		PrinterJob printJob = PrinterJob.getPrinterJob();
		printJob.setPrintable(this);
		if (printJob.printDialog())
		{
			try
			{
				printJob.print();
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
	}

	public BufferedImage getPrintImage()
	{
		PrinterJob printJob = PrinterJob.getPrinterJob();
		printJob.setPrintable(this);
		PageFormat pf = printJob.defaultPage();
		Graphics2D pig = printImage.createGraphics();
		pig.translate(pf.getImageableX(), pf.getImageableY());
		pig.drawImage(
			printImage,
			0,
			0,
			(int)pf.getImageableWidth(),
			(int)pf.getImageableHeight(),
			0,
			0,
			printImage.getWidth(),
			printImage.getHeight(),
			null);
		return printImage;
	}

	public int print(Graphics g, PageFormat pf, int pi)
	{
		if (pi > 0)
		{
			return (NO_SUCH_PAGE);
		}
		else
		{
			Graphics2D g2 = (Graphics2D)g;
			g2.translate(pf.getImageableX(), pf.getImageableY());
			g2.drawImage(
				printImage,
				0,
				0,
				(int)pf.getImageableWidth(),
				(int)pf.getImageableHeight(),
				0,
				0,
				printImage.getWidth(),
				printImage.getHeight(),
				null);
			return (PAGE_EXISTS);
		}
	}
}
