package vp.util;

import vp.gui.components.DialogFactory;
import vp.main.MainFrame;
import vp.visual.type.DefaultDecorationType1;
import vp.visual.type.DefaultDecorationType2;
import vp.visual.type.DefaultEdgeType1;
import vp.visual.type.DefaultEdgeType2;
import vp.visual.type.DefaultVertexType1;
import vp.visual.type.DefaultVertexType2;
import vp.visual.type.Type;
import java.awt.Color;
import java.awt.Font;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileFilter;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;

public class SaveType
{

    private SaveType()
    {
    }

    public static void saveType(MainFrame parent, int elementType,
        Type currentType, File file)
    {
        List typeList = null;
        switch (elementType)
        {
            case Type.DECORATION:
                typeList = parent.getPathwayGraph()
                    .getAvailableDecorationTypes();
                break;
            case Type.EDGE:
                typeList = parent.getPathwayGraph()
                    .getAvailableEdgeTypes();
                break;
            case Type.VERTEX:
                typeList = parent.getPathwayGraph()
                    .getAvailableVertexTypes();
                break;

            default:
                break;
        }

        Type match = matchTypes(currentType, typeList);

        if (match != null)
        {
            String title = "Error";
            String header = "Type exists!";
            String body = "This type already exists as\n"
                + match.getName();
            DialogFactory.createDialog(parent, IconFactory
                .getErrorIcon(), title, header, body,
                DialogFactory.OK_ONLY);
        }
        else
        {
            saveTypeNow(elementType, currentType, file);

            switch (elementType)
            {
                case Type.DECORATION:
                    parent.getPathwayGraph()
                        .updateAvailableDecorationTypes();
                    break;
                case Type.EDGE:
                    parent.getPathwayGraph()
                        .updateAvailableEdgeTypes();
                    break;
                case Type.VERTEX:
                    parent.getPathwayGraph()
                        .updateAvailableVertexTypes();
                    break;

                default:
                    break;
            }
        }
    }

    public static boolean typeFileExists(int elementType,
        String shortName)
    {
        File tempFile = getTempFile(elementType, shortName);
        return tempFile.exists();
    }

    public static void saveTypeNow(int elementType, Type currentType,
        File file)
    {
        String fileName = file.getName();
        fileName = fileName.endsWith(".type") ? fileName.substring(0,
            fileName.length() - 5) : fileName;
        String longName = fileName;
        String shortName = fileName.substring(0, Math.min(4, fileName
            .length() - 1));
        File tempFile = getTempFile(elementType, fileName);
        try
        {
            BufferedWriter out = new BufferedWriter(new FileWriter(
                tempFile));

            out.write("ShortName\t" + shortName + "\n");
            out.write("LongName\t" + longName + "\n");
            out.write("Shape\t" + currentType.getShape() + "\n");

            out.write("OutlineColor\t"
                + currentType.getOutlineColor().getRGB() + "\n");
            if (currentType.getFillColor() != null)
                out.write("FillColor\t"
                    + currentType.getFillColor().getRGB() + "\n");
            if (currentType.getFont() != null)
            {
                out.write("FontName\t"
                    + currentType.getFont().getFontName() + "\n");
                out.write("FontStyle\t"
                    + currentType.getFont().getStyle() + "\n");
                out.write("FontSize\t"
                    + currentType.getFont().getSize() + "\n");
            }
            if (currentType.getFontColor() != null)
                out.write("FontColor\t"
                    + currentType.getFontColor().getRGB() + "\n");
            out.write("Curvature\t" + currentType.getCurvature()
                + "\n");

            out.flush();
            out.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    public static ArrayList getVertexTypes()
    {
        ArrayList vertexTypes = new ArrayList();
        vertexTypes.add(new DefaultVertexType1());
        vertexTypes.add(new DefaultVertexType2());

        File[] typeFiles = new File(getPathName(Type.VERTEX))
            .listFiles((FileFilter) new TypeFileFilter());

        for (int i = 0; i < typeFiles.length; i++)
        {
            // Workaround for 'CVS' files in eclipse
            if(typeFiles[i].getName().equalsIgnoreCase("CVS"))
                continue;
            Properties prop = getProperties(typeFiles[i]);

            String outlinecolor = prop.getProperty("OutlineColor");
            Color outlineColor = new Color(Integer
                .parseInt(outlinecolor));

            String fillcolor = prop.getProperty("FillColor");
            Color fillColor = new Color(Integer.parseInt(fillcolor));

            String fontcolor = prop.getProperty("FontColor");
            Color fontColor = new Color(Integer.parseInt(fontcolor));

            String fontName = prop.getProperty("FontName");
            String fontStyle = prop.getProperty("FontStyle");
            String fontSize = prop.getProperty("FontSize");
            Font font = new Font(fontName, Integer
                .parseInt(fontStyle), Integer.parseInt(fontSize));

            String shape = prop.getProperty("Shape");
            GeneralPath drawPath;

            if (shape.equals("Ellipse"))
                drawPath = new GeneralPath(new Ellipse2D.Double(5,
                    10, 5, 10));
            else if (shape.equals("Rounded Rectangle"))
                drawPath = new GeneralPath(
                    new RoundRectangle2D.Double(5, 10, 5, 10, 5, 5));
            else
                drawPath = new GeneralPath(new Rectangle2D.Double(5,
                    10, 5, 10));

            String name = prop.getProperty("LongName");
            String shortName = prop.getProperty("ShortName");

            Type newType = new Type(outlineColor, fillColor,
                fontColor, font, drawPath, shape, name, shortName);
            vertexTypes.add(newType);
        }
        return vertexTypes;
    }

    public static ArrayList getEdgeTypes()
    {
        ArrayList edgeTypes = new ArrayList();
        edgeTypes.add(new DefaultEdgeType1());
        edgeTypes.add(new DefaultEdgeType2());

        File[] typeFiles = new File(getPathName(Type.EDGE))
            .listFiles((FileFilter) new TypeFileFilter());

        for (int i = 0; i < typeFiles.length; i++)
        {
            // Workaround for 'CVS' files in eclipse
            if(typeFiles[i].getName().equalsIgnoreCase("CVS"))
                continue;
            Properties prop = getProperties(typeFiles[i]);

            String outlinecolor = prop.getProperty("OutlineColor");
            Color outlineColor = new Color(Integer
                .parseInt(outlinecolor));

            String shape = prop.getProperty("Shape");

            String name = prop.getProperty("LongName");

            String curve = prop.getProperty("Curvature");
            float curvature = Float.parseFloat(curve);

            Type newType = new Type(outlineColor, shape, name);
            edgeTypes.add(newType);
        }

        return edgeTypes;
    }

    public static ArrayList getDecorationTypes()
    {
        ArrayList decorationTypes = new ArrayList();
        decorationTypes.add(new DefaultDecorationType1());
        decorationTypes.add(new DefaultDecorationType2(""));

        File[] typeFiles = new File(getPathName(Type.DECORATION))
            .listFiles((FileFilter) new TypeFileFilter());

        for (int i = 0; i < typeFiles.length; i++)
        {
            // Workaround for 'CVS' files in eclipse
            if(typeFiles[i].getName().equalsIgnoreCase("CVS"))
                continue;
            Properties prop = getProperties(typeFiles[i]);

            String outlinecolor = prop.getProperty("OutlineColor");
            Color outlineColor = new Color(Integer
                .parseInt(outlinecolor));

            String fillcolor = prop.getProperty("FillColor");
            Color fillColor = new Color(Integer.parseInt(fillcolor));

            String fontcolor = prop.getProperty("FontColor");
            Color fontColor = new Color(Integer.parseInt(fontcolor));

            String fontName = prop.getProperty("FontName");
            String fontStyle = prop.getProperty("FontStyle");
            String fontSize = prop.getProperty("FontSize");
            Font font = new Font(fontName, Integer
                .parseInt(fontStyle), Integer.parseInt(fontSize));

            String shape = prop.getProperty("Shape");
            GeneralPath drawPath;

            if (shape.equals("Ellipse"))
                drawPath = new GeneralPath(new Ellipse2D.Double(5,
                    10, 5, 10));
            else if (shape.equals("Rounded Rectangle"))
                drawPath = new GeneralPath(
                    new RoundRectangle2D.Double(5, 10, 5, 10, 5, 5));
            else
                drawPath = new GeneralPath(new Rectangle2D.Double(5,
                    10, 5, 10));

            String name = prop.getProperty("LongName");
            String shortName = prop.getProperty("ShortName");

            Type newType = new Type(outlineColor, fillColor,
                fontColor, font, drawPath, shape, name, shortName);
            decorationTypes.add(newType);
        }
        return decorationTypes;
    }

    public static Type matchTypes(Type test, List typeList)
    {
        Type returnType = null;

        for (int i = 0; i < typeList.size(); i++)
        {
            Type temp = (Type) typeList.get(i);

            if (matchTypes(test, temp))
            {
                returnType = temp;
                break;
            }
        }
        return returnType;
    }

    public static boolean matchTypes(Type type1, Type type2)
    {
        if ((type1.getOutlineColor().getRGB() == type2
            .getOutlineColor().getRGB())
            && (type1.getFillColor().getRGB() == type2.getFillColor()
                .getRGB())
            && (type1.getFontColor().getRGB() == type2.getFontColor()
                .getRGB())
            && (type1.getFont().getName().equals(type2.getFont()
                .getName()))
            && (type1.getFont().getStyle() == type2.getFont()
                .getStyle())
            && (type1.getFont().getSize() == type2.getFont()
                .getSize())
            && (type1.getShape().equals(type2.getShape())))
            return true;

        return false;
    }

    private static Properties getProperties(File typeFile)
    {
        Properties prop = new Properties();
        try
        {
            BufferedReader in = new BufferedReader(new FileReader(
                typeFile));
            StringTokenizer toke;
            String s, key;

            while ((s = in.readLine()) != null)
            {
                if (!(s.startsWith("//") || (s.length() == 0)))
                {
                    toke = new StringTokenizer(s, "\t");
                    key = toke.nextToken();
                    if (toke.hasMoreTokens())
                        prop.put(key, toke.nextToken());
                    else
                        prop.put(key, new String(""));
                }
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return prop;
    }

    private static File getTempFile(int elementType, String shortName)
    {
        String fileName = (shortName.endsWith(".type")) ? shortName
            : (shortName + ".type");
        return new File(getPathName(elementType) + fileName);
    }

    private static String getPathName(int elementType)
    {
        String typeDir;

        if (elementType == Type.EDGE)
            typeDir = "edge";
        else if (elementType == Type.DECORATION)
            typeDir = "decoration";
        else
            typeDir = "vertex";
        return new String("etc" + File.separator + "types"
            + File.separator + typeDir + File.separator);
    }

}
