package vp.util;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.StringTokenizer;

import vp.util.texhyphj.Hyphenator;

public class StringBreaker
{
    private static StringBuffer sb;

    private static ArrayList strings;

    private static String splitters = " -),>/]";

    private static String preSplitters = "([";

    private static int idealRowLength;

    private static Hyphenator h;
    static
    {
        h = new Hyphenator();
        try
        {
            h.loadTable(new BufferedInputStream(new FileInputStream(
                "etc" + File.separator + "hyphen.tex")));
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    private static String word;

    private static String hword;

    private static int wordbegin;

    private StringBreaker()
    {
    }

    public static String breakStringEscape(String s)
    {
        String outString = "";
        String returnString = breakString(s, 22f);
        StringTokenizer stkr = new StringTokenizer(returnString, "\n");
        while (stkr.hasMoreTokens())
        {
            String current = stkr.nextToken();
            outString += current + "\\n";
        }
        outString = outString.substring(0, outString
            .lastIndexOf("\\n"));
        return outString;
    }

    public static String breakString(String s)
    {
        String returnString = breakString(s, 22f);
        return returnString;
    }

    public static String breakString(String s, float max)
    {
        sb = new StringBuffer(s);
        String returnString = "";

        int slength = s.length();
        float f = slength / max;
        int idealNumRows = (int) Math.ceil(f);
        idealNumRows = (idealNumRows != 0) ? idealNumRows : 1;
        f = slength / idealNumRows;
        idealRowLength = (int) Math.ceil(f);

        strings = new ArrayList();
        while (sb.length() > 0)
        {
            int breakspot = 0;
            while (breakspot <= 0)
            {
                // 0. If remaining string is less than max, just
                // return
                if (sb.length() <= max)
                {
                    breakspot = sb.length();
                    break;
                }
                // 1. Look for hard breaks between ideal length and
                // max length
                int irl = idealRowLength;

                while (irl <= max)
                {
                    if (findHardBreak(irl))
                    {
                        breakspot = irl + 1;
                        break;
                    }
                    irl++;
                }

                if (breakspot > 0)
                    break;
                // 2. Look for soft breaks between ideal length and
                // max length
                // irl = idealRowLength;
                irl = idealRowLength;
                findWord(irl);
                while (irl <= max)
                {
                    if (findSoftBreak(irl))
                    {
                        breakspot = irl;
                        break;
                    }
                    irl++;
                }

                if (breakspot > 0)
                    break;
                // 3. going back from ideal length look for hard or
                // soft breaks
                irl = idealRowLength - 1;
                while (irl > 0)
                {
                    if (findHardBreak(irl))
                    {
                        breakspot = irl + 1;
                        break;
                    }
                    else if (findSoftBreak(irl))
                    {
                        breakspot = irl;
                        break;
                    }
                    irl--;
                }
            }
            strings.add(sb.substring(0, breakspot));

            // split off if not at the end
            sb.delete(0, breakspot);
        }

        for (int i = 0; i < strings.size(); i++)
        {
            returnString += strings.get(i) + "\n";
        }

        if (returnString.indexOf("\n") != -1)
            returnString = returnString.substring(0, returnString
                .lastIndexOf("\n"));
        return returnString;
    }

    private static boolean findHardBreak(int i)
    {
        if (splitters.indexOf(sb.charAt(i)) != -1)
        {
            return true;
        }
        if (i < sb.length() - 1)
        {
            if (preSplitters.indexOf(sb.charAt(i + 1)) != -1)
                return true;
        }
        return false;
    }

    private static boolean findSoftBreak(int i)
    {
        int sw = 0;
        int hw = 0;
        int wordindex = i - wordbegin;

        while (sw < wordindex)
        {
            if (hword.charAt(hw) != '\u00ad')
                sw++;
            hw++;
        }

        if (hw < hword.length())
        {
            if (hword.charAt(hw) == '\u00ad')
                return true;
        }

        return false;
    }

    private static void findWord(int i)
    {
        wordbegin = 0;
        for (int j = 0; j <= i; j++)
        {
            if (splitters.indexOf(sb.charAt(j)) != -1)
                wordbegin = j + 1;
            else if (j < i)
            {
                if (preSplitters.indexOf(sb.charAt(j + 1)) != -1)
                    wordbegin = j + 1;
            }
        }

        int wordend = -1;
        int endchar = i;
        while (wordend < 0 && endchar < (sb.length() - 1))
        {
            if (splitters.indexOf(sb.charAt(endchar)) != -1)
                wordend = endchar;
            else if (preSplitters.indexOf(sb.charAt(endchar + 1)) != -1)
                wordend = endchar;
            endchar++;
        }
        if (wordend < 0)
            wordend = sb.length();

        word = sb.substring(wordbegin, wordend);
        hword = h.hyphenate(word);
    }

    public static String breakStringForLabel(String s)
    {
        String returnString = breakStringForLabel(s, 22f);
        return returnString;
    }

    public static String breakStringForLabel(String s, float max)
    {
        if (s.length() <= max)
            return s;
        String broken = breakString(s, max);
        StringTokenizer tkzr = new StringTokenizer(broken, "\n");
        String returnString = "<html>";
        while (tkzr.hasMoreTokens())
            returnString += tkzr.nextToken() + "<br>";
        returnString = returnString.substring(0, returnString
            .lastIndexOf("<br>"));
        return returnString;
    }

    public static void main(String[] args)
    {
        long start = System.currentTimeMillis();
        String test = "N-Acetylneuraminyl-D-galactosyl-N-acetyl-D-galactosaminyl-(N- acetylneuraminyl)-D-galactosyl-D-glucosylceramide";
        String r = breakString(test);
        System.out.println("Return String: ");
        System.out.println(r);
        long end = System.currentTimeMillis();
        System.out.println();
        System.out.println("Done in " + (end - start) + " millis.");
    }
}
