package vp.util.database;

import vp.graph.DataClass;
import vp.graph.Experiment;
import vp.graph.GraphDataClass;
import vp.graph.PathwayDecoration;
import vp.graph.PathwayEdge;
import vp.graph.PathwayGraph;
import vp.graph.PathwayVertex;
import vp.util.SaveType;
import vp.visual.type.Type;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.awt.Color;
import java.awt.Font;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

public class DBQuery
{

    private DBQuery()
    {
    }

    /**
     * Finds all compound names that match the given search string.
     * 
     * @param searchString
     *            The string with which to match compounds from the
     *            database
     * @return An alphabetized list of all compounds names containing
     *         the search string
     */
    public static ArrayList findCompound(String searchString,
        PathwayGraph pg)
    {
        long start = System.currentTimeMillis();
        Statement stmt = DBAccess.getStatement();
        ArrayList compounds = new ArrayList();
        String sql = "SELECT id, name FROM hmr_compound WHERE upper(name) LIKE '%"
            + searchString.toUpperCase()
            + "%' AND id LIKE 'C%' ORDER BY name";
        Map results = new TreeMap();
        try
        {
            ResultSet rs = stmt.executeQuery(sql);

            while (rs.next())
            {
                String id = rs.getString("id");
                String name = rs.getString("name");
                results.put(name, id);

            }
        }

        catch (Exception e)
        {
            e.printStackTrace();
        }

        int i = 0;
        Iterator iter = results.entrySet().iterator();
        while (iter.hasNext())
        {
            Map.Entry entry = (Map.Entry) iter.next();
            String name = (String) entry.getKey();
            String id = (String) entry.getValue();
            DataClass compound = new DataClass(id, pg, false);
            compound.setClassID(1);
            compound.setClassName("Compound");
            compound.setObjectName(name);
            compound.setTableName("hmr_compound");
            compound.setObjectType(DBConstants.VERTEX);

            compound.put("id", id);
            compound.put("Name", name);
            compounds.add(compound);
            i++;
        }
        return compounds;
    }

    public static ArrayList findExperiment(String searchString)
    {
        Statement stmt = DBAccess.getStatement();
        ArrayList experiments = new ArrayList();
        String sql = "SELECT id, title, id_type, upload_date, public_flag, "
            + "description FROM hmr_external_header "
            + "WHERE upper(title) LIKE '%"
            + searchString.toUpperCase() + "%' ORDER BY title";
        try
        {
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next())
            {
                experiments.add(new Experiment(rs.getInt("id"), rs
                    .getString("title"), rs.getString("id_type"), rs
                    .getString("upload_date"), rs
                    .getString("public_flag"), rs
                    .getString("description")));
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return experiments;
    }

    public static ArrayList findEnzyme(String searchString,
        PathwayGraph pg)
    {
        Statement stmt = DBAccess.getStatement();
        ArrayList enzymes = new ArrayList();
        String sql;
        // if (searchString.indexOf(".") != -1)
        sql = "SELECT enzyme_id, ec, name FROM hmr_enzyme_clone WHERE ec LIKE '%"
            + searchString
            + "%' OR upper(name) LIKE '%"
            + searchString.toUpperCase() + "%'ORDER BY ec";
        // else
        // sql =
        // "SELECT ec_id, enzyme_name FROM ec_enzyme WHERE
        // upper(enzyme_name) LIKE '%"
        // + searchString.toUpperCase()
        // + "%' ORDER BY ec_id";
        try
        {
            ResultSet rs = stmt.executeQuery(sql);

            while (rs.next())
            {
                String id = rs.getString("enzyme_id");
                String ec = rs.getString("ec");
                String name = rs.getString("name");
                DataClass enzyme = new DataClass(id, pg);
                enzyme.setClassName("Enzyme");
                enzyme.put("id", id);
                enzyme.put("ECNumber", ec);
                enzyme.put("Name", name);
                PathwayDecoration pd = new PathwayDecoration(pg, -3,
                    enzyme, null, 1);
                enzymes.add(pd);
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return enzymes;
    }

    /**
     * Returns the unique identifier for the compound with the
     * specified name.
     * 
     * @see #getCompoundComment(String selectedCompound)
     */
    public static String getCompoundID(String selectedCompound)
    {
        if (selectedCompound.indexOf("'") != -1)
        {
            StringTokenizer toke = new StringTokenizer(
                selectedCompound, "'");
            selectedCompound = "";
            while (toke.hasMoreTokens())
                selectedCompound += toke.nextToken() + "''";
            selectedCompound = selectedCompound.substring(0,
                selectedCompound.length() - 2);
        }

        Statement stmt = DBAccess.getStatement();
        String selectedCompoundID = null;
        try
        {
            String sql = "SELECT id FROM hmr_compound WHERE name = '"
                + selectedCompound + "'";
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next())
                selectedCompoundID = rs.getString("id");
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        return selectedCompoundID;
    }

    public static String getEnzymeName(String ecID)
    {
        Statement stmt = DBAccess.getStatement();
        String enzymeName = null;
        try
        {
            String sql = "SELECT name FROM hmr_enzyme WHERE ec = '"
                + ecID + "'";
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next())
                enzymeName = rs.getString("name");
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return enzymeName;
    }

    public static HashMap getExpressions(Experiment experiment,
        GraphDataClass pathway)
    {
        Statement stmt = DBAccess.getStatement();
        HashMap expMap = new HashMap();
        String pathID = (String) pathway.get("id");
        pathID = pathID.substring(3, pathID.length());
        int pathNo = Integer.parseInt(pathID);

        String sql = "select e.ec, avg(d.value) value from hmr_pathway_gene p, "
            + "hmr_gene_enzyme ge, hmr_enzyme e, hmr_external_detail d "
            + "where p.pathway_id = '"
            + pathway.get("id")
            + "' and d.header_id (+) = '"
            + experiment.getExpID()
            + "' and p.gene_id = ge.gene_id and ge.enzyme_id = e.id and "
            + "p.gene_id = d.gene_id (+) group by e.ec";

        try
        {
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next())
            {
                String ec = rs.getString("ec");
                String exp = rs.getString("value");
                if (exp != null)
                {
                    expMap.put(ec, Double.valueOf(exp));
                }
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return expMap;
    }

    /**
     * @param selectedExperiment1
     * @param pathway
     * @param oldEC
     * @return
     */
    public static HashMap getExpByGene(Experiment experiment,
        GraphDataClass pathway, String oldEC, PathwayGraph pg)
    {
        Statement stmt = DBAccess.getStatement();
        HashMap expMap = new HashMap();

        // StringBuffer buff = new StringBuffer(1024);
        // buff.append()
        String sql = "select g.symbol, g.id, avg(d.value) value from hmr_pathway_gene pg, "
            + "hmr_enzyme e, hmr_gene_enzyme ge, hmr_gene g, "
            + "hmr_external_detail d where e.ec = '"
            + oldEC
            + "' and e.id = ge.enzyme_id and ge.gene_id = g.id and g.id ="
            + " pg.gene_id and pg.pathway_id = '"
            + pathway.get("id")
            + "' and d.header_id(+) = '"
            + experiment.getExpID()
            + "' and ge.gene_id = d.gene_id(+) group by g.symbol, g.id";
        try
        {
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next())
            {
                String symbol = rs.getString("symbol");
                String value = rs.getString("value");
                String id = rs.getString("id");
                id = "GE" + id;
                value = (value != null) ? value : "NA";
                DataClass gene = new DataClass(id, pg, true);
                gene.put("Name", symbol);
                gene.put("Value", value);
                gene.put("id", id);
                if (value != null)
                {
                    expMap.put(gene, value);
                }
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return expMap;
    }

    public static String getGeneValue(Experiment experiment,
        GraphDataClass pathway, String symbol, PathwayGraph pg)
    {
        String value = null;

        Statement stmt = DBAccess.getStatement();

        String sql = "select avg(d.value) value from hmr_pathway_gene pg, "
            + "hmr_gene g, "
            + "hmr_external_detail d where g.symbol = '"
            + symbol
            + "' and g.id ="
            + " pg.gene_id and pg.pathway_id = '"
            + pathway.get("id")
            + "' and d.header_id(+) = '"
            + experiment.getExpID()
            + "' and g.id = d.gene_id(+) group by g.symbol, g.id";
        try
        {
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next())
            {
                value = rs.getString("value");
                value = (value != null) ? value : "NA";

            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return value;
    }

    public static ArrayList findPathway(String searchString)
    {
        Statement stmt = DBAccess.getStatement();
        ArrayList pathways = new ArrayList();
        String sql = "SELECT id, name FROM hmr_pathway WHERE upper(name) LIKE '%"
            + searchString.toUpperCase() + "%' ORDER BY name";

        try
        {
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next())
            {
                GraphDataClass newPath = new GraphDataClass(rs
                    .getString("id"));
                newPath.put("Name", rs.getString("name"));
                pathways.add(newPath);
            }

        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return pathways;
    }

    public static ArrayList findDBPathway(String searchString,
        String organism)
    {
        Statement stmt = DBAccess.getStatement();
        ArrayList pathways = new ArrayList();
        String sql = "SELECT id, name FROM hmr_pathway WHERE organism = '"
            + organism
            + "' AND upper(name) LIKE '%"
            + searchString.toUpperCase() + "%' ORDER BY name";
        Map results = new TreeMap();

        try
        {
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next())
            {
                String id = rs.getString("id");
                String name = rs.getString("name");
                results.put(name, id);

            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        int i = 0;
        Iterator iter = results.entrySet().iterator();
        while (iter.hasNext())
        {
            Map.Entry entry = (Map.Entry) iter.next();
            String name = (String) entry.getKey();
            String id = (String) entry.getValue();
            GraphDataClass pathway = new GraphDataClass(id);
            pathway.put("id", id);
            pathway.put("Name", name);
            pathway.put("Organism", organism);
            pathways.add(pathway);
            i++;
        }
        return pathways;
    }

    public static ArrayList getHMRReactions(String searchString,
        PathwayGraph pg)
    {
        Statement stmt = DBAccess.getStatement();
        ArrayList reactions = new ArrayList();
        String sql = "select distinct pr.reaction_id,  rc.compound_id, c.name c_name, "
            + "rc.type, ec.enzyme_id, ec.ec, ec.name e_name, gc.symbol, gc.gene_id g_id "
            + "from hmr_pathway_reaction pr, "
            + "hmr_reaction_compound rc, hmr_compound c, hmr_reaction_enzyme_clone rec, "
            + "hmr_enzyme_clone ec, hmr_gene_enzyme_clone gec, hmr_gene_clone gc "
            + "where pr.pathway_id = '"
            + searchString
            + "' and pr.reaction_id = rc.reaction_id and rc.compound_id = "
            + "c.id and  rc.display_flag = 'T' and pr.reaction_id = "
            + "rec.reaction_id and rec.enzyme_id = ec.enzyme_id and ec.enzyme_id = gec.enzyme_id "
            + "and gec.gene_id = gc.gene_id and exists "
            + "(select 'x' from hmr_pathway_gene_clone pgc "
            + "where pgc.pathway_id = pr.pathway_id and pgc.gene_id = "
            + "gec.gene_id and gec.enzyme_id = rec.enzyme_id) order by "
            + "pr.reaction_id, rc.type";

        // String sql = "SELECT distinct pr.reaction_id,
        // rc.compound_id, c.name c_name, rc.type, e.ec, e.name e_name
        // FROM hmr_pathway_reaction pr, hmr_reaction_compound rc,
        // hmr_compound c, hmr_reaction_enzyme re, hmr_enzyme e WHERE
        // pr.pathway_id = '" + searchString + "' AND pr.reaction_id =
        // rc.reaction_id AND rc.compound_id = c.id AND c.display_flag
        // = 'T' AND pr.reaction_id = re.reaction_id AND re.enzyme_id
        // = e.id ORDER BY pr.reaction_id, rc.type";

        try
        {
            ResultSet rs = stmt.executeQuery(sql);
            ArrayList reactionIDs = new ArrayList();
            ArrayList compoundIDs = new ArrayList();
            ArrayList compoundNames = new ArrayList();
            ArrayList types = new ArrayList();
            ArrayList enzymeIDs = new ArrayList();
            ArrayList ecNumbers = new ArrayList();
            ArrayList enzymeNames = new ArrayList();
            ArrayList geneNames = new ArrayList();
            ArrayList geneIDs = new ArrayList();

            while (rs.next())
            {

                String currRid = rs.getString("reaction_id");

                String reactionID = rs.getString("reaction_id");
                reactionIDs.add(reactionID);
                String compoundID = rs.getString("compound_id");
                compoundIDs.add(compoundID);
                String compoundName = rs.getString("c_name");
                compoundNames.add(compoundName);
                String type = rs.getString("type");
                types.add(type);
                String enzymeID = rs.getString("enzyme_id");
                enzymeIDs.add(enzymeID);
                String ecNumber = rs.getString("ec");
                ecNumbers.add(ecNumber);
                String enzymeName = rs.getString("e_name");
                enzymeNames.add(enzymeName);
                String geneName = rs.getString("symbol");
                geneNames.add(geneName);
                String geneID = rs.getString("g_id");
                geneIDs.add(geneID);

            }

            DataClass curReaction = null;
            String rid = "";
            Set sources = new HashSet();
            Set sinks = new HashSet();
            Set enzymes = new HashSet();
            Set genes = new HashSet();
            Map compoundMap = new TreeMap();
            Map enzymeMap = new TreeMap();
            Map geneMap = new TreeMap();

            for (int i = 0; i < reactionIDs.size(); i++)
            {
                String currRid = (String) reactionIDs.get(i);
                if (!currRid.equals(rid))
                {
                    if (curReaction != null)
                    {
                        if (sources.size() > 0)
                            curReaction.put("sources", sources);
                        if (sinks.size() > 0)
                            curReaction.put("sinks", sinks);
                        if (enzymes.size() > 0)
                            curReaction.put("enzymes", enzymes);
                        if (genes.size() > 0)
                            curReaction.put("genes", genes);
                        reactions.add(curReaction);

                    }
                    curReaction = new DataClass(currRid, pg);
                    curReaction.put("id", currRid);
                    rid = currRid;

                    sources = new HashSet();
                    sinks = new HashSet();
                    enzymes = new HashSet();
                    genes = new HashSet();
                }

                String compoundID = (String) compoundIDs.get(i);
                if (!compoundMap.containsKey(compoundID))
                {
                    DataClass c = new DataClass(compoundID, pg, true);

                    c.put("id", compoundIDs.get(i));
                    c.put("Name", compoundNames.get(i));

                    compoundMap.put(compoundID, c);
                }

                if (types.get(i).equals("S"))
                {
                    sources.add(compoundMap.get(compoundID));
                }
                else if (types.get(i).equals("P"))
                {
                    sinks.add(compoundMap.get(compoundID));
                }

                String enzymeID = (String) enzymeIDs.get(i);
                if (!enzymeMap.containsKey(enzymeID))
                {
                    DataClass enzyme = new DataClass(enzymeID, pg,
                        true);
                    enzyme.put("id", enzymeIDs.get(i));
                    enzyme.put("ECNumber", ecNumbers.get(i));
                    enzyme.put("Name", enzymeNames.get(i));

                    enzymeMap.put(enzymeID, enzyme);
                }

                enzymes.add(enzymeMap.get(enzymeID));

                String geneID = (String) geneIDs.get(i);
                if (!geneMap.containsKey(geneID))
                {
                    DataClass gene = new DataClass((String) geneIDs
                        .get(i), pg, true);
                    gene.put("id", geneIDs.get(i));
                    gene.put("Name", geneNames.get(i));

                    geneMap.put(geneID, gene);
                }

                genes.add(geneMap.get(geneID));
            }
            // The last one
            if (curReaction != null)
            {

                if (sources.size() > 0)
                    curReaction.put("sources", sources);
                if (sinks.size() > 0)
                    curReaction.put("sinks", sinks);
                if (enzymes.size() > 0)
                    curReaction.put("enzymes", enzymes);
                if (genes.size() > 0)
                    curReaction.put("genes", genes);
                reactions.add(curReaction);
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return reactions;
    }

    public static int getUniqueID()
    {
        Statement stmt = DBAccess.getStatement();
        int uniqueID = 0;
        try
        {
            String sql = "SELECT pw_seq.nextval FROM dual";
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next())
                uniqueID = rs.getInt("nextval");
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return uniqueID;
    }

    /**
     * @param i
     * @return
     */
    public static Type getDefaultType(int classID)
    {
        Type tempType;
        boolean isEdge = false;
        String shape = "";
        int outlineColor = 0;
        int fillColor = 0;
        String fontFamily = "";
        int fontStyle = 0;
        int fontSize = 0;
        int fontColor = 0;

        Statement stmt = DBAccess.getStatement();
        String sql = "SELECT is_edge FROM drawing_classes WHERE class = '"
            + classID + "'";
        ResultSet rs = null;
        try
        {
            rs = stmt.executeQuery(sql);
        }
        catch (SQLException e1)
        {
            e1.printStackTrace();
        }
        try
        {
            while (rs.next())
            {
                isEdge = rs.getInt("is_edge") > 0;
            }
        }
        catch (SQLException e2)
        {
            e2.printStackTrace();
        }

        try
        {
            sql = "SELECT shape, outline_color, fill_color, font_family, "
                + "font_style, font_size, font_color FROM pathway_appearance "
                + "WHERE pathway_id = '_DEFPATH' AND object_class = '"
                + classID + "'";
            rs = stmt.executeQuery(sql);
            while (rs.next())
            {
                shape = rs.getString("shape");
                outlineColor = rs.getInt("outline_color");
                fillColor = rs.getInt("fill_color");
                fontFamily = rs.getString("font_family");
                fontStyle = rs.getInt("font_style");
                fontSize = rs.getInt("font_size");
                fontColor = rs.getInt("font_color");
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        if (isEdge)
        {
            tempType = new Type(new Color(outlineColor), shape, "");
        }
        else
        {
            GeneralPath drawPath;
            if (shape.equals("Ellipse"))
                drawPath = new GeneralPath(new Ellipse2D.Double(5, 5,
                    10, 10));
            else if (shape.equals("Round Rectangle"))
                drawPath = new GeneralPath(
                    new RoundRectangle2D.Double(5, 5, 10, 10, 2.5,
                        2.5));
            else
                drawPath = new GeneralPath(new Rectangle2D.Double(5,
                    5, 10, 10));

            tempType = new Type(new Color(outlineColor), new Color(
                fillColor), new Color(fontColor), new Font(
                fontFamily, fontStyle, fontSize), drawPath, shape,
                "", "Dflt");
        }
        return tempType;
    }

    /**
     * @param i
     * @return
     */
    public static Type getDefaultTypeForPathway(int classID,
        String pathwayID)
    {
        Type tempType;
        boolean isEdge = false;
        boolean hasType = false;
        String shape = "";
        int outlineColor = 0;
        int fillColor = 0;
        String fontFamily = "";
        int fontStyle = 0;
        int fontSize = 0;
        int fontColor = 0;

        Statement stmt = DBAccess.getStatement();
        String sql = "SELECT is_edge FROM drawing_classes WHERE class = '"
            + classID + "'";
        ResultSet rs = null;
        try
        {
            rs = stmt.executeQuery(sql);
        }
        catch (SQLException e1)
        {
            e1.printStackTrace();
        }
        try
        {
            while (rs.next())
            {
                isEdge = rs.getInt("is_edge") > 0;
            }
        }
        catch (SQLException e2)
        {
            e2.printStackTrace();
        }

        try
        {
            sql = "SELECT shape, outline_color, fill_color, font_family, "
                + "font_style, font_size, font_color FROM pathway_appearance "
                + "WHERE pathway_id = '"
                + pathwayID
                + "' AND object_class = '" + classID + "'";
            rs = stmt.executeQuery(sql);
            while (rs.next())
            {
                hasType = true;
                shape = rs.getString("shape");
                outlineColor = rs.getInt("outline_color");
                fillColor = rs.getInt("fill_color");
                fontFamily = rs.getString("font_family");
                fontStyle = rs.getInt("font_style");
                fontSize = rs.getInt("font_size");
                fontColor = rs.getInt("font_color");
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        if (hasType)
        {
            if (isEdge)
            {
                tempType = new Type(new Color(outlineColor), shape,
                    "");
            }
            else
            {
                GeneralPath drawPath;
                if (shape.equals("Ellipse"))
                    drawPath = new GeneralPath(new Ellipse2D.Double(
                        5, 5, 10, 10));
                else if (shape.equals("Round Rectangle"))
                    drawPath = new GeneralPath(
                        new RoundRectangle2D.Double(5, 5, 10, 10,
                            2.5, 2.5));
                else
                    drawPath = new GeneralPath(
                        new Rectangle2D.Double(5, 5, 10, 10));

                tempType = new Type(new Color(outlineColor),
                    new Color(fillColor), new Color(fontColor),
                    new Font(fontFamily, fontStyle, fontSize),
                    drawPath, shape, "", "Path");
            }
            return tempType;
        }
        return null;
    }

    public static Type getTypeForObject(int classID,
        String pathwayID, String objectID)
    {
        Type tempType;
        boolean hasType = false;
        boolean isEdge = false;
        String shape = "";
        int outlineColor = 0;
        int fillColor = 0;
        String fontFamily = "";
        int fontStyle = 0;
        int fontSize = 0;
        int fontColor = 0;

        Statement stmt = DBAccess.getStatement();
        String sql = "SELECT is_edge FROM drawing_classes WHERE class = '"
            + classID + "'";
        ResultSet rs = null;
        try
        {
            rs = stmt.executeQuery(sql);
        }
        catch (SQLException e1)
        {
            e1.printStackTrace();
        }
        try
        {
            while (rs.next())
            {
                isEdge = rs.getInt("is_edge") > 0;
            }
        }
        catch (SQLException e2)
        {
            e2.printStackTrace();
        }

        try
        {
            sql = "SELECT shape, outline_color, fill_color, font_family, "
                + "font_style, font_size, font_color FROM pathway_object_appearance "
                + "WHERE pathway_id = '"
                + pathwayID
                + "' AND object_class = '"
                + classID
                + "' AND object_id = '" + objectID + "'";
            rs = stmt.executeQuery(sql);
            while (rs.next())
            {
                hasType = true;
                shape = rs.getString("shape");
                outlineColor = rs.getInt("outline_color");
                fillColor = rs.getInt("fill_color");
                fontFamily = rs.getString("font_family");
                fontStyle = rs.getInt("font_style");
                fontSize = rs.getInt("font_size");
                fontColor = rs.getInt("font_color");
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        if (hasType)
        {
            if (isEdge)
            {
                tempType = new Type(new Color(outlineColor), shape,
                    "");
            }
            else
            {
                GeneralPath drawPath;
                if (shape.equals("Ellipse"))
                    drawPath = new GeneralPath(new Ellipse2D.Double(
                        5, 5, 10, 10));
                else if (shape.equals("Round Rectangle"))
                    drawPath = new GeneralPath(
                        new RoundRectangle2D.Double(5, 5, 10, 10,
                            2.5, 2.5));
                else
                    drawPath = new GeneralPath(
                        new Rectangle2D.Double(5, 5, 10, 10));

                tempType = new Type(new Color(outlineColor),
                    new Color(fillColor), new Color(fontColor),
                    new Font(fontFamily, fontStyle, fontSize),
                    drawPath, shape, "", "Obj");
            }
            return tempType;
        }
        return null;
    }

    /**
     * @param vGraph
     */
    public static void saveGraphToDB(PathwayGraph pGraph)
    {
        ListIterator li = pGraph.getVertexIterator();
        while (li.hasNext())
        {
            PathwayVertex current = (PathwayVertex) li.next();
            Type currType = current.getType();
            if ((SaveType.matchTypes(currType, pGraph.getParent()
                .getDefaultType(1)))
                || (SaveType.matchTypes(currType, pGraph
                    .getDefaultType(1))))
                continue;
            else
            {
                if (SaveType.matchTypes(currType, DBQuery
                    .getTypeForObject(0,
                        (String) ((PathwayGraph) current.getGraph())
                            .getGraphDataClass().get("id"),
                        (String) current.getDataClass().get("id"))))
                    continue;
                else
                    writeVertex(current);
            }
        }
        li = pGraph.getEdgeIterator();
        while (li.hasNext())
        {
            PathwayEdge current = (PathwayEdge) li.next();
            Type currType = current.getType();
            if ((SaveType.matchTypes(currType, pGraph.getParent()
                .getDefaultType(4)))
                || (SaveType.matchTypes(currType, pGraph
                    .getDefaultType(4))))
                continue;
            else
            {
                if ((DBQuery.getTypeForObject(4,
                    (String) ((PathwayGraph) current.getGraph())
                        .getGraphDataClass().get("id"),
                    (String) current.getDataClass().get("id"))) != null
                    && (SaveType.matchTypes(currType, DBQuery
                        .getTypeForObject(4,
                            (String) ((PathwayGraph) current
                                .getGraph()).getGraphDataClass().get(
                                "id"), (String) current
                                .getDataClass().get("id")))))
                    continue;
                else
                    writeEdge(current);
            }

            ListIterator decIter = current.getDecorationIterator();
            while (decIter.hasNext())
            {
                PathwayDecoration curDec = (PathwayDecoration) decIter
                    .next();
                Type localTypeToCompare = null;
                Type globalTypeToCompare = null;

                if (curDec.getDataClass().getClassName().equals(
                    "Enzyme"))
                {
                    globalTypeToCompare = pGraph.getParent()
                        .getDefaultType(2);
                    localTypeToCompare = pGraph.getDefaultType(2);
                }

                else if (curDec.getDataClass().getClassName().equals(
                    "PWGene"))
                {
                    globalTypeToCompare = pGraph.getParent()
                        .getDefaultType(3);
                    localTypeToCompare = pGraph.getDefaultType(3);
                }

                else
                {
                    globalTypeToCompare = pGraph.getParent()
                        .getDefaultType(5);
                    localTypeToCompare = pGraph.getDefaultType(5);
                }
                Type currDecType = curDec.getType();
                if ((SaveType.matchTypes(currDecType,
                    globalTypeToCompare))
                    || (SaveType.matchTypes(currDecType,
                        localTypeToCompare)))
                {
                    continue;
                }
                else
                {
                    if ((DBQuery
                        .getTypeForObject(
                            2,
                            (String) ((PathwayGraph) ((PathwayEdge) curDec
                                .getParent()).getGraph())
                                .getGraphDataClass().get("id"),
                            (String) curDec.getDataClass().get("id")) != null)
                        && (SaveType
                            .matchTypes(
                                currType,
                                DBQuery
                                    .getTypeForObject(
                                        2,
                                        (String) ((PathwayGraph) ((PathwayEdge) curDec
                                            .getParent()).getGraph())
                                            .getGraphDataClass().get(
                                                "id"),
                                        (String) curDec
                                            .getDataClass().get("id")))))
                        continue;
                    else
                        writeDec(curDec);
                }
            }
        }
    }

    /**
     * @param current
     */
    private static void writeDec(PathwayDecoration current)
    {
        Statement stmt = DBAccess.getStatement();
        String sql = "INSERT into pathway_object_appearance VALUES "
            + "('"
            + ((PathwayGraph) ((PathwayEdge) current.getParent())
                .getGraph()).getGraphDataClass().get("id")
            + "', '2', '" + (String) current.getDataClass().get("id")
            + "', '" + current.getShape() + "', '"
            + current.getOutlineColor().getRGB() + "', '"
            + current.getFillColor().getRGB() + "', '"
            + current.getFont().getFontName() + "', '"
            + current.getFont().getStyle() + "', '"
            + current.getFont().getSize() + "', '"
            + current.getFontColor().getRGB() + "')";
        try
        {
            stmt.executeUpdate(sql);
        }
        catch (SQLException e)
        {
            e.printStackTrace();
        }
        DBAccess.commit();
    }

    /**
     * @param current
     */
    private static void writeEdge(PathwayEdge current)
    {
        Statement stmt = DBAccess.getStatement();
        String sql = "INSERT into pathway_object_appearance VALUES "
            + "('"
            + ((PathwayGraph) current.getGraph()).getGraphDataClass()
                .get("id") + "', '4', '" + current.getID() + "', '"
            + current.getShape() + "', '"
            + current.getOutlineColor().getRGB()
            + "', '', '', '', '', '')";
        try
        {
            stmt.executeUpdate(sql);
        }
        catch (SQLException e)
        {
            e.printStackTrace();
        }
        DBAccess.commit();
    }

    /**
     * @param current
     */
    private static void writeVertex(PathwayVertex current)
    {
        Statement stmt = DBAccess.getStatement();
        // Make sure no object type exists for this vertex
        String sql = "SELECT object_id from pathway_object_appearance WHERE pathway_id = '"
            + ((PathwayGraph) current.getGraph()).getGraphDataClass()
                .get("id")
            + "' AND object_id = '"
            + (String) current.getDataClass().get("id") + "'";
        ResultSet rs = null;
        try
        {
            rs = stmt.executeQuery(sql);
        }
        catch (SQLException e1)
        {
            e1.printStackTrace();
        }
        try
        {
            while (rs.next())
            {
                String sql2 = "DELETE from pathway_object_appearance WHERE pathway_id = '"
                    + ((PathwayGraph) current.getGraph())
                        .getGraphDataClass().get("id")
                    + "' AND object_id = '"
                    + (String) current.getDataClass().get("id") + "'";
            }
        }
        catch (SQLException e2)
        {
            e2.printStackTrace();
        }
        sql = "INSERT into pathway_object_appearance VALUES "
            + "('"
            + ((PathwayGraph) current.getGraph()).getGraphDataClass()
                .get("id") + "', '1', '"
            + (String) current.getDataClass().get("id") + "', '"
            + current.getShape() + "', '"
            + current.getOutlineColor().getRGB() + "', '"
            + current.getFillColor().getRGB() + "', '"
            + current.getFont().getFontName() + "', '"
            + current.getFont().getStyle() + "', '"
            + current.getFont().getSize() + "', '"
            + current.getFontColor().getRGB() + "')";
        try
        {
            stmt.executeUpdate(sql);
        }
        catch (SQLException e)
        {
            e.printStackTrace();
        }
        DBAccess.commit();
    }

    /**
     * @param string
     * @return
     */
    public static Map getLocalTypes(String pathID)
    {
        Map localTypes = new Hashtable();
        
        Statement stmt = DBAccess.getStatement();
        String sql = "SELECT * FROM pathway_appearance WHERE pathway_id = '"
            + pathID + "'";
        ResultSet rs;
        try
        {
            rs = stmt.executeQuery(sql);
            while (rs.next())
            {
                int classID = rs.getInt("object_class");
                String shape = rs.getString("shape");
                int outlineColor = rs.getInt("outline_color");
                int fillColor = rs.getInt("fill_color");
                String fontFamily = rs.getString("font_family");
                int fontStyle = rs.getInt("font_style");
                int fontSize = rs.getInt("font_size");
                int fontColor = rs.getInt("font_color");
                GeneralPath drawPath = null;
                if (shape.equals("Ellipse"))
                    drawPath = new GeneralPath(new Ellipse2D.Double(
                        5, 5, 10, 10));
                else if (shape.equals("Round Rectangle"))
                    drawPath = new GeneralPath(
                        new RoundRectangle2D.Double(5, 5, 10, 10,
                            2.5, 2.5));
                else if (shape.equals("Rectangle"))
                    drawPath = new GeneralPath(
                        new Rectangle2D.Double(5, 5, 10, 10));

                localTypes.put(new Integer(classID), new Type(
                    new Color(outlineColor), new Color(fillColor),
                    new Color(fontColor), new Font(fontFamily,
                        fontStyle, fontSize), drawPath, shape, "",
                    "Path"));
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return localTypes;
    }

    /**
     * @param string
     * @param i
     * @return
     */
    public static List getClasses(String pathName, int elementType)
    {
        List classes = new ArrayList();
        Statement stmt = DBAccess.getStatement();
        String sql = "SELECT class FROM drawing_classes WHERE is_edge = '"
            + elementType
            // + "' AND pathway_id = '"
            // + pathName
            + "'";
        try
        {
            ResultSet rs = stmt.executeQuery(sql);

            while (rs.next())
            {
                classes.add(new Integer(rs.getInt("class")));
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return (classes.size() > 0) ? classes : null;
    }

    /**
     * @param string
     * @param classID
     * @return
     */
    public static String getTableName(String pathID, int classID)
    {
        String tableName = null;
        Statement stmt = DBAccess.getStatement();
        String sql = "SELECT table_name FROM drawing_classes WHERE class = '"
            + classID
            // + " AND pathway_id = '"
            // + pathID
            + "'";

        try
        {
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next())
            {
                tableName = rs.getString("table_name");
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return tableName;
    }

    /**
     * @param string
     * @param classID
     * @return
     */
    public static String getClassName(String pathID, int classID)
    {
        String className = null;
        Statement stmt = DBAccess.getStatement();
        String sql = "SELECT class_name FROM drawing_classes WHERE class = '"
            + classID
            // + " AND pathway_id = '"
            // + pathID
            + "'";

        try
        {
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next())
            {
                className = rs.getString("class_name");
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return className;
    }

    /**
     * @param string
     * @param classID
     * @return
     */
    public static Type getDefaultType(String pathID, int classID)
    {
        Type type = null;
        String typeName = (pathID.equals("_DEFPATH")) ? "Global default"
            : "Local default";
        Statement stmt = DBAccess.getStatement();
        String sql = "SELECT shape, outline_color, fill_color, font_family, "
            + "font_style, font_size, font_color FROM pathway_appearance "
            + "WHERE pathway_id = '"
            + pathID
            + "' AND object_class = '" + classID + "'";
        try
        {
            ResultSet rs = stmt.executeQuery(sql);
            while (rs.next())
            {
                String shape = rs.getString("shape");
                int outlineColor = rs.getInt("outline_color");
                int fillColor = rs.getInt("fill_color");
                String fontFamily = rs.getString("font_family");
                int fontStyle = rs.getInt("font_style");
                int fontSize = rs.getInt("font_size");
                int fontColor = rs.getInt("font_color");

                GeneralPath drawPath;
                if (shape.equals("Ellipse"))
                    drawPath = new GeneralPath(new Ellipse2D.Double(
                        5, 5, 10, 10));
                else if (shape.equals("Round Rectangle"))
                    drawPath = new GeneralPath(
                        new RoundRectangle2D.Double(5, 5, 10, 10,
                            2.5, 2.5));
                else
                    drawPath = new GeneralPath(
                        new Rectangle2D.Double(5, 5, 10, 10));

                type = new Type(new Color(outlineColor), new Color(
                    fillColor), new Color(fontColor), new Font(
                    fontFamily, fontStyle, fontSize), drawPath,
                    shape, typeName, "Obj");
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        return type;
    }

}
