package vp.util.geom;

/**
 * The Intersection class provides methods for determining
 * the intersection point of two lines and the intersection
 * point of a line and a GeneralPath object.
 *
 * @author		Jesus M. Salvo Jr.
 */

import java.awt.Point;
import java.awt.geom.FlatteningPathIterator;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;

public class Intersection
{

	private Intersection()
	{}

	public static Point getIntersection(Line2D line1, Line2D line2)
		throws MultipleIntersectionException
	{
		double dyline1, dxline1;
		double dyline2, dxline2, e, f;
		double x1line1, y1line1, x2line1, y2line1;
		double x1line2, y1line2, x2line2, y2line2;

		if (!line1.intersectsLine(line2))
			return null;

		x1line1 = (double)line1.getX1();
		y1line1 = (double)line1.getY1();
		x2line1 = (double)line1.getX2();
		y2line1 = (double)line1.getY2();

		x1line2 = (double)line2.getX1();
		y1line2 = (double)line2.getY1();
		x2line2 = (double)line2.getX2();
		y2line2 = (double)line2.getY2();

		if ((x1line1 == x1line2) && (y1line1 == y1line2))
		{
			return (new Point((int)x1line1, (int)y1line1));
		}
		if ((x1line1 == x2line2) && (y1line1 == y2line2))
		{
			return (new Point((int)x1line1, (int)y1line1));
		}
		if ((x2line1 == x1line2) && (y2line1 == y1line2))
		{
			return (new Point((int)x2line1, (int)y2line1));
		}
		if ((x2line1 == x2line2) && (y2line1 == y2line2))
		{
			return (new Point((int)x2line1, (int)y2line1));
		}

		dyline1 = - (y2line1 - y1line1);
		dxline1 = x2line1 - x1line1;

		dyline2 = - (y2line2 - y1line2);
		dxline2 = x2line2 - x1line2;

		e = - (dyline1 * x1line1) - (dxline1 * y1line1);
		f = - (dyline2 * x1line2) - (dxline2 * y1line2);

		if ((dyline1 * dxline2 - dyline2 * dxline1) == 0)
			throw new MultipleIntersectionException();
		return (
			new Point(
				(int) (
					- (e * dxline2 - dxline1 * f)
						/ (dyline1 * dxline2 - dyline2 * dxline1)),
				(int) (
					- (dyline1 * f - dyline2 * e)
						/ (dyline1 * dxline2 - dyline2 * dxline1))));
	}

	public static Point getIntersection(Line2D line, GeneralPath path)
	{
		Line2D.Double rectline = new Line2D.Double();
		Point intersectpoint = null;
		Point testintersectpoint = null;
		Point previousintersectpoint;
		Point lineorigin = new Point((int)line.getX1(), (int)line.getY1());
		FlatteningPathIterator pathiterator;
		double coords[] = new double[6];
		int segmenttype;
		double prevx = -1, prevy = -1;

		// Iterate through the segments
		pathiterator =
			new FlatteningPathIterator(path.getPathIterator(null), 1.0);
		while (!pathiterator.isDone())
		{
			segmenttype = pathiterator.currentSegment(coords);
			// Since we are are iterating through a GeneralPath, all segments
			// are guaranteed to be a line.
			switch (segmenttype)
			{
				case PathIterator.SEG_MOVETO :
					prevx = coords[0];
					prevy = coords[1];
					break;
				case PathIterator.SEG_QUADTO :
				case PathIterator.SEG_CUBICTO :
				case PathIterator.SEG_LINETO :
					if (prevx != -1 && prevy != 1)
					{
						rectline.setLine(prevx, prevy, coords[0], coords[1]);
						try
						{
							testintersectpoint =
								getIntersection(line, rectline);
						}
						catch (MultipleIntersectionException ex)
						{
							testintersectpoint = null;
						}

						if (testintersectpoint != null)
						{
							previousintersectpoint = intersectpoint;
							intersectpoint = testintersectpoint;
							if (previousintersectpoint != null
								&& (lineorigin.distance(intersectpoint)
									> lineorigin.distance(
										previousintersectpoint)))
							{
								intersectpoint = previousintersectpoint;
							}
						}
					}
					prevx = coords[0];
					prevy = coords[1];
					break;
			}
			pathiterator.next();
		}
		return intersectpoint;
	}
}
