package vp.util.graphviz;

import vp.graph.DataClass;
import vp.graph.GraphDataClass;
import vp.graph.PathwayDecoration;
import vp.graph.PathwayEdge;
import vp.graph.PathwayGraph;
import vp.graph.PathwayVertex;
import vp.util.StringBreaker;
import vp.util.VPProperties;
import vp.util.database.DBQuery;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.regex.Pattern;

/**
 * @author Matt Holford
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and
 * Comments
 */
/**
 * @author Matt Holford
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and
 * Comments
 */
public class GraphvizToGraph
{
    private BufferedReader br;

    private ArrayList edgesToAdd;

    private File f;

    private File outFile;

    private String graphDataID;

    private GraphDataClass graphData;

    private ArrayList reactions;

    private long startTime;

    private BufferedWriter test;

    private ArrayList verticesToAdd;

    private PathwayGraph pGraph;

    public GraphvizToGraph(PathwayGraph pGraph,
            GraphDataClass graphData)
    {
        this.pGraph = pGraph;
        this.graphData = graphData;

        verticesToAdd = new ArrayList();
        edgesToAdd = new ArrayList();

        pGraph.setGraphDataClass(graphData);
        graphDataID = (String) graphData.get("id");
    }

    private void checkEdge(PathwayVertex vertA, PathwayVertex vertB,
            Set enzymes, Set genes, String reactionID)
    {
        ListIterator it = edgesToAdd.listIterator();
        Iterator eit = enzymes.iterator();
        while (it.hasNext())
        {
            PathwayEdge current = (PathwayEdge) it.next();
            if ((current.getVertexA() == vertA)
                    && (current.getVertexB() == vertB))
                return;

            if ((current.getVertexA() == vertB)
                    && (current.getVertexB() == vertA))
            {
                PathwayEdge old = current;
                PathwayEdge pe = new PathwayEdge(old.getGraph(),
                        DBQuery.getUniqueID(), old.getDataClass(),
                        old.getVertexA(), old.getVertexB(),
                        PathwayEdge.BOTH);

                ListIterator decIter = old.getDecorationIterator();
                while (decIter.hasNext())
                {
                    try
                    {
                        pe.addDecoration((PathwayDecoration) decIter
                                .next());
                    }
                    catch (Exception e)
                    {
                        e.printStackTrace();
                    }
                }
                try
                {
                    edgesToAdd.remove(old);
                    edgesToAdd.add(pe);
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
                return;
            }
        }

        PathwayEdge newLink = new PathwayEdge(pGraph, DBQuery
                .getUniqueID(), new DataClass(reactionID, pGraph,
                true), vertA, vertB, PathwayEdge.FORWARD);

        newLink.getDataClass().put("Enzymes", new ArrayList(enzymes));
        newLink.getDataClass().put("Genes", new ArrayList(genes));

        while (eit.hasNext())
        {
            DataClass current = (DataClass) eit.next();
            PathwayDecoration pwe = new PathwayDecoration(pGraph,
                    DBQuery.getUniqueID(), current, newLink, 1);
            try
            {
                newLink.addDecoration(pwe);
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
        edgesToAdd.add(newLink);
    }

    private PathwayVertex[] checkVertices(Set verts)
    {
        PathwayVertex[] returnVerts = new PathwayVertex[verts.size()];
        Iterator it = verts.iterator();
        int i = 0;
        while (it.hasNext())
        {
            boolean add = true;
            DataClass source = (DataClass) it.next();
            ListIterator currentVerts = verticesToAdd.listIterator();
            while (currentVerts.hasNext())
            {
                PathwayVertex v = (PathwayVertex) currentVerts.next();

                if (v.getDataClass().equals(source))
                {
                    add = false;
                    returnVerts[i] = (PathwayVertex) v;
                    i++;
                    break;
                }
            }
            if (add)
            {
                returnVerts[i] = new PathwayVertex(pGraph, DBQuery
                        .getUniqueID(), source,
                        (int) (Math.random() * 400), (int) (Math
                                .random() * 200), -1, -1);

                verticesToAdd.add(returnVerts[i]);

                i++;
            }
        }
        return returnVerts;
    }

    public void generateGraph()
    {
//        try
//        {
//            outFile = new File("etc\\temp\\testOut.txt");
//            if (!outFile.exists())
//                outFile.createNewFile();
//            test = new BufferedWriter(new FileWriter(outFile));
//        }
//        catch (Exception e)
//        {
//            e.printStackTrace();
//        }

        for (int i = 0; i < reactions.size(); i++)
        {
            DataClass r = (DataClass) reactions.get(i);
            Set sources = (Set) r.get("sources");

            Set sinks = (Set) r.get("sinks");

            Set enzymes = (Set) r.get("enzymes");

            Set genes = (Set) r.get("genes");

            if (enzymes != null)
            {
                pGraph.getParent().getToolBar().getEnzymeButton()
                        .setEnabled(true);
            }

            if (genes != null)
            {
                pGraph.getParent().getToolBar().getGeneButton()
                        .setEnabled(true);
            }

            PathwayVertex[] sourceV = checkVertices(sources);
            PathwayVertex[] sinkV = checkVertices(sinks);

            for (int j = 0; j < sourceV.length; j++)
            {
                for (int k = 0; k < sinkV.length; k++)
                {

                    checkEdge(sourceV[j], sinkV[k], enzymes, genes,
                            (String) r.get("id"));
                }
            }
        }
//         try
//         {
//         test.write("Random positions\n");
//         for (int i = 0; i < verticesToAdd.size(); i++)
//         {
//         PathwayVertex current =
//         (PathwayVertex)verticesToAdd.get(i);
//         test.write(
//         current.getLabel()
//         + ": "
//         + current.getXPosition()
//         + ", "
//         + current.getYPosition()
//         + "\n");
//         test.flush();
//         }
//         test.write("\n\nAssigned Positions");
//         test.flush();
//         }
//         catch (Exception e)
//         {
//         e.printStackTrace();
//         }
//        
//         f = writeDotFile(pGraph);
    }

    public PathwayGraph getGraph()
    {
        return pGraph;
    }

    public void query()
    {
        reactions = DBQuery.getHMRReactions((String) graphData
                .get("id"), pGraph);
    }

    public void runAlgorithm() throws FileNotFoundException
    {
        for (int i = 0; i < verticesToAdd.size(); i++)
            pGraph.addVertex((PathwayVertex) verticesToAdd.get(i));
        for (int i = 0; i < edgesToAdd.size(); i++)
            pGraph.addEdge((PathwayEdge) edgesToAdd.get(i));
        
        SpringEmbeddedLayout spring = new SpringEmbeddedLayout(pGraph);
        spring.doLayout();
        
        pGraph = null;

        pGraph = spring.getPathwayGraph();

        //Shift so everything is within our graph coordinates
        //i.e. x > 0 && y > 0
        
        int lowestX = 500;
        int lowestY = 500;

        Iterator lit = pGraph.getVertexIterator();
        while (lit.hasNext())
        {
            PathwayVertex v = (PathwayVertex) lit.next();
            int xPos = v.getXPosition();
            int yPos = v.getYPosition();
            lowestX = (xPos < lowestX) ? xPos : lowestX;
            lowestY = (yPos < lowestY) ? yPos : lowestY;
        }
        int xShift = 0;
        int yShift = 0;
        if (lowestX < 0)
            xShift = Math.abs(lowestX) + 25;
        if (lowestY < 0)
            yShift = Math.abs(lowestY) + 25;

        lit = pGraph.getVertexIterator();
        while (lit.hasNext())
        {
            PathwayVertex v = (PathwayVertex) lit.next();

            int newX = v.getXPosition() + xShift;
            int newY = v.getYPosition() + yShift;

            v.setXPosition(newX);
            v.setYPosition(newY);
            v.setLocation(newX, newY);
        }
        
        pGraph.repaint();

        // setUpGraphviz(new FileInputStream(f));
    }

    private void setUpGraphviz(InputStream input)
    {
        File graphvizFile = new File(VPProperties.getFilePath()
                + "\\graphv.bat");
        try
        {
            if (graphvizFile.exists())
                graphvizFile.delete();
            graphvizFile.createNewFile();
        }
        catch (IOException e2)
        {
            e2.printStackTrace();
        }
        BufferedWriter bw = null;
        try
        {
            bw = new BufferedWriter(new FileWriter(graphvizFile));
        }
        catch (IOException e3)
        {
            e3.printStackTrace();
        }
        Object connector = null;
        String graphvizString = "neato -Gpack -Goverlap=false "
                + VPProperties.getFilePath()
                + "\\etc\\temp\\temp.dot > "
                + VPProperties.getFilePath()
                + "\\etc\\temp\\outResult.txt";
        try
        {
            bw.write(graphvizString + "\nexit\n");
            bw.flush();
            bw.close();
        }
        catch (IOException e4)
        {
            e4.printStackTrace();
        }
        try
        {
            connector = Runtime.getRuntime().exec("graphv.bat");
        }
        catch (Exception e)
        {
            System.out.println("couldn't run script!!");
            e.printStackTrace();
            connector = null;
        }

        if (connector != null)
        {
            if (connector instanceof Process)
            {
                try
                {
                    int code = ((Process) connector).waitFor();
                    if (code != 0)
                        System.out.println("process exit code is : "
                                + code);
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
            connector = null;
        }

        File outFile = new File("etc\\temp\\outResult.txt");
        try
        {
            br = new BufferedReader(new FileReader(outFile));
            String s;

            Pattern patt = Pattern.compile("(\\d+)");
            //    
            // for (
            // int i = 0; i < stringlist.length; i++) {
            // String currstr = stringlist[i];
            //    
            // // see if the current string has a match
            //    
            // Matcher match = patt.matcher(currstr);
            //    
            // // if a match, print the string text
            // // for the matching group (group 1)
            //    
            // if (match.find()) {
            // System.out.println("For \"" +
            // currstr + "\" match is: " +
            // match.group(1));
            // }

            while ((s = br.readLine()) != null)
            {
                PathwayVertex pv = null;
                s.trim();
                StringTokenizer stkr = new StringTokenizer(s);
                String firstWord = stkr.nextToken();
                if (patt.matcher(firstWord).find())
                {
                    if (s.indexOf("label") != -1)
                    {
                        String preLabel1 = s.substring(s
                                .indexOf("label=") + 6);
                        String preLabel2 = preLabel1.substring(0,
                                preLabel1.indexOf(", pos"));
                        String label = preLabel2.replaceAll("\"", "");
                        label = label.replace('\\', ' ');
                        label = label.replaceAll(" n", "");
                        StringTokenizer ltkr = new StringTokenizer(
                                label, "\n");
                        String label2 = "";
                        while (ltkr.hasMoreTokens())
                        {

                            String tok = ltkr.nextToken();
                            label2 += tok;
                        }
                        Iterator it = verticesToAdd.iterator();
                        while (it.hasNext())
                        {
                            PathwayVertex temp = (PathwayVertex) it
                                    .next();
                            if (temp.getDataClass().get("Name")
                                    .equals(label))
                            {
                                pv = temp;
                                break;
                            }
                        }
                        String prePos = s
                                .substring(s.indexOf("pos=") + 5);
                        String pos = prePos.substring(0, prePos
                                .indexOf("\""));
                        String xPos = pos.substring(0, pos
                                .indexOf(","));
                        String yPos = pos
                                .substring(pos.indexOf(",") + 1);
                        pv.setXPosition(Integer.valueOf(xPos)
                                .intValue());
                        pv.setYPosition(Integer.valueOf(yPos)
                                .intValue());
                    }
                }
            }
        }
        catch (FileNotFoundException e1)
        {
            e1.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        for (int i = 0; i < verticesToAdd.size(); i++)
            pGraph.addVertex((PathwayVertex) verticesToAdd.get(i));
        for (int i = 0; i < edgesToAdd.size(); i++)
            pGraph.addEdge((PathwayEdge) edgesToAdd.get(i));
        // vGraph.repaint();
        graphvizFile.delete();
    }

    private File writeDotFile(PathwayGraph pGraph)
    {
        File temp = null;
        try
        {
            temp = new File("etc\\temp\\temp.dot");
            BufferedWriter bw = new BufferedWriter(new FileWriter(
                    temp));
            Iterator it = edgesToAdd.iterator();
            Iterator iter = verticesToAdd.iterator();
            bw.write("graph \"temp\" { \n");
            // bw.write("node [ label = \"\\N\" ];\n");
            bw.write("edge [len=\"1.5\"];\n");
            while (iter.hasNext())
            {
                PathwayVertex current = (PathwayVertex) iter.next();
                bw.write("\t"
                        + current.getDataClass().get("id")
                        + "[label=\""
                        + StringBreaker.breakStringEscape(current
                                .getLabel()) + "\"];\n");
                bw.flush();
            }
            while (it.hasNext())
            {
                PathwayEdge current = (PathwayEdge) it.next();
                bw.write("\t"
                        + ((PathwayVertex) current.getVertexA())
                                .getDataClass().get("id")
                        + " -- "
                        + ((PathwayVertex) current.getVertexB())
                                .getDataClass().get("id") + ";\n");
                bw.flush();
            }
            bw.write("}");
            bw.flush();

        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return temp;
    }
}
