package vp.util.xml;

import vp.graph.DataClass;
import vp.graph.Decoration;
import vp.graph.GraphDataClass;
import vp.graph.PathwayDecoration;
import vp.graph.DummyVertex;
import vp.graph.PathwayEdge;
import vp.graph.PathwayGraph;
import vp.graph.PathwayVertex;
import vp.util.database.DBQuery;
import vp.visual.type.Type;
import java.awt.Color;
import java.awt.Font;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.io.File;
import java.util.ListIterator;
import java.util.StringTokenizer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

public class XMLToGraph
{
	private PathwayGraph pGraph;
	private File file;

	public static PathwayGraph loadDocument(Document doc, PathwayGraph vGraph)
	{
		return setDocument(doc, vGraph);
	}

	public static PathwayGraph loadXMLFile(PathwayGraph pGraph, File file)
	{
		Document doc = XMLFactory.getLoadDocument(file);
		return setDocument(doc, pGraph);
	}

	/**
	 * @param doc
	 * @param vGraph
	 */
	private static PathwayGraph setDocument(Document doc, PathwayGraph pGraph)
	{
		Element root = doc.getDocumentElement();
		String pathID = root.getAttribute("id");
		String pathName = root.getAttribute("name");
		String orgID = pathID.substring(0, 2);
		String orgName = root.getAttribute("organism");
		String annotation = root.getAttribute("annotation");
		GraphDataClass pathway = new GraphDataClass(pathID);
		pathway.put("id", pathID);
		pathway.put("Name", pathName);
		pathway.put("Organism", orgName);
		pGraph.setGraphDataClass(pathway);
		pGraph.setAnnotation(annotation);

		NodeList vertices = doc.getElementsByTagName("VERTEX");

		for (int i = 0; i < vertices.getLength(); i++)
		{
            Element current = (Element)vertices.item(i);
			readVertex(pGraph, current);
		}

		NodeList links = doc.getElementsByTagName("LINK");

		for (int i = 0; i < links.getLength(); i++)
		{
			Element current = (Element)links.item(i);
			readEdge(pGraph, current);
		}
		return pGraph;
	}

	private static void readEdge(PathwayGraph pGraph, Element current)
	{
		int id = Integer.parseInt(current.getAttribute("id"));
		NodeList data = current.getChildNodes();
		int vertexA = Integer.parseInt(get(data, 0));
		int vertexB = Integer.parseInt(get(data, 1));
		String direction = get(data, 2);
		float curvature = Float.parseFloat(get(data, 3));
		int linkColor = Integer.parseInt(get(data, 4));
		String solid = get(data, 5);

		String linkShape;
		if (solid.equals("N"))
			linkShape = "Dotted";
		else
			linkShape = "Solid";

		Type tempType = new Type(new Color(linkColor), linkShape, "");

		PathwayVertex vA = null;
		PathwayVertex vB = null;

		ListIterator iter = pGraph.getVertexIterator();

		while (iter.hasNext())
		{
			PathwayVertex currentV = (PathwayVertex)iter.next();
			if (currentV.getID() == vertexA)
				vA = currentV;
			else if (currentV.getID() == vertexB)
				vB = currentV;
		}

		DataClass dc = new DataClass("newEdge", pGraph);
		dc.put("id", "newEdge");
		dc.put(
			"Name",
			vA.getDataClass().get("Name")
				+ " -> "
				+ vB.getDataClass().get("Name"));

		PathwayEdge edge =
			new PathwayEdge(pGraph, id, dc, tempType, vA, vB, direction);

		NodeList decs = current.getElementsByTagName("DECORATION");

		for (int j = 0; j < decs.getLength(); j++)
		{
			Node currentE = decs.item(j);
			readDecoration(pGraph, edge, currentE);
		}

		try
		{
			pGraph.addEdge(edge);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private static void readDecoration(
		PathwayGraph pGraph,
		PathwayEdge edge,
		Node current)
	{
		int decID = Integer.parseInt(((Element)current).getAttribute("id"));
        String decClassName = ((Element)current).getAttribute("className");
        
		NodeList data = current.getChildNodes();
		Element decDataClass = (Element)data.item(0);
		String decUniqueID = decDataClass.getAttribute("enzyme_id");
		String decNameID = decDataClass.getAttribute("id");
		String decName = decDataClass.getAttribute("name");
        String decAltName = decDataClass.getAttribute("altName");
		String decAnnotation = decDataClass.getAttribute("annotation");

		String decShape = get(data, 1);
		int decOutlineColor = Integer.parseInt(get(data, 2));
		int decFillColor = Integer.parseInt(get(data, 3));
		String decFont = get(data, 4);
		int decFontSize = Integer.parseInt(get(data, 5));
		int decFontColor = Integer.parseInt(get(data, 6));

		String expression = null;

		if (data.getLength() > 7)
		{
			expression = get(data, 7);
		}

		String fontName = "";
		int fontStyle = 0;
		StringTokenizer stkr = new StringTokenizer(decFont, "_");
		while (stkr.hasMoreTokens())
		{
			fontName = stkr.nextToken();
			fontStyle = Integer.parseInt(stkr.nextToken());
		}

		GeneralPath drawPath;
		if (decShape.equals("Ellipse"))
			drawPath = new GeneralPath(new Ellipse2D.Double(5, 5, 10, 10));
		else if (decShape.equals("Round Rectangle"))
			drawPath =
				new GeneralPath(
					new RoundRectangle2D.Double(5, 5, 10, 10, 2.5, 2.5));
		else
			drawPath = new GeneralPath(new Rectangle2D.Double(5, 5, 10, 10));

		Type tempType =
			new Type(
				new Color(decOutlineColor),
				new Color(decFillColor),
				new Color(decFontColor),
				new Font(fontName, fontStyle, decFontSize),
				drawPath,
				decShape,
				"",
				"");

		DataClass dc = new DataClass(decUniqueID, pGraph);
        dc.setClassName(decClassName);
		dc.put("id", decUniqueID);
		dc.put("Name", decName);
		dc.put("enzID", decNameID);
        dc.put("ECNumber", decAltName);
		dc.put("Annotation", decAnnotation);

		PathwayDecoration d =
			new PathwayDecoration(pGraph, decID, dc, tempType, edge, 1);
		//d.setLabel((String)dc.get("ECNumber"));

		Decoration exp = null;
		if (expression != null)
		{
			DataClass expValue = new DataClass(expression, pGraph);
			exp =
				new PathwayDecoration(
					pGraph,
					DBQuery.getUniqueID(),
					expValue,
					edge,
					-1);

		}

		try
		{
			edge.addDecoration(d);
			if (exp != null)
			{
				edge.addDecoration(exp);
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private static void readVertex(PathwayGraph pGraph, Element current)
	{
		int vertexID = Integer.parseInt(current.getAttribute("id"));
		NodeList data = current.getChildNodes();
        NodeList data2 = current.getElementsByTagName("vertDataClass");

		Element vertDataClass = (Element)data2.item(0);
		String vertDataID = vertDataClass.getAttribute("id");
		String vertDataName = vertDataClass.getAttribute("name");
		String vertDataAnnotation = vertDataClass.getAttribute("annotation");

		int xposition = Integer.parseInt(get(data, 1));
		int yposition = Integer.parseInt(get(data, 2));
		String vertShape = get(data, 3);
		int vertOutlineColor = Integer.parseInt(get(data, 4));
		int vertFillColor = Integer.parseInt(get(data, 5));
		String vertFont = get(data, 6);
		int vertFontSize = Integer.parseInt(get(data, 7));
		int vertFontColor = Integer.parseInt(get(data, 8));
		int onLink = Integer.parseInt(get(data, 9));
		float onLinkPosition = Float.parseFloat(get(data, 10));

		String fontName = "";
		int fontStyle = 0;
		StringTokenizer stkr = new StringTokenizer(vertFont, "_");
		while (stkr.hasMoreTokens())
		{
			fontName = stkr.nextToken();
			fontStyle = Integer.parseInt(stkr.nextToken());
		}

		GeneralPath drawPath;
		if (vertShape.equals("Ellipse"))
			drawPath = new GeneralPath(new Ellipse2D.Double(5, 5, 10, 10));
		else if (vertShape.equals("Round Rectangle"))
			drawPath =
				new GeneralPath(
					new RoundRectangle2D.Double(5, 5, 10, 10, 2.5, 2.5));
		else
			drawPath = new GeneralPath(new Rectangle2D.Double(5, 5, 10, 10));

		Type tempType =
			new Type(
				new Color(vertOutlineColor),
				new Color(vertFillColor),
				new Color(vertFontColor),
				new Font(fontName, fontStyle, vertFontSize),
				drawPath,
				vertShape,
				"",
				"");

		DataClass c = new DataClass(vertDataID, pGraph);
		c.put("id", vertDataID);
		c.put("Name", vertDataName);
		c.put("Annotation", vertDataAnnotation);

		PathwayVertex v = null;

		if (vertDataName.equals("Dummy"))
			v =
				new DummyVertex(
					pGraph,
					xposition,
					yposition,
					onLink,
					onLinkPosition);
		else
			v =
				new PathwayVertex(
					pGraph,
					vertexID,
					c,
					tempType,
					xposition,
					yposition,
					onLink,
					onLinkPosition);

		try
		{
			pGraph.addVertex(v);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public PathwayGraph getGraph()
	{
		return pGraph;
	}

	private static String get(NodeList data, int index)
	{
		if ((((Text)data.item(index).getFirstChild()).getNodeValue()) != null)
			return (((Text)data.item(index).getFirstChild()).getNodeValue());
		return "";
	}
}
