/*
 * Created on Jul 2, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package vp.visual;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import vp.graph.PathwayDecoration;
import vp.graph.PathwayEdge;
import vp.graph.PathwayGraph;
import vp.graph.PathwayVertex;
import vp.visual.paint.DecorationPainterFactory;
import vp.visual.paint.EdgePainterFactory;
import vp.visual.paint.VertexPainterFactory;

/**
 * @author Matt Holford
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public abstract class AbstractGraphRenderer implements GraphRenderer
{
	boolean locked;
	DecorationPainterFactory dpf;
    EdgePainterFactory epf;
    VertexPainterFactory vpf;
	PathwayGraph pg;
	List containers;
	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#addContainer(java.awt.Container)
	 */
	public void addContainer(Container c)
	{
		containers.add(c);
	}

	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#getContainers()
	 */
	public List getContainers()
	{
		return containers;
	}

	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#getSelectedDecoration(int, int)
	 */
	public PathwayDecoration getSelectedDecoration(int x, int y)
	{
		PathwayEdge pe;
		PathwayDecoration pd;

		ListIterator iter = pg.getEdgeIterator();
		while (iter.hasNext())
		{
			pe = (PathwayEdge)iter.next();
			ListIterator li = pe.getDecorationIterator();
			while (li.hasNext())
			{
				pd = (PathwayDecoration)li.next();
				if (pd.containsPoint(x, y))
					return pd;
			}
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#getSelectedEdge(int, int)
	 */
	public PathwayEdge getSelectedEdge(int x, int y)
	{
		PathwayEdge pe;
		ListIterator iter = pg.getEdgeIterator();
		while (iter.hasNext())
		{
			pe = (PathwayEdge)iter.next();
			if (pe.getClickPath().contains(x, y))
				return pe;
		}
        return null;
	}
    
    /* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#getMaxSize()
	 */
	public Dimension getMaxSize()
	{
        PathwayVertex pv;
        Dimension vvDimension;
        Rectangle rect;
        
        int maxwidth = 0;
        int maxheight = 0;

        ListIterator iter = pg.getVertexIterator();
        while (iter.hasNext())
        {
            pv = (PathwayVertex)iter.next();
            rect = pv.getBounds();
            vvDimension = rect.getSize();
            if (rect.y + vvDimension.height > maxheight)
                maxheight = rect.y + vvDimension.height;
            if (rect.x + vvDimension.width > maxwidth)
                maxwidth = rect.x + vvDimension.width;
        }
        return new Dimension(maxwidth, maxheight);
	}

	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#getSelectedVertex(int, int)
	 */
	public PathwayVertex getSelectedVertex(int x, int y)
	{
        PathwayVertex pv;
        
        ListIterator iter = pg.getVertexIterator();
        
        while (iter.hasNext())
        {
            pv = (PathwayVertex)iter.next();
            if (pv.containsPoint(x, y))
                return pv;
        }
        return null;
	}

	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#getDecorationPainterFactory()
	 */
	public DecorationPainterFactory getDecorationPainterFactory()
	{
		return dpf;
	}

	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#getEdgePainterFactory()
	 */
	public EdgePainterFactory getEdgePainterFactory()
	{
		return epf;
	}

	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#getVertexPainterFactory()
	 */
	public VertexPainterFactory getVertexPainterFactory()
	{
		return vpf;
	}

	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#isLocked()
	 */
	public boolean isLocked()
	{
		return locked;
	}

	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#paint()
	 */
	public abstract void paint(Graphics2D g2);

	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#removeContainter(java.awt.Container)
	 */
	public void removeContainer(Container c)
	{
		containers.remove(c);
	}

	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#repaint()
	 */
	public abstract void repaint();
	
	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#setExperimentData(java.util.Map)
	 */
	public abstract void setExperimentData(Map data);
	
	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#setLocked(boolean)
	 */
	public void setLocked(boolean locked)
	{
		this.locked = locked;
	}

	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#setDecorationPainterFactory(vp.visual.paint.DecorationPainterFactory)
	 */
	public void setDecorationPainterFactory(DecorationPainterFactory dpf)
	{
		this.dpf = dpf;
	}

	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#setEdgePainterFactory(vp.visual.paint.EdgePainterFactory)
	 */
	public void setEdgePainterFactory(EdgePainterFactory epf)
	{
		this.epf = epf;
	}

	/* (non-Javadoc)
	 * @see vp.visual.GraphRenderer#setVertexPainterFactory(vp.visual.paint.VertexPainterFactory)
	 */
	public void setVertexPainterFactory(VertexPainterFactory vpf)
	{
		this.vpf = vpf;
	}
}
