package vp.visual;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Panel;
import java.awt.geom.AffineTransform;

/**
* An abstract implementation of <code>LabeledVisualGraphElement</code>.   This class contains default implementations 
* for methods shared by <code>VisualVertex</code> and <code>VisualDecoration</code>.  It is extended by these two 
* classes.
*
* @see edu.yale.graph.visual.LabeledVisualGraphElement
* @see edu.yale.graph.visual.VisualVertex
* @see edu.yale.graph.visual.VisualDecoration
*/
public abstract class AbstractLabeledRenderer
	extends AbstractRenderer
	implements LabeledRenderer
{

	/**
	* Returns the fill color of the visual element
	*
	* @see #setFillColor(Color fillColor)
	*/
	public Color getFillColor()
	{
		return type.getFillColor();
	}

	/**
	* Returns the font used to label the visual element.
	*
	* @see #setFont(Font font)
	*/
	public Font getFont()
	{
		return type.getFont();
	}

	/**
	* Returns the color of the font used to label the visual element.
	*
	* @see #setFontColor(Color fontColor)
	*/
	public Color getFontColor()
	{
		return type.getFontColor();
	}

	/**
	* Returns the <code>FontMetrics</code> object for the font used to label the visual element.
	*/
	public FontMetrics getFontMetrics()
	{
		return new Panel().getFontMetrics(getFont());
	}

	/**
	* Sets the fill color of the visual element to the specified value.
	*
	* @see #getFillColor()
	*/
	public void setFillColor(Color fillColor)
	{
		type.setFillColor(fillColor);
	}

	/**
	* Sets the font used to label the visual element to the specified value.
	*
	* @see #getFont()
	*/
	public void setFont(Font font)
	{
		type.setFont(font);
	}

	/**
	* Sets the color of the font used to label the visual element to the specified value.
	*
	* @see #getFontColor()
	*/
	public void setFontColor(Color fontColor)
	{
		type.setFontColor(fontColor);
	}

	/**
	* Sets the location of the visual element to the specified point.
	*
	* @param x	The x coordinate of the point.
	* @param y	The y coordinate of the point.
	*/
	public void setLocation(double x, double y)
	{
		double transX = x - getBounds2D().getCenterX();
		double transY = y - getBounds2D().getCenterY();
		AffineTransform transform = new AffineTransform();
		transform.translate(transX, transY);
		drawPath.transform(transform);
	}
    
    public abstract String getLabel();
}
