package vp.visual;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;

/**
* An extension of <code>VisualGraphElement</code> to include methods shared by visual decorations and vertices only.
* It is implemented by <code>AbstractLabeledVisualGraphElement</code> from which <code>VisualVertex</code> and 
* <code>VisualDecoration</code> are inherited.
*
* @see edu.yale.graph.visual.VisualGraphElement
* @see edu.yale.graph.visual.VisualVertex
* @see edu.yale.graph.visual.VisualDecoration
*/
public interface LabeledRenderer extends Renderer
{
	/**
	* Returns the fill color of the visual element
	*
	* @see #setFillColor(Color fillColor)
	*/
	Color getFillColor();

	/**
	* Returns the font used to label the visual element.
	*
	* @see #setFont(Font font)
	*/
	Font getFont();

	/**
	* Returns the color of the font used to label the visual element.
	*
	* @see #setFontColor(Color fontColor)
	*/
	Color getFontColor();

	/**
	* Returns the <code>FontMetrics</code> object for the font used to label the visual element.
	*/
	FontMetrics getFontMetrics();

	/**
	* Rescale the element to fit its contents.
	*/
	void rescale();

	/**
	* Sets the fill color of the visual element to the specified value.
	*
	* @see #getFillColor()
	*/
	void setFillColor(Color fillColor);

	/**
	* Sets the font used to label the visual element to the specified value.
	*
	* @see #getFont()
	*/
	void setFont(Font font);

	/**
	* Sets the color of the font used to label the visual element to the specified value.
	*
	* @see #getFontColor()
	*/
	void setFontColor(Color fontColor);

	/**
	* Sets the location of the visual element to the specified point.
	*
	* @param x	The x coordinate of the point.
	* @param y	The y coordinate of the point.
	*/
	void setLocation(double x, double y);

	String getLabel();

}
