package vp.visual.paint;

import vp.graph.Decoration;
import vp.graph.PathwayDecoration;
import vp.graph.PathwayEdge;
import vp.graph.PathwayElement;
import vp.graph.PathwayVertex;
import vp.graph.Vertex;
//import vp.src.edu.yale.graph.visual.Arrowhead;
import vp.util.GraphicsFactory;
import vp.util.geom.CurvePath;
import vp.util.geom.Equation;
import vp.util.geom.Geometry;
import vp.util.geom.Intersection;
import vp.visual.graphics.*;
import vp.visual.type.Type;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.ListIterator;

public class EdgePainterImp implements EdgePainter
{
	private Point hi, hi2, hi3, hi4, hi5, hi6;
	private Point2D.Double s1, s2, e1, e2;

	public EdgePainterImp()
	{}

	public void paint(PathwayElement element, Graphics2D g2)
	{
		PathwayEdge pEdge = (PathwayEdge)element;
		PathwayVertex vertexA = (PathwayVertex)pEdge.getVertexA();
		PathwayVertex vertexB = (PathwayVertex)pEdge.getVertexB();
		String direction = pEdge.getDirection();
		GeneralPath clickPath = pEdge.getClickPath();
		GeneralPath drawPath = pEdge.getGeneralPath();
		Rectangle vABounds = vertexA.getBounds();
		Rectangle vBBounds = vertexB.getBounds();
		Rectangle vAExtraBounds =
			new Rectangle(
				vABounds.x - 10,
				vABounds.y - 10,
				vABounds.width + 20,
				vABounds.height + 20);
		Rectangle vBExtraBounds =
			new Rectangle(
				vBBounds.x - 10,
				vBBounds.y - 10,
				vBBounds.width + 20,
				vBBounds.height + 20);
		drawPath.reset();
		clickPath.reset();

		Line2D.Double hypLine =
			new Line2D.Double(
				vABounds.getCenterX(),
				vABounds.getCenterY(),
				vBBounds.getCenterX(),
				vBBounds.getCenterY());
		Line2D.Double hypLine2 =
			new Line2D.Double(
				vABounds.getCenterX() - 8,
				vABounds.getCenterY() - 8,
				vBBounds.getCenterX() - 8,
				vBBounds.getCenterY() - 8);
		Line2D.Double hypLine3 =
			new Line2D.Double(
				vABounds.getCenterX() + 8,
				vABounds.getCenterY() + 8,
				vBBounds.getCenterX() + 8,
				vBBounds.getCenterY() + 8);
		hi =
			Intersection.getIntersection(
				hypLine,
				vertexA.getGeneralPath());
		hi2 =
			Intersection.getIntersection(
				hypLine,
				vertexB.getGeneralPath());
		hi3 =
			Intersection.getIntersection(
				hypLine2,
				new GeneralPath(vAExtraBounds));
		hi4 =
			Intersection.getIntersection(
				hypLine2,
				new GeneralPath(vBExtraBounds));
		hi5 =
			Intersection.getIntersection(
				hypLine3,
				new GeneralPath(vAExtraBounds));
		hi6 =
			Intersection.getIntersection(
				hypLine3,
				new GeneralPath(vBExtraBounds));

		if (pEdge.getCurvature() == 0)
		{
			if (hi != null && hi2 != null)
			{
				drawPath.moveTo(hi.x, hi.y);
				drawPath.lineTo(hi2.x, hi2.y);
			}
			else
				return;

			if (hi3 != null && hi4 != null && hi5 != null && hi6 != null)
			{
				clickPath.moveTo(hi3.x, hi3.y);
				clickPath.lineTo(hi4.x, hi4.y);
				clickPath.lineTo(hi6.x, hi6.y);
				clickPath.lineTo(hi5.x, hi5.y);
			}
			else
				return;
		}

		else
		{
			float pos = 0f;

			if (hi.y < hi2.y)
			{
				s1 =
					new Point2D.Double(
						vABounds.getCenterX(),
						vABounds.getY() + vABounds.getHeight());
				s2 = new Point2D.Double(s1.x, s1.y + 10);
				if (hi.x < hi2.x)
					pos = 1;
				else
					pos = -1;
			}
			else if (hi.y > hi2.y)
			{
				s1 = new Point2D.Double(vABounds.getCenterX(), vABounds.getY());
				s2 = new Point2D.Double(s1.x, s1.y - 10);
				if (hi.x < hi2.x)
					pos = -1;
				else
					pos = 1;
			}

			else
			{
				if (hi.x < hi2.x)
				{
					s1 =
						new Point2D.Double(
							vABounds.getX() + vABounds.getWidth(),
							vABounds.getCenterY());
					s2 = new Point2D.Double(s1.x + 10, s1.y);
					pos = 1;
				}
				else
				{
					s1 =
						new Point2D.Double(
							vABounds.getX(),
							vABounds.getCenterY());
					s2 = new Point2D.Double(s1.x - 10, s1.y);
					pos = -1;
				}
			}

			if (hi.x < hi2.x)
			{
				e1 = new Point2D.Double(vBBounds.getX(), vBBounds.getCenterY());
				e2 = new Point2D.Double(e1.x - 10, e1.y);
			}
            
			else if (hi.x >= hi2.x)
			{
				e1 =
					new Point2D.Double(
						vBBounds.getX() + vBBounds.getWidth(),
						vBBounds.getCenterY());
				e2 = new Point2D.Double(e1.x + 10, e1.y);
			}

			else
			{
				if (hi.y < hi2.y)
				{
					e1 =
						new Point2D.Double(
							vBBounds.getCenterX(),
							vBBounds.getY());
					e2 = new Point2D.Double(e1.x, e1.y - 10);
				}
				else
				{
					e1 =
						new Point2D.Double(
							vBBounds.getCenterX(),
							vBBounds.getY() + vBBounds.getHeight());
					e2 = new Point2D.Double(e1.x, e1.y + 10);
				}
			}

			drawPath.moveTo((float)s1.x, (float)s1.y);
			drawPath.lineTo((float)s2.x, (float)s2.y);

			drawPath.append(CurvePath.getCurvePath(s2, e2, pos), false);

			drawPath.lineTo((float)e1.x, (float)e1.y);

			clickPath =
				CurvePath.getCurveClickPath(
					new Point2D.Double(hi3.x, hi3.y),
					new Point2D.Double(hi4.x, hi4.y),
					new Point2D.Double(hi5.x, hi5.y),
					new Point2D.Double(hi6.x, hi6.y));
		}

		g2.setRenderingHint(
			RenderingHints.KEY_ANTIALIASING,
			RenderingHints.VALUE_ANTIALIAS_ON);

		if ((pEdge.getShape() != null) && (pEdge.getShape().equals(Type.DOTTED_LINE)))
			g2.setStroke(GraphicsFactory.getDashed());

		if (pEdge.getTexturePaint() != null)
			g2.setColor(Color.lightGray);
		else
			g2.setColor(pEdge.getOutlineColor());

		g2.draw(drawPath);
		g2.setStroke(new BasicStroke());

		paintDecorations(pEdge, g2);
		if (direction.equals(PathwayEdge.FORWARD))
			paintArrowhead(pEdge, g2, true);
		else if (direction.equals(PathwayEdge.BACKWARD))
			paintArrowhead(pEdge, g2, false);
		else if (direction.equals(PathwayEdge.BOTH))
		{
			paintArrowhead(pEdge, g2, true);
			paintArrowhead(pEdge, g2, false);
		}
	}

	public void paintDecorations(PathwayEdge vEdge, Graphics2D g2)
	{
		ArrayList posDecs = new ArrayList();
		ArrayList negDecs = new ArrayList();
		int j = 0;
		Decoration paintLater = null;

		ListIterator iter = vEdge.getDecorationIterator();
		while (iter.hasNext())
		{
			PathwayDecoration current = (PathwayDecoration)iter.next();
			if (current.getRelPosition() < 0)
				negDecs.add(current);
			else
				posDecs.add(current);
		}

		int noNegDecs = negDecs.size();
		for (int i = 0; i < noNegDecs; i++)
		{
			PathwayDecoration newDec = (PathwayDecoration)negDecs.get(i);
			if (newDec.hasToolTip())
			{
				j = i;
				paintLater = newDec;
			}
			else
				paintDecNow(newDec, vEdge, noNegDecs, i, g2);
		}

		if (paintLater != null)
			paintDecNow(paintLater, vEdge, noNegDecs, j, g2);

		paintLater = null;

		int noPosDecs = posDecs.size();
		for (int i = 0; i < noPosDecs; i++)
		{
			PathwayDecoration newDec = (PathwayDecoration)posDecs.get(i);
			if (newDec.hasToolTip())
			{
				j = i;
				paintLater = newDec;
			}
			else
				paintDecNow(newDec, vEdge, noPosDecs, i, g2);
		}

		if (paintLater != null)
			paintDecNow(paintLater, vEdge, noPosDecs, j, g2);

		paintLater = null;
	}

	private void paintDecNow(
		Decoration newDec,
		PathwayEdge pEdge,
		int noDecs,
		int decNo,
		Graphics2D g2)
	{
		Point2D.Double point1 =
			new Point2D.Double((double)hi.getX(), (double)hi.getY());
		Point2D.Double point2 =
			new Point2D.Double((double)hi2.getX(), (double)hi2.getY());
		PathwayDecoration current = (PathwayDecoration)newDec;
		Rectangle2D r2 = current.getBounds2D();
		double maxDist = r2.getWidth() + 3;
		double minDist = r2.getHeight() + 3;
		double avgDist = (maxDist + minDist) / 2.0d;
		double moveDist = maxDist - avgDist;
		Equation eq = Geometry.getEquation(point1, point2);
		double logg = Geometry.logg(eq.slope);
		double dist = avgDist + (logg * moveDist);

		double percent = ((double)decNo + 1) / ((double)noDecs + 1);

		Point2D.Double spot =
			Geometry.getPtOnPerp(
				point1,
				point2,
				percent,
				(int)dist,
				current.getRelPosition());
		current.setLocation(spot.x, spot.y);
		if (pEdge.getTexturePaint() != null)
			current.setTexturePaint(pEdge.getTexturePaint());
		else
			current.setTexturePaint(null);
		current.paint(g2);

	}

	public void paintArrowhead(
		PathwayEdge vEdge,
		Graphics2D g2,
		boolean forward)
	{
		Point intersection, other;

		Vertex v1 = vEdge.getVertexA();
		Vertex v2 = vEdge.getVertexB();
        
		if (vEdge.getCurvature() == 0)
		{
			intersection = forward ? hi2 : hi;
			other = forward ? hi : hi2;
		}
		else
		{
			intersection =
				forward
					? new Point((int)e1.x, (int)e1.y)
					: new Point((int)s1.x, (int)s1.y);
			other =
				forward
					? new Point((int)e2.x, (int)e2.y)
					: new Point((int)s2.x, (int)s2.y);
		}

		Point2D.Double tip = new Point2D.Double(intersection.x, intersection.y);
		Line2D.Double line =
			new Line2D.Double(tip, new Point2D.Double(other.x, other.y));

		GeneralPath ahead = Arrowhead.getArrowhead(tip, line);

		g2.setRenderingHint(
			RenderingHints.KEY_ANTIALIASING,
			RenderingHints.VALUE_ANTIALIAS_ON);
		if (vEdge.getTexturePaint() != null)
			g2.setPaint(vEdge.getTexturePaint());
		else
			g2.setColor(vEdge.getOutlineColor());

		g2.fill(ahead);
	}
}
