package vp.visual.type;

import java.awt.Color;
import java.awt.Font;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;

/**
* A default decoration type, generally used to represent expression values.
*/
public class DefaultDecorationType2 extends Type
{
	/**
	* Creates a type using the default values with the color dependent on the value.		<br><br>
	* Outline color:	black(0, 0, 0)				<br>
	* Fill color:		variable					<br>
	* Font color:		black(0, 0, 0)				<br>
	* Font:				Dialog 10-pt				<br>
	* Draw path:		Rectangle (5, 5, 10, 10)	<br>
	* Shape:			"Rectangle"					<br>
	* Name:				"Default Decoration Type 2"	<br>
	* Short name:		"Type2"						<br>
	*
	* @param value	A string representing the expression value or "NA" if unavailable.
	*/
	public DefaultDecorationType2(String value)
	{
		try
		{
			double d = Double.parseDouble(value);
			if (d >= 4)
				fillColor = new Color(255, 0, 0);
			else if (d >= 3 && d < 4)
				fillColor = new Color(255, 74, 0);
			else if (d >= 2 && d < 3)
				fillColor = new Color(255, 150, 0);
			else if (d >= 1 && d < 2)
				fillColor = new Color(255, 200, 0);
			else if (d >= .5 && d < 1)
				fillColor = new Color(255, 255, 0);
			else if (d >= .33 && d < .5)
				fillColor = new Color(128, 255, 0);
			else if (d >= .25 && d < .33)
				fillColor = new Color(0, 255, 0);
			else if (d < .25)
				fillColor = new Color(0, 229, 0);
		}
		catch (NumberFormatException nfe)
		{
			fillColor = Color.lightGray;
		}
		outlineColor = fillColor;
		fontColor = Color.black;
		font = new Font("Dialog", Font.PLAIN, 9);
		drawPath = new GeneralPath(new Rectangle2D.Float(5, 5, 10, 10));
		shape = "Rectangle";
		name = "Default Decoration Type 2";
		shortName = "Type2";
	}
}
